/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
\file CalculateLayerArea.cpp

\brief This file has the class used to calculate the geometries area of a layer.
*/

// TerraAmazon
#include "CalculateLayerArea.h"

// TerraLib
#include <terralib/common/Exception.h>
#include <terralib/dataaccess/utils/Utils.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/SimpleProperty.h>
#include <terralib/geometry/GeometryCollection.h>
#include <terralib/geometry/MultiPolygon.h>
#include <terralib/memory/DataSet.h>
#include <terralib/memory/DataSetItem.h>
#include <terralib/srs/Config.h>
#include <terralib/srs/Converter.h>
#include <terralib/srs/SpatialReferenceSystemManager.h>


ta::common::CalculateLayerArea::CalculateLayerArea() :
  m_srid(TE_UNKNOWN_SRS),
  m_createColumnArea(false),
  m_columnName(""),
  m_errorMessage(""),
  m_polyCount(0),
  m_totArea(0.0)
{
}

ta::common::CalculateLayerArea::~CalculateLayerArea()
{
}

void ta::common::CalculateLayerArea::setInputParameters(te::map::AbstractLayerPtr layer, te::common::UnitOfMeasurePtr unit, int srid, bool createColumnArea, std::string columnName)
{
  m_layer = layer;
  m_unit = unit;
  m_srid = srid;
  m_createColumnArea = createColumnArea;
  m_columnName = columnName;
}

void ta::common::CalculateLayerArea::getOutputParameters(std::size_t& polyCount, double& totArea)
{
  polyCount = m_polyCount;
  totArea = m_totArea;
}

void ta::common::CalculateLayerArea::execute()
{
  if (!checkParameters())
    throw te::common::Exception(m_errorMessage);

  //check if necessary geometry remap
  int inputSRID = m_layer->getSRID();

  bool remap = false;
  if (inputSRID != m_srid)
  {
    remap = true;
  }

  //create new column
  std::string dataSetName = "";
  std::vector<std::size_t> ids;
  std::vector< std::set<int> > propsPos;
  te::mem::DataSet* newData = 0;
  std::size_t areaPropPos = 0;

  if (m_createColumnArea)
  {
    std::auto_ptr<te::da::DataSetType> dsType = m_layer->getSchema();
    dataSetName = dsType->getTitle();

    //always getting updated data set type
    dsType = m_ds->getDataSetType(dataSetName);

    te::dt::SimpleProperty* prop = new te::dt::SimpleProperty(m_columnName, te::dt::DOUBLE_TYPE);

    m_ds->addProperty(dsType->getName(), prop);

    //get pos of attr used as pk
    te::da::PrimaryKey* pk = dsType->getPrimaryKey();

    size_t numProps = pk->getProperties().size();
    
    for (size_t i = 0; i < numProps; ++i)
    {
      std::string propName = pk->getProperties()[i]->getName();
      ids.push_back(dsType->getPropertyPosition(propName));
    }

    //get updated dataset type
    dsType = m_ds->getDataSetType(dataSetName);

    newData = new te::mem::DataSet(dsType.get());

    areaPropPos = dsType->getPropertyPosition(m_columnName);
  }

  //calculate area
  std::auto_ptr<te::da::DataSet> dataSet = m_layer->getData();
  std::size_t geomPos = te::da::GetFirstSpatialPropertyPos(dataSet.get());

  dataSet->moveBeforeFirst();
  while (dataSet->moveNext())
  {
    //get geom and convert if necessary
    std::auto_ptr<te::gm::Geometry> geom = dataSet->getGeometry(geomPos);
    geom->setSRID(inputSRID);

    if (remap)
      geom->transform(m_srid);

    //calculate area
    double area = calculateArea(geom.get());

    double convertedArea = converteArea(area);

    //update
    if (m_createColumnArea)
    {
      te::mem::DataSetItem* dsItem = new te::mem::DataSetItem(newData);

      //adding key properties
      for (std::size_t p = 0; p < ids.size(); ++p)
      {
        te::dt::AbstractData* ad = dataSet->getValue(ids[p]).release();
        dsItem->setValue(ids[p], ad);
      }

      //adding area value
      dsItem->setValue(areaPropPos, new te::dt::SimpleData<double, te::dt::DOUBLE_TYPE>(convertedArea));

      std::set<int> attrPos;
      attrPos.insert((int)areaPropPos);

      propsPos.push_back(attrPos);

      newData->add(dsItem);
    }

    m_totArea += convertedArea;
    ++m_polyCount;
  }

  if (m_createColumnArea)
  {
    m_ds->update(dataSetName, newData, propsPos, ids);
  }
}

bool ta::common::CalculateLayerArea::checkParameters()
{
  //check layer
  if (!m_layer.get())
  {
    m_errorMessage = "Input Layer not defined";
    return false;
  }

  if (m_layer->getSRID() == TE_UNKNOWN_SRS)
  {
    m_errorMessage = "Input Layer has no SRID defined";
    return false;
  }

  //check dataset type
  std::auto_ptr<te::da::DataSetType> dsType = m_layer->getSchema();

  if (!dsType->hasGeom())
  {
    m_errorMessage = "Input Layer has no Geometry information";
    return false;
  }

  //check unit
  if (!m_unit.get())
  {
    m_errorMessage = "Unit not defined";
    return false;
  }

  //check projection
  if (m_srid == TE_UNKNOWN_SRS)
  {
    m_errorMessage = "Invalid SRID.";
    return false;
  }

  if (m_createColumnArea)
  {
    //check column name
    if (m_columnName == "")
    {
      m_errorMessage = "Column name not defined.";
      return false;
    }

    std::vector<te::dt::Property*> props = dsType->getProperties();

    for (std::size_t t = 0; t < props.size(); ++t)
    {
      if (props[t]->getName() == m_columnName)
      {
        m_errorMessage = "Column name already defined.";
        return false;
      }
    }

    //get data source
    std::string dsId = m_layer->getDataSourceId();

    m_ds = te::da::GetDataSource(dsId);

    if (!m_ds.get())
    {
      m_errorMessage = "Error getting data source from layer.";
      return false;
    }
  }

  return true;
}

double ta::common::CalculateLayerArea::calculateArea(te::gm::Geometry* geom)
{
  double value = 0.0;

  te::gm::GeomType geomType = geom->getGeomTypeId();

  if (geomType == te::gm::PolygonType)
  {
    te::gm::Polygon* pol = dynamic_cast<te::gm::Polygon*>(geom);
    if (pol)
      value = pol->getArea();
  }

  if (geomType == te::gm::MultiPolygonType)
  {
    std::string type = geom->getGeometryType();

    if (type == "GeometryCollection")
    {
      te::gm::GeometryCollection* geoColl = dynamic_cast<te::gm::GeometryCollection*>(geom);

      if (geoColl)
      {
        std::vector<te::gm::Geometry*> geomVec = geoColl->getGeometries();
        for (std::size_t i = 0; i < geomVec.size(); ++i)
        {
          te::gm::GeomType inType = geomVec[i]->getGeomTypeId();
          if (inType == te::gm::PolygonType)
          {
            te::gm::Polygon* pol = dynamic_cast<te::gm::Polygon*>(geomVec[i]);
            if (pol)
              value += pol->getArea();
          }
          if (inType == te::gm::MultiPolygonType)
          {
            te::gm::MultiPolygon* pol = dynamic_cast<te::gm::MultiPolygon*>(geomVec[i]);
            if (pol)
              value += pol->getArea();
          }
        }
      }
    }
    else
    {
      te::gm::MultiPolygon* pol = dynamic_cast<te::gm::MultiPolygon*>(geom);
      if (pol)
        value = pol->getArea();
    }
  }

  return value;
}

double ta::common::CalculateLayerArea::converteArea(double value)
{
  te::common::UnitOfMeasurePtr from = te::srs::SpatialReferenceSystemManager::getInstance().getUnit(m_srid);

  if (from->getName() == "METRE")
  {
    from = te::common::UnitsOfMeasureManager::getInstance().find(9201); //square meter
  }

  te::common::UnitOfMeasurePtr to = m_unit;

  if (from->getName() == to->getName())
  {
    return value;
  }

  if (te::common::UnitsOfMeasureManager::getInstance().areConvertible(from->getName(), to->getName()))
  {
    try
    {
      double factor = te::common::UnitsOfMeasureManager::getInstance().getConversion(from->getName(), to->getName());
      value *= factor;
      return value;
    }
    catch (te::common::Exception& /*e*/)
    {
      return 0.0;
    }
  }

  // try to handle the most common: degree <-> meters and its variations
  if (from->getId() == te::common::UOM_Degree)
  {
   bool res = convertAngleToPlanar(value, to);

   if (res)
     return value;
  }

  if (to->getId() == te::common::UOM_Degree)
  {
    bool res = convertPlanarToAngle(value, from);

    if (res)
      return value;
  }

  return 0.0;
}

bool ta::common::CalculateLayerArea::convertAngleToPlanar(double& val, te::common::UnitOfMeasurePtr planar)
{
  switch (planar->getId())
  {
  case te::common::UOM_Metre:
    val *= 111000. * 111000.;            // 1 degree = 111.000 meters
    break;
  case te::common::UOM_Kilometre: 
    val *= 111. * 111.;                  // 1 degree = 111 kilometers
    break;
  case te::common::UOM_Foot:
    val *= 364173.24 * 364173.24;        //  1 feet  = 3.28084 meters
    break;
  default:
    return false;
  }
  return true;
}

bool ta::common::CalculateLayerArea::convertPlanarToAngle(double& val, te::common::UnitOfMeasurePtr planar)
{
  switch (planar->getId())
  {
  case te::common::UOM_Metre:
    val /= 111000. * 111000.;            // 1 degree = 111.000 meters
    break;
  case te::common::UOM_Kilometre:
    val /= 111. * 111.;               // 1 degree = 111 kilometers
    break;
  case te::common::UOM_Foot:
    val /= 364173.24 * 364173.24;        //  1 feet  = 3.28084 meters
    break;
  default:
    return false;
  }
  return true;
}
