/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
\file terraamazon/TerraAmazon.cpp

\brief The main class of TerraAmazon.
*/

//TerraAmazon Includes
#include "../common/qt/AboutDialog.h"
#include "../common/qt/CalculateLayerAreaDialog.h"
#include "../common/qt/ConnectLayersDockWidget.h"
#include "../common/qt/StatusBarInfoWidget.h"
#include "../common/qt/WelcomeDialog.h"
#include "../edition/qt/Utils.h"
#include "../events/WorkspaceEvents.h"
#include "../model/facade/UserFacade.h"
#include "../model/factory/AoiLayerFactory.h"
#include "../model/factory/CatalogFactory.h"
#include "../model/factory/CellFactory.h"
#include "../model/factory/ClassFactory.h"
#include "../model/factory/GroupFactory.h"
#include "../model/factory/PhaseFactory.h"
#include "../model/factory/ProcessFactory.h"
#include "../model/factory/ProjectFactory.h"
#include "../model/factory/ProjectSceneFactory.h"
#include "../model/factory/RuleFactory.h"
#include "../model/factory/RuleRuleSetFactory.h"
#include "../model/factory/RulesLayerColumnFactory.h"
#include "../model/factory/RuleSetFactory.h"
#include "../model/factory/SceneFactory.h"
#include "../model/factory/TaskFactory.h"
#include "../model/factory/TaskLogFactory.h"
#include "../model/factory/UserFactory.h"
#include "../model/item/User.h"
#include "../model/qt/accessControl/GroupControlDialog.h"
#include "../model/qt/accessControl/UserControlDialog.h"
#include "../model/qt/conceptualModel/CreateConceptualModelDialog.h"
#include "../model/qt/project/AOIManagementWizard.h"
#include "../model/qt/project/AttributeControlDialog.h"
#include "../model/qt/conceptualModel/CatalogManagerDialog.h"
#include "../model/qt/project/ClassControlDialog.h"
#include "../model/qt/project/ClassControlWidget.h"
#include "../model/qt/project/PhaseControlDialog.h"
#include "../model/qt/project/ProjectActivationDialog.h"
#include "../model/qt/project/ProjectControlDialog.h"
#include "../model/qt/project/ProjectControlWidget.h"
#include "../model/qt/project/ProjectInformationDialog.h"
#include "../model/qt/project/ProjectManagementWizard.h"
#include "../model/qt/project/RulesControlDialog.h"
#include "../model/qt/project/ProcessControlDialog.h"
#include "../model/qt/project/ProcessCellManagerDialog.h"
#include "../model/qt/project/EndProcessDialog.h"
#include "../model/Utils.h"
#include "../workspace/XMLFormatter.h"
#include "../workspace/Workspace.h"
#include "../workspace/qt/ServerLoginDialog.h"
#include "../workspace/qt/ServerManagerDialog.h"
#include "../workspace/qt/WorkspaceInfoDialog.h"
#include "../workspace/qt/WorkspaceWidgetFactory.h"
#include "TerraAmazon.h"
#include "Utils.h"

// TerraLib
#include <terralib/common/PlatformUtils.h>
#include <terralib/common/Translator.h>
#include <terralib/dataaccess/datasource/DataSourceInfoManager.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>
#include <terralib/geometry/GeometryProperty.h>
#include <terralib/maptools/DataSetLayer.h>
#include <terralib/maptools/FolderLayer.h>
#include <terralib/qt/af/ApplicationController.h>
#include <terralib/qt/af/connectors/DataSetTableDockWidget.h>
#include <terralib/qt/af/connectors/LayerExplorer.h>
#include <terralib/qt/af/connectors/MapDisplay.h>
#include <terralib/qt/af/connectors/StyleExplorer.h>
#include <terralib/qt/af/events/ApplicationEvents.h>
#include <terralib/qt/af/events/LayerEvents.h>
#include <terralib/qt/af/Utils.h>
#include <terralib/qt/widgets/canvas/MultiThreadMapDisplay.h>
#include <terralib/qt/widgets/canvas/ZoomInMapDisplayWidget.h>
#include <terralib/qt/widgets/datasource/connector/AbstractDataSourceConnector.h>
#include <terralib/qt/widgets/datasource/core/DataSourceType.h>
#include <terralib/qt/widgets/datasource/core/DataSourceTypeManager.h>
#include <terralib/qt/widgets/datasource/selector/DataSourceSelectorDialog.h>
#include <terralib/qt/widgets/exchanger/DirectExchangerDialog.h>
#include <terralib/qt/widgets/exchanger/DataExchangerWizard.h>
#include <terralib/qt/widgets/externalTable/DataPropertiesDialog.h>
#include <terralib/qt/widgets/help/HelpManager.h>
#include <terralib/qt/widgets/layer/explorer/LayerItem.h>
#include <terralib/qt/widgets/layer/explorer/LayerItemView.h>
#include <terralib/qt/widgets/layer/explorer/TreeItem.h>
#include <terralib/qt/widgets/layer/selector/AbstractLayerSelector.h>
#include <terralib/qt/widgets/layer/utils/CompositionModeMenuWidget.h>
#include <terralib/qt/widgets/plugin/manager/PluginManagerDialog.h>
#include <terralib/qt/widgets/query/QueryDataSourceDialog.h>
#include <terralib/qt/widgets/query/QueryDialog.h>
#include <terralib/qt/widgets/query/QueryLayerBuilderWizard.h>
#include <terralib/qt/widgets/se/GroupingDialog.h>
#include <terralib/qt/widgets/se/StyleDockWidget.h>
#include <terralib/qt/widgets/tools/Info.h>
#include <terralib/qt/widgets/tools/Pan.h>
#include <terralib/qt/widgets/tools/Selection.h>
#include <terralib/qt/widgets/tools/ZoomArea.h>
#include <terralib/qt/widgets/tools/ZoomClick.h>
#include <terralib/qt/widgets/Utils.h>

// STL
#include <memory>

// Qt
#include <QAction>
#include <QApplication>
#include <QCloseEvent>
#include <QDialog>
#include <QDesktopWidget>
#include <QFileDialog>
#include <QInputDialog>
#include <QMenu>
#include <QMenuBar>
#include <QMessageBox>
#include <QToolBar>

// Boost
#include <boost/filesystem.hpp>
#include <boost/format.hpp>
#include <boost/uuid/random_generator.hpp>
#include <boost/uuid/uuid_io.hpp>

QString GetWindowTitle(const ta::workspace::WorkspaceMetadata& workspace, te::qt::af::ApplicationController* app)
{
  QString title = app->getAppTitle() + " - ";
  title += TE_TR("Workspace:");
  title += " ";
  title += workspace.m_title;
  title += " - ";

  boost::filesystem::path p(workspace.m_fileName.toStdString());

  std::string filename = p.filename().string();

  title += filename.c_str();

  return title;
}

void GetWorkspacesFromSettings(QStringList& wsTitles, QStringList& wsPaths, QStringList& wsServers, QStringList& wsUsers)
{
  QSettings s(QSettings::IniFormat,
    QSettings::UserScope,
    qApp->organizationName(),
    qApp->applicationName());

  s.beginGroup("workspaces");

  s.beginGroup("most_recent");
  QString path = s.value("path").toString();
  QString title = s.value("title").toString();
  QString server = s.value("server").toString();
  QString user = s.value("user").toString();

  s.endGroup();

  if (path.isEmpty() || title.isEmpty())
    return;

  wsPaths.append(path);
  wsTitles.append(title);
  if (!server.isEmpty())
    wsServers.append(server);
  if (!user.isEmpty())
    wsUsers.append(user);

  int size = s.beginReadArray("recents");

  for (int i = 0; i<size; i++)
  {
    s.setArrayIndex(i);

    path = s.value("workspace/path").toString();
    title = s.value("workspace/title").toString();
    server = s.value("workspace/server").toString();
    user = s.value("workspace/user").toString();

    wsPaths.append(path);
    wsTitles.append(title);
    if (!server.isEmpty())
      wsServers.append(server);
    if (!user.isEmpty())
      wsUsers.append(user);
  }

  s.endArray();
  s.endGroup();
}

void WriteWorkspacesToSettings(QStringList& wsTitles, QStringList& wsPaths, QStringList& wsServers, QStringList& wsUsers)
{
  if (!wsPaths.isEmpty())
  {
    QSettings s(QSettings::IniFormat,
      QSettings::UserScope,
      qApp->organizationName(),
      qApp->applicationName());

    s.beginGroup("workspaces");
    s.beginGroup("most_recent");
    s.setValue("path", *wsPaths.begin());
    s.setValue("title", *wsTitles.begin());
    if (!wsServers.isEmpty())
      s.setValue("server", *wsServers.begin());
    if(!wsUsers.isEmpty())
      s.setValue("user", *wsUsers.begin());
    s.endGroup();

    s.beginWriteArray("recents");
    for (int i = 1; i< wsTitles.size(); ++i)
    {
      s.setArrayIndex(i - 1);

      s.setValue("workspace/path", wsPaths.at(i));
      s.setValue("workspace/title", wsTitles.at(i));
      if(!wsServers.isEmpty() && wsServers.size() > i)
        s.setValue("workspace/server", wsServers.at(i));
      if(!wsUsers.isEmpty() && wsUsers.size() > i)
        s.setValue("workspace/user", wsUsers.at(i));
    }
    s.endArray();
    s.endGroup();
  }
}

void AddRecentWorkspaceToSettings(const QString& wsTitle, const QString& wsPath, const QString& wsServer, const QString& wsUser)
{
  QStringList wsPaths, wsTitles, wsServers, wsUsers;

  GetWorkspacesFromSettings(wsTitles, wsPaths, wsServers, wsUsers);

  if (!wsPaths.contains(wsPath))
  {
    wsTitles.prepend(wsTitle);
    wsPaths.prepend(wsPath);
    if (!wsServer.isEmpty())
      wsServers.prepend(wsServer);
    if (!wsUser.isEmpty())
      wsUsers.prepend(wsUser);
  }
  else
  {
    int wsPathIdx = wsPaths.indexOf(wsPath);

    if (!wsPaths.contains(wsPath))
    {
      wsPaths.replace(wsPathIdx, wsPath);
    }

    if (!wsTitles.contains(wsTitle))
    {
      wsTitles.replace(wsPathIdx, wsTitle);
    }

    if (!wsServer.isEmpty() && !wsServers.contains(wsServer))
    {
      wsServers.replace(wsPathIdx, wsServer);
    }

    if (!wsUser.isEmpty() && !wsUsers.contains(wsUser))
    {
      wsUsers.replace(wsPathIdx, wsUser);
    }

    wsTitles.move(wsPathIdx, 0);
    wsPaths.move(wsPathIdx, 0);
    if ((!wsServers.isEmpty()) && (wsServers.size() > wsPathIdx))
      wsServers.move(wsPathIdx, 0);
    if ((!wsUsers.isEmpty()) && (wsUsers.size() > wsPathIdx))
      wsUsers.move(wsPathIdx, 0);
  }

  WriteWorkspacesToSettings(wsTitles, wsPaths, wsServers, wsUsers);
}

void ResetWorkspace(ta::workspace::WorkspaceMetadata* p)
{
  p->m_author = "INPE";
  p->m_title = QObject::tr("Default workspace");
  p->m_changed = false;
}

QModelIndex GetParent(QTreeView* view)
{
  QModelIndex res;

  QModelIndexList idxs = view->selectionModel()->selectedIndexes();

  if (idxs.size() == 1)
  {
    QModelIndex idx = idxs.at(0);
    te::qt::widgets::TreeItem* item = static_cast<te::qt::widgets::TreeItem*>(idx.internalPointer());

    if (item->getType() == "FOLDER")
      res = idx;
  }

  return res;
}

te::qt::af::DataSetTableDockWidget* GetLayerDock(const te::map::AbstractLayer* layer, const std::vector<te::qt::af::DataSetTableDockWidget*>& docs)
{
  std::vector<te::qt::af::DataSetTableDockWidget*>::const_iterator it;

  for (it = docs.begin(); it != docs.end(); ++it)
    if ((*it)->getLayer() == layer)
      return *it;

  return 0;
}

ta::common::TerraAmazon::TerraAmazon(QWidget* parent)
  : te::qt::af::BaseApplication(parent),
  m_fileMenu(0),
  m_fileNewWorkspace(0),
  m_fileOpenWorkspace(0),
  m_fileSaveWorkspace(0),
  m_fileSaveWorkspaceAs(0),
  m_filePrintPreview(0),
  m_filePrint(0),
  m_fileRecentWorkspacesMenu(0),
  m_fileRestartSystem(0),
  m_fileExit(0),
  m_showMenu(0),
  m_showToolBarsMenu(0),
  m_showConnectLayers(0),
  m_workspaceMenu(0),
  m_workspaceAddLayerDataset(0),
  m_workspaceAddLayerQueryDataSet(0),
  m_workspaceAddLayerTabularDataSet(0),
  m_workspaceAddLayerGraph(0),
  m_workspaceAddFolderLayer(0),
  m_workspaceChangeLayerDataSource(0),
  m_workspaceUpdateLayerDataSource(0),
  m_workspaceProperties(0),
  m_workspaceAddLayerMenu(0),
  m_processMenu(0),
  m_processServerControl(0),
  m_processProjectActivation(0),
  m_processControl(0),
  m_processCellsManager(0),
  m_administrationMenu(0),
  m_adminCreateConceptualModel(0),
  m_projManagementMenu(0),
  m_projManagementUserControl(0),
  m_projManagementGroupControl(0),
  m_projManagementPhaseControl(0),
  m_projManagementAttributeControl(0),
  m_projProjectControl(0),
  m_projRulesControl(0),
  m_projClassControl(0),
  m_projProjectManagementWizard(0),
  m_projAOIManagementWizard(0),
  m_projCatalogManager(0),
  m_projCreateFolder(0),
  m_projInformation(0),
  m_createDraftLayer(0),
  m_settingsMenu(0),
  m_settingsShowWelcomeDlg(0),
  m_pluginsMenu(0),
  m_pluginsManager(0),
  m_helpMenu(0),
  m_helpAbout(0),
  m_helpContents(0),
  m_helpUpdate(0),
  m_editionToolsAction(0),
  m_layerEditLegend(0),
  m_layerCompositionMode(0),
  m_layerCalculateArea(0),
  m_mapCursorSize(QSize(20, 20)),
  m_taController(0),
  m_compModeMenu(0),
  m_editToolbar(0),
  m_connectLayersDock(0),
  m_statusBarInfoWidget(0)
{
  m_workspace = new ta::workspace::WorkspaceMetadata;

  ResetWorkspace(m_workspace);
}

ta::common::TerraAmazon::~TerraAmazon()
{
  delete m_workspace;
  delete m_editToolbar;
  delete m_compModeMenu;
  delete m_taController;

  //finalize factories
  ta::workspace::qt::WorkspaceWidgetFactory::finalize();
  ta::model::AoiLayerFactory::finalize();
  ta::model::CatalogFactory::finalize();
  ta::model::CellFactory::finalize();
  ta::model::ClassFactory::finalize();
  ta::model::GroupFactory::finalize();
  ta::model::PhaseFactory::finalize();
  ta::model::ProcessFactory::finalize();
  ta::model::ProjectFactory::finalize();
  ta::model::ProjectSceneFactory::finalize();
  ta::model::SceneFactory::finalize();
  ta::model::TaskFactory::finalize();
  ta::model::TaskLogFactory::finalize();
  ta::model::UserFactory::finalize();
  ta::model::RuleFactory::finalize();
  ta::model::RuleRuleSetFactory::finalize();
  ta::model::RuleSetFactory::finalize();
  ta::model::RulesLayerColumnFactory::finalize();
}

void ta::common::TerraAmazon::makeDialog()
{
  te::qt::af::BaseApplication::makeDialog();

  addMenusActions();
  addPopUpMenu();
  initInfoStatusBar();

  //composition mode
  m_compModeMenu = new te::qt::widgets::CompositionModeMenuWidget();
  m_layerCompositionMode->setMenu(m_compModeMenu->getMenu());

  //insert connect layers dock
  m_connectLayersDock = new ta::common::qt::ConnectLayersDockWidget(this);
  m_connectLayersDock->setApplicationComponents(getLayerExplorer(), m_display);
  this->addDockWidget(Qt::LeftDockWidgetArea, m_connectLayersDock);
  this->tabifyDockWidget(m_connectLayersDock, getLayerExplorerDock());

  // Show
  connect(m_showConnectLayers, SIGNAL(toggled(bool)), m_connectLayersDock, SLOT(setVisible(bool)));

  m_display->getDisplay()->setResizePolicy(te::qt::widgets::MapDisplay::Center);
  m_styleExplorer->getExplorer()->setVisible(false);
  m_showConnectLayers->setChecked(true);

  //initialize factories
  ta::workspace::qt::WorkspaceWidgetFactory::initialize();

  ta::model::AoiLayerFactory::initialize();
  ta::model::CatalogFactory::initialize();
  ta::model::CellFactory::initialize();
  ta::model::ClassFactory::initialize();
  ta::model::GroupFactory::initialize();
  ta::model::PhaseFactory::initialize();
  ta::model::ProcessFactory::initialize();
  ta::model::ProjectFactory::initialize();
  ta::model::ProjectSceneFactory::initialize();
  ta::model::SceneFactory::initialize();
  ta::model::TaskFactory::initialize();
  ta::model::TaskLogFactory::initialize();
  ta::model::UserFactory::initialize();
  ta::model::RuleFactory::initialize();
  ta::model::RuleRuleSetFactory::initialize();
  ta::model::RuleSetFactory::initialize();
  ta::model::RulesLayerColumnFactory::initialize();
}

void ta::common::TerraAmazon::init()
{
  std::string cfg = ta::common::FindInTerraAmazonPath("share/terraamazon/config/config.xml");
  
  BaseApplication::init(cfg.c_str());

  //check user settings file
  QSettings user_settings(QSettings::IniFormat,
  QSettings::UserScope,
  QApplication::instance()->organizationName(),
  QApplication::instance()->applicationName());

  QFileInfo info(user_settings.fileName());

  if (!info.exists())
    CreateDefaultSettings();
  
  m_taController = new TerraAmazonController(m_app, cfg);
  m_taController->initializeWorkspaceMenus();

  initToolbars();
  setWindowTitle();


  ta::workspace::XMLFormatter::formatDataSourceInfos(false);

  //set terraAmazon icon
  setWindowIcon(QIcon(ta::common::FindInTerraAmazonPath("share/terraamazon/images/png/terraamazon-icon.png").c_str()));
}

bool ta::common::TerraAmazon::showWelcome()
{
  QString serverXML = m_app->getUserDataDir() + QString::fromStdString("/servers.xml");
  m_workspace->m_servers = ta::workspace::readServers(serverXML);
  m_taController->setServers(m_workspace->m_servers);

  ta::common::TaUserType taUser = ta::common::TA_GIS_USER;

  QSettings sett(QSettings::IniFormat, QSettings::UserScope, qApp->organizationName(), qApp->applicationName());
  QString keyUserType = "Welcome/UserType";
  QString keyShowDlg = "Welcome/ShowDlg";

  //check if has to show welcome dlg window
  QVariant varShowDlg = sett.value(keyShowDlg);

  bool showDlg = true;

  if (varShowDlg.isValid())
  {
    QString showDlgStr = varShowDlg.toString();

    if (showDlgStr == "FALSE")
    {
      showDlg = false;

      m_settingsShowWelcomeDlg->setChecked(false);
    }
    else
    {
      m_settingsShowWelcomeDlg->setChecked(true);
    }
  }
  else
  {
    m_settingsShowWelcomeDlg->setChecked(true);
  }

  //check current user type
  QVariant varUserType = sett.value(keyUserType);

  if (varUserType.isValid())
  {
    QString userTypeStr = varUserType.toString();

    taUser = (ta::common::TaUserType)userTypeStr.toInt();
  }

  //show dlg if necessary
  int userId = -1;
  te::da::DataSourceInfoPtr dataSource;

  if (showDlg)
  {
    ta::common::qt::WelcomeDialog dlg(this);
    dlg.setServers(m_workspace->m_servers);
    dlg.move(QApplication::desktop()->screen()->rect().center() - dlg.rect().center());

    if (dlg.exec() != QDialog::Accepted)
    {
      return false;
    }

    taUser = dlg.getOption();

    QString showDlgStr("FALSE");

    if (dlg.showWelcomeDlg())
    {
      showDlgStr = "TRUE";

      m_settingsShowWelcomeDlg->setChecked(true);
    }
    else
    {
      m_settingsShowWelcomeDlg->setChecked(false);
    }

    sett.setValue(keyUserType, QString::number(taUser));
    sett.setValue(keyShowDlg, showDlgStr);
    userId = dlg.getActiveUserId();
    dataSource = dlg.getActiveServer();
  }
  else if (!showDlg && !(taUser == ta::common::TA_GIS_USER))
  {
    ta::workspace::qt::ServerLoginDialog dlg(this);
    dlg.move(QApplication::desktop()->screen()->rect().center() - dlg.rect().center());
    dlg.setServers(m_workspace->m_servers);

    if (dlg.exec() != QDialog::Accepted)
    {
      taUser = ta::common::TA_GIS_USER;

      sett.setValue(keyUserType, QString::number(taUser));
      sett.setValue(keyShowDlg, "TRUE");
    }
    else
    {
      userId = dlg.getActiveUserId();
      dataSource = dlg.getActiveServer();
    }
  }

  //get current user and server
  if (userId != -1)
    m_taController->setCurrentUserId(userId);

  QStringList wsPaths, wsTitles, wsServers, wsUsers;
  GetWorkspacesFromSettings(wsTitles, wsPaths, wsServers, wsUsers);

  if (!wsPaths.empty())
    openWorkspace(*wsPaths.begin());

  if (dataSource)
  {
    m_taController->setActiveServer(dataSource);
    m_taController->addServer(dataSource);
  }

  //configure application using user definitions
  configureTerraAmazon(taUser);
  setWindowTitle();

  return true;
}

void ta::common::TerraAmazon::startWorkspace(const QString& wsFileName)
{
  openWorkspace(wsFileName);
}

void ta::common::TerraAmazon::workspaceChanged()
{
  m_workspace->m_changed = true;

  setWindowTitle(true);
}

void ta::common::TerraAmazon::checkAndSaveWorkspace()
{
  if (m_workspace->m_changed)
  {
    QString msg = tr("The current workspace has unsaved changes. Do you want to save them?");
    QMessageBox::StandardButton btn = QMessageBox::question(this, windowTitle(), msg, QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel, QMessageBox::Save);

    if (btn == QMessageBox::Save)
      onSaveWorkspaceTriggered();
  }
}

void ta::common::TerraAmazon::openWorkspace(const QString& wsFileName)
{
  std::list<te::map::AbstractLayerPtr> lst;
  try
  {
    ta::workspace::WorkspaceMetadata wmCheck;
    ta::workspace::LoadWorkspace(wsFileName, wmCheck, lst);

    if (wmCheck.m_currentUserId == m_taController->getCurrentUserId())
    {
      lst.clear();

      ta::workspace::LoadWorkspace(wsFileName, *m_workspace, lst);
      ta::workspace::XMLFormatter::format(m_workspace, lst, false);
      ta::workspace::XMLFormatter::formatDataSourceInfos(false);

      getLayerExplorer()->setLayers(lst);

      m_workspace->m_changed = false;
      QString serverId, userId;

      if (!m_workspace->m_servers.empty())
        m_taController->setServers(m_workspace->m_servers);

      if (m_workspace->m_currentProjId >= 0)
        m_taController->setCurrentProjectId(m_workspace->m_currentProjId);

      if (!m_workspace->m_activeServerId.empty())
      {
        serverId = QString::fromStdString(m_workspace->m_activeServerId);
        for (std::size_t i = 0; i < m_workspace->m_servers.size(); ++i)
        {
          if (m_workspace->m_servers[i]->getId() == m_workspace->m_activeServerId)
          {
            m_taController->setActiveServer(m_workspace->m_servers[i]);

            m_editToolbar->setServer(m_workspace->m_servers[i]);
            m_editToolbar->setDelegate();

            break;
          }
        }
      }

      if (m_workspace->m_currentUserId > -1)
      {
        m_taController->setCurrentUserId(m_workspace->m_currentUserId);
        userId = QString::number(m_workspace->m_currentUserId);
      }

      if (m_taController->getCurrentUserId() > -1 && m_taController->getActiveServer())
      {
        m_editToolbar->setProcess(LoadProcess(m_taController->getCurrentUserId(), m_taController->getActiveServer()));
      }
      if (m_taController->getCurrentProjectId() > -1)
      {
        m_editToolbar->setProject(m_taController->getCurrentProjectId());
      }

      setWindowTitle();
      AddRecentWorkspaceToSettings(m_workspace->m_title, m_workspace->m_fileName, serverId, userId);
      m_taController->updateRecentWorkspaces(m_workspace->m_fileName, m_workspace->m_title);

      std::list<te::map::AbstractLayerPtr> ls;
      getMapDisplay()->setLayerList(ls);
      getMapDisplay()->refresh(true);

    }
    else
      QMessageBox::information(this, m_app->getAppTitle(), tr("The user that created the workspace and the current user are not the same."));
  }
  catch (te::common::Exception&)
  {
    ResetWorkspace(m_workspace);
  }
}

void ta::common::TerraAmazon::onNewWorkspaceTriggered()
{
  checkAndSaveWorkspace();

  m_workspace->m_fileName = "";
  m_workspace->m_title = tr("Default Workspace");
  m_workspace->m_changed = false;

  setWindowTitle();

  std::list<te::map::AbstractLayerPtr> ls;

  getLayerExplorer()->setLayers(ls);

  getMapDisplay()->setLayerList(ls);

  getMapDisplay()->refresh(true);
}

void ta::common::TerraAmazon::onOpenWorkspaceTriggered()
{
  checkAndSaveWorkspace();

  QString userId = QString::number(m_workspace->m_currentUserId);
  QString serverId = QString::fromStdString(m_workspace->m_activeServerId);

  AddRecentWorkspaceToSettings(m_workspace->m_title, m_workspace->m_fileName, serverId, userId);

  QString file = QFileDialog::getOpenFileName(this, tr("Open project file"), te::qt::widgets::GetFilePathFromSettings("amazon_workspace_info"), m_taController->getExtensionFilter());

  if (file.isEmpty())
    return;

  QFileInfo info(file);

  te::qt::widgets::AddFilePathToSettings(info.absolutePath(), "amazon_workspace_info");

  try
  {
    openWorkspace(file);
  }
  catch (const te::common::Exception& e)
  {
    QString msg = tr("Fail to open project.");
    msg += " ";
    msg += e.what();
    QMessageBox::warning(this, m_app->getAppTitle(), msg);
  }
}

void ta::common::TerraAmazon::onSaveWorkspaceTriggered()
{
  QString wsFile = m_workspace->m_fileName;

  if (wsFile.isEmpty())
  {
    QFileDialog d(this);

    QString filter = tr("TerraAmazon project(*.") + m_taController->getAppWorkspaceExtension() + ")";

    QString fileName = d.getSaveFileName(this, tr("Save workspace"), te::qt::widgets::GetFilePathFromSettings("amazon_workspace_info"), filter);

    if (fileName.isEmpty())
      return;

    QFileInfo info(fileName);

    te::qt::widgets::AddFilePathToSettings(info.absolutePath(), "amazon_workspace_info");

    if (info.suffix().isEmpty())
      fileName.append("." + m_taController->getAppWorkspaceExtension());

    m_workspace->m_fileName = fileName;
    m_workspace->m_title = info.baseName();
    m_workspace->m_servers = m_taController->getServers();
    if (m_taController->getActiveServer())
      m_workspace->m_activeServerId = m_taController->getActiveServer()->getId();

    if (m_taController->getCurrentProjectId() >= 0)
    {
      m_workspace->m_currentProjId = m_taController->getCurrentProjectId();
    }
    else
    {
      m_workspace->m_currentProjId = -1;
    }
  }

  std::list<te::map::AbstractLayerPtr> lays = getLayerExplorer()->getAllLayers();

  QString serverId, userId;

  m_workspace->m_servers = m_taController->getServers();
  if (m_taController->getActiveServer())
  {
    m_workspace->m_activeServerId = m_taController->getActiveServer()->getId();
    serverId = QString::fromStdString(m_workspace->m_activeServerId);
  }

  m_workspace->m_currentProjId = m_taController->getCurrentProjectId();

  if (m_taController->getCurrentUserId() >= 0)
  {
    m_workspace->m_currentUserId = m_taController->getCurrentUserId();
    userId = QString::number(m_workspace->m_currentUserId);
  }

  m_workspace->m_changed = false;

  setWindowTitle();

  AddRecentWorkspaceToSettings(m_workspace->m_title, m_workspace->m_fileName, serverId, userId);

  m_taController->updateRecentWorkspaces(m_workspace->m_fileName, m_workspace->m_title);

  ta::workspace::XMLFormatter::format(m_workspace, lays, true);
  ta::workspace::XMLFormatter::formatDataSourceInfos(true);

  ta::workspace::SaveWorkspace(*m_workspace, lays);
  ta::workspace::writeServers(*m_workspace, m_app->getUserDataDir().toStdString());

  te::qt::af::SaveDataSourcesFile(m_app);

  ta::workspace::XMLFormatter::format(m_workspace, lays, false);
  ta::workspace::XMLFormatter::formatDataSourceInfos(false);
}

void ta::common::TerraAmazon::onSaveWorkspaceAsTriggered()
{
  if (m_workspace == 0)
    return;

  QString fileName = QFileDialog::getSaveFileName(this, tr("Save Workspace File"), te::qt::widgets::GetFilePathFromSettings("amazon_workspace_info"), m_taController->getExtensionFilter());

  if (fileName.isEmpty())
    return;

  QFileInfo info(fileName);

  te::qt::widgets::AddFilePathToSettings(info.absolutePath(), "amazon_workspace_info");

  std::string fName = fileName.toStdString();

  m_workspace->m_fileName = fName.c_str();

  if (!boost::filesystem::exists(fName) && m_workspace->m_title == "New Workspace")
  {
    m_workspace->m_title = (boost::filesystem::basename(fName)).c_str();
  }

  m_taController->updateRecentWorkspaces(fileName, m_workspace->m_title);

  m_workspace->m_changed = false;

  // Set the window title
  setWindowTitle();

  te::qt::af::SaveDataSourcesFile(m_app);
}

void ta::common::TerraAmazon::initActions()
{
  te::qt::af::BaseApplication::initActions();

  // File Menu
  initAction(m_fileNewWorkspace, "document-new", "File.New Workspace", tr("&New Workspace..."), tr(""), true, false, true, m_menubar);
  initAction(m_fileSaveWorkspace, "document-save", "File.Save Workspace", tr("&Save Workspace"), tr(""), true, false, true, m_menubar);
  initAction(m_fileSaveWorkspaceAs, "document-save-as", "File.Save Workspace As", tr("Save Workspace &As..."), tr(""), true, false, false, m_menubar);
  initAction(m_fileOpenWorkspace, "document-open", "File.Open Workspace", tr("&Open Workspace..."), tr(""), true, false, true, m_menubar);
  initAction(m_fileRestartSystem, "", "File.Restart System", tr("&Restart System..."), tr("Restart the system."), true, false, true, m_menubar);
  initAction(m_filePrintPreview, "document-print-preview", "File.Print Preview", tr("Print Pre&view..."), tr(""), true, false, false, m_menubar);
  initAction(m_filePrint, "document-print", "File.Print", tr("&Print..."), tr(""), true, false, false, m_menubar);
  initAction(m_fileExit, "system-log-out", "File.Exit", tr("E&xit"), tr(""), true, false, true, m_menubar);

  // Show Menu
  initAction(m_showConnectLayers, "", "Show.Connect Layers", tr("&Connect Layers"), tr("Show or hide the connection layers"), true, true, true, m_menubar);
  
  // Workspace Menu
  initAction(m_workspaceAddLayerDataset, "datasource", "Workspace.Add Layer.All Sources", tr("&From Data Source..."), tr("Add a new layer from all available data sources"), true, false, true, m_menubar);
  initAction(m_workspaceAddFolderLayer, "folderlayer-new", "Workspace.New Folder Layer", tr("Add &Folder Layer..."), tr("Add a new folder layer"), true, false, true, m_menubar);
  initAction(m_workspaceAddLayerQueryDataSet, "view-filter", "Workspace.Add Layer.Query Dataset", tr("&Query Dataset..."), tr("Add a new layer from a queried dataset"), true, false, true, m_menubar);
  initAction(m_workspaceAddLayerTabularDataSet, "view-data-table", "Workspace.Add Layer.Tabular File", tr("&Tabular File..."), tr("Add a new layer from a Tabular file"), true, false, true, m_menubar);
  initAction(m_workspaceChangeLayerDataSource, "", "Workspace.Change Layer Data Source", tr("&Change Layer Data Source"), tr("Chanage layer Data Source"), true, false, true, this);
  initAction(m_workspaceUpdateLayerDataSource, "", "Workspace.Update Layer Data Source", tr("&Update Layer Data Source"), tr("Update layer Data Source"), true, false, true, this);
  initAction(m_workspaceProperties, "document-info", "Workspace.Properties", tr("&Properties..."), tr("Show the project properties"), true, false, true, m_menubar);

  // Process Menu
  initAction(m_processServerControl, "", "Process.Server Control", tr("&User Information Control..."), tr("User Information Control..."), true, false, true, m_menubar);
  initAction(m_processProjectActivation, "", "Process.Project Activation", tr("&Project Activation..."), tr("Project Activation"), true, false, true, m_menubar);
  initAction(m_processControl, "", "Process.Process Control", tr("&Process Control..."), tr("Process Control"), true, false, true, m_menubar);
  initAction(m_processCellsManager, "", "Process.Cells Manager", tr("&Cells Manager..."), tr("Cells Manager"), true, false, true, m_menubar);

  // Administration Menu
  initAction(m_adminCreateConceptualModel, "", "Administration.Create Conceptual Model", tr("&Create Conceptual Model..."), tr("Create Conceptual Model"), true, false, true, m_menubar);
  initAction(m_adminDataExchanger, "datasource-exchanger", "Tools.Exchanger.All to All", tr("&DataSource Exchanger..."), tr("Exchange data sets between data sources"), true, false, true, m_menubar);
  initAction(m_adminQueryDataSource, "datasource-query", "Tools.Query Data Source", tr("&Query Data Source..."), tr("Allows you to query data in a data source"), true, false, true, m_menubar);

  //Project Management Menu
  initAction(m_projManagementUserControl, "", "Project.Management.User Control", tr("&User..."), tr("User Control"), true, false, true, m_menubar);
  initAction(m_projManagementGroupControl, "", "Project.Management.Group Control", tr("&Group..."), tr("Group Control"), true, false, true, m_menubar);
  initAction(m_projManagementPhaseControl, "", "Project.Management.Phase Control", tr("&Phase..."), tr("Phase Control"), true, false, true, m_menubar);

  initAction(m_projProjectControl, "", "Project.Management.Project Control", tr("&Project..."), tr("Project Control"), true, false, true, m_menubar);
  initAction(m_projRulesControl, "", "Project.Management.Rules Control", tr("&Rules..."), tr("Rules Control"), true, false, true, m_menubar);
  initAction(m_projManagementAttributeControl, "", "Project.Management.Attribute", tr("&Attribute..."), tr("Attribute Control"), true, false, true, m_menubar);
  initAction(m_projClassControl, "", "Project.Management.Class Control", tr("&Class..."), tr("Class Control"), true, false, true, m_menubar);
  initAction(m_projProjectManagementWizard, "", "Project.Management.Project Wizard", tr("&Project Wizard..."), tr("Project Wizard"), true, false, true, m_menubar);
  initAction(m_projAOIManagementWizard, "", "Project.Management.AOI Wizard", tr("&AOI Wizard..."), tr("AOI Management Wizard"), true, false, true, m_menubar);
  initAction(m_projCatalogManager, "library", "Project.Catalog Manager", tr("&Catalog..."), tr("Catalog Manager"), true, false, true, m_menubar);
  initAction(m_projCreateFolder, "", "Project.Create Project Folder", tr("Create Project Folder..."), tr("Create Project Folder"), true, false, true, m_menubar);
  initAction(m_projInformation, "help-about-browser", "Project.Information", tr("Information..."), tr("Project Information"), true, false, true, m_menubar);

  initAction(m_createDraftLayer, "", "Project.Create Draft Layer", tr("Create Draft Layer..."), tr("Create Draft Layer"), true, false, true, m_menubar);
  
  // Settings Menu
  initAction(m_settingsShowWelcomeDlg, "", "Settings.Show Welcome Dialog", tr("&Show Welcome Dialog on Startup..."), tr("Show Welcome"), true, true, true, m_menubar);

  // Menu -Plugins- actions
  initAction(m_pluginsManager, "plugin", "Plugins.Management", tr("&Manage Plugins..."), tr("Manage the application plugins"), true, false, true, m_menubar);

  // Help Menu
  initAction(m_helpContents, "help-browser", "Help.View Help", tr("&View Help..."), tr("Shows help dialog"), true, false, true, m_menubar);
  initAction(m_helpAbout, "help-about-browser", "Help.About", tr("About..."), tr("About"), true, false, true, m_menubar);

  // Edition Enable
  initAction(m_editionToolsAction, "edit-enable", "Edition.Management", "", tr("Edit Enable"), true, true, true, m_menubar);

  // Only popup menu
  initAction(m_layerEditLegend, "grouping", "", tr("Edit Legend..."), tr("Edit Legend"), true, false, true, m_menubar);
  initAction(m_layerCompositionMode, "layer-compose", "Layer.Composition Mode", tr("&Composition Mode..."), tr("Set the composition mode to renderer the selected layer"), true, false, true, m_menubar);
  initAction(m_layerCalculateArea, "area-measure", "Layer.Calculate Area", tr("Calculate Area..."), tr("Calculates the area of geometries of a layer using a specific projection"), true, false, true, m_menubar);

  // Menu -Layer- actions
  initAction(m_layerDataExchangerDirectPopUp, "data-exchange-direct-icon", "Tools.Exchanger.Direct", tr("&Exchange..."), tr("Exchange data sets from layers"), true, false, true, m_menubar);
  initAction(m_layerQuery, "view-filter", "Layer.Query", tr("Query..."), tr(""), true, false, true, m_menubar);

  //associate icons

  std::string iconServerControl = ta::common::FindInTerraAmazonPath("share/icons/svg/ta-user.svg");
  m_processServerControl->setIcon(QIcon(iconServerControl.c_str()));

  std::string iconActiveProject = ta::common::FindInTerraAmazonPath("share/icons/svg/project-activation.svg");
  m_processProjectActivation->setIcon(QIcon(iconActiveProject.c_str()));
  
  std::string iconDsAmazon = ta::common::FindInTerraAmazonPath("share/icons/svg/datasource-amazon.svg");
  m_adminCreateConceptualModel->setIcon(QIcon(iconDsAmazon.c_str()));

  std::string iconCreateFolder = ta::common::FindInTerraAmazonPath("share/icons/svg/folder-project.svg");
  m_projCreateFolder->setIcon(QIcon(iconCreateFolder.c_str()));

  std::string iconDraftLayer = ta::common::FindInTerraAmazonPath("share/icons/svg/draft-layer.svg");
  m_createDraftLayer->setIcon(QIcon(iconDraftLayer.c_str()));

  std::string iconConnectLayer = ta::common::FindInTerraAmazonPath("share/icons/svg/connect-eye.svg").c_str();
  m_showConnectLayers->setIcon(QIcon(iconConnectLayer.c_str()));

  std::string iconEnableEdition = ta::common::FindInTerraAmazonPath("share/icons/svg/edit-enable3.svg").c_str();
  m_editionToolsAction->setIcon(QIcon(iconEnableEdition.c_str()));

}

void ta::common::TerraAmazon::initSlotsConnections()
{
  te::qt::af::BaseApplication::initSlotsConnections();

  connect(m_fileNewWorkspace, SIGNAL(triggered()), SLOT(onNewWorkspaceTriggered()));
  connect(m_fileOpenWorkspace, SIGNAL(triggered()), SLOT(onOpenWorkspaceTriggered()));
  connect(m_fileSaveWorkspace, SIGNAL(triggered()), SLOT(onSaveWorkspaceTriggered()));
  connect(m_fileSaveWorkspaceAs, SIGNAL(triggered()), SLOT(onSaveWorkspaceAsTriggered()));
  connect(m_fileRecentWorkspacesMenu, SIGNAL(triggered(QAction*)), SLOT(onRecentWorkspacesTriggered(QAction*)));
  connect(m_fileExit, SIGNAL(triggered()), SLOT(close()));
  connect(m_fileRestartSystem, SIGNAL(triggered()), SLOT(onRestartSystemTriggered()));

  // Workspace
  connect(m_workspaceAddLayerDataset, SIGNAL(triggered()), SLOT(onAddDataSetLayerTriggered()));
  connect(m_workspaceAddLayerQueryDataSet, SIGNAL(triggered()), SLOT(onAddQueryLayerTriggered()));
  connect(m_workspaceAddLayerTabularDataSet, SIGNAL(triggered()), SLOT(onAddTabularLayerTriggered()));
  connect(m_workspaceAddFolderLayer, SIGNAL(triggered()), SLOT(onAddFolderLayerTriggered()));
  connect(m_workspaceProperties, SIGNAL(triggered()), SLOT(onWorkspacePropertiesTriggered()));

  // Process
  connect(m_processServerControl, SIGNAL(triggered()), SLOT(onWorkspaceServerControlTriggered()));
  connect(m_processProjectActivation, SIGNAL(triggered()), SLOT(onProjectActivationTriggered()));
  connect(m_processControl, SIGNAL(triggered()), SLOT(onProcessControlTriggered()));
  connect(m_processCellsManager, SIGNAL(triggered()), SLOT(onProcessCellManagerTriggered()));

  // Administration Menu
  connect(m_adminCreateConceptualModel, SIGNAL(triggered()), SLOT(onCreateCanceptualModelTriggered()));
  connect(m_adminDataExchanger, SIGNAL(triggered()), SLOT(onAdminDataExchangerTriggered()));
  connect(m_adminQueryDataSource, SIGNAL(triggered()), SLOT(onAdminQueryDatasourceTriggered()));

  //Project Management
  connect(m_projManagementUserControl, SIGNAL(triggered()), SLOT(onUserControlTriggered()));
  connect(m_projManagementGroupControl, SIGNAL(triggered()), SLOT(onGroupControlTriggered()));
  connect(m_projManagementPhaseControl, SIGNAL(triggered()), SLOT(onPhaseControlTriggered()));
  connect(m_projManagementAttributeControl, SIGNAL(triggered()), SLOT(onAttributeControlTriggered()));
  connect(m_projProjectControl, SIGNAL(triggered()), SLOT(onProjectControlTriggered()));
  connect(m_projRulesControl, SIGNAL(triggered()), SLOT(onRulesControlTriggered()));
  connect(m_projClassControl, SIGNAL(triggered()), SLOT(onClassControlTriggered()));
  connect(m_projProjectManagementWizard, SIGNAL(triggered()), SLOT(onProjectManagementWizardTriggered()));
  connect(m_projAOIManagementWizard, SIGNAL(triggered()), SLOT(onAOIManagementWizardTriggered()));
  connect(m_projCatalogManager, SIGNAL(triggered()), SLOT(onCatalogManagerTriggered()));
  connect(m_projCreateFolder, SIGNAL(triggered()), SLOT(onProjCreateFolderTriggered()));
  connect(m_projInformation, SIGNAL(triggered()), SLOT(onProjInformationTriggered()));

  connect(m_createDraftLayer, SIGNAL(triggered()), SLOT(onCreateDraftLayerTriggered()));

  //Project Management
  connect(m_settingsShowWelcomeDlg, SIGNAL(toggled(bool)), SLOT(onShowWelcomeDialogToggled(bool)));
  
  // Plugins 
  connect(m_pluginsManager, SIGNAL(triggered()), SLOT(onPluginsManagerTriggered()));

  // Help
  connect(m_helpContents, SIGNAL(triggered()), SLOT(onHelpContentsTriggered()));
  connect(m_helpAbout, SIGNAL(triggered()), SLOT(onHelpAboutTriggered()));

  // Edition
  connect(m_editionToolsAction, SIGNAL(triggered(bool)), SLOT(onEnableEditTriggered(bool)));

  // Only popup menu
  connect(m_layerEditLegend, SIGNAL(triggered()), SLOT(onLayerEditLegendTriggered()));
  connect(m_layerQuery, SIGNAL(triggered()), SLOT(onLayerQueryTriggered()));
  connect(m_layerDataExchangerDirectPopUp, SIGNAL(triggered()), SLOT(onLayerDataExchangerTriggered()));
  connect(m_layerCompositionMode, SIGNAL(hovered()), SLOT(onLayerCompositionModeTriggered()));
  connect(m_layerCalculateArea, SIGNAL(triggered()), SLOT(onLayerCalculateAreaTriggered()));
}

void ta::common::TerraAmazon::initMenus()
{
  te::qt::af::BaseApplication::initMenus();

  m_fileMenu = new QMenu(m_menubar);

  m_fileRecentWorkspacesMenu = new QMenu(m_fileMenu);
  m_fileRecentWorkspacesMenu->setObjectName("File.Recent Workspaces");
  m_fileRecentWorkspacesMenu->setTitle(tr("Recent &Workspaces"));
  m_fileRecentWorkspacesMenu->setEnabled(false);

  m_menubar->addAction(m_fileMenu->menuAction());
  m_showMenu = new QMenu(m_menubar);
  m_menubar->addAction(m_showMenu->menuAction());
  m_showToolBarsMenu = new QMenu(m_showMenu);
  m_showMenu->addAction(m_showToolBarsMenu->menuAction());
  m_workspaceMenu = new QMenu(m_menubar);
  m_menubar->addAction(m_workspaceMenu->menuAction());
  m_workspaceAddLayerMenu = new QMenu(m_workspaceMenu);
  m_workspaceMenu->addAction(m_workspaceAddLayerMenu->menuAction());
  m_processMenu = new QMenu(m_menubar);
  m_menubar->addAction(m_processMenu->menuAction());
  m_projManagementMenu = new QMenu(m_menubar);
  m_menubar->addAction(m_projManagementMenu->menuAction());
  m_administrationMenu = new QMenu(m_menubar);
  m_menubar->addAction(m_administrationMenu->menuAction());
  m_settingsMenu = new QMenu(m_menubar);
  m_menubar->addAction(m_settingsMenu->menuAction());
  m_pluginsMenu = new QMenu(m_menubar);
  m_pluginsMenu->setObjectName("Plugins");
  m_menubar->addMenu(m_pluginsMenu);
  m_helpMenu = new QMenu(m_menubar);
  m_menubar->addAction(m_helpMenu->menuAction());
}

void ta::common::TerraAmazon::initToolbars()
{
  std::vector<QToolBar*> bars = ReadToolBarsFromSettings(m_app, this);
  std::vector<QToolBar*>::iterator it;

  for (it = bars.begin(); it != bars.end(); ++it)
  {
    QToolBar* bar = *it;

    if (it == bars.end() - 1)
    {
      (*it)->addAction(m_editionToolsAction);
    }

    if (!bar->actions().isEmpty())
    {
      addToolBar(Qt::TopToolBarArea, bar);
      m_showToolBarsMenu->addAction(bar->toggleViewAction());
      m_app->registerToolBar(bar->objectName(), bar);
    }
  }

  m_editToolbar = new ta::edition::qt::EditToolBar(this);
  
  connect(m_editToolbar, SIGNAL(enableVisualizationTools(bool)), this, SLOT(onEnableVisualizationTools(bool)));

  addToolBarBreak(Qt::TopToolBarArea);
  addToolBar(Qt::TopToolBarArea, m_editToolbar->get());
  m_editToolbar->get()->hide();
}

void ta::common::TerraAmazon::initInfoStatusBar()
{
  m_statusBarInfoWidget = new ta::common::qt::StatusBarInfoWidget(this);
  m_statusbar->addWidget(m_statusBarInfoWidget);
}

void ta::common::TerraAmazon::onWorkspaceServerControlTriggered()
{
  if (m_taController->getCurrentUserId() == -1)
  {
    QMessageBox::warning(this, m_app->getAppTitle(), tr("There's no current user!"));
    return;
  }

  if (!m_taController->getActiveServer())
  {
    QMessageBox::warning(this, m_app->getAppTitle(), tr("There's no current server!"));
    return;
  }

  if (m_workspace == 0)
  {
    QMessageBox::warning(this, m_app->getAppTitle(), tr("There's no current project!"));
    return;
  }

  ta::workspace::qt::ServerManagerDialog dlg(this);
  te::da::DataSourcePtr ds = te::da::GetDataSource(m_taController->getActiveServer()->getId());
  ta::model::UserFacade uf(ds);
  std::unique_ptr<ta::model::User>user(dynamic_cast<ta::model::User*>(uf.get(m_taController->getCurrentUserId())));

  dlg.setServers(m_taController->getServers());
  dlg.setActiveServer(m_taController->getActiveServer());
  dlg.setActiveUser(std::move(user));

  int res = dlg.exec();

  if (res && dlg.hasChanges())
  {
    m_taController->setServers(dlg.getServers());
    m_taController->setCurrentUserId(dlg.getActiveUserId());
    m_taController->setActiveServer(dlg.getActiveServer());

    m_editToolbar->setServer(dlg.getActiveServer());
    m_editToolbar->setProcess(LoadProcess(dlg.getActiveUserId(), dlg.getActiveServer()));
    m_editToolbar->setProject(m_taController->getCurrentProjectId());
    m_editToolbar->setDelegate();
    
    workspaceChanged();
  }
}

void ta::common::TerraAmazon::onWorkspacePropertiesTriggered()
{
  if (m_workspace == 0)
  {
    QMessageBox::warning(this, m_app->getAppTitle(), tr("There's no current project!"));
    return;
  }

  ta::workspace::qt::WorkspaceInfoDialog editor(this);
  editor.setWorkspace(m_workspace);

  if (editor.exec() == QDialog::Accepted)
  {
    // Set window title
    if (m_workspace->m_changed)
      workspaceChanged();
  }
}

void ta::common::TerraAmazon::onAddServerTriggered()
{
  const te::qt::widgets::DataSourceType* dsType = te::qt::widgets::DataSourceTypeManager::getInstance().get("POSTGIS");

  if (dsType == 0)
    throw te::common::Exception(TE_TR("Unknown data source type!"));

  std::unique_ptr<QWidget> connectorw(dsType->getWidget(te::qt::widgets::DataSourceType::WIDGET_DATASOURCE_CONNECTOR, this));

  if (connectorw.get() == 0)
    throw te::common::Exception(TE_TR("This type of data source hasn't provide a dialog for adding a new data source"));

  te::qt::widgets::AbstractDataSourceConnector* connector = dynamic_cast<te::qt::widgets::AbstractDataSourceConnector*>(connectorw.get());

  if (connector == 0)
    throw te::common::Exception(TE_TR("Wrong type of object for adding a new data source!"));

  std::list<te::da::DataSourceInfoPtr> datasources;

  connector->connect(datasources);
  
  if (!datasources.empty())
  {
    te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().get(datasources.begin()->get()->getId());

    //set amazon server
    m_taController->addServer(dsInfo);
  }
}

void ta::common::TerraAmazon::onRestartSystemTriggered()
{
  QMessageBox msgBox(this);

  msgBox.setText(tr("The system will be restarted."));
  msgBox.setInformativeText(tr("Do you want to continue?"));
  msgBox.setWindowTitle(tr("Restart system"));

  msgBox.addButton(QMessageBox::No);
  msgBox.addButton(QMessageBox::Yes);

  msgBox.setDefaultButton(QMessageBox::Yes);

  if (msgBox.exec() == QMessageBox::Yes)
    qApp->exit(1000);
}

void ta::common::TerraAmazon::addActions(const QString& plgName, const QString& category, const QList<QAction*>& acts)
{
  if (category.compare("Processing") == 0)
  {
    QMenu* mnu = m_app->getMenu("Processing");

    if (mnu == 0)
      return;

    QMenu* p = new QMenu(plgName, mnu);

    for (QList<QAction*>::const_iterator it = acts.begin(); it != acts.end(); ++it)
      p->addAction(*it);

    mnu->addMenu(p);
  }
  else if (category.compare("Dataaccess") == 0)
  {
    for (QList<QAction*>::const_iterator it = acts.begin(); it != acts.end(); ++it)
      m_workspaceAddLayerMenu->addAction(*it);
  }
  else
  {
    for (QList<QAction*>::const_iterator it = acts.begin(); it != acts.end(); ++it)
      m_pluginsMenu->insertAction(m_app->findAction("ManagePluginsSeparator"), *it);
  }
}

void ta::common::TerraAmazon::addMenusActions()
{
  // File Menu
  m_fileMenu->setObjectName("File");
  m_fileMenu->setTitle(tr("&File"));
  m_fileMenu->addAction(m_fileNewWorkspace);
  m_fileMenu->addAction(m_fileOpenWorkspace);
  m_fileMenu->addAction(m_fileSaveWorkspace);
  m_fileMenu->addAction(m_fileSaveWorkspaceAs);
  m_fileMenu->addSeparator();  
  m_fileMenu->addMenu(m_fileRecentWorkspacesMenu);
  m_fileMenu->addSeparator();
  m_fileMenu->addAction(m_filePrintPreview);
  m_fileMenu->addAction(m_filePrint);
  m_fileMenu->addSeparator();
  m_fileMenu->addAction(m_fileRestartSystem);
  m_fileMenu->addAction(m_fileExit);

  // Show Menu
  m_showMenu->setObjectName("Show");
  m_showMenu->setTitle(tr("&Show"));

  m_showToolBarsMenu->setObjectName("Show.Toolbars");
  m_showToolBarsMenu->setTitle(tr("&Toolbars"));

  m_showMenu->addAction(m_viewDataTable);
  m_showMenu->addAction(m_viewLayerExplorer);
  m_showMenu->addAction(m_viewStyleExplorer);
  m_showMenu->addAction(m_showConnectLayers);
  m_showMenu->addSeparator();
  m_showMenu->addAction(m_viewFullScreen);
  m_showMenu->addSeparator();
  m_showMenu->addMenu(m_showToolBarsMenu);

  // Group the map tools
  QActionGroup* mapToolsGroup = new QActionGroup(this->menuBar());
  mapToolsGroup->setObjectName("Map.ToolsGroup");
  mapToolsGroup->addAction(m_mapZoomIn);
  mapToolsGroup->addAction(m_mapZoomOut);
  mapToolsGroup->addAction(m_mapPan);
  mapToolsGroup->addAction(m_mapInfo);
  mapToolsGroup->addAction(m_mapSelection);
  mapToolsGroup->addAction(m_mapRemoveSelection);

  // Workspoace
  m_workspaceMenu->setObjectName("Workspace");
  m_workspaceMenu->setTitle(tr("&Workspace"));

  m_workspaceAddLayerMenu->setObjectName("Workspace.Add Layer");
  m_workspaceAddLayerMenu->setTitle(tr("&Add Layer"));
  m_workspaceAddLayerMenu->setIcon(QIcon::fromTheme("layer-add"));
  m_workspaceAddLayerMenu->addAction(m_workspaceAddLayerTabularDataSet);
  m_workspaceAddLayerMenu->addAction(m_workspaceAddLayerDataset);
  m_workspaceAddLayerMenu->addSeparator();
  m_workspaceAddLayerMenu->addAction(m_workspaceAddLayerQueryDataSet);
  m_workspaceAddLayerMenu->addSeparator();
  m_workspaceMenu->addAction(m_workspaceAddFolderLayer);
  m_workspaceMenu->addSeparator();
  m_workspaceMenu->addAction(m_layerRemove);
  m_workspaceMenu->addAction(m_layerRename);
  m_workspaceMenu->addSeparator();
  m_workspaceMenu->addAction(m_workspaceProperties);

  // Process Menu
  m_processMenu->setObjectName("Controller");
  m_processMenu->setTitle(tr("&Controller"));

  m_processMenu->addAction(m_processServerControl);
  m_processMenu->addSeparator();
  m_processMenu->addAction(m_processProjectActivation);
  m_processMenu->addSeparator(); 
  m_processMenu->addAction(m_processControl);
  m_processMenu->addSeparator();
  m_processMenu->addAction(m_processCellsManager);

  // Administration Menu
  m_administrationMenu->setObjectName("Administration");
  m_administrationMenu->setTitle(tr("&Administration"));

  m_administrationMenu->addAction(m_adminCreateConceptualModel);
  m_administrationMenu->addAction(m_adminDataExchanger);
  m_administrationMenu->addAction(m_adminQueryDataSource);

  //Project Management Menu
  m_projManagementMenu->setObjectName("Project Management");
  m_projManagementMenu->setTitle(tr("&Project Management"));

  m_projManagementMenu->addAction(m_projManagementUserControl);
  m_projManagementMenu->addAction(m_projManagementGroupControl);
  m_projManagementMenu->addAction(m_projManagementPhaseControl);
  m_projManagementMenu->addSeparator();
  m_projManagementMenu->addAction(m_projProjectControl);
  m_projManagementMenu->addAction(m_projRulesControl);
  m_projManagementMenu->addAction(m_projManagementAttributeControl);
  m_projManagementMenu->addAction(m_projClassControl);
  m_projManagementMenu->addSeparator();
  m_projManagementMenu->addAction(m_projProjectManagementWizard);
  m_projManagementMenu->addAction(m_projAOIManagementWizard);
  m_projManagementMenu->addSeparator();
  m_projManagementMenu->addAction(m_projInformation);
  m_projManagementMenu->addSeparator();
  m_projManagementMenu->addAction(m_projCatalogManager);

  // Settings Menu
  m_settingsMenu->setObjectName("Settings");
  m_settingsMenu->setTitle(tr("&Settings"));

  m_settingsMenu->addAction(m_settingsShowWelcomeDlg);

  // Plugins Menu
  m_pluginsMenu->setObjectName("Plugins");
  m_pluginsMenu->setTitle(tr("&Plugins"));

  m_pluginsMenu->addSeparator()->setObjectName("ManagePluginsSeparator");
  m_pluginsMenu->addAction(m_pluginsManager);

  // Help Menu
  m_helpMenu->setObjectName("Help");
  m_helpMenu->setTitle(tr("&Help"));

  m_helpMenu->addAction(m_helpContents);
  m_helpMenu->addAction(m_helpAbout);
}

void ta::common::TerraAmazon::addPopUpMenu()
{
  te::qt::widgets::LayerItemView* treeView = m_layerExplorer->getExplorer();
  treeView->setAnimated(true);

  // Actions to be added to the context menu when there is no item selected
  treeView->addNoLayerAction(m_workspaceAddLayerMenu->menuAction());

  QAction* noItemSelectedSep = new QAction(this);
  noItemSelectedSep->setSeparator(true);
  treeView->addNoLayerAction(noItemSelectedSep);

  treeView->addNoLayerAction(m_workspaceAddFolderLayer);

  QAction* noItemSelectedSep2 = new QAction(this);
  noItemSelectedSep2->setSeparator(true);
  treeView->addNoLayerAction(noItemSelectedSep2);
  
  treeView->addNoLayerAction(m_projCreateFolder);
  treeView->addNoLayerAction(m_createDraftLayer);

  // Actions to be added to the context menu when there is a unique item selected
  treeView->addAllLayerAction(m_layerRemove);

  // Actions for the folder layer item
  treeView->addFolderLayerAction(m_layerRename);
  treeView->addFolderLayerAction(m_workspaceAddLayerMenu->menuAction());

  QAction* folderSep1 = new QAction(this);
  folderSep1->setSeparator(true);
  treeView->addFolderLayerAction(folderSep1);

  treeView->addFolderLayerAction(m_workspaceAddFolderLayer);

  QAction* folderSep2 = new QAction(this);
  folderSep2->setSeparator(true);
  treeView->addFolderLayerAction(folderSep2);

  // Actions for the single layer item that is not a raster layer
  treeView->addVectorLayerAction(m_layerRemoveObjectSelection);
  treeView->addVectorLayerAction(m_layerRename);

  QAction* actionStyleSep1 = new QAction(this);
  actionStyleSep1->setSeparator(true);
  treeView->addVectorLayerAction(actionStyleSep1);

  QAction* actionChartSep = new QAction(this);
  actionChartSep->setSeparator(true);
  treeView->addVectorLayerAction(actionChartSep);

  treeView->addVectorLayerAction(m_layerShowTable);
  treeView->addVectorLayerAction(m_viewStyleExplorer);
  treeView->addVectorLayerAction(m_layerEditLegend);
  treeView->addVectorLayerAction(m_layerQuery);
  treeView->addVectorLayerAction(m_layerDataExchangerDirectPopUp);

  QAction* actionStyleSep = new QAction(this);
  actionStyleSep->setSeparator(true);
  treeView->addVectorLayerAction(actionStyleSep);

  QAction* actionRemoveSep = new QAction(this);
  actionRemoveSep->setSeparator(true);
  treeView->addVectorLayerAction(actionRemoveSep);

  treeView->addVectorLayerAction(m_layerFitOnMapDisplay);
  treeView->addVectorLayerAction(m_layerFitSelectedOnMapDisplay);
  treeView->addVectorLayerAction(m_layerPanToSelectedOnMapDisplay);
  treeView->addVectorLayerAction(m_layerQuery);
  treeView->addVectorLayerAction(m_layerDataExchangerDirectPopUp);
  treeView->addVectorLayerAction(m_layerCalculateArea);

  QAction* actionFitSep = new QAction(this);
  actionFitSep->setSeparator(true);
  treeView->addVectorLayerAction(actionFitSep);

  treeView->addVectorLayerAction(m_layerSRS);

  QAction* actionSRSSep = new QAction(this);
  actionSRSSep->setSeparator(true);
  treeView->addVectorLayerAction(actionSRSSep);

  treeView->addVectorLayerAction(m_layerCompositionMode);
  treeView->addVectorLayerAction(m_layerProperties);

  // Actions for the raster layer item
  treeView->addRasterLayerAction(m_layerRename);

  actionStyleSep1 = new QAction(this);
  actionStyleSep1->setSeparator(true);
  treeView->addVectorLayerAction(actionStyleSep1);

  QAction* rasterSep1 = new QAction(this);
  rasterSep1->setSeparator(true);
  treeView->addRasterLayerAction(rasterSep1);

  treeView->addRasterLayerAction(m_viewStyleExplorer);
  treeView->addRasterLayerAction(m_layerEditLegend);

  QAction* rasterSep2 = new QAction(this);
  rasterSep2->setSeparator(true);
  treeView->addRasterLayerAction(rasterSep2);

  QAction* rasterSep3 = new QAction(this);
  rasterSep3->setSeparator(true);
  treeView->addRasterLayerAction(rasterSep3);

  treeView->addRasterLayerAction(m_layerFitOnMapDisplay);

  QAction* rasterSep4 = new QAction(this);
  rasterSep4->setSeparator(true);
  treeView->addRasterLayerAction(rasterSep4);

  treeView->addRasterLayerAction(m_layerSRS);

  QAction* rasterSep5 = new QAction(this);
  rasterSep5->setSeparator(true);
  treeView->addRasterLayerAction(rasterSep5);

  treeView->addRasterLayerAction(m_layerCompositionMode);
  treeView->addRasterLayerAction(m_layerProperties);
}

bool ta::common::TerraAmazon::checkServer()
{
  if (!m_taController->getActiveServer().get())
  {
    QMessageBox::information(this, m_app->getAppTitle(), tr("First activate a server."));

    ta::workspace::qt::ServerLoginDialog dlg(this);
    dlg.move(QApplication::desktop()->screen()->rect().center() - dlg.rect().center());
    dlg.setServers(m_workspace->m_servers);

    int res = dlg.exec();

    if (res)
    {
      m_taController->setServers(dlg.getServers());
      m_taController->setCurrentUserId(dlg.getActiveUserId());
      m_taController->setActiveServer(dlg.getActiveServer());

      m_editToolbar->setProcess(LoadProcess(dlg.getActiveUserId(), dlg.getActiveServer()));
      m_editToolbar->setProject(m_taController->getCurrentProjectId());

      workspaceChanged();
      return true;
    }
    else
      return false;
  }
  return true;
}

void ta::common::TerraAmazon::onProjectActivationTriggered()
{
  if (checkServer())
  {
    ta::model::qt::ProjectActivationDialog dlg(this);

    dlg.setServer(m_taController->getActiveServer());

    if (m_taController->getCurrentProjectId() > -1)
    {
      dlg.setActiveProjectId(m_taController->getCurrentProjectId());
    }

    dlg.exec();


    if (dlg.getActiveProjectId() != m_taController->getCurrentProjectId())
    {
      int processID = LoadProcess(m_taController->getCurrentUserId(), m_taController->getActiveServer());

      if (processID != -1)
      {
        ta::model::qt::EndProcessDialog endProcessDlg(this);

        endProcessDlg.setEndProcess(m_taController->getActiveServer(), processID);

        endProcessDlg.exec();

      }

      m_taController->setCurrentProjectId(dlg.getActiveProjectId());

      // set edition
      if (!m_editToolbar->getActiveServer().get())
        m_editToolbar->setServer(m_taController->getActiveServer());

      m_editToolbar->setProcess(LoadProcess(m_taController->getCurrentUserId(), m_taController->getActiveServer()));
      m_editToolbar->setProject(dlg.getActiveProjectId());

      //update user project
      int userId = m_taController->getCurrentUserId();
      if (userId >= 0)
      {
        te::da::DataSourcePtr ds = te::da::GetDataSource(m_taController->getActiveServer()->getId());
        ta::model::UserFacade uf(ds);

        uf.setUserProject(userId, dlg.getActiveProjectId());
      }
    }
  }

  setWindowTitle();
}

void ta::common::TerraAmazon::onProcessControlTriggered()
{
  if (checkServer())
  {
    if (!(m_taController->getCurrentProjectId() != -1))
    {
      QMessageBox::warning(this, m_app->getAppTitle(), tr("Active project not found."));
      return;
    }

    if (!(m_taController->getCurrentUserId() != -1))
    {
      QMessageBox::warning(this, m_app->getAppTitle(), tr("Active user not found."));
      return;
    }

    ta::model::qt::ProcessControlDialog dlg(this);

    dlg.setProcess(m_taController->getActiveServer(), m_taController->getCurrentUserId(), LoadProcess(m_taController->getCurrentUserId(), m_taController->getActiveServer()));

    if (dlg.exec() == QDialog::Accepted)
    {
    }

    m_editToolbar->setProcess(dlg.getProcess());
  }
}

void ta::common::TerraAmazon::onProcessCellManagerTriggered()
{
  if (checkServer())
  {
    te::da::DataSourceInfoPtr activeServer = m_taController->getActiveServer();

    std::vector<te::da::DataSourceInfoPtr> infos;
    infos.push_back(activeServer);

    std::list<te::map::AbstractLayerPtr> layers;
    te::qt::widgets::GetValidLayers(this->getLayerExplorer()->model(), QModelIndex(), layers);

    ta::model::qt::ProcessCellManagerDialog dlg(this);
    dlg.setServer(activeServer);

    dlg.getCreateCellLayerWidget()->setDataSources(infos);
    dlg.getCreateCellLayerWidget()->setLayers(layers);

    dlg.getCreateCellLayerWidget()->setActivationToDataSourceSelector(false);

    dlg.exec();
  }
}

void ta::common::TerraAmazon::onCreateCanceptualModelTriggered()
{
  std::vector<te::da::DataSourceInfoPtr> infos;
  te::da::DataSourceInfoManager::getInstance().getByType("POSTGIS", infos);

  std::list<te::map::AbstractLayerPtr> layers;
  te::qt::widgets::GetValidLayers(this->getLayerExplorer()->model(), QModelIndex(), layers);

  ta::model::qt::CreateConceptualModelDialog dlg(this);

  dlg.getWidget()->setDataSources(infos);
  dlg.getWidget()->setLayers(layers);

  if (dlg.exec() != QDialog::Accepted)
    return;

  te::da::DataSourceInfoPtr tDs = dlg.getWidget()->getTargetDataSource();
  std::string userName = dlg.getWidget()->getUserName();

  te::da::DataSourcePtr source = te::da::DataSourceManager::getInstance().get(tDs->getId(), tDs->getType(), tDs->getConnInfo());

  if (source.get())
  {
    ta::model::UserFacade u(source);
    std::unique_ptr<ta::model::User>user = u.getByName(userName);

    if (user.get())
      m_taController->setCurrentUserId(user->getId());
  }

  m_taController->addServer(tDs);
  m_taController->setActiveServer(tDs);

  te::map::AbstractLayerPtr cellLayer = dlg.getCellLayer();

  if (cellLayer)
  {
    te::qt::af::evt::LayerAdded evt(cellLayer);

    emit triggered(&evt);
  }
}

void ta::common::TerraAmazon::onAdminDataExchangerTriggered()
{
  try
  {
    te::qt::widgets::DataExchangerWizard dlg(this);
    dlg.exec();
  }
  catch (const std::exception& e)
  {
    QMessageBox::warning(this, m_app->getAppTitle(), e.what());
  }
}

void ta::common::TerraAmazon::onAdminQueryDatasourceTriggered()
{
  try
  {
    te::qt::widgets::QueryDataSourceDialog dlg(this);

    connect(&dlg, SIGNAL(createNewLayer(te::map::AbstractLayerPtr)), this, SLOT(onCreateNewLayer(te::map::AbstractLayerPtr)));

    std::list<te::map::AbstractLayerPtr> layers;
    te::qt::widgets::GetValidLayers(getLayerExplorer()->model(), QModelIndex(), layers);

    dlg.setLayerList(layers);
    dlg.setAppMapDisplay(m_display->getDisplay());

    dlg.exec();
  }
  catch (const std::exception& e)
  {
    QMessageBox::warning(this, m_app->getAppTitle(), e.what());
  }
}

void ta::common::TerraAmazon::onCatalogManagerTriggered()
{
  if (checkServer())
  {
    std::list<te::map::AbstractLayerPtr> layers;
    te::qt::widgets::GetValidLayers(getLayerExplorer()->model(), QModelIndex(), layers);

    ta::model::qt::CatalogManagerDialog dlg(this);

    te::da::DataSourcePtr server = te::da::DataSourceManager::getInstance().get(m_taController->getActiveServer()->getId(), m_taController->getActiveServer()->getType(),
      m_taController->getActiveServer()->getConnInfo());

    ta::model::UserFacade uf(server);
    std::unique_ptr<ta::model::User> curUser(dynamic_cast<ta::model::User*>(uf.get(m_taController->getCurrentUserId())));

    dlg.setServer(m_taController->getActiveServer());
    dlg.setLayers(layers);
    dlg.setUsername(curUser->getName());

    dlg.exec();

    if (!m_editToolbar->getActiveServer().get())
      m_editToolbar->setServer(m_taController->getActiveServer());

    m_editToolbar->setDelegate();
  }
}

void ta::common::TerraAmazon::onHelpContentsTriggered()
{
  te::qt::widgets::HelpManager::getInstance().showHelp("TerraAmazon/TA_frontpage.html", "dpi.inpe.br.terraamazon");
}

void ta::common::TerraAmazon::onHelpAboutTriggered()
{
  ta::common::qt::AboutDialog dlg(this);

  dlg.exec();
}

void ta::common::TerraAmazon::onShowWelcomeDialogToggled(bool status)
{
  QString flagStr("FALSE");

  if (status)
    flagStr = "TRUE";

  QSettings sett(QSettings::IniFormat, QSettings::UserScope, qApp->organizationName(), qApp->applicationName());

  QString keyShowDlg = "Welcome/ShowDlg";

  sett.setValue(keyShowDlg, flagStr);
}

void ta::common::TerraAmazon::onEnableEditTriggered(bool checked)
{
  if (checkServer())
  {
    onEnableVisualizationTools(false);

    te::map::AbstractLayer* layer = ta::edition::qt::GetSelectedLayerTreeView(getLayerExplorer());

    te::qt::af::evt::GetMapDisplay eDisplay;
    emit triggered(&eDisplay);

    // Create the edit toolbar
    if (checked)
    {
      if (layer == 0 || layer->getVisibility() != te::map::VISIBLE)
      {
        QMessageBox::information(this, m_app->getAppTitle(), tr("You must select a layer to enter the Edition Mode!"));
        m_editionToolsAction->setChecked(false);
        return;
      }

      if (layer->getSRID() <= 0)
      {
        QMessageBox::information(this, m_app->getAppTitle(), tr("SRID invalid!"));
        m_editionToolsAction->setChecked(false);
        return;
      }

      std::auto_ptr<te::da::DataSetType> dt(layer->getSchema());
      if (dt.get()->hasGeom())
      {
        te::gm::GeometryProperty* gp = te::da::GetFirstGeomProperty(dt.get());

        if ((gp->getGeometryType() != te::gm::MultiPolygonType) && (gp->getGeometryType() != te::gm::PolygonType))
        {
          QMessageBox::information(this, m_app->getAppTitle(), tr("LAYER invalid: IT'S NOT MultiPolygonType/PolygonType"));
          m_editionToolsAction->setChecked(false);
          return;
        }
      }
      else
      {
        QMessageBox::information(this, m_app->getAppTitle(), tr("LAYER invalid!"));
        m_editionToolsAction->setChecked(false);
        return;
      }

      te::da::DataSourcePtr server = te::da::DataSourceManager::getInstance().get(m_taController->getActiveServer()->getId(), m_taController->getActiveServer()->getType(), 
                                                                                  m_taController->getActiveServer()->getConnInfo());

      if (server->isOpened() == false)
        server->open();

      ta::model::CatalogFacade cf(server);
      if (cf.isCatalogLockedForEdition(layer->getTitle())) //TODO: use uri
      {
        QMessageBox::information(this, m_app->getAppTitle(), tr("Layer disabled for edition. Choose another one!"));
        m_editionToolsAction->setChecked(false);
        return;
      }

      // clear the selected items
      te::qt::af::evt::GetAvailableLayers evtClear;
      emit triggered(&evtClear);

      std::list<te::map::AbstractLayerPtr>::iterator it;
      for (it = evtClear.m_layers.begin(); it != evtClear.m_layers.end(); ++it)
      {
        (*it)->clearSelected();
        te::qt::af::evt::LayerSelectedObjectsChanged evtSelObjs((*it));
        emit triggered(&evtSelObjs);
      }

      eDisplay.m_display->getDisplay()->repaint();

      if (!m_editToolbar->getActiveServer().get())
        m_editToolbar->setServer(m_taController->getActiveServer());

      m_editToolbar->setEditionLayer(layer);

      m_editToolbar->get()->show();
    }
    else
    {
      m_editToolbar->get()->hide();
    }

    m_mapSelection->setEnabled(!checked);

    m_editToolbar->onEditActivated(checked);
  }
  else 
  {
    m_editionToolsAction->setChecked(false);
    return;
  }
}

void ta::common::TerraAmazon::closeEvent(QCloseEvent *event)
{
  int processID = -1;

  if (m_taController->getCurrentUserId() > -1)
    processID = LoadProcess(m_taController->getCurrentUserId(), m_taController->getActiveServer());

  if (m_taController->getActiveServer().get() && processID != -1)
  {
    int response = QMessageBox::question(this, "TerraAmazon",
      tr("You have a started process. Do you want to end it?"),
      tr("Yes"), tr("No"));

    if (response == 0)
    {
      //TODO: update tasklog timer
      ta::model::qt::EndProcessDialog dlg(this);

      dlg.setEndProcess(m_taController->getActiveServer(), processID);

      if (dlg.exec() != QDialog::Accepted)
      {
        event->ignore();
        dlg.close();

        return;
      }

      dlg.close();
    }
    else
    {
      //TODO:: TaskLog.. updatelogtime
    }

  }

  checkAndSaveWorkspace();

  event->accept();
}

void ta::common::TerraAmazon::onUserControlTriggered()
{
  if (checkServer())
  {
    ta::model::qt::UserControlDialog dlg(this);
    dlg.setServer(m_taController->getActiveServer());
    dlg.setUser(m_taController->getCurrentUserId());
    dlg.exec();
  }
}

void ta::common::TerraAmazon::onGroupControlTriggered()
{
  if (checkServer())
  {
    ta::model::qt::GroupControlDialog dlg(this);
    dlg.setServer(m_taController->getActiveServer());
    dlg.exec();
  }
}

void ta::common::TerraAmazon::onPhaseControlTriggered()
{
  if (checkServer())
  {
    ta::model::qt::PhaseControlDialog dlg(this);
    dlg.setServer(m_taController->getActiveServer());
    dlg.exec();
  }
}

void ta::common::TerraAmazon::onAttributeControlTriggered()
{
  if (checkServer())
  {
    ta::model::qt::AttributeControlDialog dlg(this);
    dlg.setServer(m_taController->getActiveServer());
    dlg.exec();
  }
}

void ta::common::TerraAmazon::onProjectControlTriggered()
{
  if (checkServer())
  {
    ta::model::qt::ProjectControlDialog dlg(this);

    dlg.setActiveServer(m_taController->getActiveServer());

    dlg.exec();

    if (m_taController->getCurrentProjectId() > -1)
    {
      m_editToolbar->setProject(m_taController->getCurrentProjectId());
    }
  }
}

void ta::common::TerraAmazon::onRulesControlTriggered()
{
  if (checkServer())
  {
    //get user name
    te::da::DataSourcePtr ds = te::da::GetDataSource(m_taController->getActiveServer()->getId());
    ta::model::UserFacade uf(ds);

    std::unique_ptr<ta::model::AbstractItem> item(uf.get(m_taController->getCurrentUserId()));

    ta::model::User* user = dynamic_cast<ta::model::User*>(item.get());

    //dialog
    ta::model::qt::RulesControlDialog dlg(this);

    dlg.setActiveServer(m_taController->getActiveServer());
    dlg.setUser(user->getName());

    dlg.exec();
  }
}

void ta::common::TerraAmazon::onClassControlTriggered()
{
  if (checkServer())
  {

    ta::model::qt::ClassControlDialog dlg(this);

    dlg.setActiveServer(m_taController->getActiveServer());

    dlg.exec();
  }
}

void ta::common::TerraAmazon::onProjectManagementWizardTriggered()
{
  if (checkServer())
  {
    //get user name
    te::da::DataSourcePtr ds = te::da::GetDataSource(m_taController->getActiveServer()->getId());
    ta::model::UserFacade uf(ds);

    std::unique_ptr<ta::model::AbstractItem> item(uf.get(m_taController->getCurrentUserId()));

    ta::model::User* user = dynamic_cast<ta::model::User*>(item.get());

    //wizard
    ta::model::qt::ProjectManagementWizard dlg(this);

    dlg.setActiveServer(m_taController->getActiveServer());
    dlg.setUser(user->getName());

    dlg.exec();
  }
}

void ta::common::TerraAmazon::onAOIManagementWizardTriggered()
{
  if (checkServer())
  {
    ta::model::qt::AOIManagementWizard dlg(this);
    dlg.setServer(m_taController->getActiveServer());
    dlg.exec();
  }
}

void ta::common::TerraAmazon::onProjCreateFolderTriggered()
{
  if (checkServer())
  {
    int curProjId = m_taController->getCurrentProjectId();

    if (curProjId == -1)
    {
      QMessageBox::warning(this, m_app->getAppTitle(), tr("Active project not found."));
      return;
    }

    // Get the folder layer to be added
    bool ok;
    QString text = QInputDialog::getText(this, m_app->getAppTitle(),
      tr("Project Folder name:"), QLineEdit::Normal,
      tr("Enter folder layer name"), &ok);

    if (!ok)
      return;

    if (text.isEmpty())
    {
      QMessageBox::warning(this, m_app->getAppTitle(), tr("Enter the layer name!"));
      return;
    }

    //create folder layer
    te::map::FolderLayerPtr parentLayer = ta::model::CreateProjectFolderLayer(text.toStdString(), m_taController->getActiveServer(), curProjId);

    std::list<te::map::AbstractLayerPtr> folderLayerList;
    folderLayerList.push_back(parentLayer);

    //insert in tree
    getLayerExplorer()->addLayers(folderLayerList, QModelIndex());
    workspaceChanged();

    //change the style in tree
    if (!m_editToolbar->getActiveServer().get())
      m_editToolbar->setServer(m_taController->getActiveServer());

    m_editToolbar->setDelegate();
  }
}

void ta::common::TerraAmazon::onProjInformationTriggered()
{
  if (checkServer())
  {
    ta::model::qt::ProjectInformationDialog dlg(this);

    dlg.setActiveServer(m_taController->getActiveServer());

    dlg.exec();
  }
}

void ta::common::TerraAmazon::onCreateDraftLayerTriggered()
{
  if (checkServer())
  {
    //get file path
    QString fileName = QFileDialog::getSaveFileName(this, tr("Create Draft Layer..."), QString(), tr("Shapefile (*.shp *.SHP);;"));

    if (fileName.isEmpty())
      return;

    //create draft layer
    boost::filesystem::path outfile(fileName.toStdString());
    std::string layerName = outfile.leaf().string();

    int srid = ta::model::GetServerCellLayerSRID(m_taController->getActiveServer());

    te::map::AbstractLayerPtr draftLayer = ta::model::CreateDraftLayer(fileName.toStdString(), layerName, srid);

    std::list<te::map::AbstractLayerPtr> list;
    list.push_back(draftLayer);

    //insert in tree
    getLayerExplorer()->addLayers(list, QModelIndex());

    workspaceChanged();
  }
}

void ta::common::TerraAmazon::onApplicationTriggered(te::qt::af::evt::Event* e)
{
  switch (e->m_id)
  {
  case te::qt::af::evt::NEW_ACTIONS_AVAILABLE:
  {
    te::qt::af::evt::NewActionsAvailable* evt = static_cast<te::qt::af::evt::NewActionsAvailable*>(e);
    addActions(evt->m_plgName.c_str(), evt->m_category.c_str(), evt->m_actions);
  }
  break;

  case te::qt::af::evt::LAYER_ADDED:
  case te::qt::af::evt::LAYER_REMOVED:
  case te::qt::af::evt::LAYER_VISIBILITY_CHANGED:
    workspaceChanged();
    break;

  default:
    BaseApplication::onApplicationTriggered(e);
  }
}

void ta::common::TerraAmazon::onPluginsManagerTriggered()
{
  try
  {
    te::qt::widgets::PluginManagerDialog dlg(this);
    dlg.exec();
  }
  catch (const std::exception& e)
  {
    QMessageBox::warning(this, m_app->getAppTitle(), e.what());
  }
}

void ta::common::TerraAmazon::onRecentWorkspacesTriggered(QAction* ws)
{
  QString wsFile = ws->data().toString();
  openWorkspace(wsFile);
}

void ta::common::TerraAmazon::onAddDataSetLayerTriggered()
{
  try
  {
    if (m_workspace == 0)
      throw te::common::Exception(TE_TR("Error: there is no opened workspace!"));

    QApplication::setOverrideCursor(Qt::WaitCursor);

    // Get the parent layer where the dataset layer(s) will be added.
    QModelIndex pF = GetParent(getLayerExplorer());

    // Get the layer(s) to be added
    std::unique_ptr<te::qt::widgets::DataSourceSelectorDialog> dselector(new te::qt::widgets::DataSourceSelectorDialog(this));

    QString dsTypeSett = te::qt::af::GetLastDatasourceFromSettings();

    if (!dsTypeSett.isNull() && !dsTypeSett.isEmpty())
      dselector->setDataSourceToUse(dsTypeSett);

    QApplication::restoreOverrideCursor();

    int retval = dselector->exec();

    QApplication::setOverrideCursor(Qt::WaitCursor);

    if (retval == QDialog::Rejected)
    {
      QApplication::restoreOverrideCursor();
      return;
    }

    std::list<te::da::DataSourceInfoPtr> selectedDatasources = dselector->getSelecteds();

    if (selectedDatasources.empty())
    {
      QApplication::restoreOverrideCursor();
      return;
    }

    dselector.reset(0);

    const std::string& dsTypeId = selectedDatasources.front()->getType();

    const te::qt::widgets::DataSourceType* dsType = te::qt::widgets::DataSourceTypeManager::getInstance().get(dsTypeId);

    std::unique_ptr<QWidget> lselectorw(dsType->getWidget(te::qt::widgets::DataSourceType::WIDGET_LAYER_SELECTOR, this));

    if (lselectorw.get() == 0)
    {
      QApplication::restoreOverrideCursor();
      throw te::common::Exception((boost::format(TE_TR("No layer selector widget found for this type of data source: %1%!")) % dsTypeId).str());
    }

    te::qt::widgets::AbstractLayerSelector* lselector = dynamic_cast<te::qt::widgets::AbstractLayerSelector*>(lselectorw.get());

    if (lselector == 0)
    {
      QApplication::restoreOverrideCursor();
      throw te::common::Exception(TE_TR("Wrong type of object for layer selection!"));
    }

    lselector->set(selectedDatasources);

    QApplication::restoreOverrideCursor();

    std::list<te::map::AbstractLayerPtr> layers = lselector->getLayers();

    if (layers.empty())
      return;

    QApplication::setOverrideCursor(Qt::WaitCursor);

    lselectorw.reset(0);

    getLayerExplorer()->addLayers(layers, pF);

    te::qt::af::SaveLastDatasourceOnSettings(dsTypeId.c_str());

    workspaceChanged();
  }
  catch (const std::exception& e)
  {
    QApplication::restoreOverrideCursor();
    QMessageBox::warning(this, m_app->getAppTitle(), e.what());
  }
  catch (...)
  {
    QApplication::restoreOverrideCursor();
    QMessageBox::warning(this,
      m_app->getAppTitle(),
      tr("Unknown error while trying to add a layer from a dataset!"));
  }

  QApplication::restoreOverrideCursor();
}

void ta::common::TerraAmazon::onAddQueryLayerTriggered()
{
  try
  {
    if (m_workspace == 0)
      throw te::common::Exception(TE_TR("Error: there is no opened workspace!"));

    // Get the parent layer where the dataset layer(s) will be added.
    QModelIndex par = GetParent(getLayerExplorer());

    std::unique_ptr<te::qt::widgets::QueryLayerBuilderWizard> qlb(new te::qt::widgets::QueryLayerBuilderWizard(this));

    std::list<te::map::AbstractLayerPtr> layers;
    te::qt::widgets::GetValidLayers(getLayerExplorer()->model(), QModelIndex(), layers);

    qlb->setLayerList(layers);

    int retval = qlb->exec();

    if (retval == QDialog::Rejected)
      return;

    te::map::AbstractLayerPtr layer = qlb->getQueryLayer();

    if ((m_layerExplorer != 0) && (m_layerExplorer->getExplorer() != 0))
    {
      std::list<te::map::AbstractLayerPtr> ls;
      ls.push_back(layer);

      getLayerExplorer()->addLayers(ls, par);

      workspaceChanged();
    }
  }
  catch (const std::exception& e)
  {
    QMessageBox::warning(this, m_app->getAppTitle(), e.what());
  }
  catch (...)
  {
    QMessageBox::warning(this,
      m_app->getAppTitle(),
      tr("Unknown error while trying to add a layer from a queried dataset!"));
  }

}

void ta::common::TerraAmazon::onAddTabularLayerTriggered()
{
  try
  {
    if (m_workspace == 0)
      throw te::common::Exception(TE_TR("Error: there is no opened workspace!"));

    // Get the parent layer where the tabular layer will be added.
    QModelIndex par = GetParent(getLayerExplorer());

    te::qt::widgets::DataPropertiesDialog dlg(this);
    int res = dlg.exec();

    if (res == QDialog::Accepted)
    {
      if ((m_layerExplorer != 0) && (m_layerExplorer->getExplorer() != 0))
      {
        std::list<te::map::AbstractLayerPtr> ls;
        ls.push_back(dlg.getDataSetAdapterLayer());

        getLayerExplorer()->addLayers(ls, par);

        workspaceChanged();
      }
    }
  }
  catch (const std::exception& e)
  {
    QMessageBox::warning(this, m_app->getAppTitle(), e.what());
  }
  catch (...)
  {
    QMessageBox::warning(this,
      m_app->getAppTitle(),
      tr("Unknown error while trying to add a layer from a queried dataset!"));
  }
}

void ta::common::TerraAmazon::onAddFolderLayerTriggered()
{
  // Get the parent item where the folder layer will be added.
  QModelIndex idx = GetParent(getLayerExplorer());

  // Get the folder layer to be added
  bool ok;
  QString text = QInputDialog::getText(this, m_app->getAppTitle(),
    tr("Folder layer name:"), QLineEdit::Normal,
    tr("Enter folder layer name"), &ok);

  if (!ok)
    return;

  if (text.isEmpty())
  {
    QMessageBox::warning(this, m_app->getAppTitle(), tr("Enter the layer name!"));
    return;
  }

  getLayerExplorer()->addFolder(text.toStdString(), idx);

  workspaceChanged();
}

void ta::common::TerraAmazon::configureTerraAmazon(const ta::common::TaUserType& type)
{
  if (type == ta::common::TA_OPERATOR_USER)
  {
    m_processMenu->menuAction()->setVisible(true);
    m_projManagementMenu->menuAction()->setVisible(false);
    m_administrationMenu->menuAction()->setVisible(false);

    m_projCreateFolder->setVisible(true);
    m_editionToolsAction->setVisible(true);
    m_statusBarInfoWidget->setVisible(true);
  }
  else if (type == ta::common::TA_ADMIN_USER)
  {
    m_processMenu->menuAction()->setVisible(true);
    m_projManagementMenu->menuAction()->setVisible(true);
    m_administrationMenu->menuAction()->setVisible(true);
    
    m_projCreateFolder->setVisible(true);
    m_editionToolsAction->setVisible(true);
    m_statusBarInfoWidget->setVisible(true);

    //change layout for each user type
    QPalette palette(QColor(226, 245, 230)); // green - admin
    this->setPalette(palette);
  }
  else // TA_GIS_USER
  {
    m_processMenu->menuAction()->setVisible(false);
    m_projManagementMenu->menuAction()->setVisible(false);
    m_administrationMenu->menuAction()->setVisible(false);
    
    m_projCreateFolder->setVisible(false);
    m_editionToolsAction->setVisible(false);
    m_statusBarInfoWidget->setVisible(false);

    //change layout for each user type
    QPalette palette(QColor(245, 245, 230)); // yellow - gis user
    this->setPalette(palette);
  }
}

void ta::common::TerraAmazon::setWindowTitle(bool hasChanged)
{
  //application title
  QString title = m_app->getAppTitle();

  m_statusBarInfoWidget->clear();

  //workspace title
  if (m_workspace)
  {
    m_statusBarInfoWidget->setWokspace(m_workspace->m_title.toStdString());
  }

  //server title
  te::da::DataSourceInfoPtr dsPtr = m_taController->getActiveServer();
  if (dsPtr.get())
  {
    m_statusBarInfoWidget->setServer(dsPtr->getTitle());
  }

  //user title
  int userId = m_taController->getCurrentUserId();
  if (dsPtr.get() && userId >= 0)
  {
    te::da::DataSourcePtr ds = te::da::GetDataSource(dsPtr->getId());
    ta::model::UserFacade uf(ds);

    std::unique_ptr<ta::model::AbstractItem> item(uf.get(userId));

    ta::model::User* user = dynamic_cast<ta::model::User*>(item.get());

    m_statusBarInfoWidget->setUser(user->getName());
  }

  //project title
  int projId = m_taController->getCurrentProjectId();
  if (dsPtr.get() && projId >= 0)
  {
    te::da::DataSourcePtr ds = te::da::GetDataSource(dsPtr->getId());
    ta::model::ProjectFacade pf(ds);

    std::unique_ptr<ta::model::AbstractItem> item(pf.get(projId));

    ta::model::Project* proj = dynamic_cast<ta::model::Project*>(item.get());

    m_statusBarInfoWidget->setProject(proj->getName());
  }

  //has changed
  if (hasChanged)
  {
    title += " *";
  }

  QMainWindow::setWindowTitle(title);
}

void ta::common::TerraAmazon::onLayerDataExchangerTriggered()
{
  try
  {
    te::qt::widgets::DirectExchangerDialog dlg(this);

    std::list<te::map::AbstractLayerPtr> selectedLayerItems = te::qt::widgets::GetSelectedLayersOnly(getLayerExplorer());

    if (selectedLayerItems.empty())
    {
      QMessageBox::warning(this, m_app->getAppTitle(),
        tr("Select a single layer in the layer explorer!"));
      return;
    }

    std::list<te::map::AbstractLayerPtr> layers;
    layers.push_back(selectedLayerItems.front());

    dlg.setLayers(layers);

    dlg.exec();
  }
  catch (const std::exception& e)
  {
    QMessageBox::warning(this, m_app->getAppTitle(), e.what());
  }
}

void ta::common::TerraAmazon::onLayerEditLegendTriggered()
{
  try
  {
    std::list<te::map::AbstractLayerPtr> selectedLayers = te::qt::widgets::GetSelectedLayersOnly(getLayerExplorer());

    if (selectedLayers.empty())
    {
      QMessageBox::warning(this, m_app->getAppTitle(), tr("Select a single layer in the layer explorer!"));
      return;
    }
    else
    {
      for (std::list<te::map::AbstractLayerPtr>::iterator it = selectedLayers.begin(); it != selectedLayers.end(); ++it)
      {
        if (!(*it)->isValid())
        {
          QMessageBox::warning(this, m_app->getAppTitle(), tr("There are invalid layers selected!"));

          return;
        }
      }
    }

    // The object grouping will be accomplished only on the first layer selected
    te::map::AbstractLayerPtr selectedLayer = *selectedLayers.begin();

    // Get all layer with grouping to dispose to import
    std::vector<te::map::AbstractLayerPtr> allLayers;
    te::qt::widgets::GetValidLayers(getLayerExplorer()->model(), QModelIndex(), allLayers);

    te::qt::widgets::GroupingDialog dlg(this);
    dlg.setLayers(selectedLayer, allLayers);


    if (dlg.exec() == QDialog::Accepted)
    {
      getLayerExplorer()->updateGrouping(*getLayerExplorer()->selectionModel()->selectedIndexes().begin());

      m_display->getDisplay()->refresh(true);

      workspaceChanged();
      // projectChanged();
    }
  }
  catch (const std::exception& e)
  {
    QMessageBox::warning(this, m_app->getAppTitle(), e.what());
  }
}

void ta::common::TerraAmazon::onLayerQueryTriggered()
{
  std::list<te::map::AbstractLayerPtr> selectedLayers = te::qt::widgets::GetSelectedLayersOnly(getLayerExplorer());

  if (selectedLayers.empty())
  {
    QMessageBox::warning(this, m_app->getAppTitle(),
      tr("Select a layer in the layer explorer!"));
    return;
  }
  else
  {
    std::list<te::map::AbstractLayerPtr>::iterator it = selectedLayers.begin();

    while (it != selectedLayers.end())
    {
      if (!it->get()->isValid())
      {
        QMessageBox::warning(this, m_app->getAppTitle(),
          tr("There are invalid layers selected!"));
        return;
      }

      ++it;
    }
  }

  te::qt::widgets::QueryDialog* queryDlg  = new te::qt::widgets::QueryDialog(this);
  queryDlg->setAttribute(Qt::WA_DeleteOnClose, true);

  connect(queryDlg, SIGNAL(highlightLayerObjects(const te::map::AbstractLayerPtr&, te::da::DataSet*, const QColor&)),
    SLOT(onHighlightLayerObjects(const te::map::AbstractLayerPtr&, te::da::DataSet*, const QColor&)));

  connect(queryDlg, SIGNAL(layerSelectedObjectsChanged(const te::map::AbstractLayerPtr&)),
    SLOT(onLayerSelectedObjectsChanged(const te::map::AbstractLayerPtr&)));

  std::list<te::map::AbstractLayerPtr> allLayersList;
  te::qt::widgets::GetValidLayers(getLayerExplorer()->model(), QModelIndex(), allLayersList);

  queryDlg->setLayerList(allLayersList);

  selectedLayers = GetSelectedLayersOnly(getLayerExplorer());

  if (!selectedLayers.empty())
    queryDlg->setCurrentLayer(*(selectedLayers.begin()));

  queryDlg->show();
}

void ta::common::TerraAmazon::onLayerCompositionModeTriggered()
{
  std::list<te::map::AbstractLayerPtr> selectedLayers = te::qt::widgets::GetSelectedLayersOnly(getLayerExplorer());

  if (!selectedLayers.empty())
  {
    std::list<te::map::AbstractLayerPtr>::iterator it = selectedLayers.begin();

    m_compModeMenu->setLayer(*it);
  }
}

void ta::common::TerraAmazon::onLayerCalculateAreaTriggered()
{
  try
  {
    std::list<te::map::AbstractLayerPtr> selectedLayers = te::qt::widgets::GetSelectedLayersOnly(getLayerExplorer());

    if (selectedLayers.empty())
    {
      QMessageBox::warning(this, m_app->getAppTitle(), tr("Select a single layer in the layer explorer!"));
      return;
    }
    else
    {
      te::map::AbstractLayerPtr layer = *selectedLayers.begin();

      te::gm::GeomType type = te::map::GetGeomType(layer);

      if (type != te::gm::PolygonType && type != te::gm::MultiPolygonType)
      {
        QMessageBox::warning(this, m_app->getAppTitle(), tr("Layer with invalid geometry type selected!"));

        return;
      }
    }

    // The object grouping will be accomplished only on the first layer selected
    te::map::AbstractLayerPtr selectedLayer = *selectedLayers.begin();

    ta::common::qt::CalculateLayerAreaDialog dlg(this);

    dlg.setLayer(selectedLayer);

    dlg.exec();
  }
  catch (const std::exception& e)
  {
    QMessageBox::warning(this, m_app->getAppTitle(), e.what());
  }
}

void ta::common::TerraAmazon::onCreateNewLayer(te::map::AbstractLayerPtr layer)
{
  te::qt::af::evt::LayerAdded evt(layer);
  m_app->trigger(&evt);
}

void ta::common::TerraAmazon::onEnableVisualizationTools(bool enable)
{
  m_mapZoomIn->setChecked(enable);
  m_mapZoomOut->setChecked(enable);
  m_mapPan->setChecked(enable);
  m_mapInfo->setChecked(enable);
  m_mapSelection->setChecked(enable);
  m_mapRemoveSelection->setChecked(enable);
}
