/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
\file TerraAmazonController.cpp

\brief The API for controller of TerraAmazon application.
*/

// Boost
#include <boost/foreach.hpp> // Boost => don't change this include order, otherwise you may have compiling problems! 

// TerraLib
#include <terralib/BuildConfig.h>
#include <terralib/common/SystemApplicationSettings.h>
#include <terralib/dataaccess/datasource/DataSourceInfoManager.h>
#include <terralib/qt/af/SplashScreenManager.h>
#include <terralib/qt/widgets/utils/ScopedCursor.h>

// TerraAmazon
#include "model/item/Project.h"
#include "TerraAmazonController.h"
#include "Utils.h"

// Qt
#include <QApplication>
#if QT_VERSION < 0x050000
#include <QDesktopServices>
#endif
#include <QDir>
#include <QIcon>
#include <QMenu>
#include <QMessageBox>
#include <QResource>
#if QT_VERSION >= 0x050000
#include <QStandardPaths>
#endif
#include <QWidget>

// Boost
#include <boost/filesystem.hpp>

//STL
#include <algorithm>

//#if defined(TERRALIB_APACHE_LOG4CXX_ENABLED) && defined(TERRALIB_LOGGER_ENABLED)
////Log4cxx
//#include <log4cxx/basicconfigurator.h>
//#include <log4cxx/consoleappender.h>
//#include <log4cxx/fileappender.h>
//#include <log4cxx/helpers/pool.h>
//#include <log4cxx/helpers/transcoder.h>
//#include <log4cxx/logger.h>
//#include <log4cxx/logmanager.h>
//#include <log4cxx/logstring.h>
//#include <log4cxx/simplelayout.h>
//#endif

ta::common::TerraAmazonController::TerraAmazonController(te::qt::af::ApplicationController* app, std::string appConfigFile)
  : QObject(),
  m_app(app),
  m_currentUserId(-1),
  m_currentProjId(-1)
{
  te::common::SystemApplicationSettings::getInstance().load(appConfigFile);

  m_appWorkspaceExtension = QString::fromStdString(te::common::SystemApplicationSettings::getInstance().getValue("Application.WorkspaceExtension"));
}

ta::common::TerraAmazonController::~TerraAmazonController()
{
  m_appWorkspaceExtension.clear();

  m_recentWs.clear();

  m_recentWsTitles.clear();
}

void ta::common::TerraAmazonController::initializeWorkspaceMenus()
{
  te::qt::af::SplashScreenManager::getInstance().showMessage("Loading recent workspaces...");

  try
  {
    QSettings user_settings(QSettings::IniFormat,
                            QSettings::UserScope,
                            QApplication::instance()->organizationName(),
                            QApplication::instance()->applicationName());

    QVariant wsPath = user_settings.value("workspaces/most_recent/path", "");
    QVariant wsTitle = user_settings.value("workspaces/most_recent/title", "");

    QMenu* mnu = m_app->getMenu("File.Recent Workspaces");

    if(!wsPath.toString().isEmpty())
    {
      QAction* act = mnu->addAction(wsPath.toString());
      act->setData(wsPath);

      mnu->addSeparator();

      m_recentWs.append(wsPath.toString());
      m_recentWsTitles.append(wsTitle.toString());
    }
    
    user_settings.beginGroup("workspaces");
    
    int nrc = user_settings.beginReadArray("recents");
    
    for(int i = 0; i != nrc; ++i)
    {
      user_settings.setArrayIndex(i);
      QString npath = user_settings.value("workspace/path").toString();
      QString ntitle = user_settings.value("workspace/title").toString();

      QAction* act = mnu->addAction(npath);
      act->setData(npath);
      m_recentWs.append(npath);
      m_recentWsTitles.append(ntitle);
    }

    mnu->setEnabled(true);

    te::qt::af::SplashScreenManager::getInstance().showMessage("Recent workspaces loaded!");
  }
  catch(const std::exception& e)
  {
    te::qt::widgets::ScopedCursor acursor(Qt::ArrowCursor);

    QString msgErr(tr("Error loading the registered workspaces: %1"));

    msgErr = msgErr.arg(e.what());

    QMessageBox::warning(m_app->getMainWindow(), m_app->getAppTitle(), msgErr);
  }
}

void ta::common::TerraAmazonController::updateRecentWorkspaces(const QString& wsFile, const QString& wsTitle)
{
  int pos = m_recentWs.indexOf(wsFile);

  QString author;
  int maxSaved;

  GetWorkspaceInformationsFromSettings(author, maxSaved);

  if(pos != 0)
  {
    if(pos < 0)
    {
      if(m_recentWs.size() > maxSaved) // TODO: Size of the list must be configurable.
      {
        m_recentWs.removeLast();
        m_recentWsTitles.removeLast();
      }

      m_recentWs.prepend(wsFile);
      m_recentWsTitles.prepend(wsTitle);
    }
    else
    {
      m_recentWs.move(pos, 0);
      m_recentWsTitles.move(pos, 0);
    }

    if(m_recentWs.isEmpty())
      return;

    QMenu* mnu = m_app->getMenu("File.Recent Workspaces");

    mnu->clear();

    mnu->setEnabled(true);

    QString recWs= m_recentWs.at(0);
    QAction* act = mnu->addAction(recWs);
    act->setData(recWs);

    mnu->addSeparator();

    if(m_recentWs.size() > 1)
      for(int i=1; i<m_recentWs.size(); i++)
      {
        recWs = m_recentWs.at(i);
        act = mnu->addAction(recWs);
        act->setData(recWs);
      }
  }

  QAction* act = m_app->findAction("File.Save Workspace As");

  if(act != 0)
    act->setEnabled(true);
}

const QString& ta::common::TerraAmazonController::getAppWorkspaceExtension() const
{
  return m_appWorkspaceExtension;
}

QString ta::common::TerraAmazonController::getMostRecentWorkspace() const
{
  return m_recentWs.isEmpty() ? QString("") : m_recentWs.front();
}

QString ta::common::TerraAmazonController::getExtensionFilter()
{
  QString appName = m_app->getAppName();
  QString appWorkspaceExtension = m_appWorkspaceExtension;
  QString extensionFilter = appName;
  extensionFilter += QString(" (*.");
  extensionFilter += appWorkspaceExtension + ")";

  return extensionFilter;
}

void ta::common::TerraAmazonController::addServer(te::da::DataSourceInfoPtr server)
{
  for (size_t i = 0; i < m_servers.size(); ++i)
  {
    std::string srvConnInfo = m_servers[i]->getConnInfoAsString();
    if (srvConnInfo == server->getConnInfoAsString())
      return;
  }

  m_servers.push_back(server);
}

void ta::common::TerraAmazonController::removeServer(te::da::DataSourceInfoPtr server)
{
  std::remove(m_servers.begin(), m_servers.end(), server);
}

std::vector<te::da::DataSourceInfoPtr> ta::common::TerraAmazonController::getServers()
{
  return m_servers;
}

void ta::common::TerraAmazonController::setServers(std::vector<te::da::DataSourceInfoPtr> servers)
{
  //register each server into datasourceinfomanager
  for (std::size_t t = 0; t < servers.size(); ++t)
  {
    te::da::DataSourceInfoManager::getInstance().add(servers[t]);
  }

  m_servers = servers;
}

te::da::DataSourceInfoPtr ta::common::TerraAmazonController::getActiveServer()
{
  return m_activeServer;
}

void ta::common::TerraAmazonController::setActiveServer(te::da::DataSourceInfoPtr server)
{
  m_activeServer.swap(server);
}

int ta::common::TerraAmazonController::getCurrentUserId()
{
  return m_currentUserId;
}

void ta::common::TerraAmazonController::setCurrentUserId(int userId)
{
  m_currentUserId = userId;
}

int ta::common::TerraAmazonController::getCurrentProjectId()
{
  return m_currentProjId;
}

void ta::common::TerraAmazonController::setCurrentProjectId(int projId)
{
  m_currentProjId = projId;
}
