/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file TerraAmazonController.h

  \brief The API for controller of TerraAmazon application.
*/

#ifndef __TERRAAMAZON_COMMON_TERRAAMAZONCONTROLLER_H
#define __TERRAAMAZON_COMMON_TERRAAMAZONCONTROLLER_H

// TerraAmazon
#include "Config.h"

// Terralib
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>
#include <terralib/qt/af/ApplicationController.h>

// STL
#include <map>
#include <set>
#include <vector>


// Qt
#include <QObject>
#include <QStringList>
#include <QSettings>
#include <QColor>

// Forward declarations
class QAction;
class QActionGroup;
class QMenu;
class QMenuBar;
class QToolBar;
class QWidget;

namespace ta
{
  namespace common
  {
    /*!
    \class TerraAmazonController

    \brief The API for controller of TerraAmazon application.

    */
    class TerraAmazonController : public QObject
    {
      Q_OBJECT

    public:

      /*!
      \brief Constructor.

      \param parent The parent object.
      */
      TerraAmazonController(te::qt::af::ApplicationController* app, std::string appConfigFile);

      /*!
      \brief Destructor.
      */
      virtual ~TerraAmazonController();


      /*!
      \brief Initializes the menus for the most recent open workspaces.
      */
      virtual void initializeWorkspaceMenus();

      /*!
      \brief Update the list of recent workspaces. This is commonly used when there's a new most recent workspaces.

      \param ws_file Complete file name for the workspace file.

      \param ws_title Title of the workspace.
      */
      void updateRecentWorkspaces(const QString& wsFile, const QString& wsTitle);

      /*!
      \brief Returns the application workspace extension.

      \return Application workspace extension.
      */
      const QString& getAppWorkspaceExtension() const;

      /*!
      \brief Returns the most recent workspace.

      \return Application most recent opened workspaces.
      */
      QString getMostRecentWorkspace() const;

      /*!
      \brief Returns the workspace extension filter .

      \return Filter string.
      */
      QString getExtensionFilter();

      /*!
      \brief Add a server.

      \param server DataSourceInfoPtr that will be added.
      */
      void addServer(te::da::DataSourceInfoPtr server);

      /*!
      \brief Remove a server.

      \param server DataSourceInfoPtr that will be removed.
      */
      void removeServer(te::da::DataSourceInfoPtr server);

      /*!
      \brief Get the servers connections vector.

      \return Vector of DataSourceInfoPtr.
      */
      std::vector<te::da::DataSourceInfoPtr> getServers();

      /*!
      \brief Sets the servers connections.

      \param servers Vector of DataSourceInfoPtr.
      */
      void setServers(std::vector<te::da::DataSourceInfoPtr> servers);

      /*!
      \brief Get the active server

      \return Application active server
      */
      te::da::DataSourceInfoPtr getActiveServer();

      /*!
      \brief Sets the active server

      \param server The DataSourceInfoPtr of the active server.
      */
      void setActiveServer(te::da::DataSourceInfoPtr server);

      /*!
      \brief Get the current user.

      \return Application current user.
      */
      int getCurrentUserId();

      /*!
      \brief Set the current user.

      \param user Application current user.
      */
      void setCurrentUserId(int userId);

      /*!
      \brief Get the current project.

      \return Application current project.
      */
      int getCurrentProjectId();

      /*!
      \brief Set the current project.

      \param user Application current project.
      */
      void setCurrentProjectId(int projId);

    protected:

      te::qt::af::ApplicationController* m_app;

      int                                     m_currentUserId;          //!< The id of the current user.
      int                                     m_currentProjId;          //!< THe id of the current project.
      QString                                 m_appWorkspaceExtension;  //!< Application workspace extension.
      QStringList                             m_recentWs;               //!< List of the recent workspaces.
      QStringList                             m_recentWsTitles;         //!< List of the titles of the recent workspaces.
      std::vector<te::da::DataSourceInfoPtr>  m_servers;                //!< List of servers.
      te::da::DataSourceInfoPtr               m_activeServer;           //!< Application active server.
    };
  } // end namespace common
}   // end namespace ta

#endif // __TERRAAMAZON_COMMON_TERRAAMAZONCONTROLLER_H

