/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of
    this application.
    */

/*!
  \file Utils.cpp

  \brief Utility routines for the TerraAmazon Application Framework module.
  */

// Boost
#include <boost/foreach.hpp> // Boost => don't change this include order, otherwise you may have compiling problems! 

// TerraLib
#include <terralib/Version.h>
#include <terralib/common/BoostUtils.h>
#include <terralib/common/PlatformUtils.h>
#include <terralib/se/serialization/xml/Style.h>
#include <terralib/xml/AbstractWriter.h>
#include <terralib/xml/AbstractWriterFactory.h>
#include <terralib/xml/Reader.h>
#include <terralib/xml/ReaderFactory.h>
#include "Utils.h"

// TerraAmazon
#include "../model/facade/ProcessFacade.h"
#include "../model/facade/ProjectFacade.h"
#include "../model/facade/TaskLogFacade.h"
#include "../model/facade/UserFacade.h"
#include "../model/item/Process.h"
#include "../model/item/Project.h"
#include "../model/item/TaskLog.h"
#include "../model/item/User.h"

// STL
#include <cassert>
#include <fstream>
#include <ostream>
#include <memory>

// Boost
#include <boost/algorithm/string/replace.hpp>
#include <boost/filesystem.hpp>
#include <boost/format.hpp>
#include <boost/property_tree/xml_parser.hpp>
#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/json_parser.hpp>

#include <QApplication>
#include <QSettings>
#include <QStandardPaths>

void ta::common::GetWorkspaceInformationsFromSettings(QString& defaultAuthor, int& maxSaved)
{
  QSettings sett(QSettings::IniFormat, QSettings::UserScope, qApp->organizationName(), qApp->applicationName());

  sett.beginGroup("workspaces");
  defaultAuthor = sett.value("author_name").toString();
  maxSaved = sett.value("recents_history_size").toInt();
  sett.endGroup();
}

void ta::common::SaveWorkspaceInformationsOnSettings(const QString& defaultAuthor, const int& maxSaved)
{
  QSettings sett(QSettings::IniFormat, QSettings::UserScope, qApp->organizationName(), qApp->applicationName());

  sett.beginGroup("workspaces");
  sett.setValue("author_name", defaultAuthor);
  sett.setValue("recents_history_size", maxSaved);
  sett.endGroup();
}

void ta::common::SaveOpenLastWorkspaceOnSettings(bool openLast)
{
  QSettings sett(QSettings::IniFormat, QSettings::UserScope, qApp->organizationName(), qApp->applicationName());

  sett.setValue("workspaces/openLastDataSource", openLast);
}

bool ta::common::GetOpenLastWorkspaceFromSettings()
{
  QSettings sett(QSettings::IniFormat, QSettings::UserScope, qApp->organizationName(), qApp->applicationName());

  QVariant variant = sett.value("workspaces/openLastDataSource");

  // If the option was never edited
  if (variant.isNull() || !variant.isValid())
    return true;

  return variant.toBool();
}

void ta::common::WriteDefaultWorkspaceFile(const QString& fileName)
{
  boost::property_tree::ptree p;

  std::string schema_location = ta::common::FindInTerraAmazonPath("share/terralib/schemas/terralib/qt/af/project.xsd");

  //Header
  p.add("Workspace.<xmlattr>.xmlns:xsd", "http://www.w3.org/2001/XMLSchema-instance");
  p.add("Workspace.<xmlattr>.xmlns:te_map", "http://www.terralib.org/schemas/maptools");
  p.add("Workspace.<xmlattr>.xmlns:te_qt_af", "http://www.terralib.org/schemas/qt/af");
  p.add("Workspace.<xmlattr>.xmlns", "http://www.terralib.org/schemas/qt/af");
  p.add("Workspace.<xmlattr>.xsd:schemaLocation", "http://www.terralib.org/schemas/qt/af " + schema_location);
  p.add("Workspace.<xmlattr>.version", TERRALIB_VERSION_STRING);

  //Contents
  p.add("Workspace.Title", "Default woprkspace");
  p.add("Workspace.Author", "");
  p.add("Workspace.ComponentList", "");
  p.add("Workspace.te_map:LayerList", "");

  //Store file
#if BOOST_VERSION > 105600
  boost::property_tree::xml_writer_settings<std::string> settings('\t', 1);
#else
  boost::property_tree::xml_writer_settings<char> settings('\t', 1);
#endif
  boost::property_tree::write_xml(fileName.toStdString(), p, std::locale(), settings);
}

void ta::common::CreateDefaultSettings()
{
  QSettings sett(QSettings::IniFormat, QSettings::UserScope, qApp->organizationName(), qApp->applicationName());

  sett.beginGroup("toolbars");

  sett.beginGroup("File Tool Bar");
  sett.setValue("name", "File Tool Bar");
  sett.beginWriteArray("Actions");
  sett.setArrayIndex(0);
  sett.setValue("action", "File.New Workspace");
  sett.setArrayIndex(1);
  sett.setValue("action", "File.Open Workspace");
  sett.setArrayIndex(2);
  sett.setValue("action", "File.Save Workspace");
  sett.setArrayIndex(3);
  sett.endArray();
  sett.endGroup();

  sett.beginGroup("View Tool Bar");
  sett.setValue("name", "View Tool Bar");
  sett.beginWriteArray("Actions");
  sett.setArrayIndex(0);
  sett.setValue("action", "View.Layer Explorer");
  sett.setArrayIndex(1);
  sett.setValue("action", "View.Map Display");
  sett.setArrayIndex(2);
  sett.setValue("action", "View.Data Table");
  sett.setArrayIndex(3);
  sett.setValue("action", "View.Style Explorer");
  sett.setArrayIndex(4);
  sett.setValue("action", "Show.Connect Layers");
  sett.endArray();
  sett.endGroup();

  sett.beginGroup("Map Tool Bar");
  sett.setValue("name", "Map Tool Bar");
  sett.beginWriteArray("Actions");
  sett.setArrayIndex(0);
  sett.setValue("action", "Map.Draw");
  sett.setArrayIndex(1);
  sett.setValue("action", "Map.Previous Extent");
  sett.setArrayIndex(2);
  sett.setValue("action", "Map.Next Extent");
  sett.setArrayIndex(3);
  sett.setValue("action", "Map.Zoom Extent");
  sett.setArrayIndex(4);
  sett.setValue("action", "");
  sett.setArrayIndex(5);
  sett.setValue("action", "Map.Zoom In");
  sett.setArrayIndex(6);
  sett.setValue("action", "Map.Zoom Out");
  sett.setArrayIndex(7);
  sett.setValue("action", "Map.Pan");
  sett.setArrayIndex(8);
  sett.setValue("action", "");
  sett.setArrayIndex(9);
  sett.setValue("action", "Map.Info");
  sett.setArrayIndex(10);
  sett.setValue("action", "Map.Selection");
  sett.setArrayIndex(11);
  sett.setValue("action", "Map.Remove Selection");
  sett.endArray();
  sett.endGroup();

  //sett.beginGroup("TerraAmazon Tool Bar");
  //sett.setValue("name", "TerraAmazon Tool Bar");
  //sett.beginWriteArray("Actions");
  //sett.setArrayIndex(0);
  //sett.setValue("action", "Process.Server Control");
  //sett.setArrayIndex(1);
  //sett.setValue("action", "Process.Project Activation");
  //sett.setArrayIndex(2);
  //sett.setValue("action", "Process.Process Control");
  //sett.setArrayIndex(3);
  //sett.setValue("action", "Project.Catalog Manager");
  //sett.setArrayIndex(4);
  //sett.setValue("action", "Project.Create Project Folder");
  //sett.endArray();
  //sett.endGroup();

  sett.endGroup();

  sett.beginGroup("workspaces");

  sett.setValue("author_name", "");
  sett.setValue("recents_history_size", "8");

  sett.endGroup();
}

std::map<std::string, std::string> ta::common::makePGISConnInfo(const std::string& user,
  const std::string& pass,
  const std::string& host,
  const std::string& port,
  const std::string& dsName,
  const std::string& timeout,
  const std::string& maxPoolSize,
  const std::string& minPoolSize,
  const std::string& hideRaster,
  const std::string& hideSpatialData)
{
  std::map<std::string, std::string> connInfo;

  if (!user.empty())
    connInfo["PG_USER"] = user;

  if (!pass.empty())
    connInfo["PG_PASSWORD"] = pass;

  if (!host.empty())
    connInfo["PG_HOST"] = host;
  else
    connInfo["PG_HOST"] = "localhost";

  if (!port.empty())
    connInfo["PG_PORT"] = port;
  else
    connInfo["PG_PORT"] = "5432";

  if (!dsName.empty())
    connInfo["PG_DB_NAME"] = dsName;
  else
    connInfo["PG_DB_NAME"] = "template1";

  if (!timeout.empty())
    connInfo["PG_CONNECT_TIMEOUT"] = timeout;
  else
    connInfo["PG_CONNECT_TIMEOUT"] = "5";

  if (!maxPoolSize.empty())
    connInfo["PG_MAX_POOL_SIZE"] = maxPoolSize;
  else
    connInfo["PG_MAX_POOL_SIZE"] = "4";

  if (!minPoolSize.empty())
    connInfo["PG_MIN_POOL_SIZE"] = minPoolSize;
  else
    connInfo["PG_MIN_POOL_SIZE"] = "1";

  if (!hideRaster.empty())
    connInfo["PG_HIDE_RASTER_TABLES"] = hideRaster;
  else
    connInfo["PG_HIDE_RASTER_TABLES"] = "FALSE";

  if (!hideSpatialData.empty())
    connInfo["PG_HIDE_SPATIAL_METADATA_TABLES"] = hideSpatialData;
  else
    connInfo["PG_HIDE_SPATIAL_METADATA_TABLES"] = "FALSE";

  return connInfo;
}

std::string ta::common::Write(const te::se::Style* style)
{
  QString strPath = QStandardPaths::writableLocation(QStandardPaths::DataLocation);

  if (!boost::filesystem::exists(strPath.toStdString()))
    boost::filesystem::create_directories(strPath.toStdString());

  strPath.append(QString::fromStdString("/style.xml"));

  std::string path = strPath.toStdString();

  boost::filesystem::wpath file = boost::filesystem::absolute(path);
  std::string xml;
  {
    std::unique_ptr<te::xml::AbstractWriter> writer(te::xml::AbstractWriterFactory::make());

    writer->setURI(file.string());
    writer->writeStartDocument("UTF-8", "no");

    writer->writeStartElement("StyledLayerDescriptor");

    writer->writeAttribute("xmlns", "http://www.opengis.net/sld");
    writer->writeAttribute("xmlns:ogc", "http://www.opengis.net/ogc");
    writer->writeAttribute("xmlns:se", "http://www.opengis.net/se");
    writer->writeAttribute("xmlns:xlink", "http://www.w3.org/1999/xlink");
    writer->writeAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
    writer->writeAttribute("xsi:schemaLocation", "http://www.opengis.net/sld StyledLayerDescriptor.xsd");

    writer->writeAttribute("version", "1.0");

    writer->writeStartElement("NamedLayer");
    writer->writeStartElement("UserStyle");

    te::se::serialize::Style::getInstance().write(style, *writer.get());

    writer->writeEndElement("UserStyle");
    writer->writeEndElement("NamedLayer");

    writer->writeEndElement("StyledLayerDescriptor");
    writer->writeToFile();

    std::ifstream t(file.string());
    std::stringstream buffer;
    buffer << t.rdbuf();

    xml = buffer.str();
  }

  boost::filesystem::remove(file);
  return xml;
}

std::unique_ptr<te::se::Style> ta::common::Read(std::string xml)
{
  QString strPath = QStandardPaths::writableLocation(QStandardPaths::DataLocation);

  if (!boost::filesystem::exists(strPath.toStdString()))
    boost::filesystem::create_directories(strPath.toStdString());

  strPath.append(QString::fromStdString("/style.xml"));

  std::string filePath = strPath.toStdString();

  std::unique_ptr<te::se::Style> style;
  {
    std::ofstream ot(filePath);
    ot << xml;
    ot.close();
  }

  boost::filesystem::wpath file = boost::filesystem::absolute(filePath);
  {
    std::auto_ptr<te::xml::Reader> reader(te::xml::ReaderFactory::make());
    reader->setValidationScheme(false);

    reader->read(file.string());
    reader->next();

    if ((reader->getNodeType() == te::xml::START_ELEMENT) &&
      (reader->getElementLocalName() == "StyledLayerDescriptor"))
      reader->next();

    if ((reader->getNodeType() == te::xml::START_ELEMENT) &&
      (reader->getElementLocalName() == "NamedLayer"))
      reader->next();

    if ((reader->getNodeType() == te::xml::START_ELEMENT) &&
      (reader->getElementLocalName() == "UserStyle"))
      reader->next();

    if ((reader->getNodeType() == te::xml::START_ELEMENT) &&
      (reader->getElementLocalName() == "FeatureTypeStyle"))
    {
      if (reader->getNodeType() == te::xml::START_ELEMENT)
        style.reset(te::se::serialize::Style::getInstance().read(*reader.get()));
    }
  }

  boost::filesystem::remove(file);
  return std::move(style);
}

std::string Trim(std::string str)
{
  str.erase(str.begin(), find_if(str.begin(), str.end(), [](char& ch)->bool { return !isspace(ch); }));
  str.erase(find_if(str.rbegin(), str.rend(), [](char& ch)->bool { return !isspace(ch); }).base(), str.end());
  return str;
}

const unsigned int SHA256::sha256_k[64] = //UL = uint32
{ 0x428a2f98, 0x71374491, 0xb5c0fbcf, 0xe9b5dba5,
0x3956c25b, 0x59f111f1, 0x923f82a4, 0xab1c5ed5,
0xd807aa98, 0x12835b01, 0x243185be, 0x550c7dc3,
0x72be5d74, 0x80deb1fe, 0x9bdc06a7, 0xc19bf174,
0xe49b69c1, 0xefbe4786, 0x0fc19dc6, 0x240ca1cc,
0x2de92c6f, 0x4a7484aa, 0x5cb0a9dc, 0x76f988da,
0x983e5152, 0xa831c66d, 0xb00327c8, 0xbf597fc7,
0xc6e00bf3, 0xd5a79147, 0x06ca6351, 0x14292967,
0x27b70a85, 0x2e1b2138, 0x4d2c6dfc, 0x53380d13,
0x650a7354, 0x766a0abb, 0x81c2c92e, 0x92722c85,
0xa2bfe8a1, 0xa81a664b, 0xc24b8b70, 0xc76c51a3,
0xd192e819, 0xd6990624, 0xf40e3585, 0x106aa070,
0x19a4c116, 0x1e376c08, 0x2748774c, 0x34b0bcb5,
0x391c0cb3, 0x4ed8aa4a, 0x5b9cca4f, 0x682e6ff3,
0x748f82ee, 0x78a5636f, 0x84c87814, 0x8cc70208,
0x90befffa, 0xa4506ceb, 0xbef9a3f7, 0xc67178f2 };

void SHA256::transform(const unsigned char *message, unsigned int block_nb)
{
  uint32 w[64];
  uint32 wv[8];
  uint32 t1, t2;
  const unsigned char *sub_block;
  int i;
  int j;
  for (i = 0; i < (int)block_nb; i++) {
    sub_block = message + (i << 6);
    for (j = 0; j < 16; j++) {
      SHA2_PACK32(&sub_block[j << 2], &w[j]);
    }
    for (j = 16; j < 64; j++) {
      w[j] = SHA256_F4(w[j - 2]) + w[j - 7] + SHA256_F3(w[j - 15]) + w[j - 16];
    }
    for (j = 0; j < 8; j++) {
      wv[j] = m_h[j];
    }
    for (j = 0; j < 64; j++) {
      t1 = wv[7] + SHA256_F2(wv[4]) + SHA2_CH(wv[4], wv[5], wv[6])
        + sha256_k[j] + w[j];
      t2 = SHA256_F1(wv[0]) + SHA2_MAJ(wv[0], wv[1], wv[2]);
      wv[7] = wv[6];
      wv[6] = wv[5];
      wv[5] = wv[4];
      wv[4] = wv[3] + t1;
      wv[3] = wv[2];
      wv[2] = wv[1];
      wv[1] = wv[0];
      wv[0] = t1 + t2;
    }
    for (j = 0; j < 8; j++) {
      m_h[j] += wv[j];
    }
  }
}

void SHA256::init()
{
  m_h[0] = 0x6a09e667;
  m_h[1] = 0xbb67ae85;
  m_h[2] = 0x3c6ef372;
  m_h[3] = 0xa54ff53a;
  m_h[4] = 0x510e527f;
  m_h[5] = 0x9b05688c;
  m_h[6] = 0x1f83d9ab;
  m_h[7] = 0x5be0cd19;
  m_len = 0;
  m_tot_len = 0;
}

void SHA256::update(const unsigned char *message, unsigned int len)
{
  unsigned int block_nb;
  unsigned int new_len, rem_len, tmp_len;
  const unsigned char *shifted_message;
  tmp_len = SHA224_256_BLOCK_SIZE - m_len;
  rem_len = len < tmp_len ? len : tmp_len;
  memcpy(&m_block[m_len], message, rem_len);
  if (m_len + len < SHA224_256_BLOCK_SIZE) {
    m_len += len;
    return;
  }
  new_len = len - rem_len;
  block_nb = new_len / SHA224_256_BLOCK_SIZE;
  shifted_message = message + rem_len;
  transform(m_block, 1);
  transform(shifted_message, block_nb);
  rem_len = new_len % SHA224_256_BLOCK_SIZE;
  memcpy(m_block, &shifted_message[block_nb << 6], rem_len);
  m_len = rem_len;
  m_tot_len += (block_nb + 1) << 6;
}

void SHA256::final(unsigned char *digest)
{
  unsigned int block_nb;
  unsigned int pm_len;
  unsigned int len_b;
  int i;
  block_nb = (1 + ((SHA224_256_BLOCK_SIZE - 9)
    < (m_len % SHA224_256_BLOCK_SIZE)));
  len_b = (m_tot_len + m_len) << 3;
  pm_len = block_nb << 6;
  memset(m_block + m_len, 0, pm_len - m_len);
  m_block[m_len] = 0x80;
  SHA2_UNPACK32(len_b, m_block + pm_len - 4);
  transform(m_block, block_nb);
  for (i = 0; i < 8; i++) {
    SHA2_UNPACK32(m_h[i], &digest[i << 2]);
  }
}

std::string sha256(std::string input)
{
  unsigned char digest[SHA256::DIGEST_SIZE];
  memset(digest, 0, SHA256::DIGEST_SIZE);

  SHA256 ctx = SHA256();
  ctx.init();
  ctx.update((unsigned char*)input.c_str(), input.length());
  ctx.final(digest);

  char buf[2 * SHA256::DIGEST_SIZE + 1];
  buf[2 * SHA256::DIGEST_SIZE] = 0;
  for (int i = 0; i < SHA256::DIGEST_SIZE; i++)
    sprintf(buf + i * 2, "%02x", digest[i]);
  return std::string(buf);
}

int ta::common::LoadProcess(const int& userId, te::da::DataSourceInfoPtr server)
{
  te::da::DataSourcePtr m_server = te::da::DataSourceManager::getInstance().get(server->getId(), server->getType(), server->getConnInfo());

  if (!m_server.get())
    return -1;

  if (m_server->isOpened() == false)
    m_server->open();

  ta::model::ProcessFacade pf(m_server);
  ta::model::UserFacade uf(m_server);
  ta::model::TaskLogFacade tf(m_server);

  std::unique_ptr<ta::model::User> curUser(dynamic_cast<ta::model::User*>(uf.get(userId)));

  std::vector<ta::model::Process*> vecProcess;

  if (curUser.get() == 0)
    return -1;

  if (!pf.loadProcess(curUser->getProjectId(), curUser->getId(), vecProcess))
    return -1;

  std::unique_ptr<ta::model::TaskLog> tasklog(dynamic_cast<ta::model::TaskLog*>(tf.get(vecProcess[0]->getTaskLogId())));

  if (tasklog->getStatus() == "STARTED")
    return vecProcess[0]->getId();
  else
    return -1;
}

std::string ta::common::FindInTerraAmazonPath(const std::string& path)
{
  //check install dir
  boost::filesystem::path tl_path = boost::filesystem::current_path();

  tl_path /= "..";

  boost::filesystem::path eval_path = tl_path / path;

  if (boost::filesystem::exists(eval_path))
    return eval_path.string();

  //check develop dir
  tl_path = TAMZ_SOURCE_PATH;

  eval_path = tl_path / path;

  if (boost::filesystem::exists(eval_path))
    return eval_path.string();

  //check terralib dirs
  return te::common::FindInTerraLibPath(path);
}

bool ta::common::getActiveProject(const int userId, int& projectId, te::da::DataSourceInfoPtr server)
{
  projectId = -1;

  te::da::DataSourcePtr m_server = te::da::DataSourceManager::getInstance().get(server->getId(), server->getType(), server->getConnInfo());

  if (!m_server.get())
    return false;

  m_server->open();

  ta::model::ProjectFacade pf(m_server);
  ta::model::UserFacade uf(m_server);

  std::unique_ptr<ta::model::User> curUser(dynamic_cast<ta::model::User*>(uf.get(userId)));

  std::unique_ptr<ta::model::Project> curProj(dynamic_cast<ta::model::Project*>(pf.get(curUser->getProjectId())));

  projectId = curProj->getId();

  return true;
}

std::vector<std::vector<std::string>> ta::common::loadHintConfig(const std::string& parentName, const std::vector<std::string>& keys, std::string& error)
{
  std::vector<std::vector<std::string>> result;

  std::string locale = QLocale::system().name().toLower().toStdString();
  // Hint's customized version.
  std::string jsonFile = ta::common::FindInTerraAmazonPath("share/json/hint_config_" + locale + ".json");
  if (jsonFile.empty())
  {
    // Hint's default version in English.
    jsonFile = ta::common::FindInTerraAmazonPath("share/json/hint_config.json");
  }

  try
  {
    boost::property_tree::ptree pt;
    boost::property_tree::json_parser::read_json(jsonFile, pt);
    BOOST_FOREACH(boost::property_tree::ptree::value_type &v, pt.get_child(parentName))
    {
      std::vector<std::string> vec;
      for (std::vector<int>::size_type t = 0; t != keys.size(); t++)
      {
        std::string value = v.second.get<std::string>(keys.at(t));
        vec.push_back(value);
      }
      result.push_back(vec);
    }
  }
  catch (boost::property_tree::json_parser::json_parser_error &je)
  {
    std::string filename = je.filename().c_str();
    std::string errmsg = "Error parsing: " + filename + ": " + je.message().c_str();
    error = errmsg;
  }
  catch (std::exception const& e)
  {
    error = e.what();
  }

  return result;
}
