/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

// TerraAmazon Includes
#include "../CalculateLayerArea.h"
#include "../Utils.h"
#include "CalculateLayerAreaDialog.h"
#include "ui_CalculateLayerAreaDialogForm.h"

// TerraLib Includes
#include <terralib/common/UnitsOfMeasureManager.h>
#include <terralib/qt/widgets/srs/SRSManagerDialog.h>
#include <terralib/srs/Converter.h>
#include <terralib/srs/SpatialReferenceSystemManager.h>

// Qt
#include <QIcon>
#include <QMessageBox>

Q_DECLARE_METATYPE(te::common::UnitOfMeasurePtr);

ta::common::qt::CalculateLayerAreaDialog::CalculateLayerAreaDialog(QWidget* parent)
  : QDialog(parent),
  m_ui(new Ui::CalculateLayerAreaDialogForm),
  m_layerSRID(0),
  m_outputSRID(0)
{
  // add controls
  m_ui->setupUi(this);

  std::string icon = ta::common::FindInTerraAmazonPath("share/icons/svg/calculateLayerArea-hint.svg");
  m_ui->m_imgLabel->setPixmap(QIcon(icon.c_str()).pixmap(112, 48));

  m_ui->m_srsToolButton->setIcon(QIcon::fromTheme("srs"));
  
  connect(m_ui->m_unitComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onUnitComboBoxChanged(int)));
  connect(m_ui->m_srsToolButton, SIGNAL(clicked()), this, SLOT(onSrsToolButtonClicked()));
  connect(m_ui->m_calculatePushButton, SIGNAL(clicked()), this, SLOT(onCalculatePushButtonClicked()));
}

ta::common::qt::CalculateLayerAreaDialog::~CalculateLayerAreaDialog()
{

}

void ta::common::qt::CalculateLayerAreaDialog::setLayer(te::map::AbstractLayerPtr layer)
{
  assert(layer.get());

  m_layer = layer;

  m_ui->m_layerLineEdit->setText(m_layer->getTitle().c_str());

  m_layerSRID = m_layer->getSRID();
  m_outputSRID = m_layerSRID;

  initUnitsOfMeasure();

  if (m_layerSRID > 0)
  {
    te::common::UnitOfMeasurePtr unit = te::srs::SpatialReferenceSystemManager::getInstance().getUnit(m_layerSRID);

    m_ui->m_resSRIDLabel->setText(te::srs::SpatialReferenceSystemManager::getInstance().getName(m_layerSRID).c_str());
    m_ui->m_layerSRIDLabel->setText(te::srs::SpatialReferenceSystemManager::getInstance().getName(m_layerSRID).c_str());

    setUnit(unit);
  }
  else
  {
    m_ui->m_resSRIDLabel->setText("No SRS defined");
    m_ui->m_layerSRIDLabel->setText("No SRS defined");
    m_ui->m_unitComboBox->setCurrentIndex(0);
  }
}

void ta::common::qt::CalculateLayerAreaDialog::onSrsToolButtonClicked()
{
  te::qt::widgets::SRSManagerDialog srsDialog(this);
  srsDialog.setWindowTitle(tr("Choose the SRS"));

  if (srsDialog.exec() == QDialog::Rejected)
    return;

  int newSRID = srsDialog.getSelectedSRS().first;

  if (newSRID <= 0)
  {
    m_ui->m_unitComboBox->setCurrentIndex(0);
    m_ui->m_resSRIDLabel->setText("No SRS defined");
  }
  else
  {
    std::string name = te::srs::SpatialReferenceSystemManager::getInstance().getName(newSRID);
    m_ui->m_resSRIDLabel->setText(name.c_str());
  }

  m_outputSRID = newSRID;

  te::common::UnitOfMeasurePtr unit = te::srs::SpatialReferenceSystemManager::getInstance().getUnit(m_outputSRID);

  setUnit(unit);
}

void ta::common::qt::CalculateLayerAreaDialog::onUnitComboBoxChanged(int index)
{
  if (m_outputSRID <= 0 && index != 0)
  {
    QMessageBox::warning(this, tr("Calculate Layer Area"), tr("There is no no spatial reference defined. It is not possible to select a unit."));
    m_ui->m_unitComboBox->setCurrentIndex(0);
    return;
  }

  if (index == 0)
    return;

  te::common::UnitOfMeasurePtr resUnit = te::srs::SpatialReferenceSystemManager::getInstance().getUnit(m_outputSRID);
  te::common::UnitOfMeasurePtr inputUnit = te::srs::SpatialReferenceSystemManager::getInstance().getUnit(m_layerSRID);

  if (!resUnit || !inputUnit)
  {
    m_ui->m_unitComboBox->setCurrentIndex(0);
    return;
  }

  if (resUnit == inputUnit)
    return;

  if ((te::common::UnitsOfMeasureManager::getInstance().areConvertible(inputUnit->getName(), resUnit->getName()) ||  // units are convertible
    (te::srs::SpatialReferenceSystemManager::getInstance().isGeographic(m_layerSRID) &&
    (resUnit->getId() == te::common::UOM_Metre ||
    resUnit->getId() == te::common::UOM_Foot ||
    resUnit->getId() == te::common::UOM_Kilometre)) ||
    (!te::srs::SpatialReferenceSystemManager::getInstance().isGeographic(m_layerSRID) &&
    (resUnit->getId() == te::common::UOM_Radian ||
    resUnit->getId() == te::common::UOM_Degree))))
    return;

  QMessageBox::warning(this, tr("Calculate Layer Area"), tr("Unable to convert between the selected unit and the unit of input layer"));
  m_ui->m_unitComboBox->setItemText(m_ui->m_unitComboBox->currentIndex(), QString(inputUnit->getName().c_str()));
}

void ta::common::qt::CalculateLayerAreaDialog::onCalculatePushButtonClicked()
{
  //get column name
  std::string columnName = "";

  bool createColumnArea = m_ui->m_createColumnAreaCheckBox->isChecked();

  if (createColumnArea && !m_ui->m_columnNameLineEdit->text().isEmpty())
  {
    columnName = m_ui->m_columnNameLineEdit->text().toStdString();
  }

  //get output unit
  QVariant varUnit = m_ui->m_unitComboBox->itemData(m_ui->m_unitComboBox->currentIndex(), Qt::UserRole);
  te::common::UnitOfMeasurePtr unit = varUnit.value<te::common::UnitOfMeasurePtr>();

  //calculate area
  ta::common::CalculateLayerArea cla;

  try
  {
    cla.setInputParameters(m_layer, unit, m_outputSRID, createColumnArea, columnName);

    cla.execute();
  }
  catch (te::common::Exception& e)
  {
    QMessageBox::warning(this, tr("Calculate Layer Area"), e.what());
    return;
  }
  catch (std::exception& e)
  {
    QMessageBox::warning(this, tr("Calculate Layer Area"), e.what());
    return;
  }
  catch (...)
  {
    QMessageBox::warning(this, tr("Calculate Layer Area"), tr("Internal Error"));
    return;
  }
  
  std::size_t numPols = 0;
  double totArea = 0.0;

  cla.getOutputParameters(numPols, totArea);

  m_ui->m_totAreaLineEdit->setText(QString::number(totArea, 'f', 6));
  m_ui->m_nPolsLineEdit->setText(QString::number(numPols));

  QMessageBox::information(this, tr("Calculate Layer Area"), tr("Area calculated."));
}

void ta::common::qt::CalculateLayerAreaDialog::initUnitsOfMeasure()
{
  m_ui->m_unitComboBox->clear();

  m_ui->m_unitComboBox->addItem("UNIT");

  te::common::UnitsOfMeasureManager::const_iterator it = te::common::UnitsOfMeasureManager::getInstance().begin();
  te::common::UnitsOfMeasureManager::const_iterator itend = te::common::UnitsOfMeasureManager::getInstance().end();
  while (it != itend)
  {
    te::common::UnitOfMeasurePtr uptr = it->second;
    if (it->second->getType() == te::common::Area)
      m_ui->m_unitComboBox->addItem(uptr->getName().c_str(), QVariant::fromValue(uptr));
    ++it;
  }

  if (m_outputSRID > 0)
  {
    te::common::UnitOfMeasurePtr unit = te::srs::SpatialReferenceSystemManager::getInstance().getUnit(m_outputSRID);
    if (unit.get())
      setUnit(unit);
    return;
  }

  m_ui->m_unitComboBox->setCurrentIndex(0);
}

void ta::common::qt::CalculateLayerAreaDialog::setUnit(te::common::UnitOfMeasurePtr unit)
{
  if (!unit)
  {
    m_ui->m_unitComboBox->setCurrentIndex(0);
    return;
  }

  for (int i = 0; i < m_ui->m_unitComboBox->count(); ++i)
  {
    std::string name = m_ui->m_unitComboBox->itemText(i).toStdString();

    if (unit->getName() == name)
    {
      m_ui->m_unitComboBox->setCurrentIndex(i);
      break;
    }
  }
}
