/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

#include "../Utils.h"
#include "tools/ConnectLayersBox.h"
#include "ConnectLayersDockWidget.h"
#include "ui_ConnectLayersDockWidgetForm.h"

// TerraLib
#include <terralib/common/STLUtils.h>
#include <terralib/maptools/FolderLayer.h>
#include <terralib/qt/af/ApplicationController.h>
#include <terralib/qt/widgets/canvas/Canvas.h>

Q_DECLARE_METATYPE(te::map::AbstractLayerPtr);

ta::common::qt::ConnectLayersDockWidget::ConnectLayersDockWidget(QWidget* parent)
  : QDockWidget(parent),
  m_ui(new Ui::ConnectLayersDockWidgetForm),
  m_isConnected(false),
  m_flick(false)
{
  // add controls
  m_ui->setupUi(this);

  m_ui->m_connectLayersToolButton->setIcon(QIcon(ta::common::FindInTerraAmazonPath("share/icons/svg/connect-eye.svg").c_str()));
  m_ui->m_flickToolButton->setIcon(QIcon(ta::common::FindInTerraAmazonPath("share/icons/svg/connect-flick.svg").c_str()));
  m_ui->m_boxToolButton->setIcon(QIcon(ta::common::FindInTerraAmazonPath("share/icons/svg/connect-box.svg").c_str()));
  m_ui->m_upToolButton->setIcon(QIcon::fromTheme("go-up"));
  m_ui->m_downToolButton->setIcon(QIcon::fromTheme("go-down"));
  m_ui->m_drawToolButton->setIcon(QIcon::fromTheme("map-draw"));

  m_ui->m_upToolButton->setEnabled(false);
  m_ui->m_downToolButton->setEnabled(false);

  //connects
  connect(m_ui->m_connectLayersToolButton, SIGNAL(clicked()), this, SLOT(onConnectLayersToolButtonClicked()));
  connect(m_ui->m_drawToolButton, SIGNAL(clicked()), this, SLOT(onDrawToolButtonClicked()));
  connect(m_ui->m_upToolButton, SIGNAL(clicked()), this, SLOT(onUpToolButtonClicked()));
  connect(m_ui->m_downToolButton, SIGNAL(clicked()), this, SLOT(onDownToolButtonClicked()));
  connect(m_ui->m_alphaSlider, SIGNAL(valueChanged(int)), this, SLOT(onAlphaSliderChanged(int)));
  connect(m_ui->m_verticalSlider, SIGNAL(valueChanged(int)), this, SLOT(onVSliderChanged(int)));
  connect(m_ui->m_horizontalSlider, SIGNAL(valueChanged(int)), this, SLOT(onHSliderChanged(int)));
  connect(m_ui->m_flickToolButton, SIGNAL(toggled(bool)), this, SLOT(onFlickerTolButtonClicked(bool)));
  connect(m_ui->m_boxToolButton, SIGNAL(toggled(bool)), this, SLOT(onBoxToolButtonClicked(bool)));


  //create tool for connect layers
  QActionGroup* toolsGroup = te::qt::af::AppCtrlSingleton::getInstance().findActionGroup("Map.ToolsGroup");
  assert(toolsGroup);

  QAction* action = new QAction(this);
  action->setIcon(QIcon(ta::common::FindInTerraAmazonPath("share/icons/svg/connect-box.svg").c_str()));
  action->setToolTip(tr("Create box to connect layers."));
  action->setCheckable(true);
  action->setEnabled(true);
  action->setObjectName("ConnectLayersBoxAction");

  m_ui->m_boxToolButton->setDefaultAction(action);

  toolsGroup->addAction(m_ui->m_boxToolButton->defaultAction());
}

ta::common::qt::ConnectLayersDockWidget::~ConnectLayersDockWidget()
{
  clearPixmaps();
}

void ta::common::qt::ConnectLayersDockWidget::setApplicationComponents(te::qt::widgets::LayerItemView* layerExplorer, te::qt::af::MapDisplay* mapDisplay)
{
  m_layerExplorer = layerExplorer;
  m_mapDisplay = mapDisplay;

  connect(m_mapDisplay->getDisplay(), SIGNAL(extentChanged()), this, SLOT(onMapDisplayExtentChanged()));
  connect(m_mapDisplay->getDisplay(), SIGNAL(displayPaintEvent(QPainter*)), this, SLOT(onDisplayPaintEvent(QPainter*)));

  setLayers(m_layerExplorer->getAllLayers());
}

void ta::common::qt::ConnectLayersDockWidget::setLayers(std::list<te::map::AbstractLayerPtr> layers)
{
  m_ui->m_listWidget->clear();

  m_ui->m_upToolButton->setEnabled(false);
  m_ui->m_downToolButton->setEnabled(false);

  std::list<te::map::AbstractLayerPtr>::iterator it = layers.begin();

  while (it != layers.end())
  {
    te::map::AbstractLayerPtr l = *it;

    QListWidgetItem* item = new QListWidgetItem(l->getTitle().c_str(), m_ui->m_listWidget);
    item->setData(Qt::UserRole, QVariant::fromValue(l));
    item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsUserCheckable);
    item->setCheckState(Qt::Unchecked);

    m_ui->m_listWidget->addItem(item);

    ++it;
  }
}

void ta::common::qt::ConnectLayersDockWidget::onMapDisplayExtentChanged()
{
  if (!m_isConnected)
    return;

  onDrawToolButtonClicked();
}

void ta::common::qt::ConnectLayersDockWidget::onDisplayPaintEvent(QPainter* painter)
{
  if (m_layersPixmaps.empty() || !m_isConnected)
    return;

  QImage img;
  
  if (!m_flick)
  {
    img = m_layersPixmaps[m_curLayerPix]->toImage();
  }
  else
  {
    QPixmap pix(m_mapDisplay->getDisplay()->width(), m_mapDisplay->getDisplay()->height());
    pix.fill(Qt::transparent);
    img = pix.toImage();
  }

  checkLayerItem();

  painter->setOpacity(m_ui->m_alphaSlider->value() / 255.);
  painter->drawImage(m_rect.topLeft().x(), m_rect.topLeft().y(), img, m_rect.topLeft().x(), m_rect.topLeft().y(), m_rect.width(), m_rect.height());
}

void ta::common::qt::ConnectLayersDockWidget::onConnectLayersToolButtonClicked()
{
  m_isConnected = m_ui->m_connectLayersToolButton->isChecked();

  clearPixmaps();

  if (m_isConnected)
  {
    setLayers(m_layerExplorer->getAllLayers());
    drawLayers();
  }
  else
  {
    setLayers(std::list<te::map::AbstractLayerPtr>());
  }
}

void ta::common::qt::ConnectLayersDockWidget::onDrawToolButtonClicked()
{
  if (!m_isConnected)
    return;

  clearPixmaps();

  const te::gm::Envelope& env = m_mapDisplay->getDisplay()->getExtent();

  for (int i = 0; i < m_ui->m_listWidget->count(); ++i)
  {
    QListWidgetItem* item = m_ui->m_listWidget->item(i);

    if (item->checkState() == Qt::Checked)
    {
      QVariant varLayer = item->data(Qt::UserRole);
      te::map::AbstractLayerPtr l = varLayer.value<te::map::AbstractLayerPtr>();

      QPixmap* pix = new QPixmap(m_mapDisplay->getDisplay()->width(), m_mapDisplay->getDisplay()->height());

      pix->fill(Qt::transparent);

      te::qt::widgets::Canvas canvas(m_mapDisplay->getDisplay()->width(), m_mapDisplay->getDisplay()->height());
      canvas.setDevice(pix, false);
      canvas.setWindow(env.m_llx, env.m_lly, env.m_urx, env.m_ury);

      bool cancel=false;

      if (l->getType() == "FOLDERLAYER")
      {
        te::map::FolderLayer* folder = dynamic_cast<te::map::FolderLayer*>(l.get());

        std::vector<te::map::AbstractLayer*> layers = folder->getDescendants();

        std::vector<te::map::AbstractLayer*>::reverse_iterator it;
        
        for (it = layers.rbegin(); it != layers.rend(); ++it)
        {
          te::map::AbstractLayer* folderLayer = *it;
          
          folderLayer->draw(&canvas, env, m_mapDisplay->getDisplay()->getSRID(), m_mapDisplay->getDisplay()->getScale(), &cancel);
        }
      }
      else
      {
        l->draw(&canvas, env, m_mapDisplay->getDisplay()->getSRID(), m_mapDisplay->getDisplay()->getScale(), &cancel);
      }

      m_layersPixmaps.push_back(pix);

      m_layersItems.insert(std::map<int, QListWidgetItem*>::value_type(m_layersPixmaps.size() - 1, item));
    }
  }

  m_ui->m_upToolButton->setEnabled(false);
  
  if (m_layersPixmaps.size() > 1)
    m_ui->m_downToolButton->setEnabled(true);
  else
    m_ui->m_downToolButton->setEnabled(false);

  updateSliderBarSize();
}

void ta::common::qt::ConnectLayersDockWidget::onUpToolButtonClicked()
{
  if (m_layersPixmaps.empty() || m_curLayerPix <= 0)
    return;

  --m_curLayerPix;

  if (m_curLayerPix == 0)
    m_ui->m_upToolButton->setEnabled(false);

  if (m_layersPixmaps.size() > 1)
    m_ui->m_downToolButton->setEnabled(true);

  drawLayers();
}

void ta::common::qt::ConnectLayersDockWidget::onDownToolButtonClicked()
{
  if (m_layersPixmaps.empty() || m_curLayerPix >= m_layersPixmaps.size() - 1)
    return;

  ++m_curLayerPix;

  if (m_curLayerPix == m_layersPixmaps.size() - 1)
    m_ui->m_downToolButton->setEnabled(false);

  if (m_layersPixmaps.size() > 1)
    m_ui->m_upToolButton->setEnabled(true);

  drawLayers();
}

void ta::common::qt::ConnectLayersDockWidget::onAlphaSliderChanged(int /*value*/)
{
  drawLayers();
}

void ta::common::qt::ConnectLayersDockWidget::onVSliderChanged(int value)
{
  QPoint bottomRight = m_rect.bottomRight();

  m_rect.setTopLeft(QPoint(0, 0));
  m_rect.setBottomRight(QPoint(bottomRight.x(), value));

  drawLayers();
}

void ta::common::qt::ConnectLayersDockWidget::onHSliderChanged(int value)
{
  QPoint bottomRight = m_rect.bottomRight();

  m_rect.setTopLeft(QPoint(0, 0));
  m_rect.setBottomRight(QPoint(value, bottomRight.y()));

  drawLayers();
}

void ta::common::qt::ConnectLayersDockWidget::onFlickerTolButtonClicked(bool flag)
{
  m_flick = flag;

  m_mapDisplay->getDisplay()->repaint();
}

void ta::common::qt::ConnectLayersDockWidget::onBoxToolButtonClicked(bool flag)
{
  if (!flag)
    return;

  ta::common::qt::ConnectLayersBox* tool = new ta::common::qt::ConnectLayersBox(m_mapDisplay->getDisplay());
  m_mapDisplay->setCurrentTool(tool);

  connect(tool, SIGNAL(connectLayerBoxDefined(QRect)), this, SLOT(onConnectLayerBoxDefined(QRect)));
  connect(tool, SIGNAL(connectLayerBoxUpLayer()), this, SLOT(onConnectLayerBoxUpLayer()));
  connect(tool, SIGNAL(connectLayerBoxDownLayer()), this, SLOT(onConnectLayerBoxDownLayer()));
  connect(tool, SIGNAL(connectLayerBoxFlickLayer()), this, SLOT(onConnectLayerBoxFlickLayer()));
}

void ta::common::qt::ConnectLayersDockWidget::onConnectLayerBoxDefined(QRect rect)
{
  m_rect = rect;

  disconnect(m_ui->m_verticalSlider, SIGNAL(valueChanged(int)), this, SLOT(onVSliderChanged(int)));
  disconnect(m_ui->m_horizontalSlider, SIGNAL(valueChanged(int)), this, SLOT(onHSliderChanged(int)));

  m_ui->m_horizontalSlider->setValue(m_rect.bottomRight().x());
  m_ui->m_verticalSlider->setValue(m_rect.bottomRight().y());

  connect(m_ui->m_verticalSlider, SIGNAL(valueChanged(int)), this, SLOT(onVSliderChanged(int)));
  connect(m_ui->m_horizontalSlider, SIGNAL(valueChanged(int)), this, SLOT(onHSliderChanged(int)));

  drawLayers();
}

void ta::common::qt::ConnectLayersDockWidget::onConnectLayerBoxUpLayer()
{
  onUpToolButtonClicked();
}

void ta::common::qt::ConnectLayersDockWidget::onConnectLayerBoxDownLayer()
{
  onDownToolButtonClicked();
}

void ta::common::qt::ConnectLayersDockWidget::onConnectLayerBoxFlickLayer()
{
  bool flag = m_ui->m_flickToolButton->isChecked();

  m_ui->m_flickToolButton->setChecked(!flag);

  onFlickerTolButtonClicked(!flag);
}

void ta::common::qt::ConnectLayersDockWidget::drawLayers()
{
  m_mapDisplay->getDisplay()->repaint();
}

void ta::common::qt::ConnectLayersDockWidget::updateSliderBarSize()
{
  m_ui->m_horizontalSlider->setMaximum(m_mapDisplay->getDisplay()->getWidth() - 1);
  m_ui->m_horizontalSlider->setValue(m_mapDisplay->getDisplay()->getWidth() - 1);
  m_ui->m_verticalSlider->setMaximum(m_mapDisplay->getDisplay()->getHeight() - 1);
  m_ui->m_verticalSlider->setValue(m_mapDisplay->getDisplay()->getHeight() - 1);

  m_rect.setBottomRight(QPoint(m_mapDisplay->getDisplay()->getWidth() - 1, m_mapDisplay->getDisplay()->getHeight() - 1));

  drawLayers();
}

void ta::common::qt::ConnectLayersDockWidget::clearPixmaps()
{
  te::common::FreeContents(m_layersPixmaps);
  m_layersPixmaps.clear();

  m_layersItems.clear();

  m_curLayerPix = 0;
}

void ta::common::qt::ConnectLayersDockWidget::checkLayerItem()
{
  for (int i = 0; i < m_ui->m_listWidget->count(); ++i)
  {
    QListWidgetItem* item = m_ui->m_listWidget->item(i);

    item->setTextColor(Qt::black);
  }

  QListWidgetItem* item = m_layersItems[(int)m_curLayerPix];

  item->setTextColor(Qt::blue);
}
