/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

// TerraAmazon
#include "CreateCellLayerWidget.h"
#include "ui_CreateCellLayerWidgetForm.h"

#include "../Utils.h"

// TerraLib
#include <terralib/cellspace/CellSpaceOperations.h>
#include <terralib/common/Exception.h>
#include <terralib/common/Translator.h>
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceFactory.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>
#include <terralib/dataaccess/datasource/DataSourceInfoManager.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>
#include <terralib/dataaccess/utils/Utils.h>
#include <terralib/datatype/Date.h>
#include <terralib/datatype/DateTimeInstant.h>
#include <terralib/datatype/TimeDuration.h>
#include <terralib/datatype/TimeInstant.h>
#include <terralib/geometry/GeometryProperty.h>
#include <terralib/qt/widgets/datasource/connector/AbstractDataSourceConnector.h>
#include <terralib/qt/widgets/datasource/core/DataSourceType.h>
#include <terralib/qt/widgets/datasource/core/DataSourceTypeManager.h>
#include <terralib/qt/widgets/srs/SRSManagerDialog.h>
#include <terralib/srs/SpatialReferenceSystemManager.h>

// Qt
#include <QComboBox>
#include <QLineEdit>
#include <QMessageBox>
#include <QSortFilterProxyModel>
#include <QWidget>

// Boost
#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/json_parser.hpp>

// STL
#include <fstream>
#include <memory>

Q_DECLARE_METATYPE(te::da::DataSourceInfoPtr);
Q_DECLARE_METATYPE(te::map::AbstractLayerPtr);
Q_DECLARE_METATYPE(te::gm::Envelope);

ta::common::qt::CreateCellLayerWidget::CreateCellLayerWidget(QWidget* parent, Qt::WindowFlags f)
: QWidget(parent, f),
m_ui(new Ui::CreateCellLayerWidgetForm),
m_countriesSrid(),
m_srid(-1)
{
  // add controls
  m_ui->setupUi(this);

  m_ui->m_dataSourceCreatorToolButton->setIcon(QIcon::fromTheme("datasource-new"));
  m_ui->m_dataSourceSelectorToolButton->setIcon(QIcon::fromTheme("datasource"));
  m_ui->m_projectionPushButton->setIcon(QIcon::fromTheme("srs"));

  m_ui->m_resolutionLineEdit->setValidator(new QDoubleValidator(this));

  m_ui->m_llxLineEdit->setValidator(new QDoubleValidator(this));
  m_ui->m_llyLineEdit->setValidator(new QDoubleValidator(this));
  m_ui->m_urxLineEdit->setValidator(new QDoubleValidator(this));
  m_ui->m_uryLineEdit->setValidator(new QDoubleValidator(this));

  m_ui->m_refLayerBoxCheckBox->setEnabled(false);

  connect(m_ui->m_projectionPushButton, SIGNAL(clicked()), this, SLOT(onProjectionPushButtonClicked()));

  connect(m_ui->m_refLayerBoxCheckBox, SIGNAL(toggled(bool)), this, SLOT(onRefLayerBoxCheckBoxToggled(bool)));
  connect(m_ui->m_countryBoxCheckBox, SIGNAL(toggled(bool)), this, SLOT(onCountryBoxCheckBoxToggled(bool)));

  connect(m_ui->m_refLayerBoxComboBox, SIGNAL(activated(int)), this, SLOT(onRefLayerBoxComboBoxActivated(int)));
  connect(m_ui->m_countryBoxComboBox, SIGNAL(activated(int)), this, SLOT(onCountryBoxComboBoxActivated(int)));

  connect(m_ui->m_dataSourceCreatorToolButton, SIGNAL(clicked(bool)), this, SLOT(onDataSourceCreatorToolButtonClicked(bool)));
  connect(m_ui->m_dataSourceSelectorToolButton, SIGNAL(clicked(bool)), this, SLOT(onDataSourceSelectorToolButtonClicked(bool)));

  connect(m_ui->m_resolutionLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(onResolutionLineEditTextChanged(const QString&)));

  loadCountries();
}

ta::common::qt::CreateCellLayerWidget::~CreateCellLayerWidget()
{

}

void ta::common::qt::CreateCellLayerWidget::onDataSourceCreatorToolButtonClicked(bool)
{
  const te::qt::widgets::DataSourceType* dsType = te::qt::widgets::DataSourceTypeManager::getInstance().get("POSTGIS");

  if (dsType == 0)
    throw te::common::Exception(TE_TR("Unknown data source type!"));

  std::auto_ptr<QWidget> connectorw(dsType->getWidget(te::qt::widgets::DataSourceType::WIDGET_DATASOURCE_CONNECTOR, this));

  if (connectorw.get() == 0)
    throw te::common::Exception(TE_TR("This type of data source hasn't provide a dialog for adding a new data source"));

  te::qt::widgets::AbstractDataSourceConnector* connector = dynamic_cast<te::qt::widgets::AbstractDataSourceConnector*>(connectorw.get());

  if (connector == 0)
    throw te::common::Exception(TE_TR("Wrong type of object for adding a new data source!"));

  std::list<te::da::DataSourceInfoPtr> datasources;

  connector->create(datasources);

  if (!datasources.empty())
  {
    te::da::DataSourceInfoPtr ds = te::da::DataSourceInfoManager::getInstance().get(datasources.begin()->get()->getId());
    m_ui->m_targetDataSourceComboBox->addItem(ds->getTitle().c_str(), QVariant::fromValue(ds));
  }
}

void ta::common::qt::CreateCellLayerWidget::onDataSourceSelectorToolButtonClicked(bool)
{
  const te::qt::widgets::DataSourceType* dsType = te::qt::widgets::DataSourceTypeManager::getInstance().get("POSTGIS");

  if (dsType == 0)
    throw te::common::Exception(TE_TR("Unknown data source type!"));

  std::auto_ptr<QWidget> connectorw(dsType->getWidget(te::qt::widgets::DataSourceType::WIDGET_DATASOURCE_CONNECTOR, this));

  if (connectorw.get() == 0)
    throw te::common::Exception(TE_TR("This type of data source hasn't provide a dialog for adding a new data source"));

  te::qt::widgets::AbstractDataSourceConnector* connector = dynamic_cast<te::qt::widgets::AbstractDataSourceConnector*>(connectorw.get());

  if (connector == 0)
    throw te::common::Exception(TE_TR("Wrong type of object for adding a new data source!"));

  std::list<te::da::DataSourceInfoPtr> datasources;

  connector->connect(datasources);

  if (!datasources.empty())
  {
    te::da::DataSourceInfoPtr ds = te::da::DataSourceInfoManager::getInstance().get(datasources.begin()->get()->getId());
    m_ui->m_targetDataSourceComboBox->addItem(ds->getTitle().c_str(), QVariant::fromValue(ds));
  }
}

void ta::common::qt::CreateCellLayerWidget::onRefLayerBoxComboBoxActivated(int idx)
{
  QVariant v = m_ui->m_refLayerBoxComboBox->itemData(idx, Qt::UserRole);

  te::map::AbstractLayerPtr l = v.value<te::map::AbstractLayerPtr>();

  te::gm::Envelope env = l->getExtent();

  setEnvelope(env);

  setSrid(l->getSRID());
}

void ta::common::qt::CreateCellLayerWidget::onCountryBoxComboBoxActivated(int idx)
{
  QVariant v = m_ui->m_countryBoxComboBox->itemData(idx, Qt::UserRole);

  te::gm::Envelope env = v.value<te::gm::Envelope>();

  setSrid(m_countriesSrid);

  setEnvelope(env);
}

void ta::common::qt::CreateCellLayerWidget::onCountryBoxCheckBoxToggled(bool checked)
{
  m_ui->m_countryBoxComboBox->setEnabled(checked);

  if (checked)
  {
    m_ui->m_refLayerBoxComboBox->setEnabled(false);
    m_ui->m_refLayerBoxCheckBox->setChecked(false);

    onCountryBoxComboBoxActivated(m_ui->m_countryBoxComboBox->currentIndex());
  }
}

void ta::common::qt::CreateCellLayerWidget::onRefLayerBoxCheckBoxToggled(bool checked)
{
  m_ui->m_refLayerBoxComboBox->setEnabled(checked);

  if (checked)
  {
    m_ui->m_countryBoxComboBox->setEnabled(false);
    m_ui->m_countryBoxCheckBox->setChecked(false);

    onRefLayerBoxComboBoxActivated(m_ui->m_refLayerBoxComboBox->currentIndex());
  }
}

void ta::common::qt::CreateCellLayerWidget::onProjectionPushButtonClicked()
{
  te::qt::widgets::SRSManagerDialog srsDialog(this);
  srsDialog.setWindowTitle(tr("Choose the SRS"));

  if (srsDialog.exec() == QDialog::Rejected)
    return;

  m_srid = srsDialog.getSelectedSRS().first;

  std::string name = te::srs::SpatialReferenceSystemManager::getInstance().getName(m_srid);

  m_ui->m_sridLabel->setText(name.c_str());

  setSrid(m_srid);
}

bool ta::common::qt::CreateCellLayerWidget::validateParameters(QString& error)
{
  bool hasError = false;

  QVariant variant = m_ui->m_targetDataSourceComboBox->itemData(m_ui->m_targetDataSourceComboBox->currentIndex(), Qt::UserRole);
  te::da::DataSourceInfoPtr targetDs = variant.value<te::da::DataSourceInfoPtr>();

  error = tr("Some missing informations:\n\n");

  if (!targetDs)
  {
    error += tr(" - Choose a datasource\n");
    return false;
  }

  te::da::DataSourcePtr ds = te::da::DataSourceManager::getInstance().get(targetDs->getId(), targetDs->getType(), targetDs->getConnInfo());

  if (ds)
  {
    std::string cellName = m_ui->m_cellLayerLineEdit->text().toStdString();

    if (ds->dataSetExists(cellName))
    {
      error += tr(" - There is already a layer with that name\n");
      hasError = true;
    }
  }

  if (Trim(m_ui->m_cellLayerLineEdit->text().toStdString()).empty())
  {
    error += tr(" - Fill the name of the new layer\n");
    hasError = true;
  }

  if (m_srid < 0)
  {
    error += tr(" - Fill the projection of the new layer\n");
    hasError = true;
  }

  if (Trim(m_ui->m_resolutionLineEdit->text().toStdString()).empty())
  {
    error += tr(" - Fill the resolution of the new layer\n");
    hasError = true;
  }

  if (m_ui->m_llxLineEdit->text().isEmpty())
  {
    error += tr(" - Fill the box LLX\n");
    hasError = true;
  }

  if (m_ui->m_llyLineEdit->text().isEmpty())
  {
    error += tr(" - Fill the box LLY\n");
    hasError = true;
  }

  if (m_ui->m_urxLineEdit->text().isEmpty())
  {
    error += tr(" - Fill the box URX\n");
    hasError = true;
  }

  if (m_ui->m_uryLineEdit->text().isEmpty())
  {
    error += tr(" - Fill the box URY\n");
    hasError = true;
  }

  if (!targetDs)
  {
    error += tr(" - Choice the data source\n");
    hasError = true;
  }

  if (!hasError)
  {
    error = "";
    return true;
  }

  return false;
}

void ta::common::qt::CreateCellLayerWidget::setDataSources(std::vector<te::da::DataSourceInfoPtr> sources)
{
  for (std::size_t i = 0; i < sources.size(); ++i)
  {
    m_ui->m_targetDataSourceComboBox->addItem(sources[i]->getTitle().c_str(), QVariant::fromValue(sources[i]));
  }
}

te::da::DataSourceInfoPtr ta::common::qt::CreateCellLayerWidget::getTargetDataSource()
{
  QVariant ds = m_ui->m_targetDataSourceComboBox->itemData(m_ui->m_targetDataSourceComboBox->currentIndex(), Qt::UserRole);

  return ds.value<te::da::DataSourceInfoPtr>();
}

void ta::common::qt::CreateCellLayerWidget::setLayers(std::list<te::map::AbstractLayerPtr> layers)
{
  for (std::list<te::map::AbstractLayerPtr>::iterator it = layers.begin(); it != layers.end(); ++it)
  {
    m_ui->m_refLayerBoxComboBox->addItem(it->get()->getTitle().c_str(), QVariant::fromValue(*it));
  }

  if (!layers.empty())
  {
    m_ui->m_refLayerBoxCheckBox->setEnabled(true);
  }
}

void ta::common::qt::CreateCellLayerWidget::loadCountries()
{
  m_ui->m_countryBoxComboBox->clear();

  //get boxes information (read json file)
  std::string jsonFile = ta::common::FindInTerraAmazonPath("share/json/country_boxes.json");

  try
  {
    boost::property_tree::ptree pt;
    boost::property_tree::json_parser::read_json(jsonFile, pt);

    BOOST_FOREACH(boost::property_tree::ptree::value_type &v, pt.get_child("Country Boxes"))
    {
      std::string name = v.second.get<std::string>("name");

      te::gm::Envelope env;
      env.m_llx = v.second.get<double>("box.llx");
      env.m_lly = v.second.get<double>("box.lly");
      env.m_urx = v.second.get<double>("box.urx");
      env.m_ury = v.second.get<double>("box.ury");

      m_ui->m_countryBoxComboBox->addItem(name.c_str(), QVariant::fromValue(env));
    }

    m_countriesSrid = pt.get<int>("SRID");

    std::string name = te::srs::SpatialReferenceSystemManager::getInstance().getName(m_countriesSrid);

    m_ui->m_countrySRIDLabel->setText(name.c_str());
  }
  catch (boost::property_tree::json_parser::json_parser_error &je)
  {
    QString errmsg = tr("Error parsing: ") + je.filename().c_str() + ": " + je.message().c_str();

    QMessageBox::warning(this, tr("Warning"), errmsg);

    return;
  }
  catch (std::exception const& e)
  {
    QString errmsg = e.what();

    QMessageBox::warning(this, tr("Warning"), errmsg);
  }

  
  m_ui->m_countryBoxComboBox->setCurrentIndex(0);

  ////////////////////create json file from country shp
  //boost::property_tree::ptree pt;
  //boost::property_tree::ptree children;

  //for (int i = 0; i < m_ui->m_countryBoxComboBox->count(); ++i)
  //{
  //  std::string name = m_ui->m_countryBoxComboBox->itemText(i).toStdString();

  //  QVariant v = m_ui->m_countryBoxComboBox->itemData(i, Qt::UserRole);

  //  te::gm::Envelope env = v.value<te::gm::Envelope>();

  //  boost::property_tree::ptree child;

  //  child.put("name", name);

  //  boost::property_tree::ptree box;
  //  box.put("llx", env.getLowerLeftX());
  //  box.put("lly", env.getLowerLeftY());
  //  box.put("urx", env.getUpperRightX());
  //  box.put("ury", env.getUpperRightY());
  //  child.push_back(std::make_pair("box", box));

  //  children.push_back(std::make_pair("Country", child));
  //}

  //pt.add_child("Country Boxes", children);
  //pt.put("SRID", m_countriesSrid);

  //boost::property_tree::json_parser::write_json("d:/country_boxes.json", pt);
}

te::gm::Envelope ta::common::qt::CreateCellLayerWidget::getEnvelope()
{
  double llx = boost::lexical_cast<double>(m_ui->m_llxLineEdit->text().toStdString());
  double lly = boost::lexical_cast<double>(m_ui->m_llyLineEdit->text().toStdString());
  double urx = boost::lexical_cast<double>(m_ui->m_urxLineEdit->text().toStdString());
  double ury = boost::lexical_cast<double>(m_ui->m_uryLineEdit->text().toStdString());

  return te::gm::Envelope(llx, lly, urx, ury);
}

void ta::common::qt::CreateCellLayerWidget::setEnvelope(const te::gm::Envelope& env)
{
  m_ui->m_llxLineEdit->setText(QString::number(env.getLowerLeftX()));
  m_ui->m_llyLineEdit->setText(QString::number(env.getLowerLeftY()));
  m_ui->m_urxLineEdit->setText(QString::number(env.getUpperRightX()));
  m_ui->m_uryLineEdit->setText(QString::number(env.getUpperRightY()));
}

double ta::common::qt::CreateCellLayerWidget::getResolution()
{
  return boost::lexical_cast<double>(m_ui->m_resolutionLineEdit->text().toStdString());
}

void ta::common::qt::CreateCellLayerWidget::setResolution(const double& res)
{
  m_ui->m_resolutionLineEdit->setText(QString::number(res));
}

int ta::common::qt::CreateCellLayerWidget::getSrid()
{
  return m_srid;
}

void ta::common::qt::CreateCellLayerWidget::setSrid(int srid)
{
  bool changed = false;

  if (m_srid != srid)
    changed = true;

  m_srid = srid;

  if (m_srid <= 0)
    return;

  std::string name = te::srs::SpatialReferenceSystemManager::getInstance().getName(m_srid);

  m_ui->m_sridLabel->setText(name.c_str());

  te::common::UnitOfMeasurePtr bbUnit = te::srs::SpatialReferenceSystemManager::getInstance().getUnit(m_srid);
  
  m_ui->m_unitOfMeasureLabel->setText(bbUnit->getName().c_str());

  if (changed)
  {
    clearEnvelope();
    m_ui->m_resolutionLineEdit->clear();
  }
}

te::da::DataSourceInfoPtr ta::common::qt::CreateCellLayerWidget::getCurrentDataSourceInfoPtr()
{
  QVariant variant = m_ui->m_targetDataSourceComboBox->itemData(m_ui->m_targetDataSourceComboBox->currentIndex(), Qt::UserRole);
  te::da::DataSourceInfoPtr targetDs = variant.value<te::da::DataSourceInfoPtr>();
  return targetDs;
}

std::string ta::common::qt::CreateCellLayerWidget::getCurrentCellLayerName()
{
  return m_ui->m_cellLayerLineEdit->text().toStdString();
}

double ta::common::qt::CreateCellLayerWidget::getCurrentResolution()
{
  return m_ui->m_resolutionLineEdit->text().toDouble();
}

void ta::common::qt::CreateCellLayerWidget::setActivationToDataSourceSelector(const bool& active)
{
  QComboBox* tdsCombBox = m_ui->m_targetDataSourceComboBox;
  tdsCombBox->setEnabled(active);

  QToolButton* dssButton = m_ui->m_dataSourceSelectorToolButton;
  dssButton->setEnabled(active);

  QToolButton* dscButton = m_ui->m_dataSourceCreatorToolButton;
  dscButton->setEnabled(active);
}

std::string ta::common::qt::CreateCellLayerWidget::getUserName()
{
  te::da::DataSourceInfoPtr dsInfo = getCurrentDataSourceInfoPtr();
  std::string userName = dsInfo->getConnInfo()["PG_USER"];

  if (userName.empty())
    return "";
  else
    return userName;
}

void ta::common::qt::CreateCellLayerWidget::clearEnvelope()
{
  m_ui->m_llxLineEdit->clear();
  m_ui->m_llyLineEdit->clear();
  m_ui->m_urxLineEdit->clear();
  m_ui->m_uryLineEdit->clear();
}

void ta::common::qt::CreateCellLayerWidget::onResolutionLineEditTextChanged(const QString & text)
{
  std::string aux = text.toStdString();

  if (aux.size() == 1 && aux.find('e') != std::string::npos)
    aux.clear();

  aux.erase(std::remove(aux.begin(), aux.end(), ','), aux.end());

  m_ui->m_resolutionLineEdit->setText(aux.c_str());
}