/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of
    this application.
*/

// Terralib
#include <terralib/edit/Feature.h>
#include <terralib/edit/Repository.h>
#include <terralib/edit/RepositoryManager.h>
#include <terralib/edit/qt/Renderer.h>
#include <terralib/edit/qt/core/UndoStackManager.h>
#include <terralib/edit/qt/SnapOptionsDialog.h>
#include <terralib/edit/qt/tools/AggregateAreaTool.h>
#include <terralib/edit/qt/tools/CreateContinuosPolygonTool.h>
#include <terralib/edit/qt/tools/DeleteGeometryTool.h>
#include <terralib/edit/qt/tools/EditInfoTool.h>
#include <terralib/edit/qt/tools/MergeGeometriesTool.h>
#include <terralib/edit/qt/tools/MoveGeometryTool.h>
#include <terralib/edit/qt/tools/SubtractAreaTool.h>
#include <terralib/edit/qt/tools/VertexTool.h>

// TerraAmazon
#include "EditToolBar.h"
#include "../../common/TerraAmazon.h"
#include "../../common/Utils.h"
#include "../../model/Globals.h"
#include "../../model/facade/AoiLayerFacade.h"
#include "../../model/facade/CatalogFacade.h"
#include "../../model/facade/ClassFacade.h"
#include "../../model/facade/ProcessFacade.h"
#include "../../model/facade/ProjectFacade.h"
#include "../../model/facade/RuleSetFacade.h"
#include "../../model/facade/SceneFacade.h"
#include "../../model/facade/TaskFacade.h"
#include "../../model/facade/TaskLogFacade.h"
#include "../../model/facade/UserFacade.h"
#include "../../model/item/AoiLayer.h"
#include "../../model/item/Catalog.h"
#include "../../model/item/Cell.h"
#include "../../model/item/Class.h"
#include "../../model/item/RuleSet.h"
#include "../../model/item/Task.h"
#include "edition/qt/tools/Classify.h"
#include "edition/qt/tools/DeleteTool.h"
#include "edition/qt/tools/Reclassify.h"
#include "edition/qt/tools/SelectAreaTool.h"
#include "edition/qt/tools/SelectCellTool.h"
#include "edition/qt/tools/SelectMultiLayerTool.h"

// Qt
#include <QActionGroup>
#include <QApplication>
#include <QMessageBox>
#include <QSignalMapper>
#include <QUndoCommand>

// Boost
#include <boost/ptr_container/ptr_vector.hpp>

// STL
#include <cassert>
#include <list>
#include <memory>
#include <vector>

ta::edition::qt::EditToolBar::EditToolBar(QObject* parent) :
  QObject(parent),
  m_outputLayers(std::vector<te::map::AbstractLayerPtr>()),
  m_aiGeometry(0),
  m_delegate(0),
  m_enableEdition(false),
  m_appTimer(0),
  m_toolBar(0),
  m_saveAction(0),
  m_vertexToolAction(0),
  m_createPolygonToolAction(0),
  m_moveGeometryToolAction(0),
  m_snapOptionsAction(0),
  m_deleteToolAction(0),
  m_featureAttributesAction(0),
  m_selectCellToolAction(0),
  m_selectCellsInBoxToolAction(0),
  m_unselectAllCellsToolAction(0),
  m_aggregateAreaToolAction(0),
  m_subtractAreaToolAction(0),
  m_mergeGeometriesToolAction(0),
  m_editInfoToolAction(0),
  m_selectAreaToolAction(0),
  m_selectMultiLayerToolAction(0),
  m_clearEditionAction(0),
  m_undoToolAction(0),
  m_redoToolAction(0),
  m_classifyAction(0),
  m_reclassifyAction(0),
  m_clearAllClassificationAction(0),
  m_clearClassClassificationAction(0),
  m_editionActionGroup(0),
  m_currentTool(0)
{
  initialize();
}

ta::edition::qt::EditToolBar::~EditToolBar()
{
  delete m_toolBar;

  te::qt::af::AppCtrlSingleton::getInstance().removeListener(this);
}

QToolBar* ta::edition::qt::EditToolBar::get() const
{
  return m_toolBar;
}

void ta::edition::qt::EditToolBar::initialize()
{
  // Create the main toolbar
  m_toolBar = new QToolBar;

  // Register the application framework listener
  te::qt::af::AppCtrlSingleton::getInstance().addListener(this);

  initializeActions();

  m_saveAction->setEnabled(true);

  for (int i = 0; i < m_tools.size(); ++i)
    m_tools[i]->setEnabled(true);

  m_snapOptionsAction->setEnabled(true);

  enableActions(m_tools, false);

}

void ta::edition::qt::EditToolBar::initializeActions()
{
  // Save
  createAction(m_saveAction, tr("Save edition"), "edit-save.svg", false, false, "save_edition", SLOT(onSaveInDatabase()));
  m_toolBar->addAction(m_saveAction);

  // Cancel all Edition
  createAction(m_clearEditionAction, tr("Cancel all edition [ESC]"), "clearEdition.svg", false, false, "cancel_edition", SLOT(onResetVisualizationToolActivated(bool)));
  m_toolBar->addAction(m_clearEditionAction);

  // Undo/Redo
  QUndoStack* undoStack = te::edit::UndoStackManager::getInstance().getUndoStack();

  if (undoStack)
  {
    m_undoToolAction = undoStack->createUndoAction(this, tr("&Undo"));
    m_undoToolAction->setShortcuts(QKeySequence::Undo);
    m_undoToolAction->setObjectName("redo");
    m_undoToolAction->setIcon(QIcon::fromTheme("edit-undo"));
    m_undoToolAction->setToolTip("Undo Action");

    m_redoToolAction = undoStack->createRedoAction(this, tr("&Redo"));
    m_redoToolAction->setShortcuts(QKeySequence::Redo);
    m_redoToolAction->setObjectName("undo");
    m_redoToolAction->setIcon(QIcon::fromTheme("edit-redo"));
    m_redoToolAction->setToolTip("Redo Action");
  }
  
  // TerraLib
  createAction(m_vertexToolAction, tr("Vertex Tool - Move, add and remove"), "edit-vertexTool.svg", true, false, "vertex_tool", SLOT(onVertexToolActivated(bool)));
  createAction(m_createPolygonToolAction, tr("Create Polygon [A]"), "edit-createPolygon.svg", true, false, "create_polygon", SLOT(onCreatePolygonToolActivated(bool)));
  createAction(m_moveGeometryToolAction, tr("Move Geometry [M]"), "edit-moveGeometry.svg", true, false, "move_geometry", SLOT(onMoveGeometryToolActivated(bool)));
  createAction(m_aggregateAreaToolAction, tr("Aggregate Geometry [G]"), "edit-aggregateGeometry.svg", true, false, "aggregate_geometry", SLOT(onAggregateAreaToolActivated(bool)));
  createAction(m_subtractAreaToolAction, tr("Subtract Geometry"), "edit-subtractGeometry.svg", true, false, "subtract_geometry", SLOT(onSubtractAreaToolActivated(bool)));
  createAction(m_mergeGeometriesToolAction, tr("Merge Geometries [N]"), "edit-mergeGeometries.svg", true, false, "merge_geometries", SLOT(onMergeGeometriesToolActivated(bool)));
  createAction(m_editInfoToolAction, tr("Edit Information"), "edit-Info.svg", true, false, "edit_info", SLOT(onEditInfoToolActivated(bool)));

  // TerraAmazon
  createAction(m_deleteToolAction, tr("Delete [Del]"), "edit-deleteTool.svg", false, false, "delete_geometry", SLOT(onDeleteToolActivated(bool)));
  createAction(m_selectMultiLayerToolAction, tr("Select Multi Output Layer [H]"), "edit-selectMultiLayer.svg", true, false, "select_multi_layer", SLOT(onSelectMultiLayerToolActivated(bool)));
  createAction(m_selectAreaToolAction, tr("Select Edition Layer [S]"), "edit-selectArea.svg", true, false, "select_area_tool", SLOT(onSelectAreaToolActivated(bool)));
  createAction(m_selectCellToolAction, tr("Select Cell [Y]"), "edit-selectCell.svg", true, false, "select_cell", SLOT(onSelectCellToolActivated(bool)));
  createAction(m_selectCellsInBoxToolAction, tr("Select Cells In Box [T]"), "edit-selectCellsInBox.svg", false, false, "select_cellsInBox", SLOT(onSelectCellsInBoxToolActivated(bool)));
  createAction(m_unselectAllCellsToolAction, tr("Unselect All Cells [U]"), "edit-unselectAllCells.svg", false, false, "unselect_all_cells", SLOT(onUnselectAllCellsToolActivated(bool)));

  // Get the action group of map tools.
  m_editionActionGroup = te::qt::af::AppCtrlSingleton::getInstance().findActionGroup("Map.ToolsGroup");

  m_editionActionGroup->addAction(m_selectAreaToolAction);
  m_editionActionGroup->addAction(m_selectCellToolAction);
  m_editionActionGroup->addAction(m_vertexToolAction);
  m_editionActionGroup->addAction(m_createPolygonToolAction);
  m_editionActionGroup->addAction(m_moveGeometryToolAction);
  m_editionActionGroup->addAction(m_deleteToolAction);
  m_editionActionGroup->addAction(m_aggregateAreaToolAction);
  m_editionActionGroup->addAction(m_subtractAreaToolAction);
  m_editionActionGroup->addAction(m_mergeGeometriesToolAction);
  m_editionActionGroup->addAction(m_editInfoToolAction);

  // Grouping...
  m_tools.push_back(m_saveAction);
  m_tools.push_back(m_clearEditionAction);
  m_tools.push_back(m_selectAreaToolAction);
  m_tools.push_back(m_selectCellToolAction);
  m_tools.push_back(m_selectCellsInBoxToolAction);
  m_tools.push_back(m_unselectAllCellsToolAction);
  m_tools.push_back(m_createPolygonToolAction);
  m_tools.push_back(m_moveGeometryToolAction);
  m_tools.push_back(m_vertexToolAction);
  m_tools.push_back(m_deleteToolAction);
  m_tools.push_back(m_aggregateAreaToolAction);
  m_tools.push_back(m_subtractAreaToolAction);
  m_tools.push_back(m_mergeGeometriesToolAction);
  m_tools.push_back(m_editInfoToolAction);

  // Adding tools to toolbar
  for (int i = 0; i < m_tools.size(); ++i)
  {
    m_toolBar->addAction(m_tools[i]);

    if (i == 1)
    { 
      m_toolBar->addSeparator();
      m_toolBar->addAction(m_undoToolAction);
      m_toolBar->addAction(m_redoToolAction);
      m_toolBar->addSeparator();
    }

    if (m_tools[i]->objectName() == "unselect_all_cells")
      m_toolBar->addSeparator();

  }
  
  // Snap
  createAction(m_snapOptionsAction, tr("Snap Options"), "edit-snap.svg", false, false, "snap_option", SLOT(onSnapOptionsActivated()));
  m_toolBar->addSeparator();
  m_toolBar->addAction(m_snapOptionsAction);

  // Classification
  m_toolBar->addSeparator();

  QLabel* classColumnLabel = new QLabel("Class name: ");
  m_toolBar->addWidget(classColumnLabel);

  m_classNameComboBox = new QComboBox;
  m_classNameComboBox->setEnabled(false);

  m_toolBar->addWidget(m_classNameComboBox);
  connect(m_classNameComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onClassColumnComboBoxChanged(int)));

  createAction(m_classifyAction, tr("Classify [C]"), "classify.svg", false, false, "classify", SLOT(onClassifyActivated()));
  //createAction(m_reclassifyAction, tr("Reclassify"), "reclassify.png", false, false, "reclassify", SLOT(onReclassifyActivated()));
  createAction(m_clearAllClassificationAction, tr("Clear Classification [X]"), "clearAll.svg", false, false, "clear_classification", SLOT(onClearAllClassificationActivated()));
  createAction(m_clearClassClassificationAction, tr("Clear Class Classification"), "clearClass.svg", false, false, "clear_class_classification", SLOT(onClearClassClassificationActivated()));

  m_tools.push_back(m_classifyAction);
  //m_tools.push_back(m_reclassifyAction);
  m_tools.push_back(m_clearAllClassificationAction);
  m_tools.push_back(m_clearClassClassificationAction);
  
  m_toolBar->addAction(m_classifyAction);
  //m_toolBar->addAction(m_reclassifyAction);
  m_toolBar->addAction(m_clearAllClassificationAction);
  m_toolBar->addAction(m_clearClassClassificationAction);

  m_toolBar->addSeparator();
  m_tools.push_back(m_selectMultiLayerToolAction);
  m_toolBar->addAction(m_selectMultiLayerToolAction);

  setShortCuts();

}

void ta::edition::qt::EditToolBar::createAction(QAction*& action, const QString& tooltip, const QString& icon, bool checkable, bool enabled, const QString& objName, const char* member)
{
  action = new QAction(this);
  action->setIcon(QIcon(ta::common::FindInTerraAmazonPath("share/icons/svg/" + icon.toStdString()).c_str()));
  action->setToolTip(tooltip);
  action->setCheckable(checkable);
  action->setEnabled(enabled);
  action->setObjectName(objName);
  connect(action, SIGNAL(triggered(bool)), this, member);
}

void ta::edition::qt::EditToolBar::onEditActivated(bool checked)
{
  if (!m_editLayer.get())
    return;

  te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().get(m_editLayer->getDataSourceId());

  if (!dsInfo.get())
    return;

  m_enableEdition = checked;

  enableActions(m_tools, checked);

  enableCurrentTool(checked);

  loadClassName();

  if (m_delegate == 0)
    updateDelegate(1);
  
  m_delegate->addEdited(ta::edition::qt::GetURICustomized(m_editLayer->getTitle(), dsInfo));

  updateTreeView(getLayerExplorer());

  setVisualizationScale(true);

  if (!checked && m_editLayer.get())
  {
    te::qt::af::evt::GetMapDisplay e;
    emit triggered(&e);

    //set checked to false in all tools
    for (int i = 0; i < m_tools.size(); ++i)
      m_tools[i]->setChecked(false);

    //verify polygons in memory
    ta::edition::qt::HasGeometriesInMemory(m_editLayer.get(), e.m_display->getDisplay());

    updateDelegate(2);

    //delete the current tool
    if (m_currentTool != 0)
    {
      if (m_currentTool->isInUse())
      {
        te::qt::af::evt::GetMapDisplay e;
        emit triggered(&e);

        e.m_display->setCurrentTool(0);
      }
      else
        delete m_currentTool;
    }

    if (m_appTimer)
    {
      if (m_process)
      {
        unsigned int time = (unsigned int)m_appTimer->getRealTimerSec();

        ta::model::TaskLogFacade taskLogFacade(m_server);

        if (taskLogFacade.updateLogTimer(m_process->getTaskLogId(), time))
        {
          if (m_appTimer->getStatusSleep())
            m_appTimer->stopSleep();
          if (m_appTimer->getStatusTimer())
            m_appTimer->stopTimer();
        }

      }

      delete m_appTimer;
      m_appTimer = 0;
    }

    te::qt::af::AppCtrlSingleton::getInstance().removeListener(this);
  }
  else
  {
    if (!m_appTimer)
    {
      m_appTimer = new ta::common::AppTimer();

      if (!m_appTimer->getStatusTimer())
        m_appTimer->startTimer(1 * 60000);//1 minuto
    }

    onSelectAreaToolActivated(true);
    m_selectAreaToolAction->setChecked(true);

    te::qt::af::AppCtrlSingleton::getInstance().addListener(this);
  }

}

void ta::edition::qt::EditToolBar::setServer(te::da::DataSourceInfoPtr server)
{
  m_server = te::da::DataSourceManager::getInstance().get(server->getId(), server->getType(), server->getConnInfo());

  if (!m_server.get())
    return;

  if (m_server->isOpened() == false)
    m_server->open();

  m_cellFacade.reset(new ta::model::CellFacade(m_server));

}

void ta::edition::qt::EditToolBar::setProcess(int processId)
{
  if (!m_server.get())
    return;

  if (m_process.get() != 0)
  {
    //TODO:
    //if(m_process->getId() != processId)
    //clearCurrentLogTime();
  }

  delete m_aiGeometry;
  m_aiGeometry = 0;

  ta::model::ProcessFacade pf(m_server);
  m_process.reset(dynamic_cast<ta::model::Process*>(pf.get(processId)));

  if (m_process.get() == 0)
    return;

  // GET SCENE
  ta::model::SceneFacade scenefacade(m_server);
  m_scene.reset(pf.getScene(m_process.get()));

  // GET AI GEOMETRY
  m_aiGeometry = scenefacade.getAiGeometries(m_process->getId()).release();
}

void ta::edition::qt::EditToolBar::setProject(int projectId)
{
  if (!m_server.get())
    return;

  ta::model::ProjectFacade pj(m_server);
  m_project.reset(dynamic_cast<ta::model::Project*>(pj.get(projectId)));

  loadClassName();
}

void ta::edition::qt::EditToolBar::setEditionLayer(te::map::AbstractLayerPtr layer)
{
  if (layer.get() == 0)
    return;

  if (layer->getSRID() == 0)
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("SRID invalid."));
    return;
  }

  if (m_currentTool != 0)
    m_currentTool->setLayer(layer.get());

  m_editLayer = layer;
}

void ta::edition::qt::EditToolBar::setDelegate()
{
  updateDelegate(1);
}

void ta::edition::qt::EditToolBar::onVertexToolActivated(bool)
{
  te::map::AbstractLayerPtr layer = getLayer(m_editLayer->getId());
  if(layer.get() == 0)
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Select a layer first!"));
    m_vertexToolAction->setChecked(false);
    return;
  }

  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  if (e.m_display == 0)
    return;

  setCurrentTool(new te::edit::VertexTool(e.m_display->getDisplay(), layer, 0), e.m_display);
}

void ta::edition::qt::EditToolBar::onCreatePolygonToolActivated(bool)
{
  te::map::AbstractLayerPtr layer = getLayer(m_editLayer->getId());
  if(layer.get() == 0)
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Select a layer first!"));
    m_createPolygonToolAction->setChecked(false);
    return;
  }

  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  if (e.m_display == 0)
    return;

  setCurrentTool(new te::edit::CreateContinuosPolygonTool(e.m_display->getDisplay(), layer, Qt::ArrowCursor, 0), e.m_display);

}

void ta::edition::qt::EditToolBar::onMoveGeometryToolActivated(bool)
{
  te::map::AbstractLayerPtr layer = getLayer(m_editLayer->getId());
  if(layer.get() == 0)
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Select a layer first!"));
    m_moveGeometryToolAction->setChecked(false);
    return;
  }

  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  if (e.m_display == 0)
    return;

  setCurrentTool(new te::edit::MoveGeometryTool(e.m_display->getDisplay(), layer, 0), e.m_display);
}

void ta::edition::qt::EditToolBar::onSnapOptionsActivated()
{
  te::edit::SnapOptionsDialog options(m_toolBar);

  te::qt::af::evt::GetAvailableLayers evt;
  emit triggered(&evt);

  options.setLayers(evt.m_layers);
  options.exec();
}

void ta::edition::qt::EditToolBar::onDeleteToolActivated(bool)
{
  try
  {
    te::map::AbstractLayerPtr layer = getLayer(m_editLayer->getId());
    if(layer.get() == 0)
    {
      QMessageBox::information(m_toolBar, "TerraAmazon", tr("Select a layer first!"));
      m_deleteToolAction->setChecked(false);
      return;
    }
    if (layer->getSelected() == 0)
    {
      QMessageBox::information(m_toolBar, "TerraAmazon", tr("Select a geometry first!"));
      return;
    }
    if (layer->getSelected()->size() == 0)
    {
      QMessageBox::information(m_toolBar, "TerraAmazon", tr("Select at least one geometry!"));
      return;
    }

    te::qt::af::evt::GetMapDisplay e;
    emit triggered(&e);

    if (e.m_display == 0)
      return;

    ta::edition::qt::tools::DeleteTool erase(e.m_display->getDisplay(), layer, 0);

    erase.execute();

    te::qt::af::evt::LayerSelectedObjectsChanged evt(layer);
    emit triggered(&evt);
  }
  catch(te::common::Exception& e)
  {
    QMessageBox::critical(m_toolBar, "TerraAmazon", e.what());
    return;
  }
}

void ta::edition::qt::EditToolBar::onSplitPolygonToolActivated(bool)
{
  QMessageBox::information(m_toolBar, "TerraAmazon", tr("Under Development!"));
  return;
}

void ta::edition::qt::EditToolBar::onClassifyActivated()
{
  classify();
}

void ta::edition::qt::EditToolBar::onReclassifyActivated()
{
  QMessageBox::information(m_toolBar, tr("TerraAmazon"), tr("Under Development!"));

  return;
}

void ta::edition::qt::EditToolBar::onClearAllClassificationActivated()
{
  classify(true);
}

void ta::edition::qt::EditToolBar::onClearClassClassificationActivated()
{
  classify(true, true);
}

void ta::edition::qt::EditToolBar::classify(const bool isClear, const bool isClearClass)
{
  te::map::AbstractLayerPtr draftLayer = getLayer(m_editLayer->getId());
  if (!draftLayer)
  {
    QMessageBox::information(m_toolBar, tr("TerraAmazon"), tr("Select a layer first!"));
    return;
  }

  if (m_className.empty())
  {
    QMessageBox::information(m_toolBar, tr("TerraAmazon"), tr("Were not found classes for this project. Contact the system administrator."));
    return;
  }

  if (!m_project)
  {
    QMessageBox::information(m_toolBar, tr("TerraAmazon"), tr("Project not found!"));
    return;
  }

  if (!m_process)
  {
    QMessageBox::information(m_toolBar, tr("TerraAmazon"), tr("Process not found!"));
    return;
  }

  ta::model::ProjectFacade pf(m_server);
  ta::model::RuleSetFacade rsFacade(m_server);

  std::auto_ptr<ta::model::RuleSet> rs;

  std::vector<int> ruleSets = pf.getRuleSets(m_project->getId());
  if (ruleSets.empty())
  {
    QMessageBox::information(m_toolBar, tr("TerraAmazon"), tr("RuleSets not found!"));
    return;
  }

  std::vector<te::map::AbstractLayerPtr> outputtLayerVec;

  for (std::size_t i = 0; i < ruleSets.size(); ++i)
  {
    rs.reset(dynamic_cast<ta::model::RuleSet*>(rsFacade.get(ruleSets[i])));

    if (!rs.get())
      return;

    int catalogId = rs->getCatalogId();

    ta::model::CatalogFacade catalogFacade(m_server);
    std::auto_ptr<ta::model::Catalog> outputCatalog(dynamic_cast<ta::model::Catalog*>(catalogFacade.get(catalogId)));

    te::da::DataSourcePtr dataSource = catalogFacade.getDataSource(outputCatalog.get());
    if (!dataSource.get())
      return;

    te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().get(dataSource->getId());
    if (!dsInfo.get())
    {
      QMessageBox::information(m_toolBar, tr("TerraAmazon"), tr("Output Layer Info not found!"));
      return;
    }
    outputtLayerVec.push_back(getLayer(outputCatalog->getName(), dsInfo));
  }

  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  if (e.m_display == 0)
    return;

  ta::edition::qt::tools::Classify classify(e.m_display->getDisplay(),
    m_server,
    m_project->getId(),
    m_process->getId(),
    m_className,
    draftLayer,
    outputtLayerVec,
    0);

  if (isClear)
    classify.setIsClean(true);

  if (isClearClass)
    classify.setIsCleanClass(true);

  classify.execute();
}

void ta::edition::qt::EditToolBar::onClassColumnComboBoxChanged(int index)
{
  m_className.clear();
  m_className = m_classNameComboBox->itemText(index).toStdString();
}

void ta::edition::qt::EditToolBar::onSelectCellToolActivated(bool checked)
{
  if (!m_cellFacade->loadLayer())
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Error loading cell layer!\nCheck one!"));
    m_selectCellToolAction->setChecked(false);
    return;
  }
  if (m_project.get() == 0)
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Active project not found."));
    m_selectCellToolAction->setChecked(false);
    return;
  }
  if (m_process.get() == 0)
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Start a process first!"));
    m_selectCellToolAction->setChecked(false);
    return;
  }

  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  te::map::AbstractLayerPtr cellLayer = m_cellFacade->getLayer();

  cellLayer->clearSelected();

  ta::model::Cell cell;
  cell.setProcessId(m_process->getId());

  std::vector<ta::model::Cell*> cellSet;
  if (!m_cellFacade->loadCells(cellSet, cell, true))
    return;

  m_selectAreaToolAction->setChecked(!checked);

  if (checked)
  {
    m_selectMultiLayerToolAction->setChecked(!checked);
    enableEditionTools(true);
  }
  else
    e.m_display->setCurrentTool(0);

  emit enableVisualizationTools(false);

  setCurrentTool(new ta::edition::qt::tools::SelectCellTool(e.m_display->getDisplay(), m_editLayer, cellLayer, cellSet, Qt::ArrowCursor, 0), e.m_display);

  onLayerSelectedCellsChanged(cellSet, SELECTEDCELLSTOOL);
}

void ta::edition::qt::EditToolBar::onSelectCellsInBoxToolActivated(bool)
{
  if (!m_cellFacade->loadLayer())
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Error loading cell layer!\nCheck one!"));
    return;
  }
  if (m_process.get() == 0)
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Start a process first!"));
    return;
  }

  onUnselectAllCellsToolActivated(false);

  ta::model::Cell cell;
  cell.setProcessId(m_process->getId());

  std::vector<ta::model::Cell*> cellSet;
  if (!m_cellFacade->loadCells(cellSet, cell, false))
  {
    QMessageBox::critical(m_toolBar, "TerraAmazon", tr("Error loading Cells!"));
    return;
  }

  if (m_project.get() == 0)
  {
    QMessageBox::critical(m_toolBar, "TerraAmazon", tr("Error querying active project!"));
    return;
  }

  std::size_t maxCells = m_project->getMaxCells();

  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  const te::gm::Envelope& env = e.m_display->getDisplay()->getExtent();
  if (!env.isValid())
    return;

  int srid = e.m_display->getDisplay()->getSRID();

  te::gm::Envelope reprojectedEnvelope(env);

  te::map::AbstractLayerPtr cellLayer = m_cellFacade->getLayer();

  if ((cellLayer->getSRID() != TE_UNKNOWN_SRS) && (srid != TE_UNKNOWN_SRS) &&
     (cellLayer->getSRID() != srid))
  {
    reprojectedEnvelope.transform(srid, cellLayer->getSRID());
  }

  std::vector<ta::model::Cell*> cs;
  if (!m_cellFacade->queryCellInformation(reprojectedEnvelope, cs, cellLayer))
  {
    QMessageBox::critical(m_toolBar, "TerraAmazon", tr("Error Loading Cells"));
    return;
  }

  if (cs.empty())
  {
    QMessageBox::critical(m_toolBar, "TerraAmazon", tr("0 Cells Intercepted the Box"));
    return;
  }

  // GET LAYERAI
  ta::model::CatalogFacade catalogFacade(m_server);
  std::unique_ptr<ta::model::Catalog> aiCatalog(dynamic_cast<ta::model::Catalog*>(catalogFacade.get(m_scene->getAoiLayerId())));
  
  te::da::DataSourcePtr dataSource = catalogFacade.getDataSource(aiCatalog.get());
  if (!dataSource.get())
    return;

  te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().get(dataSource->getId());
  if (!dsInfo.get())
  {
    QMessageBox::information(m_toolBar, tr("TerraAmazon"), tr("AI Layer Info not found!"));
    return;
  }

  te::map::AbstractLayerPtr layerAI = getLayer(aiCatalog->getName(), dsInfo);

  if (layerAI.get() == 0)
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Error loading ai layer!") + "\n" + tr("Call the administrator."));
    return;
  }

  std::vector<ta::model::Cell*> freeCells;
  std::vector<ta::model::Cell*> lockedCells;

  for (std::size_t i = 0; i < cs.size(); i++)
  {
    ta::model::Cell* cell = cs[i];

    if (!m_cellFacade->getCellState(*cell, true))
    {
      QMessageBox::critical(m_toolBar, "TerraAmazon", tr("Error Locking Cell"));
      return;
    }
    if (cell->getProcessId() > 0)
    {
      if (cell->getProcessId() == m_process->getId())
      {
        freeCells.push_back(cell);
      }
      else
      {
        lockedCells.push_back(cell);
      }
    }
    else
    {
      cell->setProcessId(m_process->getId());

      if (!m_cellFacade->isIntersectIA(*cell, cellLayer->getSRID(), m_process.get(), layerAI->getSRID(), m_aiGeometry))
      {
        continue;
      }
      if (!m_cellFacade->setLockState(*cell, true))
      {
        QMessageBox::critical(m_toolBar, tr("Critical"), tr("Error Locking Cell"));
        return;
      }
      freeCells.push_back(cell);
    }
    if (maxCells != 0 && freeCells.size() >= maxCells)
    {
      break;
    }
  }

  if ((m_currentTool) && (m_currentTool == dynamic_cast<ta::edition::qt::tools::SelectCellTool*>(m_currentTool)))
    dynamic_cast<ta::edition::qt::tools::SelectCellTool*>(m_currentTool)->setSelectedCells(freeCells);

  e.m_display->getDisplay()->repaint();
}

void ta::edition::qt::EditToolBar::onUnselectAllCellsToolActivated(bool)
{
  if (!m_cellFacade->loadLayer())
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Error loading cell layer!\nCheck one!"));
    return;
  }
  if (m_process.get() == 0)
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Start a process first!"));
    return;
  }

  ta::model::Cell cell;
  cell.setProcessId(m_process->getId());

  std::vector<ta::model::Cell*> cellSet;
  if (!m_cellFacade->loadCells(cellSet, cell, true))
  {
    m_unselectAllCellsToolAction->setChecked(false);
    return;
  }

  //unlock the cells
  for (unsigned int i = 0; i < cellSet.size(); i++)
  {
    ta::model::Cell cell;
    cell.setOid(cellSet[i]->getOid());

    if (!m_cellFacade->getCellState(cell, true))
    {
      QMessageBox::critical(m_toolBar, "TerraAmazon", tr("Error Locking Cell"));
      return;
    }

    if (cell.getProcessId() > 0)
    {
      if (!m_cellFacade->setLockState(cell, false))
      {
        QMessageBox::critical(m_toolBar, "TerraAmazon", tr("Error UnLocking Cell"));
        return;
      }
    }
  }

  if ((m_currentTool) && (m_currentTool == dynamic_cast<ta::edition::qt::tools::SelectCellTool*>(m_currentTool)))
    dynamic_cast<ta::edition::qt::tools::SelectCellTool*>(m_currentTool)->clearSelectedCells();

  m_cellFacade->getLayer().get()->clearSelected();

  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  e.m_display->getDisplay()->refresh(true);
}

void ta::edition::qt::EditToolBar::onAggregateAreaToolActivated(bool)
{
  te::map::AbstractLayerPtr layer = getLayer(m_editLayer->getId());
  if (layer.get() == 0)
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Select a layer first!"));
    m_aggregateAreaToolAction->setChecked(false);

    return;
  }

  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  if (e.m_display == 0)
    return;

  setCurrentTool(new te::edit::AggregateAreaTool(e.m_display->getDisplay(), layer, 0), e.m_display);
}

void ta::edition::qt::EditToolBar::onSubtractAreaToolActivated(bool)
{
  te::map::AbstractLayerPtr layer = getLayer(m_editLayer->getId());
  if (layer.get() == 0)
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Select a layer first!"));
    m_subtractAreaToolAction->setChecked(false);

    return;
  }

  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  if (e.m_display == 0)
    return;

  setCurrentTool(new te::edit::SubtractAreaTool(e.m_display->getDisplay(), layer, 0), e.m_display);
}

void ta::edition::qt::EditToolBar::onMergeGeometriesToolActivated(bool)
{
  te::map::AbstractLayerPtr layer = getLayer(m_editLayer->getId());
  if (layer.get() == 0)
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Select a layer first!"));
    m_mergeGeometriesToolAction->setChecked(false);

    return;
  }
  if (layer->getSelected() == 0)
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Select a geometry first!"));
    m_mergeGeometriesToolAction->setChecked(false);
    return;
  }
  if (layer->getSelected()->size() < 2)
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("To use this tool, you must select at least two geometry!"));
    m_mergeGeometriesToolAction->setChecked(false);
    return;
  }

  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  if (e.m_display == 0)
    return;

  setCurrentTool(new te::edit::MergeGeometriesTool(e.m_display->getDisplay(), layer, Qt::ArrowCursor, 0), e.m_display);
}

void ta::edition::qt::EditToolBar::onEditInfoToolActivated(bool)
{
  te::map::AbstractLayerPtr layer = getLayer(m_editLayer->getId());
  if (layer.get() == 0)
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Select a layer first!"));
    m_editInfoToolAction->setChecked(false);

    return;
  }

  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  if (e.m_display == 0)
    return;

  setCurrentTool(new te::edit::EditInfoTool(e.m_display->getDisplay(), layer, 0), e.m_display);
}

void ta::edition::qt::EditToolBar::onSelectAreaToolActivated(bool checked)
{
  te::map::AbstractLayerPtr layer = getLayer(m_editLayer->getId());
  if (layer.get() == 0)
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Select a layer first!"));
    m_selectAreaToolAction->setChecked(false);

    return;
  }

  m_selectCellToolAction->setChecked(!checked);
  emit enableVisualizationTools(!checked);

  if (checked)
  {
    m_selectMultiLayerToolAction->setChecked(!checked);
    enableEditionTools(true);
  }

  for (std::size_t i = 0; i < m_outputLayers.size(); i++)
    m_outputLayers[i]->clearSelected();

  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  if (e.m_display == 0)
    return;

  setCurrentTool(new ta::edition::qt::tools::SelectAreaTool(e.m_display->getDisplay(), layer, Qt::ArrowCursor, 0), e.m_display);
}

void ta::edition::qt::EditToolBar::onSelectMultiLayerToolActivated(bool checked)
{
  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  if (e.m_display == 0)
    return;

  emit enableVisualizationTools(!checked);
  enableEditionTools(!checked);

  if (checked)
  {
    ta::model::CatalogFacade catalogFacade(m_server);
    ta::model::ClassFacade classFacade(m_server);

    std::vector<ta::model::Catalog*> vecCatalogs;
    if (!classFacade.getClassificationCatalogs(m_project->getId(), vecCatalogs))
    {
      QMessageBox::information(m_toolBar, "TerraAmazon", tr("OutPut catalogs not found!"));
      m_selectMultiLayerToolAction->setChecked(false);
      return;
    }

    for (std::size_t i = 0; i < vecCatalogs.size(); i++)
    {
      te::da::DataSourcePtr dataSource = catalogFacade.getDataSource(vecCatalogs[i]);
      if (!dataSource.get())
        continue;

      te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().get(dataSource->getId());
      if (dsInfo.get())
      {
        te::map::AbstractLayerPtr layer = getLayer(vecCatalogs[i]->getName(), dsInfo);
        if (layer.get())
        {
          m_outputLayers.push_back(layer);
        }
      }
    }

    if (m_outputLayers.size() == 0)
    {
      QMessageBox::information(m_toolBar, "TerraAmazon", tr("OutPut Layers not found!"));
      m_selectMultiLayerToolAction->setChecked(false);
      return;
    }

    m_selectCellToolAction->setChecked(!checked);
    m_editLayer->clearSelected();

    setCurrentTool(new ta::edition::qt::tools::SelectMultiLayerTool(e.m_display->getDisplay(), m_outputLayers, Qt::ArrowCursor, 0), e.m_display);
  }
  else
  {
    e.m_display->setCurrentTool(0);
  }

}

void ta::edition::qt::EditToolBar::onResetVisualizationToolActivated(bool /*checked*/)
{
  // Clear the repository
  te::edit::Repository* repo = te::edit::RepositoryManager::getInstance().getRepository(m_editLayer->getId());

  if (repo)
    repo->clear();

  // Reset visualization tool
  if (m_currentTool)
    m_currentTool->resetVisualizationTool();

  // Clear the undo stack
  te::edit::UndoStackManager::getInstance().getUndoStack()->clear();

  // Repaint
  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  if (e.m_display == 0)
    return;

  e.m_display->getDisplay()->getDraftPixmap()->fill(Qt::transparent);
  e.m_display->getDisplay()->repaint();

}

void ta::edition::qt::EditToolBar::onToolDeleted()
{
  m_currentTool = 0;
}

void ta::edition::qt::EditToolBar::enableCurrentTool(const bool& enable)
{
  if (m_currentTool == 0)
    return;

  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  if (e.m_display == 0)
    return;

  if (enable)
    e.m_display->setCurrentTool(m_currentTool);
  else
    e.m_display->setCurrentTool(0, false);

  m_currentTool->setInUse(enable);
}

void ta::edition::qt::EditToolBar::setCurrentTool(te::edit::GeometriesUpdateTool* tool, te::qt::af::MapDisplay* display)
{
  display->setCurrentTool(tool);

  m_currentTool = tool;

  connect(m_currentTool, SIGNAL(geometriesEdited()), SLOT(onGeometriesChanged()));
  connect(m_currentTool, SIGNAL(toolDeleted()), SLOT(onToolDeleted()));
  connect(m_currentTool, SIGNAL(endSelectCell(const std::string&, const std::size_t&, std::size_t&)), SLOT(onEndSelectCell(const std::string&, const std::size_t&, std::size_t&)));
  
  if ((m_currentTool) &&
     ((m_currentTool == dynamic_cast<ta::edition::qt::tools::SelectAreaTool*>(m_currentTool)) || 
      (m_currentTool == dynamic_cast<ta::edition::qt::tools::SelectMultiLayerTool*>(m_currentTool))))
  {
    connect(m_currentTool, SIGNAL(triggered(te::qt::af::evt::Event*)), SIGNAL(triggered(te::qt::af::evt::Event*)));
  }

  if ((m_currentTool) &&
      (m_currentTool == dynamic_cast<ta::edition::qt::tools::SelectCellTool*>(m_currentTool)))
  {
    connect(m_currentTool, SIGNAL(layerSelectedCellsChanged(std::vector<ta::model::Cell*>&, CellStyle)), SLOT(onLayerSelectedCellsChanged(std::vector<ta::model::Cell*>&, CellStyle)));
  }

}

void ta::edition::qt::EditToolBar::loadClassName()
{
  m_classNameComboBox->clear();
  
  if (!m_server)
    return;

  if (!m_project)
    return;

  ta::model::ProjectFacade pf(m_server);
  ta::model::RuleSetFacade rsFacade(m_server);

  std::auto_ptr<ta::model::RuleSet> rs;

  std::vector<int> ruleSets = pf.getRuleSets(m_project->getId());

  for (std::size_t rsIndex = 0; rsIndex < ruleSets.size(); ++rsIndex)
  {
    if (!ruleSets.empty())
      rs.reset(dynamic_cast<ta::model::RuleSet*>(rsFacade.get(ruleSets[rsIndex])));

    if (rs.get())
    {
      std::vector<int>ruleSetClasses = rsFacade.getRuleSetClasses(rs->getCatalogId());

      ta::model::ClassFacade classFacade(m_server);

      for (std::size_t classIndex = 0; classIndex < ruleSetClasses.size(); ++classIndex)
      {
        std::auto_ptr<ta::model::Class> classItem(dynamic_cast<ta::model::Class*>(classFacade.get(ruleSetClasses[classIndex])));
        if (classItem.get())
        {
          QString className = QString(classItem->getName().c_str());
          
          // Returns the index of the item containing the given text; otherwise returns -1.
          int found = m_classNameComboBox->findText(className);

          if (found == -1)
            m_classNameComboBox->addItem(className);
        }
      }
    }
  }

  m_classNameComboBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);
  m_classNameComboBox->setEnabled(m_enableEdition);
}

void ta::edition::qt::EditToolBar::enableActions(QList<QAction*> acts, const bool& enable)
{
  for (QList<QAction*>::iterator it = acts.begin(); it != acts.end(); ++it)
    (*it)->setEnabled(enable);
}

void ta::edition::qt::EditToolBar::onApplicationTriggered(te::qt::af::evt::Event* e)
{
  switch (e->m_id)
  {
    case te::qt::af::evt::DRAWING_FINISHED:
    {
       drawStashed();
       drawCells();

       setVisualizationScale(false,true);
    }
      break;
    
    case te::qt::af::evt::LAYER_SELECTED_OBJECTS_CHANGED:
    {
       drawCells();
    }
      break;

    default:
    break;
  }
}

te::qt::widgets::LayerItemView* ta::edition::qt::EditToolBar::getLayerExplorer()
{
  te::qt::af::evt::GetLayerExplorer e;

  emit triggered(&e);

  return e.m_layerExplorer;
}

void ta::edition::qt::EditToolBar::onGeometriesChanged()
{
  if (m_editLayer == 0)
    return;

  if ((m_currentTool) &&
    (m_currentTool == dynamic_cast<te::edit::CreateContinuosPolygonTool*>(m_currentTool)))
  {
    onSaveInDatabase();
  }

  drawStashed();
}

void ta::edition::qt::EditToolBar::drawStashed()
{
  if (m_editLayer == 0)
    return;

  if (m_editLayer->getVisibility() != te::map::VISIBLE) // checked in the tree
    return;

  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  // Initialize the renderer
  te::qt::widgets::MapDisplay* display = e.m_display->getDisplay();

  if (display == 0)
    return;

  QPixmap* draft = display->getDraftPixmap();
  draft->fill(Qt::transparent);

  te::edit::Renderer& renderer = te::edit::Renderer::getInstance();
  renderer.begin(draft, display->getExtent(), display->getSRID());

  te::map::AbstractLayerPtr layer = ta::edition::qt::GetSelectedLayerTreeView(getLayerExplorer()); // selected in the tree

  if (m_editLayer->getSelected())
  {
    // Draw the edition layer geometries and activated 'create polygon tool' when the edition layer is not selected(highlight)
    if ((m_currentTool) && 
       ((m_currentTool == dynamic_cast<te::edit::CreateContinuosPolygonTool*>(m_currentTool)) ||
        (m_currentTool == dynamic_cast<ta::edition::qt::tools::SelectAreaTool*>(m_currentTool))) &&
        (layer == 0 || layer->getId() != m_editLayer->getId()))
    {
        if (m_editLayer->getSelected()->size())
        {
          const te::da::ObjectIdSet* oid = m_editLayer->getSelected();

          std::auto_ptr<te::da::DataSet> ds(m_editLayer->getData(oid));
          std::size_t pos = te::da::GetFirstSpatialPropertyPos(ds.get());

          ds->moveBeforeFirst();
          while (ds->moveNext())
          {
            te::gm::Geometry* g = (ds->getGeometry(pos)).release();

            renderer.setPolygonStyle(QColor(0, 255, 0, 128), QColor(0, 0, 0), 2);
            renderer.draw(g);
          }
        }
    }

    //if (m_editLayer->getSelected()->size())
    //{
    //  te::qt::af::evt::LayerSelectedObjectsChanged evt(m_editLayer);
    //  emit triggered(&evt);
    //}
  }

  // Draw the output layer geometries when the layer is not selected(highlight)
  if ((m_currentTool) && (m_outputLayers.size()) && 
      (m_currentTool == dynamic_cast<ta::edition::qt::tools::SelectMultiLayerTool*>(m_currentTool)))
  {
    for (std::size_t i = 0; i < m_outputLayers.size(); i++)
    {
      if (m_outputLayers[i]->getSelected())
      {
        if (m_outputLayers[i]->getSelected()->size())
        {
          const te::da::ObjectIdSet* oid = m_outputLayers[i]->getSelected();

          std::auto_ptr<te::da::DataSet> ds(m_outputLayers[i]->getData(oid));
          std::size_t pos = te::da::GetFirstSpatialPropertyPos(ds.get());

          ds->moveBeforeFirst();
          while (ds->moveNext())
          {
            te::gm::Geometry* g = (ds->getGeometry(pos)).release();

            renderer.setPolygonStyle(QColor(0, 255, 0, 128), QColor(0, 0, 0), 2);
            renderer.draw(g);
          }

          te::qt::af::evt::LayerSelectedObjectsChanged evt(m_outputLayers[i]);
          emit triggered(&evt);
        }
      }
    }
  }

  // Draw the layer edited geometries
  renderer.drawRepository(m_editLayer->getId(), display->getExtent(), display->getSRID());
  
  renderer.end();

  display->repaint();

}

void ta::edition::qt::EditToolBar::drawCells()
{
  if (m_editLayer == 0)
    return;

  if (m_process.get() == 0)
    return;

  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  te::qt::widgets::MapDisplay* display = e.m_display->getDisplay();

  if (!m_cellFacade->loadLayer())
    return;

  std::vector<te::gm::Geometry*> cells;
  if(!m_cellFacade->getSelectedCells(m_process->getId(), cells))
    return;

  // Initialize the renderer
  te::edit::Renderer& renderer = te::edit::Renderer::getInstance();

  CellStyle cellStyle = MYSELECTEDCELLS;

  if ((m_currentTool) && 
     (m_currentTool == dynamic_cast<ta::edition::qt::tools::SelectCellTool*>(m_currentTool)))
     cellStyle = SELECTEDCELLSTOOL;

  te::map::AbstractLayerPtr cellLayer = m_cellFacade->getLayer();
  std::string gproperty = te::da::GetFirstGeomProperty(cellLayer->getSchema().get())->getName();

  std::auto_ptr<te::da::DataSet> ds;

  QPixmap* qpixMap;
  if ((m_currentTool) &&
    (m_currentTool == dynamic_cast<ta::edition::qt::tools::SelectCellTool*>(m_currentTool)))
    qpixMap = display->getDraftPixmap();
  else
    qpixMap = display->getDisplayPixmap();

  //---------------DRAW 1 - OTHER SELECTED CELLS
  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::Cell::sm_processId);
  te::da::PropertyName* geomPropertyName = new te::da::PropertyName(ta::model::Cell::sm_cell);

  te::dt::AbstractData* pId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(m_process->getId()));
  te::da::Literal* literal = new te::da::Literal(pId);
  te::da::LiteralEnvelope* lenv = new te::da::LiteralEnvelope(display->getExtent(), cellLayer->getSRID());

  te::da::NotEqualTo* notEqual = new te::da::NotEqualTo(pName, literal);
  te::da::Expression* op = new te::da::ST_Intersects(geomPropertyName, lenv);

  te::da::And* andClause = new te::da::And(notEqual, op);
  te::da::Where* where = new te::da::Where(andClause);

  ds = cellLayer->getData(where->getExp());

  te::gm::Envelope env = display->getExtent();

  std::size_t gpos = te::da::GetFirstSpatialPropertyPos(ds.get());

  renderer.begin(qpixMap, env, display->getSRID());

  ta::edition::qt::SetupCellStyle(m_cellFillColor, m_cellContourColor, m_cellContourWidth, OTHERSELECTEDCELLS);

  renderer.setPolygonStyle(m_cellFillColor, m_cellContourColor, m_cellContourWidth);

  while (ds->moveNext())
  {
    te::gm::Geometry* otherCells = (ds->getGeometry(gpos)).release();

    renderer.draw(otherCells);
  }

  //---------------DRAW 2 - MY CELL SELECTED
  if (cells.size() != 0)
  {
    te::gm::Envelope finalEnv;

    te::da::In* in = new te::da::In(gproperty);
    for (std::size_t c = 0; c < cells.size(); c++)
    {
      te::gm::Envelope auxEnv(*cells[c]->getMBR());

      if (cellLayer->getSRID() != display->getSRID())
        auxEnv.transform(display->getSRID(), display->getSRID());

      finalEnv.Union(auxEnv);

      in->add(new te::da::LiteralGeom(cells[c]));
    }
    //if case the selected cells is not in env mapdisplay
    if (finalEnv.intersects(display->getExtent()))
    {
      ds = cellLayer->getData(in->clone());

      std::size_t gpos = te::da::GetFirstSpatialPropertyPos(ds.get());

      ta::edition::qt::SetupCellStyle(m_cellFillColor, m_cellContourColor, m_cellContourWidth, cellStyle);

      renderer.setPolygonStyle(m_cellFillColor, m_cellContourColor, m_cellContourWidth);

      while (ds->moveNext())
      {
        te::gm::Geometry* myCells = (ds->getGeometry(gpos)).release();

        renderer.draw(myCells);
      }
    }

  }

  renderer.end();

  display->repaint();

}

void ta::edition::qt::EditToolBar::setVisualizationScale(bool centerInAi, bool doRefresh)
{
  if (m_project.get() && m_editLayer.get() && 
      m_enableEdition)
  {
    bool offScale = false;

    te::qt::af::evt::GetMapDisplay e;
    emit triggered(&e);
    
    int scale = static_cast<int>(e.m_display->getDisplay()->getScale());

    if ((scale < m_project->getMinScale()) && m_project->getMinScale() != 0.0)
    {
      scale = m_project->getMinScale();
      offScale = true;
    }
    else if ((scale > m_project->getMaxScale()) && m_project->getMaxScale() != 0.0)
    {
      scale = m_project->getMaxScale();
      offScale = true;
    }

    if (centerInAi && offScale)
    {
      te::gm::Envelope env = m_editLayer->getExtent();
      int srid = e.m_display->getDisplay()->getSRID();

      if (m_aiGeometry)
        env = *m_aiGeometry->getMBR();

      if (env.isValid())
      {
        te::gm::Envelope reprojectedEnvelope(env);

        if ((m_editLayer->getSRID() != TE_UNKNOWN_SRS) && (srid != TE_UNKNOWN_SRS) && (m_editLayer->getSRID() != srid))
          reprojectedEnvelope.transform(srid, m_editLayer->getSRID());

        e.m_display->getDisplay()->setExtent(reprojectedEnvelope, false);

      }
    }

    if (offScale)
    {
      e.m_display->getDisplay()->setScale(scale);

      if (doRefresh)
        e.m_display->getDisplay()->refresh(true);

      return;
    }
  }

}

void ta::edition::qt::EditToolBar::updateTreeView(QTreeView* view)
{
  if (view->selectionModel()->selectedIndexes().empty())
    return;

  QModelIndex idx = *view->selectionModel()->selectedIndexes().begin();

  if (view != 0 && idx.isValid())
  {
    QVector<int> roles;
    roles << Qt::DecorationRole;

    view->dataChanged(idx, idx, roles);
  }
}

void ta::edition::qt::EditToolBar::onEndSelectCell(const std::string& objectId, const std::size_t& cellCount, std::size_t& operation)
{
  operation = 0;

  if (m_process.get() == 0)
    return;

  // GET LAYERAI
  ta::model::CatalogFacade catalogFacade(m_server);
  std::unique_ptr<ta::model::Catalog> aiCatalog(dynamic_cast<ta::model::Catalog*>(catalogFacade.get(m_scene->getAoiLayerId())));

  te::da::DataSourcePtr dataSource = catalogFacade.getDataSource(aiCatalog.get());
  if (!dataSource.get())
    return;

  te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().get(dataSource->getId());
  if (!dsInfo.get())
  {
    QMessageBox::information(m_toolBar, tr("TerraAmazon"), tr("AI Layer Info not found!"));
    return;
  }

  te::map::AbstractLayerPtr layerAI = getLayer(aiCatalog->getName(), dsInfo);

  if (!m_cellFacade->loadLayer())
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Error loading cell layer!") + "\n" + tr("Call the administrator."));
    return;
  }
  if (layerAI.get() == 0)
  {
    QMessageBox::information(m_toolBar, "TerraAmazon", tr("Error loading ai layer!") + "\n" + tr("Call the administrator."));
    return;
  }

  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  ta::model::Cell* cell(m_cellFacade->get(objectId));

  if (!m_cellFacade->getCellState(*cell, true))
    return;

  if (cell->getProcessId() > 0)//verify if has other process using this cell
  {
    if (cell->getProcessId() == m_process->getId())
    {
      if (!m_cellFacade->setLockState(*cell, false))
        return;

      operation = 2;
    }
    else
    {
      QMessageBox::information(m_toolBar, "TerraAmazon", tr("Impossible to select this cell!") + "\n" + tr("It's being used by another user!"));
      return;
    }
  }
  else
  {
    if (m_project->getMaxCells() == 0 || cellCount < m_project->getMaxCells())
    {
      cell->setProcessId(m_process->getId());

      if (!m_cellFacade->isIntersectIA(*cell, m_cellFacade->getLayer()->getSRID(), m_process.get(), layerAI->getSRID(), m_aiGeometry))
      {
        int answer = QMessageBox::question(m_toolBar, "TerraAmazon", tr("This cell does not intersect with the Interest Area!") + "\n" + tr("Do you want to move to the current IA?"), QMessageBox::Yes, QMessageBox::No);

        if (answer == QMessageBox::Yes)
          ta::edition::qt::MoveToInterestArea(e.m_display->getDisplay(), layerAI, m_server, m_process->getId());

        return;
      }

      if (!m_cellFacade->setLockState(*cell, true))
        return;

       operation = 1;
    }
    else
    {
      std::string message = tr("You already selected ").toStdString() + std::to_string(cellCount).c_str() + tr(" cells!").toStdString() + "\n" + tr("To select this cell, first remove the selection of an already selected cell!").toStdString();
      QMessageBox::information(m_toolBar, "TerraAmazon", tr(message.c_str()));
    }
  }
}

void ta::edition::qt::EditToolBar::onSaveInDatabase()
{
  if (m_editLayer == 0)
    return;

  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e);

  // Initialize the renderer
  te::qt::widgets::MapDisplay* display = e.m_display->getDisplay();

  if (display == 0)
    return;

  // saves the new geometries in the database
  ta::edition::qt::SaveGeometries(m_editLayer, display);

}

void ta::edition::qt::EditToolBar::updateDelegate(const std::size_t& option)
{
  if (!m_server.get())
    return;

  te::qt::widgets::LayerItemView* view = getLayerExplorer();

  if (view == 0)
    return;

  if (option >= 1)
  {
    if (m_delegate)
    {
      view->removeDelegate(m_delegate);
      delete m_delegate;
      m_delegate = 0;

      view->repaint();

      m_delegate = new EditDelegate((QStyledItemDelegate*)view->itemDelegate(), this);
      view->setItemDelegate(m_delegate);
    }
    else
    {
      m_delegate = new EditDelegate((QStyledItemDelegate*)view->itemDelegate(), this);
      view->setItemDelegate(m_delegate);
    }

    if (option != 2)
    {
      if (m_enableEdition)
      {
        te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().get(m_editLayer->getDataSourceId());

        if (dsInfo.get())
          m_delegate->addEdited(ta::edition::qt::GetURICustomized(m_editLayer->getTitle(), dsInfo));
      }
    }

    std::set<std::string> sts = ta::edition::qt::GetLockedLayers(m_server);

    for (std::set<std::string>::iterator it = sts.begin(); it != sts.end(); ++it)
      m_delegate->addStashed(*it);
  }

}

te::map::AbstractLayerPtr ta::edition::qt::EditToolBar::getLayer(const std::string& id)
{
  te::qt::af::evt::GetAvailableLayers evt;
  emit triggered(&evt);

  std::list<te::map::AbstractLayerPtr>::iterator it;
  for (it = evt.m_layers.begin(); it != evt.m_layers.end(); ++it)
  {
    if ((*it)->getId() == id)
      return *it;
  }

  throw te::common::Exception(TE_TR("Could not retrieve the layer."));
}

te::map::AbstractLayerPtr ta::edition::qt::EditToolBar::getLayer(const std::string& title, te::da::DataSourceInfoPtr ds)
{
  te::qt::af::evt::GetAvailableLayers evt;
  emit triggered(&evt);

  std::map<std::string, std::string> mapDs = ds->getConnInfo();

  std::list<te::map::AbstractLayerPtr>::iterator it;
  for (it = evt.m_layers.begin(); it != evt.m_layers.end(); ++it)
  {
    std::map<std::string, std::string> mapLayers = te::da::DataSourceInfoManager::getInstance().get((*it)->getDataSourceId())->getConnInfo();

    if (mapLayers["PG_DB_NAME"] == mapDs["PG_DB_NAME"] &&
       (mapLayers["PG_HOST"] == "" ? "localhost" : mapLayers["PG_HOST"]) == (mapDs["PG_HOST"] == "" ? "localhost" : mapDs["PG_HOST"]) &&
       (mapLayers["PG_PORT"] == "" ? "5432" : mapLayers["PG_PORT"]) == (mapDs["PG_PORT"] == "" ? "5432" : mapDs["PG_PORT"]) &&
        mapLayers["URI"] == mapDs["URI"])
    {
      if ((QString((*it)->getTitle().c_str()).compare(title.c_str(), Qt::CaseInsensitive) == 0) ||
          (QString(QString("public.") + (*it)->getTitle().c_str()).compare(title.c_str(), Qt::CaseInsensitive) == 0)) // temporary
      {
        return *it;
      }
    }
  }

  return 0;
}

void ta::edition::qt::EditToolBar::enableEditionTools(bool enabled)
{
  m_createPolygonToolAction->setEnabled(enabled);
  m_moveGeometryToolAction->setEnabled(enabled);
  m_vertexToolAction->setEnabled(enabled);
  m_deleteToolAction->setEnabled(enabled);
  m_aggregateAreaToolAction->setEnabled(enabled);
  m_subtractAreaToolAction->setEnabled(enabled);
  m_mergeGeometriesToolAction->setEnabled(enabled);
  m_editInfoToolAction->setEnabled(enabled);

  if (!enabled)
  {
    m_selectAreaToolAction->setChecked(enabled);
    m_selectCellToolAction->setChecked(enabled);
    m_createPolygonToolAction->setChecked(enabled);
    m_moveGeometryToolAction->setChecked(enabled);
    m_vertexToolAction->setChecked(enabled);
    m_deleteToolAction->setChecked(enabled);
    m_aggregateAreaToolAction->setChecked(enabled);
    m_subtractAreaToolAction->setChecked(enabled);
    m_mergeGeometriesToolAction->setChecked(enabled);
    m_editInfoToolAction->setChecked(enabled);
  }
}

te::da::DataSourcePtr ta::edition::qt::EditToolBar::getActiveServer()
{
  return m_server;
}

void ta::edition::qt::EditToolBar::setShortCuts()
{
  m_selectAreaToolAction->setShortcut(Qt::Key_S);
  m_selectCellToolAction->setShortcut(Qt::Key_Y);
  m_unselectAllCellsToolAction->setShortcut(Qt::Key_T);
  m_deleteToolAction->setShortcut(Qt::Key_Delete);
  m_moveGeometryToolAction->setShortcut(Qt::Key_M);
  m_createPolygonToolAction->setShortcut(Qt::Key_A);
  m_classifyAction->setShortcut(Qt::Key_C);
  m_selectMultiLayerToolAction->setShortcut(Qt::Key_H);
  m_aggregateAreaToolAction->setShortcut(Qt::Key_G);
  m_mergeGeometriesToolAction->setShortcut(Qt::Key_N);
  m_clearAllClassificationAction->setShortcut(Qt::Key_X);
  m_clearEditionAction->setShortcut(Qt::Key_Escape);
}

void ta::edition::qt::EditToolBar::onLayerSelectedCellsChanged(std::vector<ta::model::Cell*>& lockedCells, CellStyle cellStyle)
{
  te::qt::af::evt::GetMapDisplay e;
  emit triggered(&e); 

  if (e.m_display->getDisplay() == 0)
    return;

  const te::gm::Envelope& env = e.m_display->getDisplay()->getExtent();

  if (!env.isValid())
    return;

  // Clear!
  QPixmap* display = e.m_display->getDisplay()->getDraftPixmap();
  display->fill(Qt::transparent);

  // Initialize the renderer
  te::edit::Renderer& renderer = te::edit::Renderer::getInstance();
  renderer.begin(display, env, e.m_display->getDisplay()->getSRID());

  ta::edition::qt::SetupCellStyle(m_cellFillColor, m_cellContourColor, m_cellContourWidth, cellStyle);

  renderer.setPolygonStyle(m_cellFillColor, m_cellContourColor, m_cellContourWidth);

  for (std::size_t i = 0; i < lockedCells.size(); i++)
  {
    te::gm::Geometry* lockedCell = lockedCells[i]->getCell();

    renderer.draw(lockedCell);
  }

  renderer.end();

  e.m_display->getDisplay()->repaint();
}