/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

#ifndef __TERRAAMAZON_EDITION_QT_EDITTOOLBAR_H
#define __TERRAAMAZON_EDITION_QT_EDITTOOLBAR_H

// TerraLib
#include <terralib/common/Exception.h>
#include <terralib/common/PlatformUtils.h>
#include <terralib/common/Translator.h>
#include <terralib/dataaccess/dataset/DataSet.h>
#include <terralib/dataaccess/dataset/DataSetType.h>
#include <terralib/dataaccess/dataset/ObjectId.h>
#include <terralib/dataaccess/dataset/ObjectIdSet.h>
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>
#include <terralib/dataaccess/datasource/DataSourceInfoManager.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>
#include <terralib/dataaccess/query/And.h>
#include <terralib/dataaccess/query/BinaryFunction.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/In.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/dataaccess/query/LiteralGeom.h>
#include <terralib/dataaccess/query/LiteralString.h>
#include <terralib/dataaccess/query/LiteralEnvelope.h>
#include <terralib/dataaccess/query/NotEqualTo.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/ST_Intersects.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/dataaccess/utils/Utils.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/edit/qt/tools/GeometriesUpdateTool.h>
#include <terralib/geometry/GeometryProperty.h>
#include <terralib/geometry/GeometryCollection.h>
#include <terralib/maptools/DataSetLayer.h>
#include <terralib/maptools/Utils.h>
#include <terralib/memory/DataSet.h>
#include <terralib/memory/DataSetItem.h>
#include <terralib/qt/af/ApplicationController.h>
#include <terralib/qt/af/events/ApplicationEvents.h>
#include <terralib/qt/af/events/LayerEvents.h>
#include <terralib/qt/af/events/MapEvents.h>
#include <terralib/qt/widgets/canvas/MapDisplay.h>
#include <terralib/qt/widgets/layer/explorer/LayerItem.h>
#include <terralib/qt/widgets/layer/explorer/LayerItemView.h>
#include <terralib/qt/widgets/layer/explorer/TreeItem.h>
#include <terralib/qt/widgets/utils/ScopedCursor.h>

// TerraAmazon
#include "Config.h"
#include "Utils.h"
#include "../../common/AppTimer.h"
#include "../../model/facade/CellFacade.h"
#include "../../model/item/Class.h"
#include "../../model/item/Process.h"
#include "../../model/item/Project.h"
#include "../../model/item/Scene.h"
#include "../../model/item/User.h"

// Qt
#include <QAction>
#include <QComboBox>
#include <QIcon>
#include <QList>
#include <QToolBar>

// STL
#include <string>

//Testing
#include "EditDelegate.h"

namespace te
{
  namespace qt
  {
    namespace af
    {
      class MapDisplay;

      namespace evt
      {
        struct Event;
      }
    }
    namespace widgets
    {
      class LayerItemView;
    }
  }
}

namespace ta
{
  namespace edition
  {
    namespace qt
    {
      /*!
        \class EditToolBar

        \brief The main edit toolbar of Terramazon.
      */
      class EditToolBar : public QObject
      {
        Q_OBJECT

        public:

          EditToolBar(QObject* parent = 0);

          ~EditToolBar();

          QToolBar* get() const;

        public slots:

          void onEditActivated(bool checked);

        protected slots:

          void onVertexToolActivated(bool checked);

          void onCreatePolygonToolActivated(bool checked);

          void onMoveGeometryToolActivated(bool checked);

          void onSnapOptionsActivated();

          void onDeleteToolActivated(bool checked);

          void onToolDeleted();

          void onSplitPolygonToolActivated(bool checked);

          void onApplicationTriggered(te::qt::af::evt::Event* e);

          void onClassifyActivated();

          void onReclassifyActivated();

          void onClearAllClassificationActivated();

          void onClearClassClassificationActivated();

          void onClassColumnComboBoxChanged(int index);

          void onSelectCellToolActivated(bool checked);

          void onSelectCellsInBoxToolActivated(bool checked);

          void onUnselectAllCellsToolActivated(bool checked);

          void onAggregateAreaToolActivated(bool checked);

          void onSubtractAreaToolActivated(bool checked);

          void onMergeGeometriesToolActivated(bool checked);

          void onEditInfoToolActivated(bool checked);

          void onSelectAreaToolActivated(bool checked);

          void onSelectMultiLayerToolActivated(bool checked);

          void onResetVisualizationToolActivated(bool checked);

          void onGeometriesChanged();

          void onEndSelectCell(const std::string& objectId, const std::size_t& cellCount, std::size_t& operation);

          void onSaveInDatabase();

          void onLayerSelectedCellsChanged(std::vector<ta::model::Cell*>& lockedCells, CellStyle cellStyle);

      private:

          te::da::DataSourcePtr m_server;

          std::unique_ptr<ta::model::CellFacade> m_cellFacade;

          std::unique_ptr<ta::model::Project> m_project;

          std::unique_ptr<ta::model::Process> m_process;

          std::unique_ptr<ta::model::Scene> m_scene;

          te::map::AbstractLayerPtr m_editLayer;

          std::vector<te::map::AbstractLayerPtr> m_outputLayers;  //!< List of output layers.

          std::string m_className;

          te::gm::Geometry* m_aiGeometry;

          EditDelegate* m_delegate;

          bool m_enableEdition;

          ta::common::AppTimer* m_appTimer;

      Q_SIGNALS:

        void triggered(te::qt::af::evt::Event* e);

        void enableVisualizationTools(bool enable);

      protected:

        QToolBar* m_toolBar;

        QColor m_cellFillColor;
        QColor m_cellContourColor;
        std::size_t m_cellContourWidth;

        QAction* m_saveAction;
        QAction* m_vertexToolAction;
        QAction* m_createPolygonToolAction;
        QAction* m_moveGeometryToolAction;
        QAction* m_snapOptionsAction;
        QAction* m_deleteToolAction;
        QAction* m_featureAttributesAction;
        QAction* m_selectCellToolAction;
        QAction* m_selectCellsInBoxToolAction;
        QAction* m_unselectAllCellsToolAction;
        QAction* m_aggregateAreaToolAction;
        QAction* m_subtractAreaToolAction;
        QAction* m_mergeGeometriesToolAction;
        QAction* m_editInfoToolAction;
        QAction* m_selectAreaToolAction;
        QAction* m_selectMultiLayerToolAction;
        QAction* m_clearEditionAction;
        QAction* m_undoToolAction;
        QAction* m_redoToolAction;

        QAction* m_classifyAction;
        QAction* m_reclassifyAction;
        //QAction* m_classNameAction;
        QAction* m_clearAllClassificationAction;
        QAction* m_clearClassClassificationAction;

        QActionGroup* m_editionActionGroup;

        QList<QAction*> m_tools;

        QComboBox* m_classNameComboBox;

        te::edit::GeometriesUpdateTool* m_currentTool;

        void enableCurrentTool(const bool& enable);

        void setCurrentTool(te::edit::GeometriesUpdateTool* tool, te::qt::af::MapDisplay* display);

        void drawStashed();

        void drawCells();

        void setVisualizationScale(bool centerInAi = false, bool doRefresh = false);

        void enableEditionTools(bool enabled);

        void classify(const bool isClear = false, const bool isClearClass = false);

        void enableActions(QList<QAction*> acts, const bool& enable);

        void updateDelegate(const std::size_t& option);

        void initialize();

        void initializeActions();

        void createAction(QAction*& action, const QString& tooltip, const QString& icon, bool checkable, bool enabled, const QString& objName, const char* member);

        void updateTreeView(QTreeView* view);

        void setShortCuts();

        void loadClassName();

        te::qt::widgets::LayerItemView* getLayerExplorer();

        te::map::AbstractLayerPtr getLayer(const std::string& id);

        te::map::AbstractLayerPtr getLayer(const std::string& title, te::da::DataSourceInfoPtr ds);

      public:

        te::da::DataSourcePtr getActiveServer();

        void setDelegate();

        void setEditionLayer(te::map::AbstractLayerPtr layer);

        void setProcess(int processId);

        void setProject(int projectId);

        void setServer(te::da::DataSourceInfoPtr server);

      };

    }
  }
}

       // end namespace te

#endif //__TERRAAMAZON_EDITION_QT_EDITTOOLBAR_H
