/*  This file is part of the TerraAmazon application.

  See the Copyright file for details about the license of
  this application.
*/

// TerraLib
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/LiteralEnvelope.h>
#include <terralib/dataaccess/query/LiteralGeom.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/In.h>
#include <terralib/dataaccess/query/ST_Equals.h>
#include <terralib/dataaccess/query/ST_Intersects.h>
#include <terralib/dataaccess/query/ST_Within.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/dataaccess/dataset/ObjectId.h>
#include <terralib/dataaccess/dataset/ObjectIdSet.h>
#include <terralib/geometry.h>
#include <terralib/edit/Feature.h>
#include <terralib/edit/qt/core/UndoStackManager.h>
#include <terralib/edit/Repository.h>
#include <terralib/edit/RepositoryManager.h>
#include <terralib/maptools/DataSetLayer.h>
#include <terralib/qt/widgets/canvas/Canvas.h>
#include <terralib/qt/widgets/canvas/MultiThreadMapDisplay.h>
#include <terralib/qt/widgets/Utils.h>
#include <terralib/srs/Config.h>

// TerraAmazon
#include "Utils.h"
#include "../../model/facade/CatalogFacade.h"
#include "../../model/facade/SceneFacade.h"
#include "../../model/facade/TaskFacade.h"
#include "../../model/item/Catalog.h"
#include "../../model/item/Scene.h"
#include "../../model/item/Task.h"

// Qt
#include <QColor>
#include <QMessageBox>
#include <QUndoCommand>

// STL
#include <cassert>

void ta::edition::qt::HasGeometriesInMemory(const te::map::AbstractLayerPtr& layer, te::qt::widgets::MapDisplay* display)
{
  try
  {
    te::edit::Repository* repo = te::edit::RepositoryManager::getInstance().getRepository(layer->getId());

    if (repo == 0)
      return;

    // Get the edited geometries
    const std::vector<te::edit::Feature*>& features = repo->getAllFeatures();

    if (features.size())
    {
      int response = QMessageBox::question(display, "TerraAmazon",
        QObject::tr("You have geometries in memory. Do you want to save them?"),
        QMessageBox::No, QMessageBox::Yes);

      if (response == QMessageBox::No)
      {
        // Clear the repository
        repo->clear();
        // Clear the undo stack
        te::edit::UndoStackManager::getInstance().getUndoStack()->clear();
        
        //update layer
        te::qt::widgets::MultiThreadMapDisplay* mtmp = dynamic_cast<te::qt::widgets::MultiThreadMapDisplay*>(display);
        if (mtmp)
          mtmp->updateLayer(layer);
        else
          display->refresh(true);
      }
      else if (response == QMessageBox::Yes)
      {
        SaveGeometries(layer, display); // TODO: relocate
      }
    }

  }
  catch (te::common::Exception& ex)
  {
    QMessageBox::critical(0, "TerraAmazon", ex.what());
    return;
  }

}

void ta::edition::qt::SaveGeometries(const te::map::AbstractLayerPtr& layer, te::qt::widgets::MapDisplay* display)
{

  std::unique_ptr<te::da::DataSourceTransactor> t;

  try
  {
    te::edit::Repository* repo = te::edit::RepositoryManager::getInstance().getRepository(layer->getId());

    if (repo == 0)
      return;

    // Get the data souce info
    te::da::DataSourceInfoPtr info = te::da::DataSourceInfoManager::getInstance().get(layer.get()->getDataSourceId());

    // For while, use DataSetLayer to get the DataSource
    te::map::DataSetLayer* dslayer = dynamic_cast<te::map::DataSetLayer*>(layer.get());

    te::da::DataSourcePtr dsource = te::da::GetDataSource(dslayer->getDataSourceId(), true);

    // Start the transactor
    t = dsource->getTransactor();

    // Get the layer schema
    std::auto_ptr<te::map::LayerSchema> schema(layer->getSchema());

    // Get the property names that compose the object id
    std::vector<std::string> oidPropertyNames;
    te::da::GetOIDPropertyNames(schema.get(), oidPropertyNames);

    // Get the edited geometries
    const std::vector<te::edit::Feature*>& features = repo->getAllFeatures();

    // Build the DataSet that will be used to update
    std::map<te::edit::OperationType, te::mem::DataSet* > operationds;

    for (std::size_t i = 0; i < te::edit::NumberOfOperationTypes; i++)
      operationds[te::edit::OperationType(i)] = new te::mem::DataSet(schema.get());

    // Get the geometry property position
    std::size_t gpos = te::da::GetFirstSpatialPropertyPos(operationds[te::edit::GEOMETRY_CREATE]);

    // Get the geometry type
    std::auto_ptr<te::da::DataSetType> dt = layer->getSchema();

    // Get the envelope of layer
    te::gm::Envelope env(layer->getExtent());

    std::map<te::edit::OperationType, std::set<int> > propertiesPos;

    t->begin();

    for (std::size_t i = 0; i < features.size(); ++i) // for each edited feature
    {
      //if (!features[i]->getGeometry()->isValid())
      //  continue;

      // Create the new item
      te::mem::DataSetItem* item = new te::mem::DataSetItem(operationds[te::edit::GEOMETRY_CREATE]);

      // Get the object id
      te::da::ObjectId* oid = features[i]->getId();

      const boost::ptr_vector<te::dt::AbstractData>& values = oid->getValue();

      // Fill the new item
      for (std::size_t j = 0; j < values.size(); ++j)
        item->setValue(oidPropertyNames[j], values[j].clone());

      // Get the edited geometry
      te::gm::Geometry* geom = features[i]->getGeometry();

      if (geom->getSRID() != layer->getSRID())
        geom->transform(layer->getSRID());

      // Set the geometry type
      item->setGeometry(gpos, static_cast<te::gm::Geometry*>(geom->clone()));

      switch (features[i]->getOperationType())
      {
        case te::edit::GEOMETRY_CREATE:
        {
          te::mem::DataSet* tempDs = new te::mem::DataSet(schema.get());

          tempDs->add(item);

          // used to not insert the pk
          for (std::size_t j = 0; j < oidPropertyNames.size(); ++j)
          {
            std::size_t pos = te::da::GetPropertyPos(operationds[te::edit::GEOMETRY_CREATE], oidPropertyNames[j]);
            tempDs->drop(pos);
          }

          std::map<std::string, std::string> options;

          t->add(dslayer->getDataSetName(), tempDs, options);

          boost::int64_t id = t->getLastGeneratedId();

          for (std::size_t j = 0; j < values.size(); ++j)
            item->setValue(oidPropertyNames[j], new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)(id)));//TODO: convert INT32 ,64

          operationds[te::edit::GEOMETRY_CREATE]->add(item);

          env.Union(*geom->getMBR());
        }
          break;

        case te::edit::GEOMETRY_UPDATE:

          propertiesPos[te::edit::GEOMETRY_UPDATE].insert((int)gpos);

          operationds[te::edit::GEOMETRY_UPDATE]->add(item);
          break;

        case te::edit::GEOMETRY_UPDATE_ATTRIBUTES:

          if (features[i]->getOperationType() == te::edit::GEOMETRY_UPDATE_ATTRIBUTES)
          {
            if (features[i]->getData().size() > 0)
            {
              for (std::map<std::size_t, te::dt::AbstractData*>::const_iterator it = features[i]->getData().begin(); it != features[i]->getData().end(); ++it)
              {
                item->setValue(it->first, it->second);
                propertiesPos[te::edit::GEOMETRY_UPDATE_ATTRIBUTES].insert(it->first);
              }
            }
          }

          operationds[te::edit::GEOMETRY_UPDATE_ATTRIBUTES]->add(item);
          break;

        default:
          break;
        }
    }
    
    std::map<te::edit::OperationType, te::da::ObjectIdSet* > currentOids;

    if (operationds[te::edit::GEOMETRY_CREATE]->size() > 0)
    {
      currentOids[te::edit::GEOMETRY_CREATE] = te::da::GenerateOIDSet(operationds[te::edit::GEOMETRY_CREATE], schema.get());

      operationds[te::edit::GEOMETRY_CREATE]->moveBeforeFirst();
    }

    if (operationds[te::edit::GEOMETRY_UPDATE]->size() > 0)
    {
      std::vector<std::set<int> > properties;
      for (std::size_t i = 0; i < operationds[te::edit::GEOMETRY_UPDATE]->size(); i++)
        properties.push_back(propertiesPos[te::edit::GEOMETRY_UPDATE]);

      std::vector<std::size_t> oidPropertyPosition;
      for (std::size_t i = 0; i < oidPropertyNames.size(); ++i)
        oidPropertyPosition.push_back(te::da::GetPropertyPos(operationds[te::edit::GEOMETRY_UPDATE], oidPropertyNames[i]));

      currentOids[te::edit::GEOMETRY_UPDATE] = te::da::GenerateOIDSet(operationds[te::edit::GEOMETRY_UPDATE], schema.get());

      operationds[te::edit::GEOMETRY_UPDATE]->moveBeforeFirst();

      t->update(dslayer->getDataSetName(), operationds[te::edit::GEOMETRY_UPDATE], properties, oidPropertyPosition);
    }

    if (operationds[te::edit::GEOMETRY_UPDATE_ATTRIBUTES]->size() > 0)
    {
      std::vector<std::set<int> > properties;
      for (std::size_t i = 0; i < operationds[te::edit::GEOMETRY_UPDATE_ATTRIBUTES]->size(); i++)
        properties.push_back(propertiesPos[te::edit::GEOMETRY_UPDATE_ATTRIBUTES]);

      std::vector<std::size_t> oidPropertyPosition;
      for (std::size_t i = 0; i < oidPropertyNames.size(); ++i)
        oidPropertyPosition.push_back(te::da::GetPropertyPos(operationds[te::edit::GEOMETRY_UPDATE_ATTRIBUTES], oidPropertyNames[i]));

      currentOids[te::edit::GEOMETRY_UPDATE_ATTRIBUTES] = te::da::GenerateOIDSet(operationds[te::edit::GEOMETRY_UPDATE_ATTRIBUTES], schema.get());

      operationds[te::edit::GEOMETRY_UPDATE_ATTRIBUTES]->moveBeforeFirst();

      t->update(dslayer->getDataSetName(), operationds[te::edit::GEOMETRY_UPDATE_ATTRIBUTES], properties, oidPropertyPosition);
    }

    env.Union(*operationds[te::edit::GEOMETRY_UPDATE]->getExtent(gpos).get());

    layer->setExtent(env);

    // Commit the transaction
    t->commit();

    // Clear the repository
    repo->clear();

    // Clear the undo stack
    te::edit::UndoStackManager::getInstance().getUndoStack()->clear();

    //update layer
    te::qt::widgets::MultiThreadMapDisplay* mtmp = dynamic_cast<te::qt::widgets::MultiThreadMapDisplay*>(display);
    if (mtmp)
      mtmp->updateLayer(layer);
    else
      display->refresh(true);

    // Select the oids
    if (operationds[te::edit::GEOMETRY_CREATE]->size() > 0)
      layer->select(currentOids[te::edit::GEOMETRY_CREATE]);

    if (operationds[te::edit::GEOMETRY_UPDATE]->size() > 0)
      layer->select(currentOids[te::edit::GEOMETRY_UPDATE]);

    if (operationds[te::edit::GEOMETRY_UPDATE_ATTRIBUTES]->size() > 0)
      layer->select(currentOids[te::edit::GEOMETRY_UPDATE_ATTRIBUTES]);

  }
  catch (te::common::Exception& ex)
  {
    t->rollBack();
    QMessageBox::critical(0, "TerraAmazon", ex.what());
    return;
  }
}

void ta::edition::qt::RemoveGeometries(const te::map::AbstractLayerPtr& layer, te::qt::widgets::MapDisplay* display)
{
  std::unique_ptr<te::da::DataSourceTransactor> t;

  try
  {
    te::edit::Repository* repo = te::edit::RepositoryManager::getInstance().getRepository(layer->getId());

    if (repo == 0)
      return;

    // Get the data souce info
    te::da::DataSourceInfoPtr info = te::da::DataSourceInfoManager::getInstance().get(layer.get()->getDataSourceId());

    // For while, use DataSetLayer to get the DataSource
    te::map::DataSetLayer* dslayer = dynamic_cast<te::map::DataSetLayer*>(layer.get());

    te::da::DataSourcePtr dsource = te::da::GetDataSource(layer.get()->getDataSourceId(), true);

    // Start the transactor
    t = dsource->getTransactor();

    // Get the layer schema
    std::auto_ptr<te::map::LayerSchema> schema(layer->getSchema());

    // Get the property names that compose the object id
    std::vector<std::string> oidPropertyNames;
    te::da::GetOIDPropertyNames(schema.get(), oidPropertyNames);

    // Get the edited geometries
    const std::vector<te::edit::Feature*>& features = repo->getAllFeatures();

    // Build the DataSet that will be used to update
    te::mem::DataSet* memds = new te::mem::DataSet(schema.get());

    t->begin();

    for (std::size_t i = 0; i < features.size() ; ++i) // for each edited feature
    {
      if (features[i]->getOperationType() == te::edit::GEOMETRY_DELETE)
      {
        // Create the new item
        te::mem::DataSetItem* item = new te::mem::DataSetItem(memds);

        // Get the object id
        te::da::ObjectId* oid = features[i]->getId();

        const boost::ptr_vector<te::dt::AbstractData>& values = oid->getValue();

        // Fill the new item
        for (std::size_t j = 0; j < values.size(); ++j)
          item->setValue(oidPropertyNames[j], values[j].clone());

        memds->add(item);
      }
    }

    te::da::ObjectIdSet* currentOids = 0;

    if (memds->size() > 0)
    {
      currentOids = te::da::GenerateOIDSet(memds, schema.get());

      memds->moveBeforeFirst();

      t->remove(dslayer->getDataSetName(), currentOids);
    }

    // Commit the transaction
    t->commit();

    std::set<te::da::ObjectId*, te::common::LessCmp<te::da::ObjectId*> >::const_iterator it = currentOids->begin();

    //clear de repo
    while (it != currentOids->end())
    {
      repo->remove(*it);

      ++it;
    }

    //update layer
    te::qt::widgets::MultiThreadMapDisplay* mtmp = dynamic_cast<te::qt::widgets::MultiThreadMapDisplay*>(display);
    if (mtmp)
      mtmp->updateLayer(layer);
    else
      display->refresh(true);

  }
  catch (te::common::Exception& ex)
  {
    t->rollBack();
    QMessageBox::critical(0, "TerraAmazon", ex.what());
    return;
  }
}

te::map::AbstractLayer* ta::edition::qt::GetSelectedLayerTreeView(QTreeView* view)
{
  QModelIndexList idxs = view->selectionModel()->selectedIndexes();

  if (idxs.isEmpty() || idxs.size() > 1)
    return 0;

  te::qt::widgets::TreeItem* item = static_cast<te::qt::widgets::TreeItem*>((*idxs.begin()).internalPointer());

  if (item->getType() != "LAYER")
    return 0;

  return ((te::qt::widgets::LayerItem*)item)->getLayer().get();
}

std::set<std::string> ta::edition::qt::GetLockedLayers(te::da::DataSourcePtr server)
{
  std::set<std::string> res;

  ta::model::CatalogFacade cf(server);

  std::vector<ta::model::Catalog*> catalogs = cf.getCatalogs();

  for (std::size_t t = 0; t < catalogs.size(); t++)
  {
    te::da::DataSourcePtr dataSource = cf.getDataSource(catalogs[t]);
    if (!dataSource.get())
      continue;

    if (catalogs[t]->isLocked())
    {
      te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().get(dataSource->getId());

      if (dsInfo.get())
        res.insert(ta::edition::qt::GetURICustomized(catalogs[t]->getName(), dsInfo));
    }

  }

  return res;
}

void ta::edition::qt::MoveToInterestArea(te::qt::widgets::MapDisplay* display, const  te::map::AbstractLayerPtr& layerAI,
                                         te::da::DataSourcePtr server, int processID)
{
  if (processID == -1 || layerAI.get() == 0)
    return;

  // GET SCENE
  ta::model::SceneFacade scenef(server);

  try{

    te::gm::Envelope env(*scenef.getAiGeometries(processID)->getEnvelope()->getMBR());
    if (!env.isValid())
      return;

    int srid = display->getSRID();

    te::gm::Envelope reprojectedEnvelope(env);

    if ((layerAI->getSRID() != TE_UNKNOWN_SRS) && (srid != TE_UNKNOWN_SRS) && (layerAI->getSRID() != srid))
      reprojectedEnvelope.transform(srid, layerAI->getSRID());

    display->setExtent(reprojectedEnvelope, true);

  }
  catch (...)
  {
    return;
  }

}

std::string ta::edition::qt::GetURICustomized(std::string title, te::da::DataSourceInfoPtr dsInfo)
{
  std::string uri = "";

  std::map<std::string, std::string> mapDs = dsInfo->getConnInfo();

  uri = title + mapDs["PG_DB_NAME"] + (mapDs["PG_HOST"] == "" ? "localhost" : mapDs["PG_HOST"]) +
                                      (mapDs["PG_PORT"] == "" ? "5432" : mapDs["PG_PORT"]) + mapDs["URI"];

  return uri;
}

void ta::edition::qt::SetupCellStyle(QColor& m_fillColor, QColor& m_contourColor, std::size_t& m_contourWidth, CellStyle cellStyle)
{
  switch (cellStyle)
  {
    case MYSELECTEDCELLS:
      m_fillColor = QColor(255, 255, 255, 0);
      m_contourColor = QColor(0, 255, 0);
      m_contourWidth = 2;

      break;

    case SELECTEDCELLSTOOL:
      m_fillColor = QColor(0, 0, 255, 150);
      m_contourColor = QColor(255, 0, 0);
      m_contourWidth = 3;

      break;

    case OTHERSELECTEDCELLS:
      m_fillColor = QColor(255, 255, 255, 0);
      m_contourColor = QColor(255, 0, 0);
      m_contourWidth = 2;

      break;

    default:
      break;
  }

}