/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

// TerraLib
#include <terralib/common/STLUtils.h>
#include <terralib/dataaccess/dataset/DataSet.h>
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceFactory.h>
#include <terralib/dataaccess/datasource/DataSourceInfoManager.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Expression.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/In.h>
#include <terralib/dataaccess/query/LiteralGeom.h>
#include <terralib/dataaccess/query/LiteralString.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/dataaccess/utils/Utils.h>

#include <terralib/datatype/AbstractData.h>
#include <terralib/datatype/SimpleData.h>

#include <terralib/edit/qt/Utils.h>
#include <terralib/edit/Feature.h>
#include <terralib/edit/Repository.h>
#include <terralib/edit/RepositoryManager.h>
#include <terralib/edit/Utils.h>

#include <terralib/geometry/Envelope.h>
#include <terralib/geometry/Geometry.h>
#include <terralib/geometry/GeometryProperty.h>
#include <terralib/geometry/GeometryCollection.h>
#include <terralib/geometry/Enums.h>
#include <terralib/geometry/LineString.h>
#include <terralib/geometry/Polygon.h>
#include <terralib/geometry/MultiPolygon.h>
#include <terralib/geometry/Utils.h>

#include <terralib/maptools/AbstractLayer.h>

#include <terralib/memory/DataSet.h>
#include <terralib/memory/DataSetItem.h>

#include <terralib/qt/widgets/canvas/MapDisplay.h>
#include <terralib/qt/widgets/Utils.h>
#include <terralib/qt/widgets/layer/utils/DataSet2Layer.h>

#include <terralib/vp/Utils.h>

// TerraAmazon
#include "Classify.h"
#include "Utils.h"
#include "../EditToolBar.h"
#include "../../../common/Utils.h"
#include "../../../model/facade/CatalogFacade.h"
#include "../../../model/facade/ClassFacade.h"
#include "../../../model/facade/CellFacade.h"
#include "../../../model/facade/ProcessFacade.h"
#include "../../../model/facade/ProjectFacade.h"
#include "../../../model/facade/RuleFacade.h"
#include "../../../model/facade/RuleSetFacade.h"
#include "../../../model/facade/RuleRuleSetFacade.h"
#include "../../../model/facade/RulesLayerColumnFacade.h"
#include "../../../model/facade/SceneFacade.h"
#include "../../../model/facade/TaskFacade.h"
#include "../../../model/item/AoiLayer.h"
#include "../../../model/item/Catalog.h"
#include "../../../model/item/Class.h"
#include "../../../model/item/Cell.h"
#include "../../../model/item/Process.h"
#include "../../../model/item/Rule.h"
#include "../../../model/item/RuleRuleSet.h"
#include "../../../model/item/Scene.h"
#include "../../../model/item/Task.h"
#include "../../../model/Utils.h"

// Qt
#include <QMessageBox>
#include <QMouseEvent>
#include <QPainter>
#include <QPixmap>

// STL
#include <cassert>
#include <memory>

// Boost
#include <boost/uuid/random_generator.hpp>
#include <boost/uuid/uuid_io.hpp>

ta::edition::qt::tools::Classify::Classify( te::qt::widgets::MapDisplay* display,
                                            te::da::DataSourcePtr server,
                                            int projectId,
                                            int processId,
                                            std::string className,
                                            const te::map::AbstractLayerPtr& draftLayer,
                                            const std::vector<te::map::AbstractLayerPtr>& outputLayerVec,
                                            QObject* parent)
  : GeometriesUpdateTool(display, draftLayer.get(), parent),
  m_server(server),
  m_projectId(projectId),
  m_processId(processId),
  m_draftLayer(draftLayer),
  m_outputLayerVec(outputLayerVec),
  m_className(className),
  m_ruleSet(0),
  m_sourceSRID(-1),
  m_outputDataSet(0),
  m_isFinished(false),
  m_isClean(false),
  m_isCleanClass(false)
{}

ta::edition::qt::tools::Classify::~Classify()
{}

void ta::edition::qt::tools::Classify::execute()
{
// GET RULESET LAYER
  ta::model::ProjectFacade projectFacade(m_server);
  ta::model::RuleSetFacade ruleSetFacade(m_server);
  ta::model::CatalogFacade catalogFacade(m_server);

  std::vector<int> ruleSets = projectFacade.getRuleSets(m_projectId);


// GET ACTIVE CELL 
  ta::model::CellFacade cellFacade(m_server);

  if (!cellFacade.loadLayer())
  {
    QMessageBox::information(m_display, "TerraAmazon", tr("Error loading cell layer!\nCheck one!"));
    return;
  }

  std::string cellLayerName = cellFacade.getLayerName();

  std::auto_ptr<te::da::DataSetType> dsTypeCell(te::da::GetDataSetType(cellLayerName, m_server->getId()));
  te::gm::GeometryProperty* cellProp = te::da::GetFirstGeomProperty(dsTypeCell.get());

  std::vector<te::gm::Geometry*> vecCell;

  bool successCells = cellFacade.getSelectedCells(m_processId, vecCell);
  if (!successCells)
  {
    QMessageBox::information(m_display, tr("TerraAmazon"), tr("Error while geting selected cells."));
    return;
  }

  if (vecCell.empty())
  {
    QMessageBox::information(m_display, tr("TerraAmazon"), tr("Select at least one cell."));
    return;
  }

  std::auto_ptr<ta::model::Catalog> cellCatalog(dynamic_cast<ta::model::Catalog*>(catalogFacade.get(catalogFacade.getActiveCellLayer())));
  if (!cellCatalog.get())
  {
    QMessageBox::information(m_display, tr("TerraAmazon"), tr("Error while geting cell catalog."));
    return;
  }



// GET TASK/SCENE/AREA OF INTEREST
  ta::model::ProcessFacade processFacade(m_server);
  std::auto_ptr<ta::model::Process> process(dynamic_cast<ta::model::Process*>(processFacade.get(m_processId)));
  if (!process.get())
  {
    QMessageBox::information(m_display, tr("TerraAmazon"), tr("Process not found!"));
    return;
  }

  int taskId = process->getTaskId();
  ta::model::TaskFacade taskFacade(m_server);
  std::auto_ptr<ta::model::Task> task(dynamic_cast<ta::model::Task*>(taskFacade.get(taskId)));
  if (!task.get())
  {
    QMessageBox::information(m_display, tr("TerraAmazon"), tr("Task not found!"));
    return;
  }

  std::auto_ptr<ta::model::Scene> scene(taskFacade.getScene(task.get()));
  if (!scene.get())
  {
    QMessageBox::information(m_display, tr("TerraAmazon"), tr("Scene not found!"));
    return;
  }

  ta::model::SceneFacade sceneFacade(m_server);
  int sceneLayerId = scene->getAoiLayerId();
  std::auto_ptr<ta::model::AoiLayer> aoiLayer(sceneFacade.getAoiLayer(sceneLayerId));

  std::vector<std::string> aiObjectNameVec;
  aiObjectNameVec.push_back(scene->getAiObjectId());


  std::auto_ptr<ta::model::Catalog> sceneCatalog(dynamic_cast<ta::model::Catalog*>(catalogFacade.get(aoiLayer->getCatalogId())));
  if (!sceneCatalog.get())
  {
    QMessageBox::information(m_display, tr("TerraAmazon"), tr("Error while geting scene catalog."));
    return;
  }

  const te::da::ObjectIdSet* objSet = m_draftLayer->getSelected();

  if (!objSet)
  {
    QMessageBox::information(m_display, tr("TerraAmazon"), tr("Select at least one geometry."));
    return;
  }

  std::auto_ptr<te::da::DataSet> ds(m_draftLayer->getData(objSet));

  std::auto_ptr<te::da::DataSetType> dt = m_draftLayer->getSchema();

  te::gm::GeometryProperty* geomProp = te::da::GetFirstGeomProperty(dt.get());

  std::vector<te::gm::Geometry*> geomVec;

  while (ds->moveNext())
  {
    te::gm::Geometry* geom = ds->getGeometry(geomProp->getName()).release();
    if (geom)
      geomVec.push_back(geom);
  }


// Do the Classification for each RuleSet
  for (std::size_t ruleSetPos = 0; ruleSetPos < ruleSets.size(); ++ruleSetPos)
  {
    if (!ruleSets.empty())
      m_ruleSet = dynamic_cast<ta::model::RuleSet*>(ruleSetFacade.get(ruleSets[ruleSetPos]));

    if (!m_ruleSet)
    {
      QMessageBox::information(m_display, tr("TerraAmazon"), tr("RuleSet not found!"));
      return;
    }

    int ruleSetCatalogId = m_ruleSet->getCatalogId();


// Verify if the current RuleSet contains the current Class.
    std::vector<int> classIdsVec = ruleSetFacade.getRuleSetClasses(ruleSetCatalogId);
    if (!isValidClass(classIdsVec))
      continue;


    std::auto_ptr<ta::model::Catalog> outputCatalog(dynamic_cast<ta::model::Catalog*>(catalogFacade.get(ruleSetCatalogId)));

    if (!outputCatalog.get())
    {
      QMessageBox::information(m_display, tr("TerraAmazon"), tr("Catalog not found!"));
      return;
    }

    std::unique_ptr<te::da::DataSetType> outputDsType(new te::da::DataSetType(outputCatalog->getName()));

    std::auto_ptr<te::da::DataSetType> catalogDsType = catalogFacade.getDataSetType(outputCatalog.get());

    std::vector<te::dt::Property*> vecProps = catalogDsType->getProperties();
    te::gm::GeometryProperty* catalogGeomProp = te::da::GetFirstGeomProperty(catalogDsType.get());
    m_sourceSRID = catalogGeomProp->getSRID();

    te::da::PrimaryKey* pk = catalogDsType->getPrimaryKey();
    std::vector<te::dt::Property*> pkProps = pk->getProperties();

    for (std::size_t p = 0; p < vecProps.size(); ++p)
    {
      bool isPk = false;

      for (std::size_t pk = 0; pk < pkProps.size(); ++pk)
      {
        if (vecProps[p]->getName() == pkProps[pk]->getName())
          isPk = true;
      }

      if (!isPk)
        outputDsType->add(vecProps[p]->clone());
    }



    std::vector<te::edit::Feature*> outputVecFeatures;

    // Get the rules of a project
    std::vector<ta::edition::qt::tools::RulesInfo> rulesInfo;

    ta::edition::qt::tools::RulesInfo ruleInfoCell = CreateImplicitRule(-2, "INTER_CELL",
                                                                        catalogFacade.getActiveCellLayer(),
                                                                        ta::model::SO_INTERSECTION,
                                                                        cellCatalog->getCellColumn(),
                                                                        outputCatalog->getCellColumn());
    
    rulesInfo.push_back(ruleInfoCell);

    ta::edition::qt::tools::RulesInfo ruleInfoAI = CreateImplicitRule(-1, "INTER_AI",
                                                                      sceneCatalog->getId(),
                                                                      ta::model::SO_INTERSECTION);

    rulesInfo.push_back(ruleInfoAI);

    std::vector<ta::edition::qt::tools::RulesInfo> auxRuleSetInfo = ta::edition::qt::tools::GetRules(m_server, *m_ruleSet, m_isClean);
    
    if (!auxRuleSetInfo.empty())
      rulesInfo.insert(rulesInfo.end(), auxRuleSetInfo.begin(), auxRuleSetInfo.end());

    //std::vector<ta::edition::qt::tools::RulesInfo> rulesInfo = ta::edition::qt::tools::GetRules(m_server, m_projectId, m_processId, ruleSetPos, m_isClean);

    // RUN RULE BY RULE FOR EACH GEOMETRY
    for (std::size_t g = 0; g < geomVec.size(); ++g)
    {
      bool isCommited = false;

      std::vector<te::edit::Feature*> inputVecFeatures;
      te::gm::Geometry* currentGeom = dynamic_cast<te::gm::Geometry*>(geomVec[g]->clone());

      if (!currentGeom)
        continue;

      te::edit::Feature* firstFeature = new te::edit::Feature();
      firstFeature->setGeometry(currentGeom);
      inputVecFeatures.push_back(firstFeature);

      for (std::size_t r = 0; r < rulesInfo.size(); ++r)
      {
        std::auto_ptr<ta::model::Catalog> ruleCatalog(dynamic_cast<ta::model::Catalog*>(catalogFacade.get(rulesInfo[r].m_rule->getCatalogId())));

        std::string catalogName = ruleCatalog->getName();

        std::auto_ptr<te::da::DataSetType> ruleDsType = catalogFacade.getDataSetType(ruleCatalog.get());

        te::gm::GeometryProperty* geomProp(te::da::GetFirstGeomProperty(ruleDsType.get()));
        std::auto_ptr<te::da::DataSet> dataSetCatalog;

        // CONDITIONAL TO RUN THE CELL RULE.
        if (r == 0)
        {
          dataSetCatalog = m_server->query(GetCellQuery(cellLayerName,
            cellProp->getName(),
            vecCell));

        }
        // CONDITIONAL TO RUN THE AI RULE.
        else if (r == 1)
        {
          dataSetCatalog = m_server->query(GetRestrictionQuery(catalogName,
            aoiLayer->getAoiIdColumn(),
            aiObjectNameVec));

        }
        // CONDITIONAL TO RUN THE OTHER RULES.
        else
        {
          int sourceSRID = geomProp->getSRID();

          if (sourceSRID != sourceSRID)
            geomVec[g]->transform(sourceSRID);

          dataSetCatalog = m_server->getDataSet(catalogName,
            geomProp->getName(),
            geomVec[g]->getMBR(),
            te::gm::INTERSECTS);
        }

        int opId = rulesInfo[r].m_rule->getOperationId();

        switch (opId)
        {
        case ta::model::SO_INTERSECTION:
        {

                                         outputVecFeatures.clear();

                                         InputMember inputMember;

                                         inputMember.inputDataSetType = te::da::GetDataSetType(catalogName, m_server->getId());
                                         inputMember.inputDataSet = dataSetCatalog.get();
                                         inputMember.inputVecFeatures = inputVecFeatures;
                                         inputMember.rulesLayerColumnVec = rulesInfo[r].m_columns;
                                         inputMember.outputDataSetType = outputDsType.get();

                                         classifyIntersection(inputMember, outputVecFeatures);
                                         break;
        }
        case ta::model::SO_DIFFERENCE:
        {
                                       if (!dataSetCatalog->isEmpty())
                                       {
                                         outputVecFeatures.clear();
                                         classifyDifference(catalogName, dataSetCatalog, inputVecFeatures, outputVecFeatures);
                                       }
                                       break;
        }
        case ta::model::SO_UNION:
        {
                                  // UNDER DEVELOPMENT..
        }
          break;
        case ta::model::SO_CLEAN:
        {
                                  if (!dataSetCatalog->isEmpty())
                                  {
                                    outputVecFeatures.clear();

                                    clearClassification(catalogName, dataSetCatalog, inputVecFeatures, outputVecFeatures);
                                  }
        }
          break;
        default:
          break;
        }

        inputVecFeatures = outputVecFeatures;
      }

      te::gm::Envelope env(m_outputLayerVec[ruleSetPos]->getExtent());

      if (!m_isClean)
      {
        // SET CLASSCOLUMN IN ITEM AND PERSIST THE VALUE INTO THE OUTPUT LAYER.
        for (std::size_t i = 0; i < outputVecFeatures.size(); ++i)
        {
          m_outputDataSet = new te::mem::DataSet(outputDsType.get());
          te::mem::DataSetItem* item = new te::mem::DataSetItem(m_outputDataSet);

          item->setString("class_name", m_className);
          item->setInt32("scene_id", scene->getId());
          item->setInt32("task_id", taskId);
          item->setGeometry(catalogGeomProp->getName(), outputVecFeatures[i]->getGeometry());

          std::map<std::size_t, te::dt::AbstractData*> data = outputVecFeatures[i]->getData();
          std::map<std::size_t, te::dt::AbstractData*>::iterator it = data.begin();

          while (it != data.end())
          {
            std::size_t pos = it->first;
            item->setValue(pos, it->second);

            ++it;
          }

          isCommited = persistItem(outputCatalog->getName(), item);
          if (!isCommited)
            continue;

          env.Union(*outputVecFeatures[i]->getGeometry()->getMBR());
        }
      }

      m_outputLayerVec[ruleSetPos]->setExtent(env);
    }
  }

  m_draftLayer->clearSelected();
  m_display->refresh(true);
  
}

bool ta::edition::qt::tools::Classify::persistItem(const std::string& catalogName, te::mem::DataSetItem* item)
{
  m_outputDataSet->add(item);

  std::map<std::string, std::string> op;

  std::unique_ptr<te::da::DataSourceTransactor> transactor = m_server->getTransactor();

  try
  {
    transactor->add(catalogName, m_outputDataSet, op);
    transactor->commit();
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    transactor->rollBack();
    return false;
  }

  return true;
}

void ta::edition::qt::tools::Classify::classifyIntersection(const InputMember& inputMember,
                                                            std::vector<te::edit::Feature*>& outputVecFeatures)
{
  te::gm::GeometryProperty* geomProp = te::da::GetFirstGeomProperty(inputMember.inputDataSetType);

  for (std::size_t i = 0; i < inputMember.inputVecFeatures.size(); ++i)
  {
    inputMember.inputDataSet->moveBeforeFirst();

    while (inputMember.inputDataSet->moveNext())
    {
      std::auto_ptr<te::gm::Geometry> geom = inputMember.inputDataSet->getGeometry(geomProp->getName());
      std::vector<te::gm::Geometry*> geomOutputVec;

      if (!inputMember.inputVecFeatures[i]->getGeometry()->isValid())
        continue;

      intersection(inputMember.inputVecFeatures[i]->getGeometry(), geom.get(), geomOutputVec);

      for (std::size_t j = 0; j < geomOutputVec.size(); ++j)
      {
        te::edit::Feature* feature = new te::edit::Feature();
        
        if (geomOutputVec[j]->isValid())
          feature->setGeometry(geomOutputVec[j]);
        
        std::map<std::size_t, te::dt::AbstractData*> data = inputMember.inputVecFeatures[i]->getData();

        for (std::size_t k = 0; k < inputMember.rulesLayerColumnVec.size(); ++k)
        {
          std::string columnIn = inputMember.rulesLayerColumnVec[k]->getColumnIn();
          std::string columnOut = inputMember.rulesLayerColumnVec[k]->getColumnOut();
          
          std::size_t position = inputMember.outputDataSetType->getPropertyPosition(columnOut);
          te::dt::AbstractData* value = inputMember.inputDataSet->getValue(columnIn)->clone();

          if (!columnIn.empty() && !columnOut.empty())
            data.insert(std::map<std::size_t, te::dt::AbstractData*>::value_type(position, value));
        }

        if (!data.empty())
          feature->setData(data);

        outputVecFeatures.push_back(feature);
      }
    }
  }
}

void ta::edition::qt::tools::Classify::classifyDifference(const std::string& dataSetName,
                                                          const std::auto_ptr<te::da::DataSet>& dataSetCatalog,
                                                          const std::vector<te::edit::Feature*>& inputVecFeatures,
                                                          std::vector<te::edit::Feature*>& outputVecFeatures)
{
  te::da::DataSetType* dataSetType = te::da::GetDataSetType(dataSetName, m_server->getId());
  te::gm::GeometryProperty* geomProp = te::da::GetFirstGeomProperty(dataSetType);

  dataSetCatalog->moveNext();
  std::auto_ptr<te::gm::Geometry> geom;

  if (!dataSetCatalog->isAfterEnd())
    geom = dataSetCatalog->getGeometry(geomProp->getName());

  while (dataSetCatalog->moveNext())
  {
    std::auto_ptr<te::gm::Geometry> currentGeom = dataSetCatalog->getGeometry(geomProp->getName());

    if (currentGeom->isValid())
      geom.reset(geom->Union(currentGeom.get()));
  }
  
  if (!geom.get())
    return;

  for (std::size_t i = 0; i < inputVecFeatures.size(); ++i)
  {
    std::vector<te::gm::Geometry*> geomOutputVec;

    if (!inputVecFeatures[i]->getGeometry()->isValid())
      continue;

    difference(inputVecFeatures[i]->getGeometry(), geom.get(), geomOutputVec);

    for (std::size_t j = 0; j < geomOutputVec.size(); ++j)
    {
      te::edit::Feature* feature = new te::edit::Feature();
      
      if (geomOutputVec[j]->isValid())
        feature->setGeometry(geomOutputVec[j]);

      std::map<std::size_t, te::dt::AbstractData*> data = inputVecFeatures[i]->getData();
      if (!data.empty())
        feature->setData(data);

      outputVecFeatures.push_back(feature);
    }
  }
}

void ta::edition::qt::tools::Classify::clearClassification(const std::string& dataSetName,
                                                           const std::auto_ptr<te::da::DataSet>& dataSetCatalog,
                                                           const std::vector<te::edit::Feature*>& inputVecFeatures,
                                                           std::vector<te::edit::Feature*>& outputVecFeatures)
{
  te::da::DataSetType* dataSetType = te::da::GetDataSetType(dataSetName, m_server->getId());
  te::gm::GeometryProperty* geomProp = te::da::GetFirstGeomProperty(dataSetType);

  te::gm::Geometry* unionDraft = 0;
  for (std::size_t i = 0; i < inputVecFeatures.size(); ++i)
  {
    if (i == 0)
      unionDraft = inputVecFeatures[i]->getGeometry();
    else
      unionDraft = unionDraft->Union(inputVecFeatures[i]->getGeometry());
  }


  std::auto_ptr<te::da::DataSetType> dsType = m_server->getDataSetType(dataSetName);

  std::unique_ptr<te::da::DataSetType> outputDsType(new te::da::DataSetType(dataSetName));

  std::vector<te::dt::Property*> vecProps = dsType->getProperties();
  te::gm::GeometryProperty* catalogGeomProp = te::da::GetFirstGeomProperty(dsType.get());
  m_sourceSRID = catalogGeomProp->getSRID();

  te::da::PrimaryKey* pk = dsType->getPrimaryKey();
  std::vector<te::dt::Property*> pkProps = pk->getProperties();

  for (std::size_t p = 0; p < vecProps.size(); ++p)
  {
    bool isPk = false;

    for (std::size_t pk = 0; pk < pkProps.size(); ++pk)
    {
      if (vecProps[p]->getName() == pkProps[pk]->getName())
        isPk = true;
    }

    if (!isPk)
      outputDsType->add(vecProps[p]->clone());
  }

  while (dataSetCatalog->moveNext())
  {
    if (m_isCleanClass && dataSetCatalog->getString("class_name") != m_className)
    {
      continue;
    }

    std::auto_ptr<te::gm::Geometry> geom = dataSetCatalog->getGeometry(geomProp->getName());

    std::vector<te::gm::Geometry*> geomOutputVec;

    clear(unionDraft, geom.get(), geomOutputVec);

    //Salva

    te::da::ObjectId* objId = new te::da::ObjectId;
    objId->addValue(new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(dataSetCatalog->getInt32("object_id"))));

    te::da::ObjectIdSet* os = new te::da::ObjectIdSet;
    os->add(objId);
    m_server->remove(dataSetName, os);

    std::auto_ptr<te::mem::DataSet> newDs(new te::mem::DataSet(outputDsType.get()));

    for (std::size_t i = 0; i < geomOutputVec.size(); ++i)      

    {
      std::auto_ptr<te::mem::DataSetItem> item(new te::mem::DataSetItem(newDs.get()));
      std::vector<te::dt::Property*> props = outputDsType->getProperties();
      for (std::size_t p = 0; p < props.size(); p++)
      {
        if (props[p]->getType() == te::dt::GEOMETRY_TYPE)
          item->setGeometry(props[p]->getName(), geomOutputVec[i]);
        else
          item->setValue(props[p]->getName(), dataSetCatalog->getValue(props[p]->getName()).release());
      }

      newDs->add(item.release());

    }

    if (!geomOutputVec.empty())
    {
      std::map<std::string, std::string> op;
      m_server->add(dataSetName, newDs.release(), op);
    }
  }
}

void ta::edition::qt::tools::Classify::intersection(te::gm::Geometry* gIn,
                                                    te::gm::Geometry* geomSource,
                                                    std::vector<te::gm::Geometry*>& gOut)
{
// Check SRID
  int currentSRID = gIn->getSRID();
  int catalogSRID = geomSource->getSRID();
  
  if (!gIn->getEnvelope()->intersects(geomSource->getEnvelope()))
    return;

  if (currentSRID != m_sourceSRID)
    gIn->transform(m_sourceSRID);

  if (catalogSRID != m_sourceSRID)
    geomSource->transform(m_sourceSRID);

// Split Multigeometry.
  te::gm::Geometry* geomResult;
  
  std::vector<te::gm::Geometry*> gInVec;
  te::gm::Multi2Single(gIn, gInVec);

// Do Intersection  
  for (std::size_t i = 0; i < gInVec.size(); ++i)
  {
    std::vector<te::gm::Geometry*> auxGeomVec;

    geomResult = gInVec[i]->intersection(geomSource);

    te::gm::Multi2Single(geomResult, auxGeomVec);

    for (std::size_t j = 0; j < auxGeomVec.size(); ++j)
    {
      te::gm::GeomType gAuxType = auxGeomVec[j]->getGeomTypeId();
      bool valid = (gAuxType == te::gm::PolygonType) || (gAuxType == te::gm::PolygonZType) ||
        (gAuxType == te::gm::PolygonMType) || (gAuxType == te::gm::PolygonZMType);

      if (!valid)
        continue;

      te::gm::Geometry* auxGeom = auxGeomVec[j];
      auxGeomVec[j] = GetValidPolygon((te::gm::Polygon*)auxGeom);
      auxGeomVec[j]->setSRID(m_sourceSRID);
      delete auxGeom;
    }

    gOut.insert(gOut.end(), auxGeomVec.begin(), auxGeomVec.end());
  }
}

void ta::edition::qt::tools::Classify::difference(te::gm::Geometry* gIn,
                                                  te::gm::Geometry* geomSource,
                                                  std::vector<te::gm::Geometry*>& gOut)
{
// Check SRID
  int currentSRID = gIn->getSRID();
  int sourceSRID = geomSource->getSRID();

  if (currentSRID != sourceSRID)
    gIn->transform(sourceSRID);

// Split Multigeometry.
  te::gm::Geometry* geomResult;

  std::vector<te::gm::Geometry*> gInVec;
  te::gm::Multi2Single(gIn, gInVec);

// Do Difference
  for (std::size_t i = 0; i < gInVec.size(); ++i)
  {
    std::vector<te::gm::Geometry*> auxGeomVec;

    geomResult = gInVec[i]->difference(geomSource);

    te::gm::Multi2Single(geomResult, auxGeomVec);

    for (std::size_t j = 0; j < auxGeomVec.size(); ++j)
    {
      te::gm::GeomType gAuxType = auxGeomVec[j]->getGeomTypeId();
      bool valid = (gAuxType == te::gm::PolygonType) || (gAuxType == te::gm::PolygonZType) ||
        (gAuxType == te::gm::PolygonMType) || (gAuxType == te::gm::PolygonZMType);

      if (!valid)
        continue;

      te::gm::Geometry* auxGeom = auxGeomVec[j];
      auxGeomVec[j] = GetValidPolygon((te::gm::Polygon*)auxGeom);
      auxGeomVec[j]->setSRID(sourceSRID);
      delete auxGeom;
    }

    gOut.insert(gOut.end(), auxGeomVec.begin(), auxGeomVec.end());
  }
}

void ta::edition::qt::tools::Classify::clear(te::gm::Geometry* gIn,
                                             te::gm::Geometry* geomSource,
                                             std::vector<te::gm::Geometry*>& gOut)
{
  // Check SRID
  int currentSRID = gIn->getSRID();
  int sourceSRID = geomSource->getSRID();

  if (currentSRID != sourceSRID)
    gIn->transform(sourceSRID);

  // Split Multigeometry.
  te::gm::Geometry* geomResult;

  std::vector<te::gm::Geometry*> gInVec;
  te::gm::Multi2Single(gIn, gInVec);

  // Do Difference
  for (std::size_t i = 0; i < gInVec.size(); ++i)
  {
    std::vector<te::gm::Geometry*> auxGeomVec;

    geomResult = geomSource->difference(gInVec[i]);//gInVec[i]->difference(geomSource);

    te::gm::Multi2Single(geomResult, auxGeomVec);

    for (std::size_t j = 0; j < auxGeomVec.size(); ++j)
    {
      te::gm::GeomType gAuxType = auxGeomVec[j]->getGeomTypeId();
      bool valid = (gAuxType == te::gm::PolygonType) || (gAuxType == te::gm::PolygonZType) ||
        (gAuxType == te::gm::PolygonMType) || (gAuxType == te::gm::PolygonZMType);

      if (!valid)
        continue;

      te::gm::Geometry* auxGeom = auxGeomVec[j];
      auxGeomVec[j] = GetValidPolygon((te::gm::Polygon*)auxGeom);
      auxGeomVec[j]->setSRID(sourceSRID);
      delete auxGeom;
    }

    gOut.insert(gOut.end(), auxGeomVec.begin(), auxGeomVec.end());
  }
}

void ta::edition::qt::tools::Classify::setIsClean(const bool& isClean)
{
  m_isClean = isClean;
}

void ta::edition::qt::tools::Classify::setIsCleanClass(const bool& isCleanClass)
{
  m_isCleanClass = isCleanClass;
}

bool ta::edition::qt::tools::Classify::isValidClass(std::vector<int>& classIdsVec)
{
  ta::model::ClassFacade classFacade(m_server);
  
  for (std::size_t i = 0; i < classIdsVec.size(); ++i)
  {
    std::auto_ptr<ta::model::Class> currentClass(dynamic_cast<ta::model::Class*>(classFacade.get(classIdsVec[i])));
    std::string className = currentClass->getName();
    if (m_className == className)
      return true;
  }

  return false;
}