/*  This file is part of the TerraAmazon application.

  See the Copyright file for details about the license of
  this application.

*/
#ifndef __TERRAAMAZON_EDITION_QT_TOOLS_CLASSIFY_H
#define __TERRAAMAZON_EDITION_QT_TOOLS_CLASSIFY_H

// TerraLib
#include <terralib/edit/qt/tools/GeometriesUpdateTool.h>
#include <terralib/geometry/Coord2D.h>
#include <terralib/maptools/AbstractLayer.h>
#include <terralib/memory/DataSetItem.h>

// TerraAmazon
#include "model/item/Catalog.h"
#include "model/item/RuleSet.h"
#include "model/item/RulesLayerColumn.h"

#include "Config.h"

// STL
#include <vector>

namespace te
{
  namespace gm
  {
    class Geometry;
  }

  namespace qt
  {
    namespace widgets
    {
      class MapDisplay;
    }
  }
}

namespace ta
{
  namespace edition
  {
    namespace qt
    {
      namespace tools
      {
        /*!
          \class Classify

          \brief This class implements a concrete classification tool.
          */
        class Classify : public te::edit::GeometriesUpdateTool
        {
          Q_OBJECT

          public:

            /** @name Initializer Methods
              *  Methods related to instantiation and destruction.
              */
            //@{

            /*!
              \brief It constructs a classification tool associated with the given map display.

              \param display   The map display associated with the tool.
              \param server   Active server.
              \param projectId   Active project id.
              \param processId   Active  process id.
              \param className   Class name to set in class column.
              \param draftLayer   Draft layer to get drawn  geometries.
              \param outputLayer   Output layer to get/set extent.
              \param parent   The tool's parent.

              \note The tool will NOT take the ownership of the given pointers.
              */
            Classify( te::qt::widgets::MapDisplay* display,
                      te::da::DataSourcePtr server,
                      int projectId,
                      int processId,
                      std::string className,
                      const te::map::AbstractLayerPtr& draftLayer,
                      const std::vector<te::map::AbstractLayerPtr>& outputLayerVec,
                      QObject* parent = 0);

            /*! \brief Destructor. */
            ~Classify();

            /*
            \brief Main function.
            */
            void execute();

            /*!
            \brief It define if the operation is "clear"

            \param isClean True fi is "clear"
            */
            void setIsClean(const bool& isClean);

            /*!
            \brief It define if the operation is "clear class"

            \param isCleanClass True fi is "clear class"
            */
            void setIsCleanClass(const bool& isCleanClass);

            bool isValidClass(std::vector<int>& classIdsVec);

          private:

            struct InputMember
            {
              te::da::DataSetType* inputDataSetType;
              te::da::DataSet* inputDataSet;
              std::vector<te::edit::Feature*> inputVecFeatures;
              std::vector<ta::model::RulesLayerColumn*> rulesLayerColumnVec;
              te::da::DataSetType* outputDataSetType;
            };

            /*
            \brief Persist item in a catalog.

            \param[in] catalogName   Catalog name where the item is saved.
            \param[in] item   Item with the values that will be saved.

            \return bool   Return true if item is persisted
            */
            bool persistItem(const std::string& catalogName, te::mem::DataSetItem* item);

           te::gm::Polygon* getValidPolygon(te::gm::Polygon* gm);

           void classifyIntersection( const InputMember& inputMember,
                                      std::vector<te::edit::Feature*>& outputVecFeatures);

           void classifyDifference( const std::string& dataSetName,
                                    const std::auto_ptr<te::da::DataSet>& dataSetCatalog,
                                    const std::vector<te::edit::Feature*>& inputVecFeatures,
                                    std::vector<te::edit::Feature*>& outputVecFeatures);

           void clearClassification(const std::string& dataSetName,
                                    const std::auto_ptr<te::da::DataSet>& dataSetCatalog,
                                    const std::vector<te::edit::Feature*>& inputVecFeatures,
                                    std::vector<te::edit::Feature*>& outputVecFeatures);

           void intersection(te::gm::Geometry* gIn,
                             te::gm::Geometry* geomSource,
                             std::vector<te::gm::Geometry*>& gOut);

           void difference(te::gm::Geometry* gIn,
                           te::gm::Geometry* geomSource,
                           std::vector<te::gm::Geometry*>& gOut);

           void clear(te::gm::Geometry* gIn,
                      te::gm::Geometry* geomSource,
                      std::vector<te::gm::Geometry*>& gOut);

          protected:

            te::da::DataSourcePtr m_server;             //!< Active server.
            int m_projectId;                            //!< Active project.
            int m_processId;                            //!< Active process.
                        
            te::map::AbstractLayerPtr m_draftLayer;                     //!< Draft layer.
            std::vector<te::map::AbstractLayerPtr> m_outputLayerVec;    //!< Output Layer Vector.
            te::map::AbstractLayerPtr m_cellLayer;                      //!< Abstract cell layer.

            std::string m_className;                    //!< Class name to label the classified polygon.
            ta::model::RuleSet* m_ruleSet;              //!< An Object that represents the output catalog and its rules.
            int m_sourceSRID;                           //!< SRID from output catalog.
            
            te::mem::DataSet* m_outputDataSet;          //!< Output dataset.
            std::vector<te::edit::Feature*> m_feature;  //!< A vector of features with values and geometries already classified.
            bool m_isFinished;                          //!< A flag that indicates if the operations was finished.

            std::string m_errorMessage;

            bool m_isClean;
            bool m_isCleanClass;
        };
      }
    }
  }
}

#endif  // __TERRAAMAZON_EDITION_QT_TOOLS_CLASSIFY_H

  