/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

// TerraLib
#include <terralib/edit/RepositoryManager.h>
#include <terralib/edit/Utils.h>
#include <terralib/edit/qt/Renderer.h>
#include <terralib/edit/qt/Utils.h>
#include <terralib/geometry/Envelope.h>
#include <terralib/geometry/Geometry.h>
#include <terralib/geometry/LinearRing.h>
#include <terralib/geometry/LineString.h>
#include <terralib/geometry/Point.h>
#include <terralib/maptools/AbstractLayer.h>
#include <terralib/qt/widgets/canvas/MapDisplay.h>
#include <terralib/qt/widgets/Utils.h>

// TerraAmazon
#include "Reclassify.h"

// Qt
#include <QMessageBox>
#include <QMouseEvent>
#include <QPainter>
#include <QPixmap>

// STL
#include <cassert>
#include <memory>

ta::edition::qt::tools::Reclassify::Reclassify(te::qt::widgets::MapDisplay* display, const te::map::AbstractLayerPtr& layer, const QCursor& cursor, QObject* parent)
  : GeometriesUpdateTool(display, layer.get(), parent),
  m_continuousMode(false),
  m_isFinished(false)
{
  setCursor(cursor);

  // Signals & slots
  connect(m_display, SIGNAL(extentChanged()), SLOT(onExtentChanged()));

  draw();
}

ta::edition::qt::tools::Reclassify::~Reclassify()
{
  QPixmap* draft = m_display->getDraftPixmap();
  draft->fill(Qt::transparent);
}

bool ta::edition::qt::tools::Reclassify::mousePressEvent(QMouseEvent* e)
{
  if(e->button() != Qt::LeftButton)
    return false;

  if(m_isFinished) // Is Finished?! So, start again...
  {
    clear();
    m_isFinished = false;
  }

  QPointF pw = m_display->transform(te::edit::GetPosition(e));

  te::gm::Coord2D coord = te::gm::Coord2D(pw.x(), pw.y());

  te::edit::TrySnap(coord, m_display->getSRID());

  m_coords.push_back(coord);

  return true;
}

bool ta::edition::qt::tools::Reclassify::mouseMoveEvent(QMouseEvent* e)
{
  if(m_coords.size() < 1 || m_isFinished)
    return false;

  QPointF pos = te::edit::GetPosition(e);

  QPointF pw = m_display->transform(pos);

  te::gm::Coord2D coord = te::gm::Coord2D(pw.x(), pw.y());

  te::edit::TrySnap(coord, m_display->getSRID());

  m_coords.push_back(coord);

  m_lastPos = te::gm::Coord2D(coord.x, coord.y);

  if (e->buttons() & Qt::LeftButton)
    m_continuousMode = true;
  else
    m_continuousMode = false;

  draw();

  return false;
}

bool ta::edition::qt::tools::Reclassify::mouseDoubleClickEvent(QMouseEvent* e)
{
  if(e->button() != Qt::LeftButton)
    return false;

  m_isFinished = true;

  storeNewGeometry();

  return true;
}

void ta::edition::qt::tools::Reclassify::draw()
{
  const te::gm::Envelope& env = m_display->getExtent();
  if(!env.isValid())
    return;

  // Clear!
  QPixmap* draft = m_display->getDraftPixmap();
  draft->fill(Qt::transparent);

  // Initialize the renderer
  te::edit::Renderer& renderer = te::edit::Renderer::getInstance();
  renderer.begin(draft, env, m_display->getSRID());

  // Draw the layer edited geometries
  renderer.drawRepository(m_layer->getId(), env, m_display->getSRID());

  if(!m_coords.empty())
  {
    // Draw the geometry being created
    te::gm::Geometry* line = buildLine();
    renderer.draw(line, true);

    if(m_continuousMode == false)
      m_coords.pop_back();

  }

  renderer.end();

  m_display->repaint();
  
}

void ta::edition::qt::tools::Reclassify::clear()
{
  m_coords.clear();
}

te::gm::Geometry* ta::edition::qt::tools::Reclassify::buildLine()
{
  te::gm::LineString* line = new te::gm::LineString(m_coords.size(), te::gm::LineStringType);
  for(std::size_t i = 0; i < m_coords.size(); ++i)
    line->setPoint(i, m_coords[i].x, m_coords[i].y);

  line->setSRID(m_display->getSRID());

  if(line->getSRID() == m_layer->getSRID())
    return line;

  // else, need conversion...
  line->transform(m_layer->getSRID());

  return line;
}

void ta::edition::qt::tools::Reclassify::storeNewGeometry()
{
  te::edit::RepositoryManager::getInstance().addGeometry(m_layer->getId(), buildLine(), te::edit::GEOMETRY_CREATE);
  emit geometriesEdited();
}

void ta::edition::qt::tools::Reclassify::onExtentChanged()
{
  if(m_coords.empty())
    return;

  m_coords.push_back(m_lastPos);

  draw();
}
