/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

// TerraLib
#include <terralib/dataaccess/dataset/ObjectId.h>
#include <terralib/dataaccess/dataset/ObjectIdSet.h>
#include <terralib/dataaccess/utils/Utils.h>
#include <terralib/edit/Utils.h>
#include <terralib/edit/qt/Renderer.h>
#include <terralib/edit/qt/Utils.h>
#include <terralib/geometry/Envelope.h>
#include <terralib/geometry/Geometry.h>
#include <terralib/geometry/GeometryProperty.h>
#include <terralib/geometry/Utils.h>
#include <terralib/maptools/AbstractLayer.h>
#include <terralib/qt/widgets/canvas/MapDisplay.h>
#include <terralib/qt/widgets/utils/ScopedCursor.h>

// TerraAmazon
#include "SelectAreaTool.h"

// Qt
#include <QKeyEvent>
#include <QMessageBox>

ta::edition::qt::tools::SelectAreaTool::SelectAreaTool(te::qt::widgets::MapDisplay* display, const te::map::AbstractLayerPtr& layer, const QCursor& cursor, QObject* parent)
: GeometriesUpdateTool(display, layer.get() , parent),
  m_continuousMode(true),
  m_keepPreviousSelection(false),
  m_selectionStarted(false),
  m_selectionByPointing(false),
  m_geom(0)
{
  setCursor(cursor);
}

ta::edition::qt::tools::SelectAreaTool::~SelectAreaTool()
{
  QPixmap* draft = m_display->getDraftPixmap();
  draft->fill(Qt::transparent);
}

bool ta::edition::qt::tools::SelectAreaTool::mousePressEvent(QMouseEvent* e)
{
  if (e->button() != Qt::LeftButton)
    return false;

  m_selectionStarted = true;

  QPointF pw = m_display->transform(te::edit::GetPosition(e));

  te::gm::Coord2D coord = te::gm::Coord2D(pw.x(), pw.y());

  te::edit::TrySnap(coord, m_display->getSRID());

  m_coords.push_back(coord);

  return true;
}

bool ta::edition::qt::tools::SelectAreaTool::mouseMoveEvent(QMouseEvent* e)
{
  if (m_coords.size() < 1 || !m_selectionStarted)
    return false;

  QPointF pos = te::edit::GetPosition(e);
  pos += QPointF(0.0001, 0.0001); // To avoid collinear points on polygon

  QPointF pw = m_display->transform(pos);

  te::gm::Coord2D coord = te::gm::Coord2D(pw.x(), pw.y());

  te::edit::TrySnap(coord, m_display->getSRID());

  m_coords.push_back(coord);

  m_lastPos = te::gm::Coord2D(coord.x, coord.y);

  draw();

  return false;

}
bool ta::edition::qt::tools::SelectAreaTool::mouseReleaseEvent(QMouseEvent* e)
{
  te::qt::widgets::ScopedCursor cursor(Qt::WaitCursor);

  m_selectionStarted = false;

  clear();

  m_keepPreviousSelection = false;

  Qt::KeyboardModifiers keys = e->modifiers();

  if (keys & Qt::ControlModifier)
    m_keepPreviousSelection = true;

  m_selectionByPointing = false;

  te::gm::Envelope env;
  if (m_geom == 0)
  {
    m_selectionByPointing = true;
    env = buildEnvelope(te::edit::GetPosition(e));
  }
  else
  {
    env = *m_geom->getMBR();
  }

  // Select the draft layer
  executeSelection(m_layer, env);

  delete m_geom;
  m_geom = 0;

  emit geometriesEdited();

  return true;
}

void ta::edition::qt::tools::SelectAreaTool::draw()
{
  const te::gm::Envelope& env = m_display->getExtent();
  if (!env.isValid())
    return;

  // Clear!
  QPixmap* display = m_display->getDraftPixmap();
  display->fill(Qt::transparent);

  // Initialize the renderer
  te::edit::Renderer& renderer = te::edit::Renderer::getInstance();
  renderer.begin(display, env, m_display->getSRID());

  renderer.setPolygonStyle(QColor(255, 255, 255, 0), QColor(255, 0, 0), 1); //TODO drashline?!?!

  if (!m_coords.empty())
  {
    // Draw the geometry being created
    if (m_coords.size() > 2)
      drawPolygon();

    if (m_continuousMode == false)
      m_coords.pop_back();
  }
  
  renderer.end();

  m_display->repaint();
}

void ta::edition::qt::tools::SelectAreaTool::drawPolygon()
{
  // Build the geometry
  m_geom = buildPolygon();

  // Draw the current geometry and the vertexes
  te::edit::Renderer& renderer = te::edit::Renderer::getInstance();
  renderer.draw(m_geom);

}

te::gm::Geometry* ta::edition::qt::tools::SelectAreaTool::buildPolygon()
{
  // Build the geometry
  te::gm::LinearRing* ring = new te::gm::LinearRing(m_coords.size() + 1, te::gm::LineStringType);
  for (std::size_t i = 0; i < m_coords.size(); ++i)
    ring->setPoint(i, m_coords[i].x, m_coords[i].y);
  ring->setPoint(m_coords.size(), m_coords[0].x, m_coords[0].y); // Closing...

  te::gm::Polygon* polygon = new te::gm::Polygon(1, te::gm::PolygonType);
  polygon->setRingN(0, ring);

  polygon->setSRID(m_display->getSRID());

  if (polygon->getSRID() == m_layer->getSRID())
    return polygon;

  // else, need conversion...
  polygon->transform(m_layer->getSRID());

  return polygon;
}


void ta::edition::qt::tools::SelectAreaTool::clear()
{
  m_coords.clear();

  QPixmap* draft = m_display->getDraftPixmap();
  draft->fill(Qt::transparent);

  m_display->repaint();

}

void ta::edition::qt::tools::SelectAreaTool::executeSelection(const te::map::AbstractLayerPtr& layer, const te::gm::Envelope& e)
{
  if (layer->getVisibility() != te::map::VISIBLE || !layer->isValid() || layer->getType() == "FOLDERLAYER")
    return;

  std::auto_ptr<te::da::DataSetType> dsType = layer->getSchema();

  if (!dsType->hasGeom())
    return;

  te::gm::Envelope reprojectedEnvelope(e);

  if ((layer->getSRID() != TE_UNKNOWN_SRS) && (m_display->getSRID() != TE_UNKNOWN_SRS) && (layer->getSRID() != m_display->getSRID()))
    reprojectedEnvelope.transform(m_display->getSRID(), layer->getSRID());

  // Clear previous selection?
  if (!m_keepPreviousSelection)
  {
    layer->clearSelected();

    te::qt::af::evt::LayerSelectedObjectsChanged evt(layer);
    emit triggered(&evt);
  }

  if (!reprojectedEnvelope.intersects(layer->getExtent()))
    return;

  try
  {
    // Gets the layer schema
    std::auto_ptr<const te::map::LayerSchema> schema(layer->getSchema());
    if (!schema->hasGeom())
      return;

    te::gm::GeometryProperty* gp = te::da::GetFirstGeomProperty(schema.get());

    // Gets the dataset
    std::auto_ptr<te::da::DataSet> dataset = layer->getData(gp->getName(), &reprojectedEnvelope, te::gm::INTERSECTS);
    assert(dataset.get());

    // Let's generate the oids
    te::da::ObjectIdSet* oids = 0;
    te::da::GetEmptyOIDSet(schema.get(), oids);
    assert(oids);

    std::vector<std::string> pnames;
    te::da::GetOIDPropertyNames(schema.get(), pnames);

    // Generates a geometry from the given extent. It will be used to refine the results
    std::auto_ptr<te::gm::Geometry> geometryFromEnvelope(te::gm::GetGeomFromEnvelope(&reprojectedEnvelope, layer->getSRID()));

    if (m_selectionByPointing == false)
    {
      while (dataset->moveNext())
      {
        std::auto_ptr<te::gm::Geometry> g(dataset->getGeometry(gp->getName()));

        if (g->getSRID() == TE_UNKNOWN_SRS)
          g->setSRID(layer->getSRID());

        if (m_geom->getSRID() != g->getSRID())
          m_geom->setSRID(g->getSRID());

        if (!g->intersects(m_geom))
          continue;

        // Feature found!
        oids->add(te::da::GenerateOID(dataset.get(), pnames));
      }

      oids->setExpressionByInClause();
    }
    else
    {
      // The restriction point. It will be used to refine the results
      te::gm::Coord2D center = reprojectedEnvelope.getCenter();
      te::gm::Point point(center.x, center.y, layer->getSRID());

      while (dataset->moveNext())
      {
        std::auto_ptr<te::gm::Geometry> g(dataset->getGeometry(gp->getName()));

        if (g->getSRID() == TE_UNKNOWN_SRS)
          g->setSRID(layer->getSRID());

        if (g->contains(&point) || g->crosses(geometryFromEnvelope.get()) || geometryFromEnvelope->contains(g.get()))
        {
          // Feature found!
          oids->add(te::da::GenerateOID(dataset.get(), pnames));
        }
      }

      oids->setExpressionByInClause();
    }

    assert(oids);

    // Adjusts the layer selection
    const te::da::ObjectIdSet* currentOids = layer->getSelected();
    if (currentOids == 0 || !m_keepPreviousSelection)
    {
      layer->select(oids);
      te::qt::af::evt::LayerSelectedObjectsChanged evt(layer);
      emit triggered(&evt);

      return;
    }

    te::da::ObjectIdSet* finalSelected = currentOids->clone();
    finalSelected->symDifference(oids);

    delete oids;

    layer->clearSelected();

    layer->select(finalSelected);

    te::qt::af::evt::LayerSelectedObjectsChanged evt(layer);
    emit triggered(&evt);

    return;
  }
  catch (std::exception& e)
  {
    QMessageBox::critical(m_display, tr("Error"), QString(tr("The selection cannot be retrieved from the layer. Details:") + " %1.").arg(e.what()));
    // TODO: catch the exceptions...
  }
}

te::gm::Envelope ta::edition::qt::tools::SelectAreaTool::buildEnvelope(const QPointF& pos)
{
  QPointF pixelOffset(4.0, 4.0);

  QRectF rect(pos - pixelOffset, pos + pixelOffset);

  // Converts rect boundary to world coordinates
  QPointF ll(rect.left(), rect.bottom());
  QPointF ur(rect.right(), rect.top());
  ll = m_display->transform(ll);
  ur = m_display->transform(ur);

  te::gm::Envelope env(ll.x(), ll.y(), ur.x(), ur.y());

  return env;
}
