/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

// TerraLib
#include <terralib/dataaccess/dataset/ObjectId.h>
#include <terralib/dataaccess/dataset/ObjectIdSet.h>
#include <terralib/edit/Utils.h>
#include <terralib/edit/qt/Renderer.h>
#include <terralib/edit/qt/Utils.h>
#include <terralib/geometry/Envelope.h>
#include <terralib/geometry/Geometry.h>
#include <terralib/maptools/AbstractLayer.h>
#include <terralib/qt/widgets/canvas/MapDisplay.h>

// TerraAmazon
#include "SelectCellTool.h"

// Qt
#include <QMessageBox>

ta::edition::qt::tools::SelectCellTool::SelectCellTool(te::qt::widgets::MapDisplay* display, const te::map::AbstractLayerPtr& layer, 
                                                       const te::map::AbstractLayerPtr& cellLayer, std::vector<ta::model::Cell*> cells, const QCursor& cursor, QObject* parent)
: GeometriesUpdateTool(display, layer.get() , parent),
  m_lockedCells(cells),
  m_cellLayer(cellLayer.get())
{
  setCursor(cursor);

  emit layerSelectedCellsChanged(m_lockedCells, SELECTEDCELLSTOOL);
}

ta::edition::qt::tools::SelectCellTool::~SelectCellTool()
{
  emit layerSelectedCellsChanged(m_lockedCells, MYSELECTEDCELLS);

  m_lockedCells.clear(); 
}

bool ta::edition::qt::tools::SelectCellTool::mousePressEvent(QMouseEvent* e)
{
  if (e->button() != Qt::LeftButton)
    return false;

  if (!m_cellLayer->isValid())
    return false;

  if (m_cellLayer->getVisibility() != te::map::VISIBLE)
  {
    QMessageBox::critical(m_display, tr("Error"), QString(tr("The cell layer is not checked!")));
    return false;
  }

  pickFeature(te::edit::GetPosition(e));

  return true;
}

bool ta::edition::qt::tools::SelectCellTool::mouseReleaseEvent(QMouseEvent* e)
{
  te::gm::Envelope env = buildEnvelope(te::edit::GetPosition(e));

  try
  {
    if (m_feature)
      locateCell();
  }
  catch (std::exception& e)
  {
    QMessageBox::critical(m_display, tr("Error"), QString(tr("The geometry cannot be selected from the layer. Details:") + " %1.").arg(e.what()));
  }

  return true;
}

void ta::edition::qt::tools::SelectCellTool::locateCell()
{
  try
  {
    std::size_t operation = 1;
    emit endSelectCell(m_feature->getId()->getValueAsString(), m_lockedCells.size(), operation);

    if (operation ==  1)
    {
      ta::model::Cell* cell = new ta::model::Cell();
      cell->setOid(m_feature->getId()->getValueAsString());
      cell->setCell(*m_feature->getGeometry());

      m_lockedCells.push_back(cell);
    }
    else if (operation == 2)
    {
        for (std::size_t c = 0; c < m_lockedCells.size(); c++)
        {
          if (m_lockedCells.at(c)->getOid() == m_feature->getId()->getValueAsString())
          {
            m_lockedCells.erase(m_lockedCells.begin() + c);
            break;
          }
        }
    }

    emit layerSelectedCellsChanged(m_lockedCells, SELECTEDCELLSTOOL);
  }
  catch (std::exception& e)
  {
    QMessageBox::critical(m_display, tr("Error"), QString(tr("The selection cannot be retrieved from the layer. Details:") + " %1.").arg(e.what()));
  }
}

void ta::edition::qt::tools::SelectCellTool::pickFeature(const QPointF& pos)
{
  delete m_feature;
  m_feature = 0;

  te::gm::Envelope env = buildEnvelope(pos);

  try
  {
    m_feature = PickFeature(m_cellLayer, env, m_display->getSRID(), te::edit::GEOMETRY_UPDATE);

  }
  catch (std::exception& e)
  {
    QMessageBox::critical(m_display, tr("Error"), QString(tr("The geometry cannot be selected from the layer. Details:") + " %1.").arg(e.what()));
  }
}

te::gm::Envelope ta::edition::qt::tools::SelectCellTool::buildEnvelope(const QPointF& pos)
{
  QPointF pixelOffset(4.0, 4.0);

  QRectF rect(pos - pixelOffset, pos + pixelOffset);

  // Converts rect boundary to world coordinates
  QPointF ll(rect.left(), rect.bottom());
  QPointF ur(rect.right(), rect.top());
  ll = m_display->transform(ll);
  ur = m_display->transform(ur);

  te::gm::Envelope env(ll.x(), ll.y(), ur.x(), ur.y());

  return env;
}

void ta::edition::qt::tools::SelectCellTool::clearSelectedCells()
{
  m_lockedCells.clear();
}

void ta::edition::qt::tools::SelectCellTool::setSelectedCells(std::vector<ta::model::Cell*> cells)
{
  if (m_lockedCells.size()==0)
    m_lockedCells = cells;
}