/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

// TerraLib
#include <terralib/common/Exception.h>
#include <terralib/common/Translator.h>
#include <terralib/dataaccess/utils/Utils.h>
#include <terralib/geometry/GeometryCollection.h>

// TerraAmazon
#include "../../../model/Enums.h"
#include "../../../model/facade/CatalogFacade.h"
#include "../../../model/facade/CellFacade.h"
#include "../../../model/facade/ProcessFacade.h"
#include "../../../model/facade/ProjectFacade.h"
#include "../../../model/facade/RuleFacade.h"
#include "../../../model/facade/RuleRuleSetFacade.h"
#include "../../../model/facade/RuleSetFacade.h"
#include "../../../model/facade/RulesLayerColumnFacade.h"
#include "../../../model/facade/SceneFacade.h"
#include "../../../model/facade/TaskFacade.h"
#include "../../../model/item/Catalog.h"
#include "../../../model/item/Scene.h"
#include "../../../model/item/Process.h"
#include "../../../model/item/Rule.h"
#include "../../../model/item/RuleSet.h"
#include "../../../model/item/RulesLayerColumn.h"
#include "../../../model/item/Task.h"
#include "Utils.h"

te::da::Select ta::edition::qt::tools::GetCellQuery(const std::string& catalogName,
                                                    const std::string& columnName,
                                                    const std::vector<te::gm::Geometry*>& cellVec)
{
  te::da::Fields* fields = new te::da::Fields;
  te::da::Field* f_all = new te::da::Field("*");
  fields->push_back(f_all);

  te::da::From* from = new te::da::From;
  te::da::FromItem* f_item = new te::da::DataSetName(catalogName);
  from->push_back(f_item);

  te::da::In* in = new te::da::In(columnName);
  for (std::size_t i = 0; i < cellVec.size(); ++i)
  {
    te::da::Literal* value = new te::da::LiteralGeom(*cellVec[i]);
    in->add(value);
  }

  te::da::Where* where = new te::da::Where(in);

  return te::da::Select(fields, from, where);
}

ta::edition::qt::tools::RulesInfo ta::edition::qt::tools::CreateImplicitRule(const int& Id,
                                                                            const std::string& name,
                                                                            const int& catalogId,
                                                                            const int& operationId,
                                                                            const std::string columnIn,
                                                                            const std::string columnOut)
{
  ta::model::Rule* rule = new ta::model::Rule();
  rule->setId(Id);
  rule->setName(name);
  rule->setCatalogId(catalogId);
  rule->setOperationId(operationId);

  RulesInfo ruleInfo;
  ruleInfo.m_id = Id;
  ruleInfo.m_rule = rule;

  if (!columnIn.empty() && !columnOut.empty())
  {
    ta::model::RulesLayerColumn* ruleCellColumn = new ta::model::RulesLayerColumn();
    
    ruleCellColumn->setColumnIn(columnIn);
    ruleCellColumn->setColumnOut(columnOut);
    
    ruleInfo.m_columns.push_back(ruleCellColumn);
  }

  return ruleInfo;
}

std::vector<ta::edition::qt::tools::RulesInfo> ta::edition::qt::tools::GetRules(te::da::DataSourcePtr server,
                                                                                const ta::model::RuleSet& ruleSet,
                                                                                const bool& clean)
{
  std::vector<ta::edition::qt::tools::RulesInfo> rulesInfo;

  ta::model::RuleRuleSetFacade rule_ruleSetFacade(server);
  std::vector<int>rulesIds = rule_ruleSetFacade.getRules(ruleSet.getId());

  for (std::size_t i = 0; i < rulesIds.size(); ++i)
  {
    ta::model::RuleFacade ruleFacade(server);
    std::auto_ptr<ta::model::Rule> rule(dynamic_cast<ta::model::Rule*>(ruleFacade.get(rulesIds[i])));
    if (rule.get())
    {
      int op = rule->getOperationId();

      if (clean)
      {
        if (op != ta::model::SO_CLEAN)
          continue;
      }
      else
      {
        if (op == ta::model::SO_CLEAN)
          continue;
      }

      RulesInfo currInfo;
      currInfo.m_id = rule->getId();
      currInfo.m_rule = rule.release();

      if (op == ta::model::SO_INTERSECTION)
      {
        std::vector<ta::model::RulesLayerColumn*> rulesColumnVec;

        ta::model::RulesLayerColumnFacade rulesLayerColumnFacade(server);
        rulesColumnVec = rulesLayerColumnFacade.getByRuleSetIdAndRuleId(ruleSet.getId(), rule->getId());

        currInfo.m_columns = rulesColumnVec;
      }

      rulesInfo.push_back(currInfo);
    }

  }
  return rulesInfo;
}

std::vector<ta::edition::qt::tools::RulesInfo> ta::edition::qt::tools::GetRules(te::da::DataSourcePtr server, 
                                                                                const int& projectId, 
                                                                                const int& processId, 
                                                                                const int& ruleSetId,
                                                                                const bool& clean)
{
  std::vector<ta::edition::qt::tools::RulesInfo> infos;

  // GET RULESET LAYER
  ta::model::ProjectFacade projectFacade(server);
  ta::model::RuleSetFacade ruleSetFacade(server);

  std::vector<int> ruleSets = projectFacade.getRuleSets(projectId);
  ta::model::RuleSet* ruleSet = 0;
  if (!ruleSets.empty())
    ruleSet = dynamic_cast<ta::model::RuleSet*>(ruleSetFacade.get(ruleSets[ruleSetId]));

  if (!ruleSet)
    te::common::Exception(TE_TR("RuleSet not found!"));

  int ruleSetCatalogId = ruleSet->getCatalogId();

  ta::model::CatalogFacade catalogFacade(server);
  std::auto_ptr<ta::model::Catalog> outputCatalog(dynamic_cast<ta::model::Catalog*>(catalogFacade.get(ruleSetCatalogId)));

  if (!outputCatalog.get())
    te::common::Exception(TE_TR("Catalog not found!"));

  // GET ACTIVE CELL 
  ta::model::CellFacade cellFacade(server);

  if (!cellFacade.loadLayer())
    te::common::Exception(TE_TR("Error loading cell layer!\nCheck one!"));

  std::string cellLayerName = cellFacade.getLayerName();

  std::auto_ptr<te::da::DataSetType> dsTypeCell(te::da::GetDataSetType(cellLayerName, server->getId()));
  
  std::vector<te::gm::Geometry*> vecCell;

  bool successCells = cellFacade.getSelectedCells(processId, vecCell);
  if (!successCells)
    te::common::Exception(TE_TR("Error while geting selected cells."));

  if (vecCell.empty())
    te::common::Exception(TE_TR("Select at least one cell."));

  std::auto_ptr<ta::model::Catalog> cellCatalog(dynamic_cast<ta::model::Catalog*>(catalogFacade.get(catalogFacade.getActiveCellLayer())));
  if (!cellCatalog.get())
    te::common::Exception(TE_TR("Error while geting cell catalog."));


  // GET TASK/SCENE/AREA OF INTEREST
  ta::model::ProcessFacade processFacade(server);
  std::auto_ptr<ta::model::Process> process(dynamic_cast<ta::model::Process*>(processFacade.get(processId)));
  if (!process.get())
    te::common::Exception(TE_TR("Process not found!"));

  int taskId = process->getTaskId();
  ta::model::TaskFacade taskFacade(server);
  std::auto_ptr<ta::model::Task> task(dynamic_cast<ta::model::Task*>(taskFacade.get(taskId)));
  if (!task.get())
    te::common::Exception(TE_TR("Task not found!"));

  std::auto_ptr<ta::model::Scene> scene(taskFacade.getScene(task.get()));
  if (!scene.get())
    te::common::Exception(TE_TR("Scene not found!"));

  ta::model::SceneFacade sceneFacade(server);
  int sceneLayerId = scene->getAoiLayerId();
  std::auto_ptr<ta::model::AoiLayer> aoiLayer(sceneFacade.getAoiLayer(sceneLayerId));

  std::vector<std::string> aiObjectNameVec;
  aiObjectNameVec.push_back(scene->getAiObjectId());


  std::auto_ptr<ta::model::Catalog> sceneCatalog(dynamic_cast<ta::model::Catalog*>(catalogFacade.get(aoiLayer->getCatalogId())));
  if (!sceneCatalog.get())
    te::common::Exception(TE_TR("Error while geting scene catalog."));
  
  // SET INPLICIT RULES
  // CELL RULE
  ta::model::Rule* ruleCELL = new ta::model::Rule();
  ruleCELL->setId(-2);
  ruleCELL->setName("INTER_CELL");
  ruleCELL->setCatalogId(catalogFacade.getActiveCellLayer());
  ruleCELL->setOperationId(ta::model::SO_INTERSECTION);

  std::string inCellColumn = cellCatalog->getCellColumn();
  std::string outCellColumn = outputCatalog->getCellColumn();

  ta::model::RulesLayerColumn* ruleCellColumn = new ta::model::RulesLayerColumn();
  ruleCellColumn->setColumnIn(inCellColumn);
  ruleCellColumn->setColumnOut(outCellColumn);

  RulesInfo cellInfo;
  cellInfo.m_id = -2;
  cellInfo.m_rule = ruleCELL;
  cellInfo.m_columns.push_back(ruleCellColumn);

  infos.push_back(cellInfo);


  // AOI RULE
  ta::model::Rule* ruleAoI = new ta::model::Rule();
  ruleAoI->setId(-1);
  ruleAoI->setName("INTER_AI");
  ruleAoI->setCatalogId(sceneCatalog->getId());
  ruleAoI->setOperationId(ta::model::SO_INTERSECTION);

  RulesInfo aoiInfo;
  aoiInfo.m_id = -1;
  aoiInfo.m_rule = ruleAoI;

  infos.push_back(aoiInfo);


  // GET RULES
  ta::model::RuleRuleSetFacade rule_ruleSetFacade(server);
  std::vector<int>rulesIds = rule_ruleSetFacade.getRules(ruleSet->getId());

  for (std::size_t i = 0; i < rulesIds.size(); ++i)
  {
    ta::model::RuleFacade ruleFacade(server);
    std::auto_ptr<ta::model::Rule> rule(dynamic_cast<ta::model::Rule*>(ruleFacade.get(rulesIds[i])));
    if (rule.get())
    {
      int op = rule->getOperationId();

      if (clean)
      {
        if (op != ta::model::SO_CLEAN)
          continue;
      }
      else
      {
        if (op == ta::model::SO_CLEAN)
          continue;
      }

      RulesInfo currInfo;
      currInfo.m_id = rule->getId();
      currInfo.m_rule = rule.release();

      if (op == ta::model::SO_INTERSECTION)
      {
        std::vector<ta::model::RulesLayerColumn*> rulesColumnVec;

        ta::model::RulesLayerColumnFacade rulesLayerColumnFacade(server);
        rulesColumnVec = rulesLayerColumnFacade.getByRuleSetIdAndRuleId(ruleSet->getId(), rule->getId());

        currInfo.m_columns = rulesColumnVec;
      }

      infos.push_back(currInfo);
    }

  }

  return infos;
}

te::da::Select ta::edition::qt::tools::GetRestrictionQuery(const std::string& catalogName,
                                                           const std::string& columnName,
                                                           const std::vector<std::string>& aiObjectName)
{
  te::da::Fields* fields = new te::da::Fields;
  te::da::Field* f_all = new te::da::Field("*");
  fields->push_back(f_all);

  te::da::From* from = new te::da::From;
  te::da::FromItem* f_item = new te::da::DataSetName(catalogName);
  from->push_back(f_item);

  te::da::In* in = new te::da::In(columnName);
  for (std::size_t i = 0; i < aiObjectName.size(); ++i)
  {
    te::da::Literal* value = new te::da::LiteralString(aiObjectName[i]);
    in->add(value);
  }

  te::da::Where* where = new te::da::Where(in);

  return te::da::Select(fields, from, where);
}

te::gm::Polygon* ta::edition::qt::tools::GetValidPolygon(te::gm::Polygon* gm)
{
  te::gm::Polygon* pol = new te::gm::Polygon(0, te::gm::PolygonType);

  for (size_t k = 0; k < gm->getNumRings(); k++)
  {
    te::gm::LineString* c = (te::gm::LineString*)gm->getRingN(k);

    size_t ct = c->getNPoints();

    te::gm::LineString* l = new te::gm::LinearRing(ct, te::gm::LineStringType);

    for (size_t i = 0; i < ct; i++)
    {
      std::auto_ptr<te::gm::Point> pt(c->getPointN(i));
      l->setPoint(i, pt->getX(), pt->getY());
    }

    pol->add(l);
  }

  return pol;
}