/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of
    this application.
    */

/*!
\file terraAmazon/model/Utils.h

\brief Utility routines for the TerraAmazon Model module.
*/

// TerraAmazon
#include "common/Utils.h"
#include "model/facade/AoiLayerFacade.h"
#include "model/facade/CatalogFacade.h"
#include "model/facade/ClassFacade.h"
#include "model/facade/ProjectFacade.h"
#include "model/facade/RuleFacade.h"
#include "model/facade/RuleSetFacade.h"
#include "model/item/AoiLayer.h"
#include "model/item/Catalog.h"
#include "model/item/Class.h"
#include "model/item/Project.h"
#include "model/item/Rule.h"
#include "model/item/User.h"
#include "Enums.h"
#include "Globals.h"
#include "Utils.h"

// TerraLib
#include <terralib/common/STLUtils.h>
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceFactory.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>
#include <terralib/dataaccess/datasource/DataSourceInfoManager.h>
#include <terralib/dataaccess/query/And.h>
#include <terralib/dataaccess/query/BinaryFunction.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/LiteralString.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/dataaccess/utils/Utils.h>
#include <terralib/datatype/SimpleProperty.h>
#include <terralib/geometry/GeometryProperty.h>
#include <terralib/maptools/Grouping.h>
#include <terralib/maptools/GroupingItem.h>
#include <terralib/maptools/Utils.h>
#include <terralib/qt/widgets/layer/utils/DataSet2Layer.h>
#include <terralib/se/Utils.h>

// Boost
#include <boost/filesystem.hpp>
#include <boost/uuid/random_generator.hpp>
#include <boost/uuid/uuid_io.hpp>

std::string ta::model::GetSpatialOperationPrefix(const int& soEnum)
{
  std::string prefix = "";

  switch (soEnum)
  {
    case ta::model::SO_DIFFERENCE:
      prefix = "DIFF";
      break;
    case ta::model::SO_INTERSECTION:
      prefix = "INTER";
      break;
    case ta::model::SO_UNION:
      prefix = "UNION";
      break;
    case ta::model::SO_CLEAN:
      prefix = "CLEAN";
      break;
    default:
      break;
  }

  return prefix;
}

std::string ta::model::GetSpatialOperationString(const int& soEnum)
{
  std::string name = "";

  switch (soEnum)
  {
  case ta::model::SO_DIFFERENCE:
    name = ta::model::Globals::sm_soDifference;
    break;
  case ta::model::SO_INTERSECTION:
    name = ta::model::Globals::sm_soIntersection;
    break;
  case ta::model::SO_UNION:
    name = ta::model::Globals::sm_soUnion;
    break;
  case ta::model::SO_CLEAN:
    name = ta::model::Globals::sm_soClean;
    break;
  default:
    break;
  }

  return name;
}

std::list<te::map::AbstractLayerPtr> ta::model::GetProjectLayers(te::da::DataSourceInfoPtr dsInfo, const int& projId)
{
  std::list<te::map::AbstractLayerPtr> list;

  //get server
  te::da::DataSourcePtr ds = te::da::GetDataSource(dsInfo->getId());

  ta::model::ProjectFacade pf(ds);
  ta::model::RuleSetFacade rsf(ds);
  ta::model::RuleFacade rf(ds);
  ta::model::CatalogFacade cf(ds);

  //get rules from project
  std::vector<int> rulesIds = pf.getRules(projId);

  std::set<int> layers;

  for (std::size_t t = 0; t < rulesIds.size(); ++t)
  {
    //get rule
    ta::model::Rule* r = dynamic_cast<ta::model::Rule*>(rf.get(rulesIds[t]));

    int catalogId = r->getCatalogId();

    if (layers.find(catalogId) != layers.end())
      continue;

    layers.insert(catalogId);

    //get catalog
    ta::model::Catalog* c = dynamic_cast<ta::model::Catalog*>(cf.get(catalogId));

    std::string driverName = c->getType();
    std::string connInfoStr = c->getUri();
    std::string dataSetName = c->getName();
    std::string styleStr = c->getStyle();

    te::da::DataSourceInfoPtr catalogDsInfo = te::da::DataSourceInfoManager::getInstance().getByConnInfo(connInfoStr);

    if (!catalogDsInfo.get())
    {
      static boost::uuids::basic_random_generator<boost::mt19937> gen;
      boost::uuids::uuid valU = gen();
      std::string id = boost::uuids::to_string(valU);

      catalogDsInfo.reset(new te::da::DataSourceInfo);
      catalogDsInfo->setConnInfoFromString(connInfoStr);
      catalogDsInfo->setId(id);
      catalogDsInfo->setTitle(dataSetName);
      catalogDsInfo->setAccessDriver(driverName);
      catalogDsInfo->setType(driverName);

      te::da::DataSourceInfoManager::getInstance().add(catalogDsInfo);
    }

    te::da::DataSourcePtr dsPtr = te::da::GetDataSource(catalogDsInfo->getId());

    if (!dsPtr.get())
    {
      dsPtr.reset(te::da::DataSourceFactory::make(driverName).release());
      dsPtr->setConnectionInfo(catalogDsInfo->getConnInfo());
      dsPtr->open();
      dsPtr->setId(catalogDsInfo->getId());

      te::da::DataSourceManager::getInstance().insert(dsPtr);
    }

    //create layer
    te::da::DataSetTypePtr dsType(dsPtr->getDataSetType(dataSetName).release());

    te::qt::widgets::DataSet2Layer ds2l(dsPtr->getId());

    te::map::AbstractLayerPtr layer = ds2l(dsType);

    if (!styleStr.empty())
    {
      std::unique_ptr<te::se::Style> style = ta::common::Read(styleStr);
      layer->setStyle(style.release());
    }

    //generate legend
    std::vector<int> classes = rsf.getRuleSetClasses(catalogId);

    if (!classes.empty())
    {
      CreateOutputLayerLegend(ds, layer, classes);
    }

    list.push_back(layer);

    delete c;
    delete r;
  }

  return list;
}

std::list<te::map::AbstractLayerPtr> ta::model::GetServerAOILayers(te::da::DataSourceInfoPtr dsInfo)
{
  std::list<te::map::AbstractLayerPtr> list;

  te::da::DataSourcePtr ds = te::da::GetDataSource(dsInfo->getId());

  ta::model::CatalogFacade cf(ds);
  ta::model::AoiLayerFacade aoif(ds);

  std::vector<ta::model::AbstractItem*> aoiItems = aoif.get();

  for (std::size_t t = 0; t < aoiItems.size(); ++t)
  {
    ta::model::AoiLayer* aoiLayer = dynamic_cast<ta::model::AoiLayer*>(aoiItems[t]);

    std::auto_ptr<ta::model::Catalog> catalog(dynamic_cast<ta::model::Catalog*>(cf.get(aoiLayer->getCatalogId())));

    std::auto_ptr<te::da::DataSetType> dst = cf.getDataSetType(catalog.get());

    te::qt::widgets::DataSet2Layer converter(ds->getId());

    te::da::DataSetTypePtr dt(dst.release());

    te::map::AbstractLayerPtr layer = converter(dt);

    if (layer.get())
    {
      te::se::Style* s = GetCatalogLayerStyle(aoiLayer->getCatalogId(), ds);

      layer->setStyle(s);
    }

    list.push_back(layer);
  }

  //clear aoiItems
  te::common::FreeContents(aoiItems);
  aoiItems.clear();

  return list;
}

te::map::AbstractLayerPtr ta::model::GetServerCellLayer(te::da::DataSourceInfoPtr dsInfo)
{
  te::da::DataSourcePtr ds = te::da::GetDataSource(dsInfo->getId());

  ta::model::CatalogFacade cf(ds);

  int cellLayerId = cf.getActiveCellLayer();

  std::auto_ptr<ta::model::Catalog> catalog(dynamic_cast<ta::model::Catalog*>(cf.get(cellLayerId)));

  std::auto_ptr<te::da::DataSetType> dst = cf.getDataSetType(catalog.get());

  te::qt::widgets::DataSet2Layer converter(ds->getId());

  te::da::DataSetTypePtr dt(dst.release());

  te::map::AbstractLayerPtr layer = converter(dt);

  if (layer.get())
  {
    te::se::Style* s = GetCatalogLayerStyle(cellLayerId, ds);

    layer->setStyle(s);
  }

  return layer;
}

int ta::model::GetServerCellLayerSRID(te::da::DataSourceInfoPtr dsInfo)
{
  te::da::DataSourcePtr ds = te::da::GetDataSource(dsInfo->getId());

  ta::model::CatalogFacade cf(ds);

  int cellLayerId = cf.getActiveCellLayer();

  std::auto_ptr<ta::model::Catalog> catalog(dynamic_cast<ta::model::Catalog*>(cf.get(cellLayerId)));

  std::auto_ptr<te::da::DataSetType> dst = cf.getDataSetType(catalog.get());

  te::gm::GeometryProperty* geomProp = te::da::GetFirstGeomProperty(dst.get());
  assert(geomProp);

  return geomProp->getSRID();
}

te::map::FolderLayerPtr ta::model::CreateProjectFolderLayer(const std::string& name, te::da::DataSourceInfoPtr dsInfo, const int& projId)
{
  //generate id
  static boost::uuids::basic_random_generator<boost::mt19937> gen;
  boost::uuids::uuid valU = gen();
  std::string id = boost::uuids::to_string(valU);

  te::map::FolderLayerPtr folder(new te::map::FolderLayer(id, name));

  //create layers
  std::list<te::map::AbstractLayerPtr> layers = GetProjectLayers(dsInfo, projId);

  std::list<te::map::AbstractLayerPtr>::iterator it;

  for (it = layers.begin(); it != layers.end(); ++it)
    folder->add(*it);

  //get cell layer
  te::map::AbstractLayerPtr cellLayer = GetServerCellLayer(dsInfo);
  folder->add(cellLayer);

  //get AOI layers
  std::list<te::map::AbstractLayerPtr> aoiLayers = GetServerAOILayers(dsInfo);

  for (it = aoiLayers.begin(); it != aoiLayers.end(); ++it)
    folder->add(*it);


  return folder;
}

te::map::AbstractLayerPtr ta::model::CreateDraftLayer(const std::string& fileName, const std::string& layerName, const int& srid)
{
  te::map::AbstractLayerPtr layer;

  //create data source info
  std::string outputdataset = layerName;
  std::size_t idx = outputdataset.find(".");
  if (idx != std::string::npos)
    outputdataset = outputdataset.substr(0, idx);

  std::map<std::string, std::string> connInfo;
  connInfo["URI"] = fileName;


  //create dataSetType
  std::auto_ptr<te::da::DataSetType> dsType(new te::da::DataSetType(layerName));

  te::dt::SimpleProperty* idProperty = new te::dt::SimpleProperty("id", te::dt::INT32_TYPE);
  dsType->add(idProperty);

  te::gm::GeometryProperty* geomProperty = new te::gm::GeometryProperty("geom", srid, te::gm::PolygonType);
  dsType->add(geomProperty);

  std::string pkName = "pk_id";
  pkName += "_" + layerName;
  te::da::PrimaryKey* pk = new te::da::PrimaryKey(pkName, dsType.get());
  pk->add(idProperty);

  {
    std::auto_ptr<te::da::DataSource> ds = te::da::DataSourceFactory::make("OGR");
    ds->setConnectionInfo(connInfo);
    ds->open();

    std::map<std::string, std::string> nopt;

    try
    {
      ds->createDataSet(dsType.get(), nopt);
    }
    catch (...)
    {
      return layer;
    }
  }


  // let's include the new datasource in the managers
  boost::uuids::basic_random_generator<boost::mt19937> gen;
  boost::uuids::uuid u = gen();
  std::string id = boost::uuids::to_string(u);

  te::da::DataSourceInfoPtr dsInfo(new te::da::DataSourceInfo);
  dsInfo->setConnInfo(connInfo);
  dsInfo->setTitle(layerName);
  dsInfo->setAccessDriver("OGR");
  dsInfo->setType("OGR");
  dsInfo->setDescription(layerName);
  dsInfo->setId(id);

  te::da::DataSourceInfoManager::getInstance().add(dsInfo);

  te::da::DataSourcePtr dsPtr = te::da::GetDataSource(id);

  //create layer
  te::qt::widgets::DataSet2Layer converter(id);

  te::da::DataSetTypePtr dt(dsType.release());

  layer = converter(dt);

  return layer;
}

te::se::Style* ta::model::GetCatalogLayerStyle(const int& catalogId, te::da::DataSourcePtr ds)
{
  ta::model::CatalogFacade cf(ds);

  std::auto_ptr<ta::model::Catalog> c(dynamic_cast<ta::model::Catalog*>(cf.get(catalogId)));

  std::string styleStr = c->getStyle();

  if (styleStr.empty())
    return 0;

  std::unique_ptr<te::se::Style> style = ta::common::Read(styleStr);

  return style.release();
}

void ta::model::CreateOutputLayerLegend(te::da::DataSourcePtr ds, te::map::AbstractLayerPtr layer, const std::vector<int>& classes)
{
  std::string groupingName = "class_name";
  int attrType = te::dt::STRING_TYPE;
  int prec = 0;
  int geomType = te::map::GetGeomType(layer);

  //create grouping items
  ta::model::ClassFacade cf(ds);

  std::vector<te::map::GroupingItem*> legend;

  for (std::size_t t = 0; t < classes.size(); ++t)
  {
    std::auto_ptr<ta::model::Class> c(dynamic_cast<ta::model::Class*>(cf.get(classes[t])));

    std::vector<te::se::Symbolizer*> symbVec;

    te::se::Symbolizer* s = te::se::CreateSymbolizer((te::gm::GeomType)geomType, c->getStyle());

    symbVec.push_back(s);

    te::map::GroupingItem* legendItem = new te::map::GroupingItem;
    legendItem->setValue(c->getName());
    legendItem->setCount(1);
    legendItem->setSymbolizers(symbVec);
    legend.push_back(legendItem);
  }

  //create grouping
  te::map::Grouping* group = new te::map::Grouping(groupingName, te::map::UNIQUE_VALUE);
  group->setPropertyType(attrType);
  group->setPrecision(prec);
  group->setStdDeviation(0.);
  group->setGroupingItems(legend);

  layer->setGrouping(group);
}

bool ta::model::HasTerraAmazonUser(te::da::DataSourcePtr ds)
{
  bool isTaUser = false;

  try
  {
    te::da::Field* field = new te::da::Field("*");
    te::da::Fields* fields = new te::da::Fields;
    fields->push_back(field);

    te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_user);
    te::da::From* from = new te::da::From;
    from->push_back(fromItem);

    te::da::PropertyName* pName = new te::da::PropertyName(ta::model::User::sm_name);

    te::da::Literal* literalName = new te::da::LiteralString(ta::model::Globals::sm_taUser);

    te::da::EqualTo* equalName = new te::da::EqualTo(pName, literalName);

    te::da::PropertyName* pPass = new te::da::PropertyName(ta::model::User::sm_password);

    te::da::Literal* literalPass = new te::da::LiteralString(sha256(ta::model::Globals::sm_taPassword));

    te::da::EqualTo* equalPass = new te::da::EqualTo(pPass, literalPass);

    te::da::And* andClause = new te::da::And(equalName, equalPass);

    te::da::Where* where = new te::da::Where(andClause);

    te::da::Select* select = new te::da::Select(fields, from, where);

    std::unique_ptr<te::da::DataSet> res = ds->query(select);

    if (!res->isEmpty())
      isTaUser = true;
  }
  catch (const std::exception& /*e*/)
  {
    return false;
  }

  return isTaUser;
}
