/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraAmazon/model/Utils.h

  \brief Utility routines for the TerraAmazon Model module.
*/

#ifndef __TERRAAMAZON_MODEL_UTILS_H
#define __TERRAAMAZON_MODEL_UTILS_H

// TerraLib
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>
#include <terralib/maptools/AbstractLayer.h>
#include <terralib/maptools/FolderLayer.h>
#include <terralib/se/Style.h>

// STL
#include <string>

namespace ta
{
  namespace model
  {
    /*
    \brief Get string prefix for a specific spatial operation

    \param[in] Spatial Operation Enum (ta::model::Enums) 

    \return String with the prefix value.
    */
    std::string GetSpatialOperationPrefix(const int& soEnum);

    /*
    \brief Get string for a specific spatial operation

    \param[in] Spatial Operation Enum (ta::model::Enums)

    \return String with the so name.
    */
    std::string GetSpatialOperationString(const int& soEnum);

    /*
    \brief Get catalog layers of a project

    \param[in] dsInfo   Information about current server
    \param[in] projId   Information about current project

    \return List with all layers.
    */
    std::list<te::map::AbstractLayerPtr> GetProjectLayers(te::da::DataSourceInfoPtr dsInfo, const int& projId);

    /*
    \brief Get servers AOI layers

    \param[in] dsInfo   Information about current server

    \return List with all layers.
    */
    std::list<te::map::AbstractLayerPtr> GetServerAOILayers(te::da::DataSourceInfoPtr dsInfo);

    /*
    \brief Get the active cell layer from server

    \param[in] dsInfo   Information about current server

    \return Cell layer.
    */
    te::map::AbstractLayerPtr GetServerCellLayer(te::da::DataSourceInfoPtr dsInfo);

    /*
    \brief Get the active cell layer srid from server

    \param[in] dsInfo   Information about current server

    \return srid from cell layer.
    */
    int GetServerCellLayerSRID(te::da::DataSourceInfoPtr dsInfo);
    
    /*
    \brief Get folder with catalog layers of a project

    \param[in] name     Folder name;
    \param[in] dsInfo   Information about current server
    \param[in] projId   Information about current project

    \return List with all layers.
    */
    te::map::FolderLayerPtr CreateProjectFolderLayer(const std::string& name, te::da::DataSourceInfoPtr dsInfo, const int& projId);

    /*
    \brief Create a draft layer as SHP dataset

    \param[in] fileName     Data source path;

    \return A draft layer.
    */
    te::map::AbstractLayerPtr CreateDraftLayer(const std::string& fileName, const std::string& layerName, const int& srid);

    /*
    \brief Get the default style associated to the catalog

    \param[in] catalogId  Catalog identification
    \param[in] ds         Server to access the catalog table

    \return Style definition or zero if the catalog does not have style.
    */
    te::se::Style* GetCatalogLayerStyle(const int& catalogId, te::da::DataSourcePtr ds);

    /*
    \brief Create a grouping legend to a output layer

    \param[in] ds         Server to access the class table
    \param[in] layer     Layer to create legend;
    \param[in] classes   The classes id to be add to layer legend

    */
    void CreateOutputLayerLegend(te::da::DataSourcePtr ds, te::map::AbstractLayerPtr layer, const std::vector<int>& classes);


    bool HasTerraAmazonUser(te::da::DataSourcePtr ds);

  } // end namespace model
}   // end namespace ta

#endif  // __TERRAAMAZON_MODEL_UTILS_H

