/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/AbstractFacade.cpp

\brief
*/

// TerraAmazon
#include "AbstractFacade.h"
#include "../factory/AbstractItemFactory.h"
#include "../item/AbstractItem.h"
#include "../item/ItemProperties.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>
#include <terralib/dataaccess/dataset/ObjectId.h>
#include <terralib/dataaccess/dataset/ObjectIdSet.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/memory/DataSet.h>
#include <terralib/memory/DataSetItem.h>


ta::model::AbstractFacade::AbstractFacade(te::da::DataSourcePtr ds) : m_dataSource(ds), m_errorMessage("")
{

}

ta::model::AbstractFacade::~AbstractFacade()
{

}

std::string ta::model::AbstractFacade::getDataSetName()
{
  return m_datasetName;
}

bool ta::model::AbstractFacade::save(ta::model::AbstractItem* item)
{
  if (!checkProperties(item))
  {
    return false;
  }

  ta::model::ItemPropertyVector props = item->toProperties();

  ta::model::ItemPropertyVector::iterator it;

  std::unique_ptr<te::da::DataSetType> newDst(new te::da::DataSetType(m_datasetName));

  for (it = props.begin(); it != props.end(); ++it)
    newDst->add((*it)->getProperty()->clone());

  te::mem::DataSet* newData = new te::mem::DataSet(newDst.get());

  te::mem::DataSetItem* dsItem = new te::mem::DataSetItem(newData);

  for (it = props.begin(); it != props.end(); ++it)
  {
    dsItem->setValue((*it)->getProperty()->getName(), (*it)->getData()->clone());
  }


  newData->add(dsItem);

  std::map<std::string, std::string> op;

  std::unique_ptr<te::da::DataSourceTransactor> transactor = m_dataSource->getTransactor();

  try
  {
    transactor->add(m_datasetName, newData, op);

    boost::int64_t id = transactor->getLastGeneratedId();

    item->setId((int)id);
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    transactor->rollBack();
    return false;
  }

  transactor->commit();

  return true;
}

bool ta::model::AbstractFacade::save(ta::model::AbstractItem* item, te::da::DataSourceTransactor* transactor)
{
  if (!checkProperties(item))
  {
    return false;
  }

  ta::model::ItemPropertyVector props = item->toProperties();

  ta::model::ItemPropertyVector::iterator it;

  std::unique_ptr<te::da::DataSetType> newDst(new te::da::DataSetType(m_datasetName));

  for (it = props.begin(); it != props.end(); ++it)
    newDst->add((*it)->getProperty()->clone());

  te::mem::DataSet* newData = new te::mem::DataSet(newDst.get());

  te::mem::DataSetItem* dsItem = new te::mem::DataSetItem(newData);

  for (it = props.begin(); it != props.end(); ++it)
  {
    dsItem->setValue((*it)->getProperty()->getName(), (*it)->getData()->clone());
  }

  newData->add(dsItem);

  std::map<std::string, std::string> op;

  try
  {
    transactor->add(m_datasetName, newData, op);

    boost::int64_t id = transactor->getLastGeneratedId();

    item->setId((int)id);
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";
    return false;
  }

  return true;
}

bool ta::model::AbstractFacade::update(ta::model::AbstractItem* item)
{
  if (!checkProperties(item))
  {
    return false;
  }

  //get original datasettype
  std::unique_ptr<te::da::DataSetType> dsType = m_dataSource->getDataSetType(m_datasetName);

  //get pos of attr used as pk
  te::da::PrimaryKey* pk = dsType->getPrimaryKey();

  size_t numProps = pk->getProperties().size();
  std::vector<std::size_t> ids;

  for (size_t i = 0; i < numProps; ++i)
  {
    std::string propName = pk->getProperties()[i]->getName();
    ids.push_back(dsType->getPropertyPosition(propName));
  }

  //create dataset memory with updated values
  ta::model::ItemPropertyVector props = item->toProperties();

  ta::model::ItemPropertyVector::iterator it;

  std::unique_ptr<te::da::DataSetType> newDst(new te::da::DataSetType(m_datasetName));

  for (it = props.begin(); it != props.end(); ++it)
    newDst->add((*it)->getProperty()->clone());

  te::mem::DataSet* newData = new te::mem::DataSet(newDst.get());

  te::mem::DataSetItem* dsItem = new te::mem::DataSetItem(newData);

  std::set<int> attrPos;

  for (it = props.begin(); it != props.end(); ++it)
  {
    attrPos.insert((int)newDst->getPropertyPosition((*it)->getProperty()->getName()));

    dsItem->setValue((*it)->getProperty()->getName(), (*it)->getData()->clone());
  }

  //get properties position that will be updated
  std::vector< std::set<int> > propsPos;

  propsPos.push_back(attrPos);

  //update
  newData->add(dsItem);

  std::unique_ptr<te::da::DataSourceTransactor> transactor = m_dataSource->getTransactor();

  try
  {
    transactor->update(m_datasetName, newData, propsPos, ids);
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    transactor->rollBack();
    return false;
  }
  
  transactor->commit();

  return true;
}

bool ta::model::AbstractFacade::remove(const int& id)
{
  if (hasDependecy(id))
  {
    return false;
  }

  //get original datasettype
  std::unique_ptr<te::da::DataSetType> dsType = m_dataSource->getDataSetType(m_datasetName);

  //get name of attr used as pk
  te::da::PrimaryKey* pk = dsType->getPrimaryKey();

  std::string firstPropName = pk->getProperties()[0]->getName();

  //create objectIdSet to remove element
  te::da::ObjectIdSet* objIdSet = new te::da::ObjectIdSet();

  te::da::ObjectId* objId = new te::da::ObjectId();
  objId->addValue(new te::dt::SimpleData<int, te::dt::INT32_TYPE>(id));

  objIdSet->add(objId);
  
  std::unique_ptr<te::da::DataSourceTransactor> transactor = m_dataSource->getTransactor();

  //remove element
  try
  {
    transactor->remove(m_datasetName, objIdSet);
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    transactor->rollBack();
    return false;
  }

  transactor->commit();

  return true;
}

ta::model::AbstractItem* ta::model::AbstractFacade::get(const int& id)
{
  std::unique_ptr<te::da::DataSetType> dsType = m_dataSource->getDataSetType(m_datasetName);

  if (!dsType.get())
    return 0;

  te::da::PrimaryKey* pk = dsType->getPrimaryKey();

  if (!pk)
    return 0;

  //CHECK
  std::string firstPropName = pk->getProperties()[0]->getName();

  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(m_datasetName);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(firstPropName);

  te::dt::AbstractData* dataId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(id));
  te::da::Literal* literal = new te::da::Literal(dataId);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::unique_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return 0;
  }

  if (dataSet->size() != 1)
  {
    return 0;
  }

  dataSet->moveFirst();

  ta::model::AbstractItem* ai = ta::model::AbstractItemFactory::make(m_datasetName, dataSet.get());

  return ai;
}

std::vector<ta::model::AbstractItem*> ta::model::AbstractFacade::get()
{
  std::vector<ta::model::AbstractItem*> vec;

  te::da::Fields fields = te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields.push_back(field);

  te::da::From from = te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(m_datasetName);
  from.push_back(fItem);

  te::da::Select select = new te::da::Select(fields, from);

  std::unique_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    ta::model::AbstractItem* ai = ta::model::AbstractItemFactory::make(m_datasetName, dataSet.get());

    vec.push_back(ai);
  }

  return vec;
}

bool ta::model::AbstractFacade::exists(const int& id)
{
  std::unique_ptr<te::da::DataSetType> dsType = m_dataSource->getDataSetType(m_datasetName);

  if (!dsType.get())
    return 0;

  te::da::PrimaryKey* pk = dsType->getPrimaryKey();

  if (!pk)
    return 0;

  //CHECK
  std::string firstPropName = pk->getProperties()[0]->getName();

  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(m_datasetName);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(firstPropName);

  te::dt::AbstractData* dataId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(id));
  te::da::Literal* literal = new te::da::Literal(dataId);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::unique_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return false;
  }

  if (dataSet->size() != 0)
  {
    return true;
  }

  return false;
}

std::string ta::model::AbstractFacade::getErrorMessage()
{
  return m_errorMessage;
}
