/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/AoiLayerFacade.cpp

\brief
*/

// TerraAmazon
#include "../factory/AbstractItemFactory.h"
#include "../item/Catalog.h"
#include "../item/Scene.h"
#include "../item/AoiLayer.h"
#include "../Globals.h"
#include "AoiLayerFacade.h"
#include "CatalogFacade.h"
#include "SceneFacade.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>
#include <terralib/dataaccess/dataset/ObjectId.h>
#include <terralib/dataaccess/dataset/ObjectIdSet.h>
#include <terralib/dataaccess/query/And.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/Join.h>
#include <terralib/dataaccess/query/JoinConditionOn.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/dataaccess/query/LiteralInt32.h>
#include <terralib/dataaccess/query/LiteralString.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/memory/DataSet.h>
#include <terralib/memory/DataSetItem.h>


ta::model::AoiLayerFacade::AoiLayerFacade(te::da::DataSourcePtr ds) : ta::model::AbstractFacade(ds)
{
  m_datasetName = ta::model::Globals::sm_aoiLayer;
}

ta::model::AoiLayerFacade::~AoiLayerFacade()
{
}

bool ta::model::AoiLayerFacade::save(ta::model::AbstractItem* item)
{
  if (!checkProperties(item))
  {
    return false;
  }

  ta::model::ItemPropertyVector props = item->toProperties();

  ta::model::ItemPropertyVector::iterator it;

  std::unique_ptr<te::da::DataSetType> newDst(new te::da::DataSetType(m_datasetName));

  for (it = props.begin(); it != props.end(); ++it)
    newDst->add((*it)->getProperty()->clone());

  te::mem::DataSet* newData = new te::mem::DataSet(newDst.get());

  te::mem::DataSetItem* dsItem = new te::mem::DataSetItem(newData);

  for (it = props.begin(); it != props.end(); ++it)
    dsItem->setValue((*it)->getProperty()->getName(), (*it)->getData()->clone());

  newData->add(dsItem);

  std::map<std::string, std::string> op;

  std::unique_ptr<te::da::DataSourceTransactor> transactor = m_dataSource->getTransactor();

  try
  {
    transactor->add(m_datasetName, newData, op);

  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    transactor->rollBack();
    return false;
  }

  transactor->commit();

  return true;
}

const std::string ta::model::AoiLayerFacade::getAoiIdColumn(const int& layerId)
{
  CatalogFacade cf(m_dataSource);
  std::unique_ptr<ta::model::Catalog> catalog(dynamic_cast<ta::model::Catalog*>(cf.get(layerId)));
  std::string column = "-";
  if (catalog.get())
  {
    //Acquiring the aoi column, if it has been defined laready

    te::da::Fields* fields = new te::da::Fields();
    te::da::Field* field = new te::da::Field(te::da::PropertyName(ta::model::AoiLayer::sm_aoiIdColumn));
    fields->push_back(field);

    te::da::From* from = new te::da::From();
    te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_aoiLayer);
    from->push_back(fItem);

    te::da::PropertyName* pName = new te::da::PropertyName(ta::model::AoiLayer::sm_catalogId);
    te::dt::AbstractData* id = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(catalog->getId()));
    te::da::Literal* literal = new te::da::Literal(id);

    te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);

    te::da::Where* where = new te::da::Where(equal);

    te::da::Select select = new te::da::Select(fields, from, where);

    std::unique_ptr<te::da::DataSet> dataSet;

    try
    {
      dataSet = m_dataSource->query(select);
    }
    catch (...)
    {
      return column;
    }

    if (dataSet->moveFirst())
      return dataSet->getString(ta::model::AoiLayer::sm_aoiIdColumn);
    else
      return column;
  }
  else
    return column;
}

/*
bool ta::model::AoiLayerFacade::remove(const int& projectId, const int& sceneId)
{
  //TODO: refactoring...

  std::string sql = "DELETE FROM ";
  sql += m_datasetName;
  sql += " WHERE ";
  sql += ta::model::AoiLayer::sm_projectId;
  sql += " = ";
  sql += te::common::Convert2String(projectId);
  sql += " AND ";
  sql += ta::model::AoiLayer::sm_sceneId;
  sql += " = ";
  sql += te::common::Convert2String(sceneId);

  //remove element
  try
  {
    m_dataSource->execute(sql);
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    return false;
  }

  return true;
}
*/

bool ta::model::AoiLayerFacade::checkProperties(ta::model::AbstractItem* /*item*/)
{
  return true;
}

bool ta::model::AoiLayerFacade::hasDependecy(const int& /*id*/)
{
  return false;
}
