/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/CatalogFacade.cpp

\brief
*/

// TerraAmazon
#include "../../common/Utils.h"
#include "../Globals.h"
#include "../item/Catalog.h"
#include "CatalogFacade.h"
#include "RuleFacade.h"
#include "RuleSetFacade.h"
#include "../factory/AbstractItemFactory.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/dataset/ObjectId.h>
#include <terralib/dataaccess/dataset/ObjectIdSet.h>
#include <terralib/dataaccess/datasource/DataSourceInfoManager.h>
#include <terralib/dataaccess/datasource/DataSourceInfo.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>
#include <terralib/dataaccess/query/And.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/Join.h>
#include <terralib/dataaccess/query/JoinConditionOn.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/dataaccess/query/LiteralBool.h>
#include <terralib/dataaccess/query/LiteralString.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/dataaccess/utils/Utils.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/StringProperty.h>
#include <terralib/datatype/TimeInstant.h>
#include <terralib/geometry/GeometryProperty.h>
#include <terralib/maptools/AbstractLayer.h>
#include <terralib/memory/DataSet.h>
#include <terralib/memory/DataSetItem.h>
#include <terralib/se/Utils.h>

// BOOST
#include <boost/uuid/random_generator.hpp>
#include <boost/uuid/uuid_io.hpp>

const std::string ta::model::CatalogFacade::sm_rulesLayerParams_catalogId("catalog_id");
const std::string ta::model::CatalogFacade::sm_rulesLayerParams_classColumn("class_column");

const std::string ta::model::CatalogFacade::sm_cellLayer_catalogId("catalog_id");
const std::string ta::model::CatalogFacade::sm_cellLayer_resolution("resolution");
const std::string ta::model::CatalogFacade::sm_cellLayer_active("active");

const std::string ta::model::CatalogFacade::sm_aoiLayer_catalogId("catalog_id");
const std::string ta::model::CatalogFacade::sm_aoiLayer_aoiIdColumn("aoi_id_column");

ta::model::CatalogFacade::CatalogFacade(te::da::DataSourcePtr ds) : ta::model::AbstractFacade(ds)
{
  m_datasetName = ta::model::Globals::sm_catalog;
}

ta::model::CatalogFacade::~CatalogFacade()
{

}

bool ta::model::CatalogFacade::saveAsCellLayer(ta::model::Catalog* catalog, const double& resolution, const bool& active)
{
  if (!save(catalog))
    return false;

  std::auto_ptr<te::da::DataSetType> newDst = m_dataSource->getDataSetType(ta::model::Globals::sm_cellLayer);

  te::mem::DataSet* newData = new te::mem::DataSet(newDst.get());

  te::mem::DataSetItem* dsItem = new te::mem::DataSetItem(newData);

  dsItem->setValue(ta::model::CatalogFacade::sm_cellLayer_catalogId, new te::dt::SimpleData<int, te::dt::INT32_TYPE>(catalog->getId()));
  dsItem->setValue(ta::model::CatalogFacade::sm_cellLayer_resolution, new te::dt::SimpleData<double, te::dt::DOUBLE_TYPE>(resolution));
  dsItem->setValue(ta::model::CatalogFacade::sm_cellLayer_active, new te::dt::SimpleData<bool, te::dt::BOOLEAN_TYPE>(active));

  newData->add(dsItem);

  std::map<std::string, std::string> op;

  std::auto_ptr<te::da::DataSourceTransactor> transactor = m_dataSource->getTransactor();

  try
  {
    transactor->add(ta::model::Globals::sm_cellLayer, newData, op);
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    transactor->rollBack();
    return false;
  }

  transactor->commit();

  return true;
}

bool ta::model::CatalogFacade::remove(const int& id)
{
  if (hasDependecy(id))
  {
    return false;
  }

  // sql delele ta_rules_layer_params
  std::string deleteParamsSql = "DELETE FROM ";
  deleteParamsSql += ta::model::Globals::sm_rulesLayerParams;
  deleteParamsSql += " WHERE ";
  deleteParamsSql += ta::model::CatalogFacade::sm_rulesLayerParams_catalogId;
  deleteParamsSql += " = ";
  deleteParamsSql += te::common::Convert2String(id);

  // sql delele ta_cell_layer
  std::string deleteCellLayer = "DELETE FROM ";
  deleteCellLayer += ta::model::Globals::sm_cellLayer;
  deleteCellLayer += " WHERE ";
  deleteCellLayer += ta::model::CatalogFacade::sm_cellLayer_catalogId;
  deleteCellLayer += " = ";
  deleteCellLayer += te::common::Convert2String(id);

  //get original datasettype
  std::unique_ptr<te::da::DataSetType> dsType = m_dataSource->getDataSetType(m_datasetName);

  //get name of attr used as pk
  te::da::PrimaryKey* pk = dsType->getPrimaryKey();

  std::string firstPropName = pk->getProperties()[0]->getName();

  //create objectIdSet to remove element
  te::da::ObjectIdSet* objIdSet = new te::da::ObjectIdSet();

  te::da::ObjectId* objId = new te::da::ObjectId();
  objId->addValue(new te::dt::SimpleData<int, te::dt::INT32_TYPE>(id));

  objIdSet->add(objId);

  std::unique_ptr<te::da::DataSourceTransactor> transactor = m_dataSource->getTransactor();

  //remove element
  try
  {
    transactor->execute(deleteParamsSql);
    transactor->execute(deleteCellLayer);
    transactor->remove(m_datasetName, objIdSet);
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    transactor->rollBack();
    return false;
  }

  transactor->commit();

  return true;
}

te::da::DataSourcePtr ta::model::CatalogFacade::getDataSource(ta::model::Catalog* catalog)
{
  te::da::DataSourcePtr dsPtr;

  try
  {
    std::string connInfo = catalog->getUri();

    te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().getByConnInfo(connInfo);

    //data source already registed
    if (dsInfo.get())
    {
      return te::da::GetDataSource(dsInfo->getId());
    }

    //register data source
    boost::uuids::basic_random_generator<boost::mt19937> gen;
    boost::uuids::uuid u = gen();
    std::string id = boost::uuids::to_string(u);

    te::da::DataSourceInfoPtr ds(new te::da::DataSourceInfo);
    ds->setConnInfoFromString(connInfo);
    ds->setTitle(catalog->getName());
    ds->setAccessDriver(catalog->getType());
    ds->setType(catalog->getType());
    ds->setDescription(catalog->getDescription());
    ds->setId(id);

    te::da::DataSourceInfoManager::getInstance().add(ds);

    dsPtr = te::da::GetDataSource(ds->getId());
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();
  }

  return dsPtr;
}

std::auto_ptr<te::da::DataSetType> ta::model::CatalogFacade::getDataSetType(ta::model::Catalog* catalog)
{
  te::da::DataSourcePtr dataSource = getDataSource(catalog);

  std::auto_ptr<te::da::DataSetType> dsType;

  try
  {
    dsType = dataSource->getDataSetType(catalog->getName());
  }
  catch (...)
  {
    return std::auto_ptr<te::da::DataSetType>();
  }

  return dsType;
}

std::auto_ptr<te::da::DataSet> ta::model::CatalogFacade::getDataSet(ta::model::Catalog* catalog)
{
  te::da::DataSourcePtr dataSource = getDataSource(catalog);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = dataSource->getDataSet(catalog->getName());
  }
  catch (...)
  {
    return std::auto_ptr<te::da::DataSet>();
  }

  return dataSet;
}

std::auto_ptr<te::da::DataSet> ta::model::CatalogFacade::getDataSet(ta::model::Catalog* catalog, std::string oid, std::string propName)
{
  te::da::DataSourcePtr dataSource = getDataSource(catalog);

  std::auto_ptr<te::da::DataSetType> dsType = dataSource->getDataSetType(catalog->getName());

  te::da::PrimaryKey* pk = dsType->getPrimaryKey();

  if (propName == "")
    propName = pk->getProperties()[0]->getName();

  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(catalog->getName());
  from->push_back(fItem);

  te::da::PropertyName* pName = new te::da::PropertyName(propName);
  te::da::Literal* literal = new te::da::LiteralString(oid);
  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = dataSource->query(select);
  }
  catch (...)
  {
    return std::auto_ptr<te::da::DataSet>();
  }

  return dataSet;
}

bool ta::model::CatalogFacade::addClassColumn(const int& catalogId, const std::string& className)
{
  std::auto_ptr<te::da::DataSetType> newDst = m_dataSource->getDataSetType(ta::model::Globals::sm_rulesLayerParams);

  te::mem::DataSet* newData = new te::mem::DataSet(newDst.get());

  te::mem::DataSetItem* dsItem = new te::mem::DataSetItem(newData);

  dsItem->setValue(ta::model::CatalogFacade::sm_rulesLayerParams_catalogId, new te::dt::SimpleData<int, te::dt::INT32_TYPE>(catalogId));
  dsItem->setValue(ta::model::CatalogFacade::sm_rulesLayerParams_classColumn, new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(className));

  newData->add(dsItem);

  std::map<std::string, std::string> op;

  std::auto_ptr<te::da::DataSourceTransactor> transactor = m_dataSource->getTransactor();

  try
  {
    transactor->add(ta::model::Globals::sm_rulesLayerParams, newData, op);

  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    transactor->rollBack();
    return false;
  }

  transactor->commit();

  return true;
}

bool ta::model::CatalogFacade::removeClassColumn(const int& catalogId, const std::string& className)
{
  //TODO: refactoring...

  std::string sql = "DELETE FROM ";
  sql += ta::model::Globals::sm_rulesLayerParams;
  sql += " WHERE ";
  sql += ta::model::CatalogFacade::sm_rulesLayerParams_catalogId;
  sql += " = ";
  sql += te::common::Convert2String(catalogId);
  sql += " AND ";
  sql += ta::model::CatalogFacade::sm_rulesLayerParams_classColumn;
  sql += " = ";
  sql += className;

  //remove element
  try
  {
    m_dataSource->execute(sql);
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    return false;
  }

  return true;
}

bool ta::model::CatalogFacade::getClassColumn(const int& catalogId, std::string& className)
{
  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_rulesLayerParams);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::CatalogFacade::sm_rulesLayerParams_catalogId);

  te::dt::AbstractData* dataId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(catalogId));
  te::da::Literal* literal = new te::da::Literal(dataId);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  if (dataSet->size() != 1)
    return false;

  dataSet->moveFirst();

  className = dataSet->getString(ta::model::CatalogFacade::sm_rulesLayerParams_classColumn);

  return true;
}

std::vector<std::string> ta::model::CatalogFacade::getClassColumns(const int& catalogId)
{
  std::vector<std::string> result;

  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_rulesLayerParams);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::CatalogFacade::sm_rulesLayerParams_catalogId);

  te::dt::AbstractData* dataId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(catalogId));
  te::da::Literal* literal = new te::da::Literal(dataId);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    std::string classColumn = dataSet->getString(ta::model::CatalogFacade::sm_rulesLayerParams_classColumn);
    result.push_back(classColumn);
  }
  return result;
}

std::vector<std::string> ta::model::CatalogFacade::getAoiIdColumns(const int& catalogId)
{
  std::vector<std::string> result;

  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_aoiLayer);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::CatalogFacade::sm_aoiLayer_catalogId);

  te::dt::AbstractData* dataId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(catalogId));
  te::da::Literal* literal = new te::da::Literal(dataId);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    std::string aoiIdColumn = dataSet->getString(ta::model::CatalogFacade::sm_aoiLayer_aoiIdColumn);
    result.push_back(aoiIdColumn);
  }
  return result;
}

ta::model::Catalog* ta::model::CatalogFacade::getByUriAndName(const std::string& uri, const std::string& name)
{
  ta::model::Catalog* result = 0;

  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_catalog);
  from->push_back(fItem);

  te::da::PropertyName* pUri = new te::da::PropertyName(ta::model::Catalog::sm_uri);
  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::Catalog::sm_name);

  te::da::LiteralString* lUri = new te::da::LiteralString(uri);
  te::da::LiteralString* lName = new te::da::LiteralString(name);

  te::da::EqualTo* e1 = new te::da::EqualTo(pUri, lUri);
  te::da::EqualTo* e2 = new te::da::EqualTo(pName, lName);

  te::da::And* and1 = new te::da::And(e1, e2);

  te::da::Where* where1 = new te::da::Where(and1);

  te::da::Select select1 = new te::da::Select(fields, from, where1);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select1);

  dataSet->moveBeforeFirst();

  if (dataSet->moveNext())
  {
    ta::model::AbstractItem* ai = ta::model::AbstractItemFactory::make(ta::model::Globals::sm_catalog, dataSet.get());
    result = dynamic_cast<ta::model::Catalog*>(ai);
  }

  return result;
}

bool ta::model::CatalogFacade::isCellLayer(const int& catalogId)
{
  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_cellLayer);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pId = new te::da::PropertyName(sm_cellLayer_catalogId);
  te::dt::AbstractData* id = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(catalogId));
  te::da::Literal* lId = new te::da::Literal(id);

  te::da::EqualTo* equal = new te::da::EqualTo(pId, lId);

  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return false;
  }

  if (dataSet->size() != 1)
  {
    return false;
  }

  if (dataSet->moveFirst())
  {
    return true;
  }

  return false;
}

int ta::model::CatalogFacade::getActiveCellLayer()
{
  int result = -1;

  te::da::Field* field = new te::da::Field(sm_cellLayer_catalogId);
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_cellLayer);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pActive = new te::da::PropertyName(sm_cellLayer_active);
  te::da::Literal* lActive = new te::da::LiteralBool(true);

  te::da::EqualTo* equal = new te::da::EqualTo(pActive, lActive);

  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return false;
  }

  if (dataSet->size() != 1)
  {
    return false;
  }

  if (dataSet->moveFirst())
  {
    std::auto_ptr<te::dt::AbstractData> ad = dataSet->getValue(0);
    te::dt::SimpleData<int, te::dt::INT32_TYPE>* sd = dynamic_cast<te::dt::SimpleData<int, te::dt::INT32_TYPE>*>(ad.get());
    result = sd->getValue();
  }

  return result;
}

bool ta::model::CatalogFacade::setActiveCellLayer(const int& catalogId)
{
  std::string sql1 = "UPDATE " + ta::model::Globals::sm_cellLayer + " SET " + sm_cellLayer_active + " = FALSE WHERE " + sm_cellLayer_active + " = TRUE;";
  std::string sql2 = "UPDATE " + ta::model::Globals::sm_cellLayer + " SET " + sm_cellLayer_active + " = TRUE WHERE " + sm_cellLayer_catalogId + " = " + boost::lexical_cast<std::string>(catalogId)+";";

  std::auto_ptr<te::da::DataSourceTransactor> transactor = m_dataSource->getTransactor();
  try
  {
    transactor->begin();
    transactor->execute(sql1);
    transactor->execute(sql2);
    transactor->commit();
    return true;
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
}

std::vector<ta::model::CellLayer*> ta::model::CatalogFacade::getCellLayers()
{
  std::vector<ta::model::CellLayer*> result;

  te::da::Field* field1 = new te::da::Field(ta::model::Catalog::sm_id);
  te::da::Field* field2 = new te::da::Field(ta::model::Catalog::sm_name);
  te::da::Field* field3 = new te::da::Field(sm_cellLayer_resolution);
  te::da::Field* field4 = new te::da::Field(sm_cellLayer_active);

  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field1);
  fields->push_back(field2);
  fields->push_back(field3);
  fields->push_back(field4);

  te::da::FromItem* fromItem1 = new te::da::DataSetName(ta::model::Globals::sm_catalog);
  te::da::FromItem* fromItem2 = new te::da::DataSetName(ta::model::Globals::sm_cellLayer);

  te::da::Expression* exp1 = new te::da::PropertyName(ta::model::Catalog::sm_id);
  te::da::Expression* exp2 = new te::da::PropertyName(sm_cellLayer_catalogId);
  te::da::Expression* expression = new te::da::BinaryFunction("=", exp1, exp2);

  te::da::Join* join = new te::da::Join(fromItem1, fromItem2, te::da::INNER_JOIN, new te::da::JoinConditionOn(expression));

  te::da::From* from = new te::da::From;
  from->push_back(join);

  te::da::Select select = new te::da::Select(fields, from);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    //TODO: launch error
    return result;
  }

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    int catalogId = dataSet->getInt32(ta::model::Catalog::sm_id);
    std::string name = dataSet->getString(ta::model::Catalog::sm_name);
    double resolution = dataSet->getDouble(sm_cellLayer_resolution);
    bool active = dataSet->getBool(sm_cellLayer_active);

    ta::model::CellLayer* cellLayer = new CellLayer();
    cellLayer->setCatalogId(catalogId);
    cellLayer->setName(name);
    cellLayer->setResolution(resolution);
    cellLayer->setActive(active);

    result.push_back(cellLayer);
  }

  return result;
}

ta::model::CellLayer* ta::model::CatalogFacade::getCellLayerData(const int& catalogId)
{
  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_cellLayer);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pId = new te::da::PropertyName(sm_cellLayer_catalogId);
  te::dt::AbstractData* id = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(catalogId));
  te::da::Literal* lId = new te::da::Literal(id);

  te::da::EqualTo* equal = new te::da::EqualTo(pId, lId);

  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    //TODO: launch error
    return NULL;
  }

  dataSet->moveBeforeFirst();

  if (dataSet->moveNext())
  {
    int catalogId = dataSet->getInt32(ta::model::CellLayer::sm_catalogId);
    double resolution = dataSet->getDouble(ta::model::CellLayer::sm_resolution);
    bool active = dataSet->getBool(ta::model::CellLayer::sm_active);

    ta::model::CellLayer* cellLayer = new CellLayer();
    cellLayer->setCatalogId(catalogId);
    cellLayer->setResolution(resolution);
    cellLayer->setActive(active);

    return cellLayer;
  }

  return NULL;
}


void ta::model::CatalogFacade::unlockCellLayerItems(const int& catalogId, const std::vector<std::string>& cellOids)
{
  ta::model::Catalog* c = dynamic_cast<ta::model::Catalog*>(get(catalogId));

  if (!c)
  {
    //TODO: launch exception
    return;
  }

  std::string list = "";
  for (std::size_t i = 0; i < cellOids.size(); i++)
  {
    list += "'" + cellOids.at(i) + "'";
    if (!(i + 1 == cellOids.size())){
      list += ",";
    }
  }

  //TODO: refactor
  std::string sql = "UPDATE ";
  sql += c->getName();
  sql += " SET process_id = NULL ";
  sql += "WHERE cell_oid IN (";
  sql += list;
  sql += ");";

  try
  {
    m_dataSource->execute(sql);
  }
  catch (...)
  {
    //TODO: launch exception
  }

  delete c;
}

bool ta::model::CatalogFacade::isCatalogLockedForEdition(const std::string& layerName)
{
  bool isLocked = false;

  //TODO: refactoring
  std::string select = "SELECT " + ta::model::Catalog::sm_locked + " FROM TERRAAMAZON.TA_CATALOG B ";
  select = select + "WHERE LOWER(B.NAME) = LOWER('" + layerName + "')";

  /*
  te::da::Field* field = new te::da::Field(ta::model::Catalog::sm_locked);
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_catalog);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::Catalog::sm_name);
  te::da::Literal* lName = new te::da::LiteralString(layerName);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, lName);

  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);
  */
  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return false;
  }

  if (dataSet->size() > 0)
  {
    dataSet->moveFirst();
    isLocked = dataSet->getBool(ta::model::Catalog::sm_locked);
  }

  return isLocked;
}

std::vector<ta::model::Catalog*> ta::model::CatalogFacade::getCatalogs()
{
  std::vector<ta::model::Catalog*> result;

  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_catalog);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::Select* select = new te::da::Select(fields, from);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return result;
  }

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    ta::model::AbstractItem* ai = ta::model::AbstractItemFactory::make(m_datasetName, dataSet.get());

    result.push_back(dynamic_cast<ta::model::Catalog*>(ai));
  }

  return result;
}

bool ta::model::CatalogFacade::createAmazonLayer(const std::string& layerName, const std::string& userName, int& newLayerId)
{
  //create properties
  std::vector<te::dt::SimpleProperty*> props;

  {//object id
    te::dt::SimpleProperty* sp = new te::dt::SimpleProperty("object_id", te::dt::INT32_TYPE, true);
    sp->setAutoNumber(true);
    props.push_back(sp);
  }

  {//cell_column
    te::dt::StringProperty* sp = new te::dt::StringProperty("cell_oid", te::dt::STRING, 255, false);
    sp->setSubtype(te::dt::VAR_STRING);
    props.push_back(sp);
  }

  {//class_column
    te::dt::StringProperty* sp = new te::dt::StringProperty("class_name", te::dt::STRING, 255, false);
    sp->setSubtype(te::dt::VAR_STRING);
    props.push_back(sp);
  }

  {//scene_id
    te::dt::SimpleProperty* sp = new te::dt::SimpleProperty("scene_id", te::dt::INT32_TYPE, false);
    props.push_back(sp);
  }

  {//task_id
    te::dt::SimpleProperty* sp = new te::dt::SimpleProperty("task_id", te::dt::INT32_TYPE, false);
    props.push_back(sp);
  }

  {//spatial_data
    int cellLayerId = getActiveCellLayer();
    std::auto_ptr<ta::model::Catalog> catalog(dynamic_cast<ta::model::Catalog*>(get(cellLayerId)));
    assert(catalog.get());

    std::auto_ptr<te::da::DataSetType> dsType = getDataSetType(catalog.get());
    te::gm::GeometryProperty* geomProp = te::da::GetFirstGeomProperty(dsType.get());
    assert(geomProp);

    int srid = geomProp->getSRID();

    te::dt::SimpleProperty* sp = new te::gm::GeometryProperty("spatial_data", srid, te::gm::PolygonType, false);
    props.push_back(sp);
  }

  //create dataSetType
  std::auto_ptr<te::da::DataSetType> dsType(new te::da::DataSetType(layerName));

  for (std::size_t t = 0; t < props.size(); ++t)
  {
    dsType->add(props[t]);
  }

  dsType->setTitle(layerName);

  //create primary key
  te::dt::Property* ppk = dsType->getProperty("object_id");

  te::da::PrimaryKey* pk = new te::da::PrimaryKey(dsType.get());

  std::string name = dsType->getTitle() + "_" + ppk->getName() + "_pk";

  boost::replace_all(name, ".", "_");

  pk->setName(name);
  pk->add(ppk->clone());

  //create spatial index
  te::gm::GeometryProperty* pGeom = te::da::GetFirstGeomProperty(dsType.get());

  te::da::Index* idx = new te::da::Index(dsType.get());

  name = dsType->getTitle() + "_" + pGeom->getName() + "_idx";

  boost::replace_all(name, ".", "_");

  idx->setName(name);
  idx->setIndexType(te::da::R_TREE_TYPE);
  idx->add(pGeom->clone());


  //create datasettype
  std::map<std::string, std::string> nopt;

  try
  {
    m_dataSource->createDataSet(dsType.get(), nopt);
  }
  catch (const std::exception& ex)
  {
    m_errorMessage = ex.what();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Error creating layer.";
    return false;
  }

  //create style
  std::string strStyle = "";

  te::gm::GeometryProperty* gProp = te::da::GetFirstGeomProperty(dsType.get());

  if (gProp)
  {
    std::auto_ptr<te::se::Style> style(te::se::CreateFeatureTypeStyle(gProp->getGeometryType()));

    if (style.get())
    {
      strStyle = ta::common::Write(style.get());
    }
  }

  //add new layer to catalog
  te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().get(m_dataSource->getId());

  ta::model::Catalog* item = new ta::model::Catalog();
  item->setName("public." + layerName);
  item->setUri(dsInfo->getConnInfoAsString());
  item->setType(dsInfo->getType());
  item->setAuthor(userName);
  item->setSource("Amazon Layer.");
  item->setPermission(1);
  item->setStyle(strStyle);
  item->setLocked(true);
  item->setCellColumn("cell_oid");

  boost::posix_time::ptime boostLocalTime = boost::posix_time::second_clock::local_time();
  te::dt::TimeInstant* localTime = new te::dt::TimeInstant(boostLocalTime);
  item->setLastEdition(localTime);

  item->setScaleMin(1);
  item->setScaleMax(1000000);

  bool res = save(item);

  if (res)
  {
    addClassColumn(item->getId(), "class_name");
    newLayerId = item->getId();
  }

  return res;
}

bool ta::model::CatalogFacade::checkProperties(ta::model::AbstractItem* /*item*/)
{
  return true;
}

bool ta::model::CatalogFacade::hasDependecy(const int& id)
{
  ta::model::RuleFacade rf(m_dataSource);
  ta::model::RuleSetFacade rsf(m_dataSource);

  if (rsf.existsByCatalogId(id))
    return true;
  if (rf.existsByCatalogId(id))
    return true;
  if (getActiveCellLayer() == id)
    return true;

  return false;
}
