/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/CellFacade.cpp

\brief
*/

// TerraAmazon
#include "../item/AoiLayer.h"
#include "../item/Catalog.h"
#include "../item/Cell.h"
#include "../item/Process.h"
#include "../item/Scene.h"
#include "../item/Task.h"
#include "../facade/AoiLayerFacade.h"
#include "../facade/CatalogFacade.h"
#include "../facade/ProcessFacade.h"
#include "../facade/SceneFacade.h"
#include "../facade/TaskFacade.h"
#include "../factory/AbstractItemFactory.h"
#include "../Globals.h"
#include "../../edition/qt/Utils.h"
#include "CellFacade.h"


// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/datasource/DataSourceInfoManager.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/LiteralBool.h>
#include <terralib/dataaccess/query/LiteralEnvelope.h>
#include <terralib/dataaccess/query/LiteralString.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/ST_Intersects.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/dataaccess/query/OrderBy.h>
#include <terralib/dataaccess/query/OrderByItem.h>
#include <terralib/dataaccess/utils/Utils.h>
#include <terralib/datatype/SimpleData.h>

ta::model::CellFacade::CellFacade(te::da::DataSourcePtr ds) : ta::model::AbstractFacade(ds),
 m_layerName(""),
 m_layerURI(""),
 m_layer(0)
{
  // Register the application framework listener
  te::qt::af::AppCtrlSingleton::getInstance().addListener(this);

  getLayerCell(m_layerName, m_layerURI);

  m_datasetName = m_layerName;

 }

ta::model::CellFacade::~CellFacade()
{
  te::qt::af::AppCtrlSingleton::getInstance().removeListener(this);
}

bool ta::model::CellFacade::loadCells(std::vector<ta::model::Cell*>& vecCell, ta::model::Cell& cell, const bool& getCellInfo)
{
  vecCell.clear();

  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("ID"));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItemA = new te::da::DataSetName(m_datasetName);
  from->push_back(fItemA);

  te::da::PropertyName* pName = new te::da::PropertyName(m_datasetName + "." + ta::model::Cell::sm_processId);

  te::dt::AbstractData* id = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(cell.getProcessId()));
  te::da::Literal* literal = new te::da::Literal(id);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);

  te::da::Where* where = new te::da::Where(equal);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);

    if (getCellInfo)
    {
      dataSet->moveBeforeFirst();

      while (dataSet->moveNext())
      {
        ta::model::Cell* cell(get(dataSet->getString("ID")));
        vecCell.push_back(cell);
      }
    }
  }
  catch (...)
  {
    return false;
  }

  return true;

  //SELECT a.cell_id, a.process_id, a.linkcolumn FROM cell a WHERE a.process_id = 48 ORDER BY a.cell_id
}

ta::model::Cell* ta::model::CellFacade::get(const std::string& id)
{
  ta::model::Cell* cell = new ta::model::Cell();

  std::unique_ptr<te::da::DataSetType> dsType = m_dataSource->getDataSetType(m_datasetName);

  if (!dsType.get())
    return 0;

  te::da::PrimaryKey* pk = dsType->getPrimaryKey();

  if (!pk)
    return 0;

  //CHECK
  std::string firstPropName = pk->getProperties()[0]->getName();

  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields();
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(m_datasetName);
  te::da::From* from = new te::da::From();
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(firstPropName);

  te::da::Literal* literal = new te::da::LiteralString(id);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::unique_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return 0;
  }

  if (dataSet->size() != 1)
  {
    return 0;
  }

  dataSet->moveFirst();

  if (!dataSet->isNull(ta::model::Cell::sm_processId))
    cell->setProcessId(dataSet->getInt32(ta::model::Cell::sm_processId));
  
  cell->setOid(dataSet->getString(ta::model::Cell::sm_oid));
  cell->setCell(*dataSet->getGeometry(ta::model::Cell::sm_cell).release());
  
  return cell;
}

bool ta::model::CellFacade::isIntersectIA(ta::model::Cell& cell, int cellSRID, ta::model::Process* process, int aiSRID, te::gm::Geometry* aiGeometry)
{
  if (process->getId() <= 0)
    return false;

  if ((cell.getOid() != "") && (cell.getOid().size() == 0))
    return false;

  if (cellSRID != aiSRID)
    aiGeometry->transform(cellSRID);

  if (cell.getCell()->intersects(aiGeometry))
    return true;
  else
    return false;
}

bool ta::model::CellFacade::setLockState(ta::model::Cell& cell, const bool& lockState)
{
  if (cell.getProcessId() <= 0 && cell.getOid().length() == 0)
    return false;

  std::string lockValue = "NULL";
  if (lockState)
    lockValue = std::to_string(cell.getProcessId());

  std::unique_ptr<te::da::DataSourceTransactor> t = m_dataSource->getTransactor();

  //TODO: Query
  std::string sql = "UPDATE " + m_datasetName + " SET PROCESS_ID = " + lockValue + " WHERE ID = '" + cell.getOid() + "';";

  try
  {
    t->begin();

    t->execute(sql);

    t->commit();
  }
  catch (...)
  {
    t->rollBack();

    return false;
  }

  return true;

}

bool ta::model::CellFacade::getCellState(ta::model::Cell& cell, const bool& /*lockForConcurrency*/)
{
  if (cell.getOid().length() == 0)
    return false;

  //TODO: Use Query
  std::string sql = "SELECT A.ID, A.PROCESS_ID FROM " + m_datasetName + " A WHERE A.ID = '" + cell.getOid() + "' ORDER BY A.ID";

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(sql);
  }
  catch (...)
  {
    return false;
  }

  if (dataSet->size() < 1)
  {
    return false;
  }

  dataSet->moveFirst();

  cell.setOid(dataSet->getString(ta::model::Cell::sm_oid));
  if (!dataSet->isNull(ta::model::Cell::sm_processId))
    cell.setProcessId(dataSet->getInt32(ta::model::Cell::sm_processId));

  return true;
}

bool ta::model::CellFacade::getSelectedCells(const unsigned int& processId, std::vector<te::gm::Geometry*>& cs)
{
  cs.clear();

  ta::model::Cell cell;
  cell.setProcessId(processId);

  std::vector<ta::model::Cell*> vecCells;
  if (loadCells(vecCells, cell, true))
  {
    for (std::size_t i = 0; i < vecCells.size(); i++)
    {
      cs.push_back(vecCells[i]->getCell());
    }
  }

  return true;

}

bool ta::model::CellFacade::queryCellInformation(const te::gm::Envelope& env, std::vector<ta::model::Cell*>& cellSet, const te::map::AbstractLayerPtr& layer)
{
  if (layer == 0)
    return false;

  std::auto_ptr<te::da::DataSetType> dt(layer->getSchema());

  te::gm::GeometryProperty* gp = te::da::GetFirstGeomProperty(dt.get());

  // create expression to order the cells
  te::da::Field* field = new te::da::Field(gp->getName());
  te::da::Field* fieldB = new te::da::Field(ta::model::Cell::sm_oid);
  te::da::Fields* fields = new te::da::Fields();
  fields->push_back(field);
  fields->push_back(fieldB);

  te::da::FromItem* t = new te::da::DataSetName(m_datasetName);
  te::da::From* from = new te::da::From();
  from->push_back(t);

  te::da::LiteralEnvelope* lenv = new te::da::LiteralEnvelope(env, layer->getSRID());
  te::da::ST_Intersects* intersects = new te::da::ST_Intersects(field->getExpression(), lenv);

  te::da::Where* wh = new te::da::Where(intersects);

  te::da::OrderByItem* obItem = new te::da::OrderByItem(new te::da::PropertyName(ta::model::Cell::sm_oid));
  te::da::OrderBy* ob = new te::da::OrderBy();
  ob->push_back(obItem);

  te::da::Select select = new te::da::Select(fields, from, wh, ob);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);

    while (dataSet->moveNext())
    {
      ta::model::Cell* cell = new ta::model::Cell();
      std::auto_ptr<te::gm::Geometry> g(dataSet->getGeometry(gp->getName()));

      if (!dataSet->isNull(ta::model::Cell::sm_oid))
        cell->setOid(dataSet->getString(ta::model::Cell::sm_oid));

      cell->setCell(*g.get());

      cellSet.push_back(cell);
    }
  }
  catch (...)
  {
    return false;
  }

  if (cellSet.empty())
    return false;

  return true;
}

bool ta::model::CellFacade::getLayerCell(std::string& layerCell, std::string& uri)
{
  layerCell = "";
  uri = "";

  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields();
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_cellLayer);
  te::da::From* from = new te::da::From();
  from->push_back(fromItem);
  
  te::da::PropertyName* pActive = new te::da::PropertyName(ta::model::CatalogFacade::sm_cellLayer_active);
  te::dt::AbstractData* active = new te::dt::SimpleData<bool, te::dt::BOOLEAN_TYPE>(true);
  te::da::Literal* lActive = new te::da::LiteralBool(active);

  te::da::EqualTo* equal = new te::da::EqualTo(pActive, lActive);

  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);

    if (dataSet->moveNext())
    {
      ta::model::CatalogFacade cf(m_dataSource);
      
      std::unique_ptr<ta::model::Catalog> catalog(dynamic_cast<ta::model::Catalog*>(cf.get(dataSet->getInt32(ta::model::CatalogFacade::sm_cellLayer_catalogId))));

      layerCell = catalog->getName();

      te::da::DataSourcePtr dataSource = cf.getDataSource(catalog.release());
      if (dataSource.get())
      {
        te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().get(dataSource->getId());

        if (dsInfo.get())
          uri = ta::edition::qt::GetURICustomized(layerCell, dsInfo);

      }

    }
  }
  catch (...)
  {
    return false;
  }

  return true;
}

bool ta::model::CellFacade::loadLayer()
{  
  std::string connectionlayer = "";

  if (m_layer) 
    return true;

  te::qt::af::evt::GetAvailableLayers evt;
  emit triggered(&evt);

  std::list<te::map::AbstractLayerPtr>::iterator it;
  for (it = evt.m_layers.begin(); it != evt.m_layers.end(); ++it)
  {
    te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().get((*it)->getDataSourceId());

    if (dsInfo.get())
      connectionlayer = ta::edition::qt::GetURICustomized((*it)->getTitle(), dsInfo);

    if (connectionlayer == m_layerURI && 
        (*it)->getVisibility() == te::map::VISIBLE)
    {
      m_layer = *it;
      break;
    }

  }

  if (m_layer == 0)
    return false;

  return true;
}

bool ta::model::CellFacade::unlockCells(const unsigned int& processId)
{
  ta::model::Cell cell;
  cell.setProcessId(processId);

  std::vector<ta::model::Cell*> vecCells;
  if (!loadCells(vecCells, cell, true))
    return false;

  for (std::size_t i = 0; i < vecCells.size(); i++)
  {
    if (!setLockState(*vecCells[i], false))
    {
      return false;
    }
  }

  return true;
}

std::string ta::model::CellFacade::getLayerName()
{
  return m_layerName;
}

std::string ta::model::CellFacade::getLayerURI()
{
  return m_layerURI;
}

te::map::AbstractLayerPtr ta::model::CellFacade::getLayer()
{
  if (loadLayer() == false)
  {
    return 0;
  }

  return m_layer;
}

bool ta::model::CellFacade::checkProperties(ta::model::AbstractItem* /*item*/)
{
  return true;
}

bool ta::model::CellFacade::hasDependecy(const int& /*id*/)
{
  return false;
}

void ta::model::CellFacade::onApplicationTriggered(te::qt::af::evt::Event* /*e*/)
{}
