/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/ClassFacade.cpp

\brief
*/

// TerraAmazon
#include "../Globals.h"
#include "../../common/Utils.h"
#include "../item/Catalog.h"
#include "../item/Class.h"
#include "../item/RuleSet.h"
#include "../factory/AbstractItemFactory.h"
#include "CatalogFacade.h"
#include "ClassFacade.h"
#include "RuleSetFacade.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/Distinct.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/IsNull.h>
#include <terralib/dataaccess/query/Join.h>
#include <terralib/dataaccess/query/JoinConditionOn.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/dataaccess/query/LiteralInt32.h>
#include <terralib/dataaccess/query/LiteralString.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/dataaccess/query/Where.h>

ta::model::ClassFacade::ClassFacade(te::da::DataSourcePtr ds) : ta::model::AbstractFacade(ds)
{
  m_datasetName = ta::model::Globals::sm_class;
}

ta::model::ClassFacade::~ClassFacade()
{}

std::vector<ta::model::Class*> ta::model::ClassFacade::getNotAssociatedClasses()
{
  std::vector<ta::model::Class*> vec;

  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fItem1 = new te::da::DataSetName(m_datasetName);
  te::da::FromItem* fItem2 = new te::da::DataSetName(ta::model::Globals::sm_rulesetClass);

  te::da::Expression* exp1 = new te::da::PropertyName(ta::model::Class::sm_id);
  te::da::Expression* exp2 = new te::da::PropertyName("class_id");
  te::da::Expression* expression = new te::da::BinaryFunction("=", exp1, exp2);

  te::da::Join* join = new te::da::Join(fItem1, fItem2, te::da::LEFT_JOIN, new te::da::JoinConditionOn(expression));

  te::da::From* from = new te::da::From();
  from->push_back(join);

  te::dt::AbstractData* function = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>("class_id " + te::da::FunctionNames::sm_IsNull));
  te::da::Expression* isNull = new te::da::Literal(function);
  te::da::Where* where = new te::da::Where(isNull);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    ta::model::AbstractItem* ai = ta::model::AbstractItemFactory::make(m_datasetName, dataSet.get());

    vec.push_back(dynamic_cast<ta::model::Class*>(ai));
  }

  return vec;
}

std::vector<ta::model::Class*> ta::model::ClassFacade::getNotAssociatedClasses(const int& catalogId)
{
  ta::model::RuleSetFacade rsf(m_dataSource);
  ta::model::RuleSet* rs = rsf.getByCatalogId(catalogId);
  std::vector<int> classIds;
  if (rs != NULL)
    classIds = getByRuleSetId(rs->getId());

  std::vector<ta::model::Class*> vec;

  te::da::FromItem* fItem1 = new te::da::DataSetName(m_datasetName);
  te::da::FromItem* fItem2 = new te::da::DataSetName(ta::model::Globals::sm_rulesetClass);

  te::da::Expression* exp1 = new te::da::PropertyName(ta::model::Class::sm_id);
  te::da::Expression* exp2 = new te::da::PropertyName("class_id");
  te::da::Expression* expression = new te::da::BinaryFunction("=", exp1, exp2);

  te::da::Join* join = new te::da::Join(fItem1, fItem2, te::da::LEFT_JOIN, new te::da::JoinConditionOn(expression));

  te::da::Distinct* dist = new te::da::Distinct;
  te::da::Expression* propName = new te::da::PropertyName(m_datasetName + ".*");
  dist->push_back(propName);

  te::da::From* from = new te::da::From();
  from->push_back(join);

  te::da::Select* select = new te::da::Select();
  select->setDistinct(dist);
  select->setFrom(from);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    ta::model::Class* c = dynamic_cast<ta::model::Class*>(ta::model::AbstractItemFactory::make(m_datasetName, dataSet.get()));

    bool associatedClass = false;
    for (std::size_t t = 0; t < classIds.size(); ++t)
    {
      if (c->getId() == classIds.at(t))
      {
        associatedClass = true;
        break;
      }
    }
    if (!associatedClass)
    {
      vec.push_back(c);
    }
  }

  return vec;
}

bool ta::model::ClassFacade::existsByName(const std::string& className)
{
  te::da::Fields fields = te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields.push_back(field);

  te::da::From from = te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_class);
  from.push_back(fItem);

  te::da::Select select = new te::da::Select(fields, from);

  std::unique_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    std::string name = dataSet->getString(ta::model::Class::sm_name);

    std::string nameConvert = te::common::Convert2UCase(name);
    std::string classNameConvert = te::common::Convert2UCase(className);

    if (Trim(nameConvert) == Trim(classNameConvert))
      return true;
  }

  return false;
}

bool ta::model::ClassFacade::existsByName(const std::string& className, te::da::DataSourceTransactor* transactor)
{
  te::da::Fields fields = te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields.push_back(field);

  te::da::From from = te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_class);
  from.push_back(fItem);

  te::da::Select select = new te::da::Select(fields, from);

  std::unique_ptr<te::da::DataSet> dataSet = transactor->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    std::string name = dataSet->getString(ta::model::Class::sm_name);

    std::string nameConvert = te::common::Convert2UCase(name);
    std::string classNameConvert = te::common::Convert2UCase(className);

    if (Trim(nameConvert) == Trim(classNameConvert))
      return true;
  }

  return false;
}

int ta::model::ClassFacade::getByName(const std::string& className)
{
  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_class);
  from->push_back(fItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::Class::sm_name);
  te::da::LiteralString* literal = new te::da::LiteralString(className);
  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::unique_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  if (dataSet->moveNext())
  {
    return dataSet->getInt32(ta::model::Class::sm_id);
  }

  return 0;
}

int ta::model::ClassFacade::getByName(const std::string& className, te::da::DataSourceTransactor* transactor)
{
  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_class);
  from->push_back(fItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::Class::sm_name);
  te::da::LiteralString* literal = new te::da::LiteralString(className);
  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::unique_ptr<te::da::DataSet> dataSet = transactor->query(select);

  dataSet->moveBeforeFirst();

  if (dataSet->moveNext())
  {
    return dataSet->getInt32(ta::model::Class::sm_id);
  }

  return 0;
}

std::vector<int> ta::model::ClassFacade::getByRuleSetId(const int& ruleSetId)
{
  std::vector<int> vec;

  te::da::Field* field = new te::da::Field(m_datasetName + ".*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fItem1 = new te::da::DataSetName(m_datasetName);
  te::da::FromItem* fItem2 = new te::da::DataSetName(ta::model::Globals::sm_rulesetClass);

  te::da::Expression* exp1 = new te::da::PropertyName(ta::model::Class::sm_id);
  te::da::Expression* exp2 = new te::da::PropertyName("class_id");
  te::da::Expression* expression = new te::da::BinaryFunction("=", exp1, exp2);

  te::da::Join* join = new te::da::Join(fItem1, fItem2, te::da::INNER_JOIN, new te::da::JoinConditionOn(expression));

  te::da::From* from = new te::da::From();
  from->push_back(join);

  te::da::PropertyName* pName = new te::da::PropertyName("ruleset_id");
  te::da::Literal* literal = new te::da::LiteralInt32(ruleSetId);
  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    int class_id = dataSet->getInt32(ta::model::Class::sm_id);
    vec.push_back(class_id);
  }

  return vec;
}

bool ta::model::ClassFacade::hasAssociation(const int& classId)
{
  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fItem1 = new te::da::DataSetName(m_datasetName);
  te::da::FromItem* fItem2 = new te::da::DataSetName(ta::model::Globals::sm_rulesetClass);

  te::da::Expression* exp1 = new te::da::PropertyName(ta::model::Class::sm_id);
  te::da::Expression* exp2 = new te::da::PropertyName("class_id");
  te::da::Expression* expression = new te::da::BinaryFunction("=", exp1, exp2);

  te::da::Join* join = new te::da::Join(fItem1, fItem2, te::da::INNER_JOIN, new te::da::JoinConditionOn(expression));

  te::da::From* from = new te::da::From();
  from->push_back(join);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::Class::sm_id);
  te::da::Literal* literal = new te::da::LiteralInt32(classId);
  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();
  return dataSet->moveNext();
}

bool ta::model::ClassFacade::getClassificationCatalogs(const int& projectId, std::vector<ta::model::Catalog*>& vecCatalogs)
{
  if (projectId == -1)
    return false;

  //TODO: Use Query
  std::string sql = "SELECT DISTINCT RS.CATALOG_ID AS ID FROM TERRAAMAZON.TA_PROJECT P";
  sql += " INNER JOIN TERRAAMAZON.TA_RULESET_PROJECT RP ON RP.PROJECT_ID = P.ID";
  sql += " INNER JOIN TERRAAMAZON.TA_RULESET RS ON RS.ID = RP.RULESET_ID";
  sql += " WHERE P.ID = " + te::common::Convert2String(projectId);

  std::auto_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(sql);
  }
  catch (...)
  {
    return false;
  }

  if (dataSet->size() < 1)
  {
    return false;
  }

  ta::model::CatalogFacade cf(m_dataSource);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    ta::model::Catalog* c = dynamic_cast<ta::model::Catalog*>(cf.get(dataSet->getInt32("ID")));

    vecCatalogs.push_back(c);
  }

  return true;
}

bool ta::model::ClassFacade::checkProperties(ta::model::AbstractItem* /*item*/)
{
  return true;
}

bool ta::model::ClassFacade::hasDependecy(const int& /*id*/)
{
  return false;
}
