/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/GroupFacade.cpp

\brief
*/

// TerraAmazon
#include "../Globals.h"
#include "../item/Group.h"
#include "../factory/AbstractItemFactory.h"
#include "GroupFacade.h"
#include "PhaseFacade.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/query/And.h>
#include <terralib/dataaccess/dataset/ObjectId.h>
#include <terralib/dataaccess/dataset/ObjectIdSet.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/NotEqualTo.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/dataaccess/query/LiteralString.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/dataaccess/query/Where.h>

ta::model::GroupFacade::GroupFacade(te::da::DataSourcePtr ds) : ta::model::AbstractFacade(ds)
{
  m_datasetName = ta::model::Globals::sm_group;
}

ta::model::GroupFacade::~GroupFacade()
{

}

bool ta::model::GroupFacade::addAssociatePhase(const int &groupId, const int &phaseId)
{
  //TODO: refactoring...
  try
  {
    ta::model::PhaseFacade phaseFacade(m_dataSource);
    if (!phaseFacade.isAssociated(phaseId))
    {
      std::string sql = "INSERT INTO ";
      sql += ta::model::Globals::sm_groupPhase;
      sql += " (group_id, phase_id) VALUES (";
      sql += te::common::Convert2String(groupId);
      sql += ", ";
      sql += te::common::Convert2String(phaseId);
      sql += ")";

      m_dataSource->execute(sql);
    }
  }
  catch (...)
  {
    return false;
  }
  return true;
}

bool ta::model::GroupFacade::existsByName(const std::string& groupName)
{
  te::da::Fields fields = te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields.push_back(field);

  te::da::From from = te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_group);
  from.push_back(fItem);

  te::da::Select select = new te::da::Select(fields, from);

  std::unique_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    std::string name = dataSet->getString(ta::model::Group::sm_name);

    if (te::common::Convert2UCase(name) == te::common::Convert2UCase(groupName))
      return true;
  }

  return false;
}

std::vector<ta::model::Group*> ta::model::GroupFacade::getAllowedGroups()
{
  std::vector<ta::model::Group*> vec;

  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(m_datasetName);
  from->push_back(fItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::Group::sm_name);

  te::da::LiteralString* lit1 = new te::da::LiteralString(ta::model::Globals::sm_adminGroup);
  te::da::LiteralString* lit2 = new te::da::LiteralString(ta::model::Globals::sm_projectManagerGroup);

  te::da::NotEqualTo* notEqual1 = new te::da::NotEqualTo(pName, lit1);
  te::da::NotEqualTo* notEqual2 = new te::da::NotEqualTo(pName, lit2);

  te::da::And* andClause = new te::da::And(notEqual1, notEqual2);

  te::da::Where* where = new te::da::Where(andClause);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::unique_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    ta::model::AbstractItem* ai = ta::model::AbstractItemFactory::make(m_datasetName, dataSet.get());
    vec.push_back(dynamic_cast<ta::model::Group*>(ai));
  }

  return vec;
}

std::vector<ta::model::Phase*> ta::model::GroupFacade::getAssociatedPhases(te::da::DataSourcePtr ds, int groupId)
{
  assert(ds.get());

  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_groupPhase);
  from->push_back(fItem);

  te::da::PropertyName* pName = new te::da::PropertyName("group_id");
  te::dt::AbstractData* id = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(groupId));
  te::da::Literal* literal = new te::da::Literal(id);
  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::unique_ptr<te::da::DataSet> dataSet = ds->query(select);

  dataSet->moveBeforeFirst();

  std::vector<ta::model::Phase*> phases;

  ta::model::PhaseFacade pf(ds);

  while (dataSet->moveNext())
  {
    int id = dataSet->getInt32("phase_id");

    ta::model::Phase* p = dynamic_cast<ta::model::Phase*>(pf.get(id));

    phases.push_back(p);
  }

  return phases;
}

std::unique_ptr<ta::model::Group> ta::model::GroupFacade::getByName(const std::string& groupName)
{
  std::unique_ptr<ta::model::Group> group;

  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(m_datasetName);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::Group::sm_name);

  te::dt::AbstractData* dataId = new te::dt::SimpleData<std::string>("'" + groupName + "'");
  te::da::Literal* literal = new te::da::Literal(dataId);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::unique_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return 0;
  }

  if (dataSet->size() != 1)
  {
    return 0;
  }

  dataSet->moveFirst();
  ta::model::AbstractItem* ai = ta::model::AbstractItemFactory::make(m_datasetName, dataSet.get());
  group.reset(dynamic_cast<ta::model::Group*>(ai));

  return group;
}

bool ta::model::GroupFacade::removeAssociatePhase(const int &groupId, const int &phaseId)
{
  //TODO: refactoring...
  try
  {
    std::string sql = "DELETE FROM ";
    sql += ta::model::Globals::sm_groupPhase;
    sql += " WHERE group_id = ";
    sql += te::common::Convert2String(groupId);
    sql += " AND phase_id = ";
    sql += te::common::Convert2String(phaseId);

    m_dataSource->execute(sql);
  }
  catch (...)
  {
    return false;
  }
  return true;
}

bool ta::model::GroupFacade::checkProperties(ta::model::AbstractItem* /*item*/)
{
  return true;
}

bool ta::model::GroupFacade::hasDependecy(const int& /*id*/)
{
  return false;
}
