/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/ProjectFacade.cpp

\brief
*/

// TerraAmazon
#include "../Globals.h"
#include "../item/Project.h"
#include "../item/Rule.h"
#include "../item/RuleSet.h"
#include "../item/Scene.h"
#include "AoiLayerFacade.h"
#include "CatalogFacade.h"
#include "ClassFacade.h"
#include "ProjectFacade.h"
#include "ProjectSceneFacade.h"
#include "RuleFacade.h"
#include "RuleSetFacade.h"
#include "RulesLayerColumnFacade.h"
#include "../qt/conceptualModel/ProjectImportUtils.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>
#include <terralib/dataaccess/query/And.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/memory/DataSet.h>
#include <terralib/memory/DataSetItem.h>

const std::string ta::model::ProjectFacade::sm_ruleSetProject_projectId("project_id");
const std::string ta::model::ProjectFacade::sm_ruleSetProject_ruleSetId("ruleset_id");

ta::model::ProjectFacade::ProjectFacade(te::da::DataSourcePtr ds) : ta::model::AbstractFacade(ds)
{
  m_datasetName = ta::model::Globals::sm_project;
}

ta::model::ProjectFacade::~ProjectFacade()
{

}

bool ta::model::ProjectFacade::hasScaleDefined(ta::model::Project* proj)
{
  if (proj->getMaxScale() != -std::numeric_limits<double>::max() &&
    proj->getMinScale() != std::numeric_limits<double>::max())
    return true;

  return false;
}

void ta::model::ProjectFacade::setNoScale(ta::model::Project* proj)
{
  proj->setMaxScale(-std::numeric_limits<double>::max());
  proj->setMinScale(std::numeric_limits<double>::max());
}

bool ta::model::ProjectFacade::existsByName(const std::string& projName)
{
  te::da::Fields fields = te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields.push_back(field);

  te::da::From from = te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_project);
  from.push_back(fItem);

  te::da::Select select = new te::da::Select(fields, from);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    std::string name = dataSet->getString(ta::model::Project::sm_name);

    if (te::common::Convert2UCase(name) == te::common::Convert2UCase(projName))
      return true;
  }

  return false;
}

bool ta::model::ProjectFacade::addAssociateRuleSet(const int &projectId, const int &ruleSetId)
{
  std::auto_ptr<te::da::DataSetType> dstype = m_dataSource->getDataSetType(ta::model::Globals::sm_rulesetProject);

  te::mem::DataSet* newData = new te::mem::DataSet(dstype.get());

  te::mem::DataSetItem* dsItem = new te::mem::DataSetItem(newData);

  dsItem->setValue(sm_ruleSetProject_ruleSetId, new te::dt::SimpleData<int, te::dt::INT32_TYPE>(ruleSetId));
  dsItem->setValue(sm_ruleSetProject_projectId, new te::dt::SimpleData<int, te::dt::INT32_TYPE>(projectId));

  newData->add(dsItem);

  std::map<std::string, std::string> op;

  std::auto_ptr<te::da::DataSourceTransactor> transactor = m_dataSource->getTransactor();

  try
  {
    transactor->add(ta::model::Globals::sm_rulesetProject, newData, op);
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    transactor->rollBack();
    return false;
  }

  transactor->commit();

  return true;
}

bool ta::model::ProjectFacade::addAssociateRuleSet(const int &projectId, const int &ruleSetId, te::da::DataSourceTransactor* transactor)
{
  std::auto_ptr<te::da::DataSetType> dstype = m_dataSource->getDataSetType(ta::model::Globals::sm_rulesetProject);

  te::mem::DataSet* newData = new te::mem::DataSet(dstype.get());

  te::mem::DataSetItem* dsItem = new te::mem::DataSetItem(newData);

  dsItem->setValue(sm_ruleSetProject_ruleSetId, new te::dt::SimpleData<int, te::dt::INT32_TYPE>(ruleSetId));
  dsItem->setValue(sm_ruleSetProject_projectId, new te::dt::SimpleData<int, te::dt::INT32_TYPE>(projectId));

  newData->add(dsItem);

  std::map<std::string, std::string> op;

  try
  {
    transactor->add(ta::model::Globals::sm_rulesetProject, newData, op);
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";
    return false;
  }

  return true;
}

bool ta::model::ProjectFacade::removeAssociateRuleSet(const int &projectId, const int &ruleSetId)
{
  //TODO: refactoring...

  std::string sql = "DELETE FROM ";
  sql += ta::model::Globals::sm_rulesetProject;
  sql += " WHERE ";
  sql += sm_ruleSetProject_ruleSetId;
  sql += " = ";
  sql += te::common::Convert2String(ruleSetId);
  sql += " AND ";
  sql += sm_ruleSetProject_projectId;
  sql += " = ";
  sql += te::common::Convert2String(projectId);

  //remove element
  try
  {
    m_dataSource->execute(sql);
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    return false;
  }

  return true;
}

std::vector<int> ta::model::ProjectFacade::getRuleSets(const int& projectId)
{
  std::vector<int> ids;

  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_rulesetProject);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::ProjectFacade::sm_ruleSetProject_projectId);

  te::dt::AbstractData* dataId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(projectId));
  te::da::Literal* literal = new te::da::Literal(dataId);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    int id = dataSet->getInt32(ta::model::ProjectFacade::sm_ruleSetProject_ruleSetId);

    ids.push_back(id);
  }

  return ids;
}

std::vector<int> ta::model::ProjectFacade::getRules(const int& projectId)
{
  std::vector<int> rulesIds;

  std::vector<int> rulesSetsIds = getRuleSets(projectId);

  ta::model::RuleSetFacade rsf(m_dataSource);

  for (std::size_t t = 0; t < rulesSetsIds.size(); ++t)
  {
    std::vector<int> rulesRuleSet = rsf.getRulesRuleSet(rulesSetsIds[t]);

    rulesIds.insert(rulesIds.end(), rulesRuleSet.begin(), rulesRuleSet.end());
  }

  return rulesIds;
}

std::vector<int> ta::model::ProjectFacade::getRules(const int& projectId, const ta::model::SpatialOperation& soType)
{
  std::vector<int> rulesIds;

  std::vector<int> rulesSetsIds = getRuleSets(projectId);

  ta::model::RuleSetFacade rsf(m_dataSource);

  for (std::size_t t = 0; t < rulesSetsIds.size(); ++t)
  {
    std::vector<int> rulesRuleSet = rsf.getRulesRuleSet(rulesSetsIds[t]);

    for (std::size_t p = 0; p < rulesRuleSet.size(); ++p)
    {
      ta::model::RuleFacade rf(m_dataSource);

      ta::model::Rule* r = dynamic_cast<ta::model::Rule*>(rf.get(rulesRuleSet[p]));

      if (r->getOperationId() == soType)
        rulesIds.push_back(rulesRuleSet[p]);

      delete r;
    }
  }

  return rulesIds;
}

bool ta::model::ProjectFacade::addAssocietedScene(const int &projectId, const int &sceneId)
{
  ta::model::ProjectSceneFacade psf(m_dataSource);

  ta::model::ProjectScene ps;
  ps.setProjectId(projectId);
  ps.setSceneId(sceneId);
  ps.setLocked(false); //default value

  if (!psf.save(&ps))
  {
    m_errorMessage = psf.getErrorMessage();
    return false;
  }

  return true;
}

std::vector<int> ta::model::ProjectFacade::getOutputCatalogs(const int& projectId)
{
  std::vector<int> catalogIds;

  std::vector<int> rulesSetsIds = getRuleSets(projectId);

  ta::model::RuleSetFacade rsf(m_dataSource);

  for (std::size_t t = 0; t < rulesSetsIds.size(); ++t)
  {
    ta::model::RuleSet* rs = dynamic_cast<ta::model::RuleSet*>(rsf.get(rulesSetsIds[t]));

    catalogIds.push_back(rs->getCatalogId());

    delete rs;
  }

  return catalogIds;
}

bool ta::model::ProjectFacade::hasCatalog(const int& projectId, const int& catalogId)
{
  std::vector<int> rulesSetsIds = getRuleSets(projectId);

  ta::model::RuleSetFacade rsf(m_dataSource);

  for (std::size_t t = 0; t < rulesSetsIds.size(); ++t)
  {
    ta::model::RuleSet* rs = dynamic_cast<ta::model::RuleSet*>(rsf.get(rulesSetsIds[t]));

    if (rs->getCatalogId() == catalogId)
    {
      delete rs;

      return true;
    }

    delete rs;
  }

  return false;
}

bool  ta::model::ProjectFacade::hasCatalogRule(const int& projectId, const int& catalogId, const ta::model::SpatialOperation& soType)
{
  std::vector<int> rulesSetsIds = getRuleSets(projectId);

  ta::model::RuleSetFacade rsf(m_dataSource);

  for (std::size_t t = 0; t < rulesSetsIds.size(); ++t)
  {
    std::vector<int> rulesRuleSet = rsf.getRulesRuleSet(rulesSetsIds[t]);

    for (std::size_t p = 0; p < rulesRuleSet.size(); ++p)
    {
      ta::model::RuleFacade rf(m_dataSource);

      ta::model::Rule* r = dynamic_cast<ta::model::Rule*>(rf.get(rulesRuleSet[p]));

      if (r->getCatalogId() == catalogId && r->getOperationId() == soType)
      {
        delete r;

        return true;
      }

      delete r;
    }
  }

  return false;
}

bool ta::model::ProjectFacade::importToDatabase(ta::model::ProjectMem* projectMem, std::string& errorMessage, std::string& warningMessage)
{
  ta::model::ProjectImportUtils* projectImport = new ProjectImportUtils();
  return projectImport->importToDatabase(projectMem, errorMessage, warningMessage, m_dataSource);
}

bool ta::model::ProjectFacade::checkProperties(ta::model::AbstractItem* /*item*/)
{
  return true;
}

bool ta::model::ProjectFacade::hasDependecy(const int& /*id*/)
{
  return false;
}
