/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/ProjectSceneFacade.cpp

\brief
*/

// TerraAmazon
#include "../factory/AbstractItemFactory.h"
#include "../item/Project.h"
#include "../item/Scene.h"
#include "../item/ProjectScene.h"
#include "../Globals.h"
#include "ProjectSceneFacade.h"
#include "SceneFacade.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>
#include <terralib/dataaccess/dataset/ObjectId.h>
#include <terralib/dataaccess/dataset/ObjectIdSet.h>
#include <terralib/dataaccess/query/And.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/Join.h>
#include <terralib/dataaccess/query/JoinConditionOn.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/dataaccess/query/LiteralInt32.h>
#include <terralib/dataaccess/query/LiteralString.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/memory/DataSet.h>
#include <terralib/memory/DataSetItem.h>


ta::model::ProjectSceneFacade::ProjectSceneFacade(te::da::DataSourcePtr ds) : ta::model::AbstractFacade(ds)
{
  m_datasetName = ta::model::Globals::sm_projectScene;
}

ta::model::ProjectSceneFacade::~ProjectSceneFacade()
{
}

bool ta::model::ProjectSceneFacade::save(ta::model::AbstractItem* item)
{
  if (!checkProperties(item))
  {
    return false;
  }

  ta::model::ItemPropertyVector props = item->toProperties();

  ta::model::ItemPropertyVector::iterator it;

  std::unique_ptr<te::da::DataSetType> newDst(new te::da::DataSetType(m_datasetName));

  for (it = props.begin(); it != props.end(); ++it)
    newDst->add((*it)->getProperty()->clone());

  te::mem::DataSet* newData = new te::mem::DataSet(newDst.get());

  te::mem::DataSetItem* dsItem = new te::mem::DataSetItem(newData);

  for (it = props.begin(); it != props.end(); ++it)
    dsItem->setValue((*it)->getProperty()->getName(), (*it)->getData()->clone());

  newData->add(dsItem);

  std::map<std::string, std::string> op;

  std::unique_ptr<te::da::DataSourceTransactor> transactor = m_dataSource->getTransactor();

  try
  {
    transactor->add(m_datasetName, newData, op);

  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    transactor->rollBack();
    return false;
  }

  transactor->commit();

  return true;
}

bool ta::model::ProjectSceneFacade::lock(const int& projectId, const int& sceneId, bool locked)
{
  ta::model::ProjectScene ps;
  ps.setProjectId(projectId);
  ps.setSceneId(sceneId);
  ps.setLocked(locked);

  if (!update(&ps))
  {
    m_errorMessage = getErrorMessage();
    return false;
  }

  return true;
}

bool ta::model::ProjectSceneFacade::isAssociated(const int& projectId, const int& sceneId)
{
  std::vector<int> sceneIds = getScenes(projectId);

  ta::model::SceneFacade scnf(m_dataSource);
  size_t scnNmbr = sceneIds.size();

  for (std::size_t t = 0; t < scnNmbr; ++t)
  {
    std::unique_ptr<ta::model::Scene> scene(dynamic_cast<ta::model::Scene*>(scnf.get(sceneIds[t])));

    if (scene->getId() == sceneId)
    {
      return true;
    }
  }

  return false;
}

bool ta::model::ProjectSceneFacade::remove(const int& projectId, const int& sceneId)
{
  //TODO: refactoring...

  std::string sql = "DELETE FROM ";
  sql += m_datasetName;
  sql += " WHERE ";
  sql += ta::model::ProjectScene::sm_projectId;
  sql += " = ";
  sql += te::common::Convert2String(projectId);
  sql += " AND ";
  sql += ta::model::ProjectScene::sm_sceneId;
  sql += " = ";
  sql += te::common::Convert2String(sceneId);

  //remove element
  try
  {
    m_dataSource->execute(sql);
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    return false;
  }

  return true;
}

std::vector<int> ta::model::ProjectSceneFacade::getScenes(const int& projectId)
{
  std::vector<int> scenesIds;

  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_projectScene);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::ProjectScene::sm_projectId);

  te::dt::AbstractData* dataId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(projectId));
  te::da::Literal* literal = new te::da::Literal(dataId);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::unique_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    int id = dataSet->getInt32(ta::model::ProjectScene::sm_sceneId);

    scenesIds.push_back(id);
  }

  return scenesIds;
}

std::vector<int> ta::model::ProjectSceneFacade::getProjects(const int& sceneId)
{
  std::vector<int> projIds;

  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_projectScene);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::ProjectScene::sm_sceneId);

  te::dt::AbstractData* dataId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(sceneId));
  te::da::Literal* literal = new te::da::Literal(dataId);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::unique_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    int id = dataSet->getInt32(ta::model::ProjectScene::sm_projectId);

    projIds.push_back(id);
  }

  return projIds;
}

bool ta::model::ProjectSceneFacade::checkProperties(ta::model::AbstractItem* /*item*/)
{
  return true;
}

bool ta::model::ProjectSceneFacade::hasDependecy(const int& /*id*/)
{
  return false;
}
