/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/RuleFacade.cpp

\brief
*/

// TerraAmazon
#include "../Globals.h"
#include "../Utils.h"
#include "../item/Catalog.h"
#include "../item/Rule.h"
#include "../item/RuleRuleSet.h"
#include "../item/RuleSet.h"
#include "../factory/AbstractItemFactory.h"
#include "CatalogFacade.h"
#include "RuleFacade.h"
#include "RuleRuleSetFacade.h"
#include "RuleSetFacade.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/query/And.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/Join.h>
#include <terralib/dataaccess/query/JoinConditionOn.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/memory/DataSet.h>
#include <terralib/memory/DataSetItem.h>


ta::model::RuleFacade::RuleFacade(te::da::DataSourcePtr ds) : ta::model::AbstractFacade(ds)
{
  m_datasetName = ta::model::Globals::sm_rule;
}

ta::model::RuleFacade::~RuleFacade()
{

}

std::string ta::model::RuleFacade::createRuleName(const int& catalogId, const int& spatialOperationId)
{
  std::string checkName = "";

  ta::model::CatalogFacade cf(m_dataSource);
  ta::model::Catalog* c = dynamic_cast<ta::model::Catalog*>(cf.get(catalogId));

  if (c)
  {
    std::string name = ta::model::GetSpatialOperationPrefix(spatialOperationId);
    name += "_";
    name += c->getName();

    checkName = name;

    int count = 1;
    while (existsByName(checkName))
    {
      checkName = name + "_" + te::common::Convert2String(count);

      ++count;
    }
  }

  delete c;


  return checkName;
}

bool ta::model::RuleFacade::existsByName(const std::string& ruleName)
{
  te::da::Fields fields = te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields.push_back(field);

  te::da::From from = te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_rule);
  from.push_back(fItem);

  te::da::Select select = new te::da::Select(fields, from);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    std::string name = dataSet->getString(ta::model::Rule::sm_name);

    if (te::common::Convert2UCase(name) == te::common::Convert2UCase(ruleName))
      return true;
  }

  return false;
}

bool ta::model::RuleFacade::existsByCatalogId(const int& catalogId)
{
  te::da::Fields fields = te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields.push_back(field);

  te::da::From from = te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_rule);
  from.push_back(fItem);

  te::da::Select select = new te::da::Select(fields, from);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    int cid = dataSet->getInt32(ta::model::Rule::sm_catalog_id);

    if (cid == catalogId)
      return true;
  }

  return false;
}

std::vector<ta::model::Rule*> ta::model::RuleFacade::getRulesWithIntersectionByRuleSet(const int &ruleSetId)
{
  std::vector<ta::model::Rule*> vec;

  te::da::Field* field = new te::da::Field(ta::model::Globals::sm_ruleAlias + "." + "*");
  te::da::Fields* fields = new te::da::Fields();
  fields->push_back(field);

  te::da::FromItem* fItem1 = new te::da::DataSetName(ta::model::Globals::sm_catalog, ta::model::Globals::sm_catalogAlias);
  te::da::FromItem* fItem2 = new te::da::DataSetName(ta::model::Globals::sm_rule, ta::model::Globals::sm_ruleAlias);
  te::da::FromItem* fItem3 = new te::da::DataSetName(ta::model::Globals::sm_ruleRuleset, ta::model::Globals::sm_ruleRulesetAlias);
  te::da::FromItem* fItem4 = new te::da::DataSetName(ta::model::Globals::sm_ruleset, ta::model::Globals::sm_rulesetAlias);

  te::da::Expression* exp1 = new te::da::PropertyName(ta::model::Globals::sm_catalogAlias + "." + ta::model::Catalog::sm_id);
  te::da::Expression* exp2 = new te::da::PropertyName(ta::model::Globals::sm_ruleAlias + "." + ta::model::Rule::sm_catalog_id);
  te::da::Expression* exp3 = new te::da::PropertyName(ta::model::Globals::sm_ruleAlias + "." + ta::model::Rule::sm_id);
  te::da::Expression* exp4 = new te::da::PropertyName(ta::model::Globals::sm_ruleRulesetAlias + "." + ta::model::RuleRuleSet::sm_rule_id);
  te::da::Expression* exp5 = new te::da::PropertyName(ta::model::Globals::sm_ruleRulesetAlias + "." + ta::model::RuleRuleSet::sm_ruleset_id);
  te::da::Expression* exp6 = new te::da::PropertyName(ta::model::Globals::sm_rulesetAlias + "." + ta::model::RuleSet::sm_id);

  te::da::Expression* expression1 = new te::da::BinaryFunction("=", exp1, exp2);
  te::da::Expression* expression2 = new te::da::BinaryFunction("=", exp3, exp4);
  te::da::Expression* expression3 = new te::da::BinaryFunction("=", exp5, exp6);

  te::da::Join* join1 = new te::da::Join(fItem1, fItem2, te::da::INNER_JOIN, new te::da::JoinConditionOn(expression1));
  te::da::Join* join2 = new te::da::Join(join1, fItem3, te::da::INNER_JOIN, new te::da::JoinConditionOn(expression2));
  te::da::Join* join3 = new te::da::Join(join2, fItem4, te::da::INNER_JOIN, new te::da::JoinConditionOn(expression3));

  te::da::From* from = new te::da::From();
  from->push_back(join3);

  te::da::PropertyName* pName1 = new te::da::PropertyName(ta::model::Globals::sm_ruleAlias + "." + ta::model::Rule::sm_operation_id);
  te::dt::AbstractData* opId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(ta::model::SO_INTERSECTION));
  te::da::Literal* literal1 = new te::da::Literal(opId);

  te::da::PropertyName* pName2 = new te::da::PropertyName(ta::model::Globals::sm_rulesetAlias + "." + ta::model::RuleSet::sm_id);
  te::dt::AbstractData* rsId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(ruleSetId));
  te::da::Literal* literal2 = new te::da::Literal(rsId);

  te::da::EqualTo* equal1 = new te::da::EqualTo(pName1, literal1);
  te::da::EqualTo* equal2 = new te::da::EqualTo(pName2, literal2);

  te::da::And* andClause = new te::da::And(equal1, equal2);

  te::da::Where* where = new te::da::Where(andClause);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    ta::model::AbstractItem* ai = ta::model::AbstractItemFactory::make(ta::model::Globals::sm_rule, dataSet.get());

    vec.push_back(dynamic_cast<ta::model::Rule*>(ai));
  }

  return vec;
}

bool ta::model::RuleFacade::checkProperties(ta::model::AbstractItem* /*item*/)
{
  return true;
}

bool ta::model::RuleFacade::hasDependecy(const int& /*id*/)
{
  return false;
}
