/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/RuleSetFacade.cpp

\brief
*/

// TerraAmazon
#include "../Globals.h"
#include "../item/Catalog.h"
#include "../item/Rule.h"
#include "../item/RuleRuleSet.h"
#include "../item/RuleSet.h"
#include "../factory/AbstractItemFactory.h"
#include "CatalogFacade.h"
#include "RuleFacade.h"
#include "RuleRuleSetFacade.h"
#include "RuleSetFacade.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>
#include <terralib/dataaccess/query/And.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/Join.h>
#include <terralib/dataaccess/query/JoinConditionOn.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/dataaccess/query/LiteralInt32.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/memory/DataSet.h>
#include <terralib/memory/DataSetItem.h>

const std::string ta::model::RuleSetFacade::sm_ruleSetClass_ruleSetId("ruleset_id");
const std::string ta::model::RuleSetFacade::sm_ruleSetClass_classId("class_id");


ta::model::RuleSetFacade::RuleSetFacade(te::da::DataSourcePtr ds) : ta::model::AbstractFacade(ds)
{
  m_datasetName = ta::model::Globals::sm_ruleset;
}

ta::model::RuleSetFacade::~RuleSetFacade()
{

}

std::string ta::model::RuleSetFacade::createRuleSetName(const int& catalogId)
{
  std::string checkName = "";

  ta::model::CatalogFacade cf(m_dataSource);
  ta::model::Catalog* c = dynamic_cast<ta::model::Catalog*>(cf.get(catalogId));

  if (c)
  {
    std::string name = "RS";
    name += "_";
    name += c->getName();

    checkName = name;

    int count = 1;
    while (existsByName(checkName))
    {
      checkName = name + "_" + te::common::Convert2String(count);

      ++count;
    }
  }

  delete c;

  return checkName;
}

bool ta::model::RuleSetFacade::existsByName(const std::string& ruleSetName)
{
  te::da::Fields fields = te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields.push_back(field);

  te::da::From from = te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_ruleset);
  from.push_back(fItem);

  te::da::Select select = new te::da::Select(fields, from);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    std::string name = dataSet->getString(ta::model::RuleSet::sm_name);

    if (te::common::Convert2UCase(name) == te::common::Convert2UCase(ruleSetName))
      return true;
  }

  return false;
}

bool ta::model::RuleSetFacade::existsByCatalogId(const int& catalogId)
{
  return getByCatalogId(catalogId) != NULL;
}

ta::model::RuleSet* ta::model::RuleSetFacade::getByCatalogId(const int& catalogId)
{
  ta::model::RuleSet* rs = NULL;

  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_ruleset);
  from->push_back(fItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::RuleSet::sm_catalog_id);
  te::da::Literal* literal = new te::da::LiteralInt32(catalogId);
  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  if (dataSet->moveNext())
  {
    rs = dynamic_cast<ta::model::RuleSet*>(ta::model::AbstractItemFactory::make(m_datasetName, dataSet.get()));
  }

  return rs;
}

bool ta::model::RuleSetFacade::createOutputRule(const int& ruleSetId, const int& catalogId, const ta::model::SpatialOperation& so)
{
  //create rule
  ta::model::RuleFacade rf(m_dataSource);

  ta::model::Rule r;
  r.setName(rf.createRuleName(catalogId, so));
  r.setCatalogId(catalogId);
  r.setOperationId(so);

  if (!rf.save(&r))
  {
    m_errorMessage = rf.getErrorMessage();
    return false;
  }

  //create association
  ta::model::RuleRuleSetFacade rrsf(m_dataSource);

  ta::model::RuleRuleSet rrs;
  rrs.setRuleId(r.getId());
  rrs.setRuleSetId(ruleSetId);
  rrs.setPriority(0); //default value

  if (!rrsf.save(&rrs))
  {
    m_errorMessage = rrsf.getErrorMessage();
    return false;
  }

  return true;
}

bool ta::model::RuleSetFacade::createAssociation(const int& ruleId, const int& ruleSetId, const int& priority)
{
  ta::model::RuleRuleSetFacade rrsf(m_dataSource);

  ta::model::RuleRuleSet rrs;
  rrs.setRuleId(ruleId);
  rrs.setRuleSetId(ruleSetId);
  rrs.setPriority(priority);

  if (!rrsf.save(&rrs))
  {
    m_errorMessage = rrsf.getErrorMessage();
    return false;
  }

  return true;
}

bool ta::model::RuleSetFacade::isOutputWithIntersection(const int& ruleSetId, ta::model::RuleSet* &output)
{
  te::da::Field* field = new te::da::Field(ta::model::Globals::sm_rulesetAlias + "." + "*");
  te::da::Fields* fields = new te::da::Fields();
  fields->push_back(field);

  te::da::FromItem* fItem1 = new te::da::DataSetName(ta::model::Globals::sm_catalog, ta::model::Globals::sm_catalogAlias);
  te::da::FromItem* fItem2 = new te::da::DataSetName(ta::model::Globals::sm_ruleset, ta::model::Globals::sm_rulesetAlias);
  te::da::FromItem* fItem3 = new te::da::DataSetName(ta::model::Globals::sm_ruleRuleset, ta::model::Globals::sm_ruleRulesetAlias);
  te::da::FromItem* fItem4 = new te::da::DataSetName(ta::model::Globals::sm_rule, ta::model::Globals::sm_ruleAlias);

  te::da::Expression* exp1 = new te::da::PropertyName(ta::model::Globals::sm_catalogAlias + "." + ta::model::Catalog::sm_id);
  te::da::Expression* exp2 = new te::da::PropertyName(ta::model::Globals::sm_rulesetAlias + "." + ta::model::RuleSet::sm_catalog_id);
  te::da::Expression* exp3 = new te::da::PropertyName(ta::model::Globals::sm_rulesetAlias + "." + ta::model::RuleSet::sm_id);
  te::da::Expression* exp4 = new te::da::PropertyName(ta::model::Globals::sm_ruleRulesetAlias + "." + ta::model::RuleRuleSet::sm_ruleset_id);
  te::da::Expression* exp5 = new te::da::PropertyName(ta::model::Globals::sm_ruleRulesetAlias + "." + ta::model::RuleRuleSet::sm_rule_id);
  te::da::Expression* exp6 = new te::da::PropertyName(ta::model::Globals::sm_ruleAlias + "." + ta::model::Rule::sm_id);

  te::da::Expression* expression1 = new te::da::BinaryFunction("=", exp1, exp2);
  te::da::Expression* expression2 = new te::da::BinaryFunction("=", exp3, exp4);
  te::da::Expression* expression3 = new te::da::BinaryFunction("=", exp5, exp6);

  te::da::Join* join1 = new te::da::Join(fItem1, fItem2, te::da::INNER_JOIN, new te::da::JoinConditionOn(expression1));
  te::da::Join* join2 = new te::da::Join(join1, fItem3, te::da::INNER_JOIN, new te::da::JoinConditionOn(expression2));
  te::da::Join* join3 = new te::da::Join(join2, fItem4, te::da::INNER_JOIN, new te::da::JoinConditionOn(expression3));

  te::da::From* from = new te::da::From();
  from->push_back(join3);

  te::da::PropertyName* pName1 = new te::da::PropertyName(ta::model::Globals::sm_ruleAlias + "." + ta::model::Rule::sm_operation_id);
  te::dt::AbstractData* opId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(ta::model::SO_INTERSECTION));
  te::da::Literal* literal1 = new te::da::Literal(opId);

  te::da::PropertyName* pName2 = new te::da::PropertyName(ta::model::Globals::sm_rulesetAlias + "." + ta::model::RuleSet::sm_id);
  te::dt::AbstractData* rsId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(ruleSetId));
  te::da::Literal* literal2 = new te::da::Literal(rsId);

  te::da::EqualTo* equal1 = new te::da::EqualTo(pName1, literal1);
  te::da::EqualTo* equal2 = new te::da::EqualTo(pName2, literal2);

  te::da::And* andClause = new te::da::And(equal1, equal2);

  te::da::Where* where = new te::da::Where(andClause);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  if (dataSet->moveNext())
  {
    ta::model::AbstractItem* ai = ta::model::AbstractItemFactory::make(ta::model::Globals::sm_ruleset, dataSet.get());
    output = dynamic_cast<ta::model::RuleSet*>(ai);
    return true;
  }

  return false;
}

bool ta::model::RuleSetFacade::removeAssociation(const int& ruleId, const int& ruleSetId)
{
  ta::model::RuleRuleSetFacade rrsf(m_dataSource);

  if (!rrsf.remove(ruleId, ruleSetId))
  {
    m_errorMessage = rrsf.getErrorMessage();
    return false;
  }

  return true;
}

std::vector<int> ta::model::RuleSetFacade::getRulesRuleSet(const int& ruleSetId)
{
  std::vector<int> ids;

  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_ruleRuleset);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::RuleRuleSet::sm_ruleset_id);

  te::dt::AbstractData* dataId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(ruleSetId));
  te::da::Literal* literal = new te::da::Literal(dataId);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    int id = dataSet->getInt32(ta::model::RuleRuleSet::sm_rule_id);

    ids.push_back(id);
  }

  return ids;
}

bool ta::model::RuleSetFacade::getRuleSetId(const int& catalogId, int& ruleSetId)
{
  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_ruleset);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::RuleSet::sm_catalog_id);

  te::dt::AbstractData* dataId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(catalogId));
  te::da::Literal* literal = new te::da::Literal(dataId);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  if (dataSet->isEmpty())
    return false;

  dataSet->moveFirst();

  ruleSetId = dataSet->getInt32(ta::model::RuleSet::sm_id);

  return true;
}

bool ta::model::RuleSetFacade::hasCatalog(const int& catalogId)
{
  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_ruleset);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::RuleSet::sm_catalog_id);

  te::dt::AbstractData* dataId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(catalogId));
  te::da::Literal* literal = new te::da::Literal(dataId);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  return !dataSet->isEmpty();
}

std::vector<int> ta::model::RuleSetFacade::getRuleSetClasses(const int& catalogId)
{
  std::vector<int> ids;

  int ruleSetId;
  if (getRuleSetId(catalogId, ruleSetId))
  {
    te::da::Field* field = new te::da::Field("*");
    te::da::Fields* fields = new te::da::Fields;
    fields->push_back(field);

    te::da::FromItem* fromItem = new te::da::DataSetName(ta::model::Globals::sm_rulesetClass);
    te::da::From* from = new te::da::From;
    from->push_back(fromItem);

    te::da::PropertyName* pName = new te::da::PropertyName(ta::model::RuleSetFacade::sm_ruleSetClass_ruleSetId);

    te::dt::AbstractData* dataId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(ruleSetId));
    te::da::Literal* literal = new te::da::Literal(dataId);

    te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
    te::da::Where* where = new te::da::Where(equal);

    te::da::Select* select = new te::da::Select(fields, from, where);

    std::auto_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

    dataSet->moveBeforeFirst();

    while (dataSet->moveNext())
    {
      ids.push_back(dataSet->getInt32(ta::model::RuleSetFacade::sm_ruleSetClass_classId));
    }
  }

  return ids;
}

bool ta::model::RuleSetFacade::addRuleSetClass(const int& catalogId, const int& classId)
{
  std::auto_ptr<te::da::DataSetType> newDst = m_dataSource->getDataSetType(ta::model::Globals::sm_rulesetClass);

  te::mem::DataSet* newData = new te::mem::DataSet(newDst.get());

  te::mem::DataSetItem* dsItem = new te::mem::DataSetItem(newData);

  int ruleSetId;
  if (!getRuleSetId(catalogId, ruleSetId))
  {
    m_errorMessage = "RuleSet not found for this Catalog.";
    return false;
  }

  dsItem->setValue(ta::model::RuleSetFacade::sm_ruleSetClass_ruleSetId, new te::dt::SimpleData<int, te::dt::INT32_TYPE>(ruleSetId));
  dsItem->setValue(ta::model::RuleSetFacade::sm_ruleSetClass_classId, new te::dt::SimpleData<int, te::dt::INT32_TYPE>(classId));

  newData->add(dsItem);

  std::map<std::string, std::string> op;

  std::auto_ptr<te::da::DataSourceTransactor> transactor = m_dataSource->getTransactor();

  try
  {
    transactor->add(ta::model::Globals::sm_rulesetClass, newData, op);

  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    transactor->rollBack();
    return false;
  }

  transactor->commit();

  return true;
}

bool ta::model::RuleSetFacade::addRuleSetClass(const int& ruleSetId, const int& classId, te::da::DataSourceTransactor* transactor)
{
  std::auto_ptr<te::da::DataSetType> newDst = m_dataSource->getDataSetType(ta::model::Globals::sm_rulesetClass);

  te::mem::DataSet* newData = new te::mem::DataSet(newDst.get());

  te::mem::DataSetItem* dsItem = new te::mem::DataSetItem(newData);

  dsItem->setValue(ta::model::RuleSetFacade::sm_ruleSetClass_ruleSetId, new te::dt::SimpleData<int, te::dt::INT32_TYPE>(ruleSetId));
  dsItem->setValue(ta::model::RuleSetFacade::sm_ruleSetClass_classId, new te::dt::SimpleData<int, te::dt::INT32_TYPE>(classId));

  newData->add(dsItem);

  std::map<std::string, std::string> op;

  try
  {
    transactor->add(ta::model::Globals::sm_rulesetClass, newData, op);

  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";
    return false;
  }

  return true;
}

bool ta::model::RuleSetFacade::removeRuleSetClass(const int& catalogId, const int& classId)
{
  int ruleSetId;
  if (!getRuleSetId(catalogId, ruleSetId))
  {
    m_errorMessage = "RuleSet not defined for this Catalog.";
    return false;
  }

  //TODO: refactoring...

  std::string sql = "DELETE FROM ";
  sql += ta::model::Globals::sm_rulesetClass;
  sql += " WHERE ";
  sql += ta::model::RuleSetFacade::sm_ruleSetClass_ruleSetId;
  sql += " = ";
  sql += te::common::Convert2String(ruleSetId);
  sql += " AND ";
  sql += ta::model::RuleSetFacade::sm_ruleSetClass_classId;
  sql += " = ";
  sql += te::common::Convert2String(classId);

  //remove element
  try
  {
    m_dataSource->execute(sql);
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    return false;
  }

  return true;
}

bool ta::model::RuleSetFacade::checkProperties(ta::model::AbstractItem* /*item*/)
{
  return true;
}

bool ta::model::RuleSetFacade::hasDependecy(const int& /*id*/)
{
  return false;
}
