/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/facade/UserFacade.cpp

\brief
*/

// TerraAmazon
#include "../Globals.h"
#include "../factory/AbstractItemFactory.h"
#include "../item/Group.h"
#include "../item/User.h"
#include "GroupFacade.h"
#include "UserFacade.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>
#include <terralib/dataaccess/query/And.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/Literal.h>
#include <terralib/dataaccess/query/LiteralString.h>
#include <terralib/dataaccess/query/NotEqualTo.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/memory/DataSet.h>
#include <terralib/memory/DataSetItem.h>

const std::string ta::model::UserFacade::sm_groupUser_groupId("group_id");
const std::string ta::model::UserFacade::sm_groupUser_userId("user_id");

ta::model::UserFacade::UserFacade(te::da::DataSourcePtr ds) : ta::model::AbstractFacade(ds)
{
  m_datasetName = ta::model::Globals::sm_user;
}

ta::model::UserFacade::~UserFacade()
{

}

bool ta::model::UserFacade::addAssociatedGroup(const int &groupId, const int &userId)
{
  std::unique_ptr<te::da::DataSetType> dstype = m_dataSource->getDataSetType(ta::model::Globals::sm_groupUser);

  te::mem::DataSet* newData = new te::mem::DataSet(dstype.get());

  te::mem::DataSetItem* dsItem = new te::mem::DataSetItem(newData);

  dsItem->setValue(sm_groupUser_groupId, new te::dt::SimpleData<int, te::dt::INT32_TYPE>(groupId));
  dsItem->setValue(sm_groupUser_userId, new te::dt::SimpleData<int, te::dt::INT32_TYPE>(userId));

  newData->add(dsItem);

  std::map<std::string, std::string> op;

  std::unique_ptr<te::da::DataSourceTransactor> transactor = m_dataSource->getTransactor();

  try
  {
    transactor->add(ta::model::Globals::sm_groupUser, newData, op);
  }
  catch (const te::common::Exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (const std::exception& e)
  {
    m_errorMessage = e.what();

    transactor->rollBack();
    return false;
  }
  catch (...)
  {
    m_errorMessage = "Unknown Excption.";

    transactor->rollBack();
    return false;
  }

  transactor->commit();
  return true;
}

bool ta::model::UserFacade::isAssociated(const int &groupId, const int &userId)
{
  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_groupUser);
  from->push_back(fItem);

  te::da::PropertyName* gPName = new te::da::PropertyName(ta::model::UserFacade::sm_groupUser_groupId);
  te::dt::AbstractData* gId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(groupId));
  te::da::Literal* gLiteral = new te::da::Literal(gId);
  te::da::EqualTo* gEqual = new te::da::EqualTo(gPName, gLiteral);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::UserFacade::sm_groupUser_userId);
  te::dt::AbstractData* uId = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(userId));
  te::da::Literal* pLiteral = new te::da::Literal(uId);
  te::da::EqualTo* pEqual = new te::da::EqualTo(pName, pLiteral);

  te::da::And* andClause = new te::da::And(gEqual, pEqual);

  te::da::Where* where = new te::da::Where(andClause);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::unique_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  if (dataSet->moveNext())
    return true;

  return false;
}

bool ta::model::UserFacade::existsByName(const std::string& userName)
{
  te::da::Fields fields = te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields.push_back(field);

  te::da::From from = te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_user);
  from.push_back(fItem);

  te::da::Select select = new te::da::Select(fields, from);

  std::unique_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    std::string name = dataSet->getString(ta::model::User::sm_name);

    if (te::common::Convert2UCase(name) == te::common::Convert2UCase(userName))
      return true;
  }

  return false;
}

std::vector<ta::model::User*> ta::model::UserFacade::getAllowedUsers()
{
  std::unique_ptr<ta::model::User> tmzUser = getByName(ta::model::Globals::sm_taUser);

  std::vector<ta::model::User*> vec;

  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(m_datasetName);
  from->push_back(fItem);

  te::da::Select select = new te::da::Select(fields, from);

  std::unique_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  while (dataSet->moveNext())
  {
    ta::model::AbstractItem* ai = ta::model::AbstractItemFactory::make(m_datasetName, dataSet.get());
    ta::model::User* user = dynamic_cast<ta::model::User*>(ai);
    if (tmzUser->getId() != user->getId())
      vec.push_back(dynamic_cast<ta::model::User*>(ai));
  }

  return vec;
}

std::unique_ptr<ta::model::User> ta::model::UserFacade::getByName(const std::string& userName)
{
  std::unique_ptr<ta::model::User> user;

  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName(m_datasetName);
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName(ta::model::User::sm_name);

  te::dt::AbstractData* dataId = new te::dt::SimpleData<std::string>("'" + userName + "'");
  te::da::Literal* literal = new te::da::Literal(dataId);

  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select* select = new te::da::Select(fields, from, where);

  std::unique_ptr<te::da::DataSet> dataSet;

  try
  {
    dataSet = m_dataSource->query(select);
  }
  catch (...)
  {
    return 0;
  }

  if (dataSet->size() != 1)
  {
    return 0;
  }

  dataSet->moveFirst();
  ta::model::AbstractItem* ai = ta::model::AbstractItemFactory::make(m_datasetName, dataSet.get());
  user.reset(dynamic_cast<ta::model::User*>(ai));

  return user;
}

std::vector<std::unique_ptr<ta::model::Group> > ta::model::UserFacade::getAssociatedGroups(int userId)
{
  te::da::Fields* fields = new te::da::Fields();
  te::da::Field* field = new te::da::Field(te::da::PropertyName("*"));
  fields->push_back(field);

  te::da::From* from = new te::da::From();
  te::da::FromItem* fItem = new te::da::DataSetName(ta::model::Globals::sm_groupUser);
  from->push_back(fItem);

  te::da::PropertyName* pName = new te::da::PropertyName("user_id");
  te::dt::AbstractData* id = new te::dt::SimpleData<std::string>(boost::lexical_cast<std::string>(userId));
  te::da::Literal* literal = new te::da::Literal(id);
  te::da::EqualTo* equal = new te::da::EqualTo(pName, literal);
  te::da::Where* where = new te::da::Where(equal);

  te::da::Select select = new te::da::Select(fields, from, where);

  std::unique_ptr<te::da::DataSet> dataSet = m_dataSource->query(select);

  dataSet->moveBeforeFirst();

  std::vector<std::unique_ptr<ta::model::Group> > groups;

  ta::model::GroupFacade gf(m_dataSource);

  while (dataSet->moveNext())
  {
    int id = dataSet->getInt32("group_id");
    groups.emplace_back(dynamic_cast<ta::model::Group*>(gf.get(id)));
  }

  return groups;
}

int ta::model::UserFacade::getPrivilege(int userId)
{
  std::vector<std::unique_ptr<ta::model::Group> > groups = getAssociatedGroups(userId);

  size_t numGroups = groups.size();
  int privilege = 0;

  for (size_t i = 0; i < numGroups; ++i)
  {
    int curPriv = groups[i]->getPrivileges();
    if (privilege < curPriv)
      privilege = curPriv;
  }

  return privilege;
}

bool ta::model::UserFacade::removeAssociatedGroup(const int &groupId, const int &userId)
{
  try
  {
    std::string sql = "DELETE FROM ";
    sql += ta::model::Globals::sm_groupUser;
    sql += " WHERE group_id = ";
    sql += te::common::Convert2String(groupId);
    sql += " AND user_id = ";
    sql += te::common::Convert2String(userId);

    m_dataSource->execute(sql);
  }
  catch (...)
  {
    return false;
  }
  return true;
}

bool ta::model::UserFacade::setUserProject(const int& user, const int& projId)
{
  std::unique_ptr<ta::model::User> userItem(dynamic_cast<ta::model::User*>(get(user)));

  if (!userItem.get())
    return false;

  userItem->setProjectId(projId);

  return update(userItem.get());
}

bool ta::model::UserFacade::checkProperties(ta::model::AbstractItem* /*item*/)
{
  return true;
}

bool ta::model::UserFacade::hasDependecy(const int& /*id*/)
{
  return false;
}
