/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/item/Project.h

\brief This file defines the Project model.
*/

// TerraAmazon
#include "ItemProperties.h"
#include "Project.h"

// TerraLib
#include <terralib/datatype/DateTimeInstant.h>
#include <terralib/datatype/DateTimeProperty.h>
#include <terralib/datatype/SimpleData.h>
#include <terralib/datatype/StringProperty.h>

const std::string ta::model::Project::sm_id("id");
const std::string ta::model::Project::sm_name("name");
const std::string ta::model::Project::sm_active("active");
const std::string ta::model::Project::sm_minScale("min_scale");
const std::string ta::model::Project::sm_maxScale("max_scale");
const std::string ta::model::Project::sm_created("created_on");
const std::string ta::model::Project::sm_updated("updated_on");
const std::string ta::model::Project::sm_maxCells("maxcells");

ta::model::Project::Project()
  : AbstractItem(),
  m_name(""),
  m_active(false),
  m_minScale(std::numeric_limits<double>::max()),
  m_maxScale(-std::numeric_limits<double>::max()),
  m_created(0),
  m_updated(0),
  m_maxCells(0)
{

}

ta::model::Project::~Project()
{
  delete m_created;
  delete m_updated;
}

ta::model::ItemPropertyVector ta::model::Project::toProperties()
{
  ta::model::ItemPropertyVector ipv;

  {
    if (m_id != -1)
    {
      te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Project::sm_id, te::dt::INT32_TYPE);
      te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_id);
      ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Project::sm_id);
      mep->setProperty(p);
      mep->setData(ad);

      addProperty(ta::model::Project::sm_id, mep, ipv);
    }
  }

  {
    te::dt::Property* p = new te::dt::StringProperty(ta::model::Project::sm_name, te::dt::VAR_STRING, 255);
    te::dt::AbstractData* ad = new te::dt::SimpleData<std::string, te::dt::STRING_TYPE>(m_name);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Project::sm_name);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Project::sm_name, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Project::sm_minScale, te::dt::DOUBLE_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<double, te::dt::DOUBLE_TYPE>(m_minScale);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Project::sm_minScale);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Project::sm_minScale, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Project::sm_maxScale, te::dt::DOUBLE_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<double, te::dt::DOUBLE_TYPE>(m_maxScale);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Project::sm_maxScale);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Project::sm_maxScale, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::DateTimeProperty(ta::model::Project::sm_created, te::dt::DateTimeType::DATE);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Project::sm_created);
    mep->setProperty(p);
    mep->setData(m_created->clone());

    addProperty(ta::model::Project::sm_created, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::DateTimeProperty(ta::model::Project::sm_updated, te::dt::DateTimeType::DATE);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Project::sm_updated);
    mep->setProperty(p);
    mep->setData(m_updated->clone());

    addProperty(ta::model::Project::sm_updated, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Project::sm_maxCells, te::dt::INT32_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<int, te::dt::INT32_TYPE>((int)m_maxCells);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Project::sm_maxCells);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Project::sm_maxCells, mep, ipv);
  }

  {
    te::dt::Property* p = new te::dt::SimpleProperty(ta::model::Project::sm_active, te::dt::BOOLEAN_TYPE);
    te::dt::AbstractData* ad = new te::dt::SimpleData<bool, te::dt::BOOLEAN_TYPE>(m_active);
    ta::model::ItemProperties* mep = new ta::model::ItemProperties(ta::model::Project::sm_active);
    mep->setProperty(p);
    mep->setData(ad);

    addProperty(ta::model::Project::sm_active, mep, ipv);
  }

  return ipv;
}

std::string ta::model::Project::getName()
{
  return m_name;
}

double ta::model::Project::getMinScale()
{
  return m_minScale;
}

double ta::model::Project::getMaxScale()
{
  return m_maxScale;
}

te::dt::Date* ta::model::Project::getCreated()
{
  return m_created;
}

te::dt::Date* ta::model::Project::getUpdated()
{
  return m_updated;
}

std::size_t ta::model::Project::getMaxCells()
{
  return m_maxCells;
}

bool ta::model::Project::isActive()
{
  return m_active;
}

void ta::model::Project::setName(const std::string& name)
{
  m_name = name;
}

void ta::model::Project::setMinScale(const double& scale)
{
  m_minScale = scale;
}

void ta::model::Project::setMaxScale(const double& scale)
{
  m_maxScale = scale;
}

void ta::model::Project::setCreated(te::dt::Date* date)
{
  m_created = date;
}

void ta::model::Project::setUpdated(te::dt::Date* date)
{
  m_updated = date;
}

void ta::model::Project::setMaxCells(const std::size_t& maxCells)
{
  m_maxCells = maxCells;
}

void ta::model::Project::setActive(const bool& active)
{
  m_active = active;
}
