/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

//Terralib
#include <terralib/dataaccess/datasource/DataSourceManager.h>

// TerraAmazon
#include "../../facade/GroupFacade.h"
#include "../../item/Group.h"
#include "GroupControlWidget.h"
#include "ui_GroupControlWidgetForm.h"

//Qt
#include <QCheckBox>
#include <QMessageBox>
#include <QToolButton>

ta::model::qt::GroupControlWidget::GroupControlWidget(QWidget* parent, Qt::WindowFlags f)
  : QWidget(parent, f),
  m_ui(new Ui::GroupControlWidgetForm)
{
  m_ui->setupUi(this);

  m_ui->m_addGroupToolButton->setIcon(QIcon::fromTheme("list-add"));

  //mapper initializer
  m_updateMapper = new QSignalMapper(this);
  m_removeMapper = new QSignalMapper(this);

  connect(m_ui->m_addGroupToolButton, SIGNAL(clicked()), this, SLOT(onAddGroupButtonClicked()));
  connect(m_updateMapper, SIGNAL(mapped(int)), this, SLOT(onUpdateGroupExecuted(int)));
  connect(m_removeMapper, SIGNAL(mapped(int)), this, SLOT(onRemoveGroupExecuted(int)));
}

ta::model::qt::GroupControlWidget::~GroupControlWidget()
{
}

void ta::model::qt::GroupControlWidget::loadGroups()
{
  if (!m_activeServer.get())
    return;

  //Loading all the users from the active server
  std::vector<ta::model::AbstractItem*> groups = m_facade->get();
  std::vector<std::unique_ptr<ta::model::Group> > gPtrs;

  size_t grpNmbr = groups.size();
  for (std::size_t t = 0; t < grpNmbr; ++t)
    gPtrs.emplace_back(dynamic_cast<ta::model::Group*>(groups[t]));

  for (std::size_t i = 0; i < grpNmbr; ++i)
  {
    addTableRow(m_ui->m_groupTableWidget->rowCount(), m_ui->m_groupTableWidget, gPtrs[i]->getName(), gPtrs[i]->getPrivileges(), true);
    m_ui->m_groupTableWidget->item((int)i, 1)->setData(Qt::UserRole, QVariant::fromValue(gPtrs[i]->getId()));
  }
  gPtrs.clear();
}

void ta::model::qt::GroupControlWidget::setServer(te::da::DataSourceInfoPtr server)
{
  if (server.get())
  {
    //get data source
    m_activeServer = te::da::DataSourceManager::getInstance().get(server->getId(), server->getType(), server->getConnInfo());

    if (!m_activeServer.get())
      return;

    m_activeServer->open();
    m_facade.reset(new ta::model::GroupFacade(m_activeServer));
    loadGroups();
  }
}

void ta::model::qt::GroupControlWidget::onAddGroupButtonClicked()
{
  if (m_ui->m_groupNameLineEdit->text().isEmpty())
  {
    QMessageBox::warning(this, tr("Warning"), tr("Inform the name of the new group."));
    return;
  }

  if (m_activeServer.get())
  {
    std::unique_ptr<ta::model::Group> newGroup (new ta::model::Group());
    newGroup->setName(m_ui->m_groupNameLineEdit->text().toStdString());
    if (m_facade->existsByName(newGroup->getName()))
      QMessageBox::warning(this, "Warning", tr("The informed group already exists."));
    else
    {
      addTableRow(m_ui->m_groupTableWidget->rowCount(), m_ui->m_groupTableWidget, newGroup->getName());

      if (m_facade->save(newGroup.get()))
      {
        m_ui->m_groupTableWidget->item((m_ui->m_groupTableWidget->rowCount() - 1), 1)->setData(Qt::UserRole, QVariant::fromValue(newGroup->getId()));
        added((m_ui->m_groupTableWidget->rowCount() - 1));
      }
      else
        QMessageBox::warning(this, windowTitle(), m_facade->getErrorMessage().c_str());


      m_ui->m_groupNameLineEdit->setText("");
    }
  }
  else
  {
    QMessageBox::warning(this, "Warning", tr("There is no active server."));
  }
}

void ta::model::qt::GroupControlWidget::onUpdateGroupExecuted(int row)
{
  //Acquiring the name of the cconfigured group and it's privileges
  QCheckBox* admCheck = dynamic_cast<QCheckBox*>(m_ui->m_groupTableWidget->cellWidget(row, 2));
  QCheckBox* projManCheck = dynamic_cast<QCheckBox*>(m_ui->m_groupTableWidget->cellWidget(row, 3));
  std::string name = m_ui->m_groupTableWidget->item(row, 1)->text().toStdString();
  int id = m_ui->m_groupTableWidget->item(row, 1)->data(Qt::UserRole).toInt();
  std::unique_ptr<ta::model::Group> curGroup(dynamic_cast<ta::model::Group*>(m_facade->get(id)));

  if (curGroup.get())
  {
    curGroup->setName(name);
    if (admCheck->isChecked())
    {
      projManCheck->setChecked(true);
      curGroup->setPrivileges(7);
    }
    else if (projManCheck->isChecked())
      curGroup->setPrivileges(3);
    else
      curGroup->setPrivileges(1);

    if(!m_facade->update(curGroup.get()))
      QMessageBox::warning(this, windowTitle(), m_facade->getErrorMessage().c_str());
    else
      QMessageBox::information(this, windowTitle(), (curGroup->getName() + " information updated.").c_str());
  }
}

void ta::model::qt::GroupControlWidget::onRemoveGroupExecuted(int row)
{
  if (row >= 0)
  {
    int id = m_ui->m_groupTableWidget->item(row, 1)->data(Qt::UserRole).toInt();
    if (m_facade->remove(id))
    {
      removed(row);
      m_ui->m_groupTableWidget->removeRow(row);
    }
    else
      QMessageBox::warning(this, windowTitle(), m_facade->getErrorMessage().c_str());
  }
}

void ta::model::qt::GroupControlWidget::addTableRow(int newRow, QTableWidget* groupTable, std::string groupName, int groupPrivilege, bool fromDb)
{
  groupTable->insertRow(newRow);

  QTableWidgetItem* nameItem = new QTableWidgetItem(QString::fromStdString(groupName));
  nameItem->setFlags(Qt::ItemIsEnabled | Qt::ItemIsEditable);
  groupTable->setItem(newRow, 1, nameItem);

  //A checkbox used to know the permissions associated with thew administrator group
  QCheckBox* admCheck = new QCheckBox(groupTable);

  groupTable->setCellWidget(newRow, 2, admCheck);

  //A checkbox used to know the permissions associated with thew operator group
  QCheckBox* projManCheck = new QCheckBox(groupTable);

  groupTable->setCellWidget(newRow, 3, projManCheck);

  //A checkbox used to know the permissions associated with thew operator group
  QCheckBox* opCheck = new QCheckBox(groupTable);
  opCheck->setCheckState(Qt::Checked);
  opCheck->setDisabled(true);

  groupTable->setCellWidget(newRow, 4, opCheck);

  if (groupPrivilege == 7)
  {
    admCheck->setCheckState(Qt::Checked);
    projManCheck->setCheckState(Qt::Checked);
  }
  else if (groupPrivilege == 3)
    projManCheck->setCheckState(Qt::Checked);

  //actions
  QWidget* wActions = new QWidget(groupTable);

  //A ToolButton used to save a group from the list
  QToolButton* updateButton = new QToolButton(groupTable);
  updateButton->setObjectName("UPDATE");
  updateButton->setAutoRaise(true);
  updateButton->setEnabled(fromDb);
  updateButton->setIcon(QIcon::fromTheme("view-refresh"));

  //A ToolButton used to remove a group from the list
  QToolButton* remButton = new QToolButton(groupTable);
  remButton->setObjectName("REMOVE");
  remButton->setAutoRaise(true);
  remButton->setEnabled(fromDb);
  remButton->setIcon(QIcon::fromTheme("list-remove"));

  QGridLayout* layout = new QGridLayout(wActions);
  layout->setContentsMargins(0, 0, 0, 0);

  layout->addWidget(updateButton, 0, 0);
  layout->addWidget(remButton, 0, 1);

  groupTable->setCellWidget(newRow, 0, wActions);

  //Mapping the signals

  m_updateMapper->setMapping(updateButton, newRow);
  connect(updateButton, SIGNAL(clicked()), m_updateMapper, SLOT(map()));

  m_removeMapper->setMapping(remButton, newRow);
  connect(remButton, SIGNAL(clicked()), m_removeMapper, SLOT(map()));

  groupTable->resizeColumnToContents(0);
  groupTable->resizeColumnToContents(1);
  groupTable->horizontalHeader()->setSectionResizeMode(2, QHeaderView::Stretch);
  groupTable->horizontalHeader()->setSectionResizeMode(3, QHeaderView::Stretch);
  groupTable->horizontalHeader()->setSectionResizeMode(4, QHeaderView::Stretch);
}

void ta::model::qt::GroupControlWidget::added(int row)
{
  QWidget* w = m_ui->m_groupTableWidget->cellWidget(row, 0);

  if (!w)
    return;

  QToolButton* updateToolButton = w->findChild<QToolButton*>("UPDATE");
  updateToolButton->setEnabled(true);

  QToolButton* removeToolButton = w->findChild<QToolButton*>("REMOVE");
  removeToolButton->setEnabled(true);
}

void ta::model::qt::GroupControlWidget::removed(int row)
{
  size_t rows = m_ui->m_groupTableWidget->rowCount();

  for (size_t i = (row + 1); row <= rows; ++i)
  {
    QWidget* w = m_ui->m_groupTableWidget->cellWidget((int)i, 0);

    if (!w)
      return;

    //Re-mapping the signals
    QToolButton* updateToolButton = w->findChild<QToolButton*>("UPDATE");
    m_updateMapper->setMapping(updateToolButton, ((int)i - 1));

    QToolButton* removeToolButton = w->findChild<QToolButton*>("REMOVE");
    m_removeMapper->setMapping(removeToolButton, ((int)i - 1));
  }
}
