/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

//Terralib
#include <terralib/common/STLUtils.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>

// TerraAmazon
#include "../../facade/GroupFacade.h"
#include "../../facade/UserFacade.h"
#include "../../Globals.h"
#include "../../item/Group.h"
#include "../../item/User.h"
#include "common/Utils.h"
#include "UserControlWidget.h"
#include "ui_UserControlWidgetForm.h"

//Qt
#include <QMessageBox>
#include <QToolButton>

ta::model::qt::UserControlWidget::UserControlWidget(QWidget* parent, Qt::WindowFlags f)
  : QWidget(parent, f),
  m_ui(new Ui::UserControlWidgetForm)
{
  m_ui->setupUi(this);
  m_ui->m_addUserToolButton->setIcon(QIcon::fromTheme("list-add"));


  //mapper initializer
  m_updateMapper = new QSignalMapper(this);

  connect(m_ui->m_addUserToolButton, SIGNAL(clicked()), this, SLOT(onAddUserButtonClicked()));
  connect(m_ui->m_userTableWidget, SIGNAL(cellClicked(int, int)), this, SLOT(onCellClicked(int, int)));
  connect(m_ui->m_userTableWidget->horizontalHeader(), SIGNAL(sectionClicked(int)), this, SLOT(onSortOccurred(int)));
  connect(m_updateMapper, SIGNAL(mapped(int)), this, SLOT(onUpdateUserExecuted(int)));
}

ta::model::qt::UserControlWidget::~UserControlWidget()
{
}

void ta::model::qt::UserControlWidget::loadUsers()
{
  if (!m_activeServer.get())
    return;

  //Loading all the users from the active server
  std::vector<ta::model::User*> users = m_facade->getAllowedUsers();
  std::vector<std::unique_ptr<ta::model::User> > uPtrs;

  size_t usrNmbr = users.size();

  for (std::size_t t = 0; t < usrNmbr; ++t)
    uPtrs.emplace_back(users[t]);

  for (std::size_t i = 0; i < usrNmbr; ++i)
  {
    addTableRow(m_ui->m_userTableWidget->rowCount(), m_ui->m_userTableWidget, uPtrs[i].get(), true);
    m_ui->m_userTableWidget->item((int)i, 1)->setData(Qt::UserRole, QVariant::fromValue(uPtrs[i]->getId()));
  }

  uPtrs.clear();
}

void ta::model::qt::UserControlWidget::setUser(int userId)
{
  if (m_activeServer.get())
  {
   std::unique_ptr<ta::model::User> user (dynamic_cast<ta::model::User*>(m_facade->get(userId)));
   if (user.get())
     m_activeUser = std::move(user);
  }
}

void ta::model::qt::UserControlWidget::setServer(te::da::DataSourceInfoPtr server)
{
  if (server.get())
  {
    //get data source
    m_activeServer = te::da::DataSourceManager::getInstance().get(server->getId(), server->getType(), server->getConnInfo());

    if (!m_activeServer.get())
      return;

    m_activeServer->open();
    m_facade.reset(new ta::model::UserFacade(m_activeServer));

    loadUsers();

        //Loading the group list
    ta::model::GroupFacade gf(m_activeServer);
    std::vector<ta::model::AbstractItem*> groups = gf.get();

    size_t gSize = groups.size();
    std::unique_ptr<ta::model::Group> operatorGroup = gf.getByName(ta::model::Globals::sm_operatorGroup);


    for (size_t i = 0; i < gSize; ++i)
    {
      std::unique_ptr<ta::model::Group> curGroup(dynamic_cast<ta::model::Group*>(groups[i]));
      QListWidgetItem* groupItem = new QListWidgetItem(QString::fromStdString(curGroup->getName()), m_ui->m_groupListWidget);
      groupItem->setData(Qt::UserRole, curGroup->getId());

      if (curGroup->getId() == operatorGroup->getId())
      {
        groupItem->setCheckState(Qt::Checked);
        groupItem->setFlags(Qt::NoItemFlags);
      }
      else
      {
        groupItem->setFlags(groupItem->flags() | Qt::ItemIsUserCheckable);
        groupItem->setCheckState(Qt::Unchecked);
      }

      m_ui->m_groupListWidget->addItem(groupItem);
    }
  }
}

void ta::model::qt::UserControlWidget::onAddUserButtonClicked()
{
  if (m_ui->m_userNameLineEdit->text().isEmpty())
  {
    QMessageBox::warning(this, tr("Warning"), tr("Inform the name of the new user."));
    return;
  }

  if (m_activeServer.get())
  {
    std::unique_ptr<ta::model::User> newUser(new ta::model::User());
    newUser->setName(m_ui->m_userNameLineEdit->text().toStdString());
    newUser->setPassword(sha256(ta::model::Globals::sm_taDefaultPassword));

    if (m_facade->existsByName(newUser->getName()))
      QMessageBox::warning(this, "Warning", tr("The informed user already exists."));
    else
    {
      addTableRow(m_ui->m_userTableWidget->rowCount(), m_ui->m_userTableWidget, newUser.get());
      
      if (m_facade->save(newUser.get()))
      {
        m_ui->m_userTableWidget->item((m_ui->m_userTableWidget->rowCount() -1), 1)->setData(Qt::UserRole, QVariant::fromValue(newUser->getId()));

        size_t gsize = m_ui->m_groupListWidget->count();

        for (size_t i = 0; i < gsize; ++i)
        {
          if (m_ui->m_groupListWidget->item((int)i)->checkState() == Qt::Checked)
            m_facade->addAssociatedGroup(m_ui->m_groupListWidget->item((int)i)->data(Qt::UserRole).toInt(), newUser->getId());
        }

        added((m_ui->m_userTableWidget->rowCount() - 1));
      }
      else
        QMessageBox::warning(this, windowTitle(), m_facade->getErrorMessage().c_str());

      m_ui->m_userNameLineEdit->setText("");
    }
  }
  else
  {
    QMessageBox::warning(this, "Warning", tr("There is no active server."));
  }
  onSortOccurred(0);
}

void ta::model::qt::UserControlWidget::onCellClicked(int row, int /*column*/)
{
  int uId = m_ui->m_userTableWidget->item(row, 1)->data(Qt::UserRole).toInt();
  ta::model::GroupFacade gf(m_activeServer);
  std::unique_ptr<ta::model::Group> operatorGroup = gf.getByName(ta::model::Globals::sm_operatorGroup);

  size_t gsize = m_ui->m_groupListWidget->count();
  
  for (size_t i = 0; i < gsize; ++i)
  {
    int gId = m_ui->m_groupListWidget->item((int)i)->data(Qt::UserRole).toInt();
    if (gId != operatorGroup->getId())
    {
      if (m_facade->isAssociated(gId, uId))
        m_ui->m_groupListWidget->item((int)i)->setCheckState(Qt::Checked);
      else
        m_ui->m_groupListWidget->item((int)i)->setCheckState(Qt::Unchecked);
    }
  }
}

void ta::model::qt::UserControlWidget::onUpdateUserExecuted(int row)
{
  //Acquiring the name and the eMail of the cconfigured user
  std::string name, eMail;
  name = m_ui->m_userTableWidget->item(row, 1)->text().toStdString();
  eMail = m_ui->m_userTableWidget->item(row, 2)->text().toStdString();

  int id = m_ui->m_userTableWidget->item(row, 1)->data(Qt::UserRole).toInt();
  std::unique_ptr<ta::model::User> curUser(dynamic_cast<ta::model::User*>(m_facade->get(id)));

  if (curUser.get())
  {
    curUser->setName(name);
    curUser->setEMail(eMail);
    if (m_facade->update(curUser.get()))
    {
      size_t gsize = m_ui->m_groupListWidget->count();

      for (size_t i = 0; i < gsize; ++i)
      {
        if (m_ui->m_groupListWidget->item((int)i)->checkState() == Qt::Checked)
          m_facade->addAssociatedGroup(m_ui->m_groupListWidget->item((int)i)->data(Qt::UserRole).toInt(), curUser->getId());
        else
          m_facade->removeAssociatedGroup(m_ui->m_groupListWidget->item((int)i)->data(Qt::UserRole).toInt(), curUser->getId());
      }
      QMessageBox::information(this, windowTitle(), (curUser->getName() + " information updated.").c_str());
    }
    else
      QMessageBox::warning(this, windowTitle(), m_facade->getErrorMessage().c_str());
  }
}

void ta::model::qt::UserControlWidget::onSortOccurred(int /*column*/)
{
  size_t rows = m_ui->m_userTableWidget->rowCount();

  for (size_t row = 0; row <= rows; ++row)
  {
    QWidget* w = m_ui->m_userTableWidget->cellWidget((int)row, 0);

    if (!w)
      return;

    //Re-mapping the signals
    QToolButton* updateToolButton = w->findChild<QToolButton*>("UPDATE");
    m_updateMapper->setMapping(updateToolButton, (int)row);
  }
}

void ta::model::qt::UserControlWidget::addTableRow(int newRow, QTableWidget* userTable, ta::model::User* newUser, bool fromDb)
{
  m_ui->m_userTableWidget->setSortingEnabled(false);
  userTable->insertRow(newRow);

  QTableWidgetItem* nameItem = new QTableWidgetItem(QString::fromStdString(newUser->getName()));
  userTable->setItem(newRow, 1, nameItem);

  QTableWidgetItem* eMailItem = new QTableWidgetItem(QString::fromStdString(newUser->getEMail()));
  userTable->setItem(newRow, 2, eMailItem);

  //actions
  QWidget* wActions = new QWidget(userTable);

  //A ToolButton used to update an user's data
  QToolButton* updateButton = new QToolButton(userTable);
  updateButton->setObjectName("UPDATE");
  updateButton->setAutoRaise(true);
  updateButton->setEnabled(fromDb);
  updateButton->setIcon(QIcon::fromTheme("view-refresh"));

  QGridLayout* layout = new QGridLayout(wActions);
  layout->setContentsMargins(0, 0, 0, 0);

  layout->addWidget(updateButton);

  userTable->setCellWidget(newRow, 0, wActions);

  //Mapping the signals

  m_updateMapper->setMapping(updateButton, newRow);
  connect(updateButton, SIGNAL(clicked()), m_updateMapper, SLOT(map()));

  userTable->resizeColumnsToContents();
  userTable->horizontalHeader()->setSectionResizeMode(2, QHeaderView::Stretch);

  m_ui->m_userTableWidget->setSortingEnabled(true);
}

void ta::model::qt::UserControlWidget::added(int row)
{
  QWidget* w = m_ui->m_userTableWidget->cellWidget(row, 0);

  if (!w)
    return;

  QToolButton* updateToolButton = w->findChild<QToolButton*>("UPDATE");
  updateToolButton->setEnabled(true);
}
