/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/CatalogManagerDialog.cpp

\brief
*/

// TerraAmazon
#include "../../../common/Utils.h"
#include "../../facade/CatalogFacade.h"
#include "../../facade/UserFacade.h"
#include "CatalogManagerDialog.h"
#include "ui_CatalogManagerDialogForm.h"

// TerraLib
#include <terralib/common/PlatformUtils.h>
#include <terralib/common/StringUtils.h>
#include <terralib/datatype/Enums.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>
#include <terralib/dataaccess/dataset/DataSetAdapter.h>
#include <terralib/dataaccess/dataset/DataSetTypeConverter.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>
#include <terralib/dataaccess/datasource/DataSourceInfoManager.h>
#include <terralib/dataaccess/utils/Utils.h>
#include <terralib/datatype/TimeInstant.h>
#include <terralib/geometry/GeometryProperty.h>
#include <terralib/qt/widgets/se/PolygonSymbolizerProperty.h>
#include <terralib/qt/widgets/se/LineSymbolizerProperty.h>
#include <terralib/qt/widgets/se/PointSymbolizerProperty.h>
#include <terralib/qt/widgets/se/RasterSymbolizerWidget.h>
#include <terralib/qt/widgets/se/SymbolPreviewWidget.h>
#include <terralib/qt/widgets/se/StyleControllerWidget.h>
#include <terralib/qt/widgets/se/StyleExplorer.h>
#include <terralib/se/Rule.h>

// Qt
#include <QMessageBox>

Q_DECLARE_METATYPE(te::map::AbstractLayerPtr);

ta::model::qt::CatalogManagerDialog::CatalogManagerDialog(QWidget* parent, Qt::WindowFlags f)
: QDialog(parent, f),
m_ui(new Ui::CatalogManagerDialogForm)
{
  m_ui->setupUi(this);

  std::string icon = ta::common::FindInTerraAmazonPath("share/icons/svg/terraAmazom-colorido-horizontal.svg");
  m_ui->m_imgLabel->setPixmap(QIcon(icon.c_str()).pixmap(112, 100));
  m_ui->m_imgLabel->hide();

  //create layer
  m_createLayerUi = new te::qt::widgets::CreateLayerWidget(m_ui->m_createLayerWidget);
  QGridLayout* createLayerLayout = new QGridLayout(m_ui->m_createLayerWidget);
  createLayerLayout->setContentsMargins(0, 0, 0, 0);
  createLayerLayout->addWidget(m_createLayerUi);

  //mapper initializer
  m_updateButtonMapper = new QSignalMapper(this);
  m_removeButtonMapper = new QSignalMapper(this);

  m_ui->m_addLayerToolButton->setIcon(QIcon::fromTheme("list-add"));

  m_ui->m_createLayerPushButton->setIcon(QIcon::fromTheme("list-add"));

  m_ui->m_updateItemToolButton->setIcon(QIcon::fromTheme("document-save"));
  m_ui->m_updateItemToolButton->setVisible(false);
  m_ui->m_catalogsTableWidget->setSelectionBehavior(QAbstractItemView::SelectRows);

  connect(m_ui->m_addLayerToolButton, SIGNAL(pressed()), this, SLOT(onAddLayerToolButtonPressed()));
  connect(m_ui->m_createLayerPushButton, SIGNAL(clicked()), this, SLOT(onCreateLayerPushButtonClicked()));
  connect(m_ui->m_catalogStyleComboBox, SIGNAL(activated(int)), this, SLOT(onCatalogStyleComboBoxActivated(int)));
  connect(m_updateButtonMapper, SIGNAL(mapped(int)), this, SLOT(onUpdateButtonClicked(int)));
  connect(m_removeButtonMapper, SIGNAL(mapped(int)), this, SLOT(onRemoveButtonClicked(int)));
  connect(m_ui->m_catalogsTableWidget, SIGNAL(currentCellChanged(int, int, int, int)), this, SLOT(onCurrentCellTableWidgetChanged(int, int, int, int)));
  connect(m_ui->m_applyStylePushButton, SIGNAL(clicked()), this, SLOT(onApplyStylePushButtonClicked()));


  buildStyleWidgets();
}

ta::model::qt::CatalogManagerDialog::~CatalogManagerDialog()
{
}

void ta::model::qt::CatalogManagerDialog::setServer(te::da::DataSourceInfoPtr server)
{
  if (server.get())
  {
    //m_activeServer = te::da::DataSourceManager::getInstance().get(server->getId(), server->getType(), server->getConnInfo());

    m_activeServer = te::da::GetDataSource(server->getId());

    if (!m_activeServer.get())
      return;

    m_catalogFacade.reset(new ta::model::CatalogFacade(m_activeServer));
    m_ruleFacade.reset(new ta::model::RuleFacade(m_activeServer));
    m_ruleSetFacade.reset(new ta::model::RuleSetFacade(m_activeServer));
    m_userFacade.reset(new ta::model::UserFacade(m_activeServer));

    m_createLayerUi->setDataSource(server);

    updateCatalogTableWidget();
  }
}

void ta::model::qt::CatalogManagerDialog::setUsername(std::string userName)
{
  m_userName = userName;
}

void ta::model::qt::CatalogManagerDialog::onUpdateButtonClicked(int row)
{
  std::string str = "Are you sure you want to update the layer (";
  str += m_ui->m_catalogsTableWidget->item(row, 1)->text().toStdString();
  str += ")?";
  int reply = QMessageBox::question(0, "TerraAmazon", tr(str.c_str()), QMessageBox::No, QMessageBox::Yes);
  if (reply == QMessageBox::Yes)
  {
    QTableWidget* tw = m_ui->m_catalogsTableWidget;

    int id = tw->item(row, 1)->data(Qt::UserRole).toInt();
    ta::model::Catalog* c = dynamic_cast<ta::model::Catalog*>(m_catalogFacade->get(id));

    QComboBox* cbLocked = dynamic_cast<QComboBox*>(tw->cellWidget(row, 2));
    bool locked = false;
    if (cbLocked->currentData().toInt() == 0)
      locked = false;
    else if (cbLocked->currentData().toInt() == 1)
      locked = true;

    QComboBox* cbCellColumn = dynamic_cast<QComboBox*>(tw->cellWidget(row, 3));
    std::string cell_column = cbCellColumn->currentData(Qt::UserRole).toString().toStdString();
    std::string source = tw->item(row, 4)->text().toStdString();
    std::string description = m_ui->m_descriptionLineEdit->text().toStdString();

    c->setLocked(locked);
    c->setCellColumn(cell_column);
    c->setSource(source);
    c->setDescription(description);

    m_catalogFacade->update(c);

    updateCatalogTableWidget();
  }
}

void ta::model::qt::CatalogManagerDialog::onRemoveButtonClicked(int row)
{
  std::string str = "Are you sure you want to remove the layer (";
  str += m_ui->m_catalogsTableWidget->item(row, 1)->text().toStdString();
  str += ")?";
  int reply = QMessageBox::question(0, "TerraAmazon", tr(str.c_str()), QMessageBox::No, QMessageBox::Yes);
  if (reply == QMessageBox::Yes)
  {
    int id = m_ui->m_catalogsTableWidget->item(row, 1)->data(Qt::UserRole).toInt();

    if (m_catalogFacade->remove(id))
    {
      updateCatalogTableWidget();
      setLayers(m_layers);
    }
    else
    {
      QMessageBox::warning(this, "TerraAmazon", "This layer can not be removed.");
    }
  }
}

void ta::model::qt::CatalogManagerDialog::onCatalogStyleComboBoxActivated(int /*idx*/)
{
  int catalogId = m_ui->m_catalogStyleComboBox->currentData().toInt();

  ta::model::CatalogFacade cf(m_activeServer);

  ta::model::Catalog* c = dynamic_cast<ta::model::Catalog*>(cf.get(catalogId));

  if (c)
  {
    std::string strStyle = c->getStyle();

    if (!strStyle.empty())
    {
      m_style = ta::common::Read(strStyle);

      m_styleController->setStyle(m_style.get());

      onMapRefresh();
    }
    else
    {
      m_styleController->setStyle(0);

      m_symbolPreview->clear();

      setTabStatus(false);
    }
  }

  delete c;
}

void ta::model::qt::CatalogManagerDialog::onCreateLayerPushButtonClicked()
{
  std::string errorMessage = "";

  bool res = m_createLayerUi->createLayer(errorMessage);

  if (!res)
  {
    QMessageBox::warning(this, tr("Warning"), errorMessage.c_str());
    return;
  }

  te::map::AbstractLayerPtr layer = m_createLayerUi->getLayer();

  if (!layer.get())
  {
    QMessageBox::warning(this, tr("Warning"), tr("Error creating layer."));
    return;
  }

  m_createLayerUi->clear();

  //get geometry type

  std::string style = generateStyle(layer);

  //add new layer to catalog
  te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().get(m_activeServer->getId());

  ta::model::Catalog* item = new ta::model::Catalog();
  item->setName(layer->getTitle());
  item->setUri(dsInfo->getConnInfoAsString());
  item->setType(dsInfo->getType());
  item->setAuthor("unknow user");
  item->setSource("Created with the Create Layer Interface.");
  item->setPermission(1);
  item->setStyle(style);
  item->setLocked(true);

  boost::posix_time::ptime boostLocalTime = boost::posix_time::second_clock::local_time();
  te::dt::TimeInstant* localTime = new te::dt::TimeInstant(boostLocalTime);
  item->setLastEdition(localTime);

  item->setScaleMin(1);
  item->setScaleMax(1000000);

  //create facade
  ta::model::CatalogFacade cf(m_activeServer);

  if (!cf.save(item))
  {
    QMessageBox::warning(this, tr("Warning"), cf.getErrorMessage().c_str());
    return;
  }

  updateCatalogTableWidget();

  QMessageBox::information(this, tr("Warning"), tr("Layer Created"));
}

void ta::model::qt::CatalogManagerDialog::onSymbolizerSelected(te::se::Symbolizer* s)
{
  setTabStatus(false);

  if (s->getType() == "PolygonSymbolizer")
  {
    m_ui->m_styleTabWidget->setTabEnabled(1, true);
    m_polyWidget->setSymbolizer(dynamic_cast<te::se::PolygonSymbolizer*>(s));
  }
  else if (s->getType() == "LineSymbolizer")
  {
    m_ui->m_styleTabWidget->setTabEnabled(2, true);
    m_lineWidget->setSymbolizer(dynamic_cast<te::se::LineSymbolizer*>(s));
  }
  else if (s->getType() == "PointSymbolizer")
  {
    m_ui->m_styleTabWidget->setTabEnabled(3, true);
    m_pointWidget->setSymbolizer(dynamic_cast<te::se::PointSymbolizer*>(s));
  }
}

void ta::model::qt::CatalogManagerDialog::onPolygonSymbolizerChanged()
{
  te::se::Symbolizer* s = m_polyWidget->getSymbolizer();

  emit symbolizerChanged(s);
}

void ta::model::qt::CatalogManagerDialog::onLineSymbolizerChanged()
{
  te::se::Symbolizer* s = m_lineWidget->getSymbolizer();

  emit symbolizerChanged(s);
}

void ta::model::qt::CatalogManagerDialog::onPointSymbolizerChanged()
{
  te::se::Symbolizer* s = m_pointWidget->getSymbolizer();

  emit symbolizerChanged(s);
}

void ta::model::qt::CatalogManagerDialog::onMapRefresh()
{
  //repaint preview
  if (m_style.get() && !m_style->getRules().empty())
  {
    te::se::Rule* r = m_style->getRule(0); //always getting the first rule

    m_symbolPreview->updatePreview(r->getSymbolizers());
  }
}

void ta::model::qt::CatalogManagerDialog::setLayers(std::list<te::map::AbstractLayerPtr> layers)
{

  m_ui->m_addLayerToolButton->setEnabled(false);

  m_layers = layers;

  m_ui->m_layerComboBox->clear();

  std::vector<ta::model::AbstractItem*> catalogs = m_catalogFacade->get();

  for (std::list<te::map::AbstractLayerPtr>::iterator it = layers.begin(); it != layers.end(); ++it)
  {
    te::map::DataSetLayer* dsl = dynamic_cast<te::map::DataSetLayer*>(it->get());
    if (dsl)
    {
      std::string dataSourceId = dsl->getDataSourceId();
      te::da::DataSourceInfoPtr dsi = te::da::DataSourceInfoManager::getInstance().get(dataSourceId);

      std::string uri = dsi->getConnInfoAsString();
      std::string name = dsl->getTitle();
      std::string type = dsi->getType();

      bool existsCatalog = false;
      for (std::size_t i = 0; i < catalogs.size(); ++i)
      {
        ta::model::Catalog* c = dynamic_cast<ta::model::Catalog*>(catalogs.at(i));

        std::string uriConverted = te::common::Convert2UCase(uri);
        std::string uriCapConverted = te::common::Convert2UCase(c->getUri());
        std::string nameConverted = te::common::Convert2UCase(name);
        std::string nameCapConverted = te::common::Convert2UCase(c->getName());

        if (Trim(uriConverted).compare(Trim(uriCapConverted)) == 0)
        {
          if (Trim(nameConverted).compare(Trim(nameCapConverted)) == 0)
          {
            existsCatalog = true;
            break;
          }
        }
      }

      std::string completeName = name + " (" + type + ") ";

      if (!existsCatalog){
        m_ui->m_layerComboBox->addItem(completeName.c_str(), QVariant::fromValue(*it));
        m_ui->m_addLayerToolButton->setEnabled(true);
      }
    }
  }
}

void ta::model::qt::CatalogManagerDialog::onAddLayerToolButtonPressed()
{
  QRadioButton* irb = m_ui->m_importRadioButton;
  QRadioButton* arb = m_ui->m_associateRadioButton;

  QVariant varLayer = m_ui->m_layerComboBox->currentData(Qt::UserRole);
  te::map::AbstractLayerPtr alp = varLayer.value<te::map::AbstractLayerPtr>();

  if (!alp.get())
  {
    QMessageBox::warning(this, tr("Warning"), tr("Error getting selected layer."));
    return;
  }

  te::map::DataSetLayer* dsl = dynamic_cast<te::map::DataSetLayer*>(alp.get());

  if (dsl->isValid())
  {
    if (irb->isChecked())
    {
      importLayer(dsl);
    }
    else if (arb->isChecked())
    {
      associateLayer(dsl);
    }

    updateCatalogTableWidget();
    setLayers(m_layers);
  }
}

void ta::model::qt::CatalogManagerDialog::onCurrentCellTableWidgetChanged(int currentRow, int /*currentColumn*/, int previousRow, int /*previousColumn*/)
{
  if (currentRow >= 0 && currentRow != previousRow)
  {
    QTableWidget* tw = m_ui->m_catalogsTableWidget;

    QTableWidgetItem* item = tw->item(currentRow, 1);
    if (!item)
      return;
    int id = item->data(Qt::UserRole).toInt();
    if (id > 0)
    {
      ta::model::Catalog* c = dynamic_cast<ta::model::Catalog*>(m_catalogFacade->get(id));

      m_ui->m_descriptionLineEdit->setText(c->getDescription().c_str());
      m_ui->m_uriLineEdit->setText(c->getUri().c_str());
      m_ui->m_sourceTypeLineEdit->setText(c->getType().c_str());
      te::dt::TimeInstant* ti = c->getLastEdition();

      QDate lastUpdateDate(ti->getDate().getYear(),
        ti->getDate().getMonth(),
        ti->getDate().getDay());

      QTime lastUpdateTime(ti->getTime().getHours(),
        ti->getTime().getMinutes(),
        ti->getTime().getSeconds());

      m_ui->m_lastUpdateDateTimeEdit->setDate(lastUpdateDate);
      m_ui->m_lastUpdateDateTimeEdit->setTime(lastUpdateTime);

      m_ui->m_authorLineEdit->setText(c->getAuthor().c_str());
    }
  }
  else
  {
  }
}

void ta::model::qt::CatalogManagerDialog::onApplyStylePushButtonClicked()
{
  if (!m_style.get())
  {
    QMessageBox::warning(this, windowTitle(), tr("Style not defined."));
    return;
  }

  int catalogId = m_ui->m_catalogStyleComboBox->currentData().toInt();

  ta::model::CatalogFacade cf(m_activeServer);

  ta::model::Catalog* c = dynamic_cast<ta::model::Catalog*>(cf.get(catalogId));

  if (c && m_style.get())
  {
    std::string strStyle = ta::common::Write(m_style.get());

    c->setStyle(strStyle);

    ta::model::CatalogFacade cf(m_activeServer);

    if (!cf.update(c))
    {
      QMessageBox::warning(this, windowTitle(), cf.getErrorMessage().c_str());
      return;
    }
  }

  delete c;

  QMessageBox::information(this, windowTitle(), tr("Style updated."));
}

void ta::model::qt::CatalogManagerDialog::updateCatalogTableWidget()
{
  std::vector<ta::model::AbstractItem*> catalogs = m_catalogFacade->get();

  QTableWidget* tw = m_ui->m_catalogsTableWidget;

  tw->clearContents();
  tw->setRowCount(0);
  int row = tw->rowCount();

  //update catalog combo box
  m_ui->m_catalogStyleComboBox->clear();

  for (std::size_t t = 0; t < catalogs.size(); ++t)
  {
    tw->insertRow(row);

    // buttons
    QWidget* wActions = new QWidget(tw);

    QToolButton* updateButton = new QToolButton(tw);
    updateButton->setAutoRaise(true);
    updateButton->setCheckable(false);
    updateButton->setIcon(QIcon::fromTheme("view-refresh"));

    QToolButton* removeButton = new QToolButton(tw);
    removeButton->setAutoRaise(true);
    removeButton->setCheckable(false);
    removeButton->setIcon(QIcon::fromTheme("delete"));

    QGridLayout* layout = new QGridLayout(wActions);
    layout->setContentsMargins(0, 0, 0, 0);

    layout->addWidget(updateButton, 0, 0);
    layout->addWidget(removeButton, 0, 1);

    tw->setCellWidget(row, 0, wActions);
    m_updateButtonMapper->setMapping(updateButton, row);
    m_removeButtonMapper->setMapping(removeButton, row);
    connect(updateButton, SIGNAL(pressed()), m_updateButtonMapper, SLOT(map()));
    connect(removeButton, SIGNAL(pressed()), m_removeButtonMapper, SLOT(map()));

    ta::model::Catalog* c = dynamic_cast<ta::model::Catalog*>(catalogs.at(t));

    if (c)
    {
      // catalog name
      QTableWidgetItem* name = new QTableWidgetItem();
      name->setText(c->getName().c_str());
      name->setData(Qt::UserRole, QVariant::fromValue(c->getId()));
      name->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);

      // catalog locked
      QComboBox* cbLocked = new QComboBox(tw);
      cbLocked->insertItem(0, "FALSE", QVariant::fromValue(0));
      cbLocked->insertItem(1, "TRUE", QVariant::fromValue(1));
      if (!c->isLocked())
        cbLocked->setCurrentIndex(0);
      else
        cbLocked->setCurrentIndex(1);

      // catalog cellcolumn
      QComboBox* cbCellColumn = new QComboBox(tw);
      if (!c->getCellColumn().empty())
      {
        cbCellColumn->insertItem(0, c->getCellColumn().c_str(), QVariant(c->getCellColumn().c_str()));
        cbCellColumn->setEnabled(false);
      }
      else{
        cbCellColumn->insertItem(0, "select...", QVariant(""));
        std::vector<te::dt::Property*> properties = getAllowedAttributesFromLayer(c);
        for (std::size_t i = 0; i < properties.size(); ++i)
        {
          cbCellColumn->addItem(properties.at(i)->getName().c_str(), QVariant(properties.at(i)->getName().c_str()));
        }
      }

      // catalog source
      QTableWidgetItem* source = new QTableWidgetItem();
      source->setText(c->getSource().c_str());
      source->setData(Qt::UserRole, QVariant(c->getSource().c_str()));
      source->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsEditable);

      tw->setItem(row, 1, name);
      tw->setCellWidget(row, 2, cbLocked);
      tw->setCellWidget(row, 3, cbCellColumn);
      tw->setItem(row, 4, source);

      row = tw->rowCount();
    }

    //add into catalog combo box
    m_ui->m_catalogStyleComboBox->addItem(c->getName().c_str(), QVariant(c->getId()));
  }

  resizeCatalogsTableWidget();

  if (!catalogs.empty())
  {
    onCatalogStyleComboBoxActivated(0);
  }

  te::common::FreeContents(catalogs);
  catalogs.clear();
}

void ta::model::qt::CatalogManagerDialog::resizeCatalogsTableWidget()
{
  QTableWidget* tw = m_ui->m_catalogsTableWidget;

  tw->resizeColumnsToContents();
  tw->horizontalHeader()->setSectionResizeMode(1, QHeaderView::Stretch);
  tw->horizontalHeader()->setSectionResizeMode(2, QHeaderView::Stretch);
  tw->horizontalHeader()->setSectionResizeMode(3, QHeaderView::Stretch);
}

void ta::model::qt::CatalogManagerDialog::buildStyleWidgets()
{
  // StyleExplorer
  m_styleController = new te::qt::widgets::StyleControllerWidget(m_ui->m_styleTabWidget);
  m_ui->m_styleTabWidget->addTab(m_styleController, tr("Explorer"));
  connect(m_styleController, SIGNAL(mapRefresh()), this, SLOT(onMapRefresh()));
  connect(m_styleController->getStyleExplorer(), SIGNAL(symbolizerClicked(te::se::Symbolizer*)), this, SLOT(onSymbolizerSelected(te::se::Symbolizer*)));

  // PolygonSymbolizer Property
  m_polyWidget = new te::qt::widgets::PolygonSymbolizerProperty(m_ui->m_styleTabWidget);
  m_ui->m_styleTabWidget->addTab(m_polyWidget, tr("Polygons"));
  connect(m_polyWidget, SIGNAL(symbolizerChanged()), this, SLOT(onPolygonSymbolizerChanged()));

  // LineSymbolizer Property
  m_lineWidget = new te::qt::widgets::LineSymbolizerProperty(m_ui->m_styleTabWidget);
  m_ui->m_styleTabWidget->addTab(m_lineWidget, tr("Lines"));
  connect(m_lineWidget, SIGNAL(symbolizerChanged()), this, SLOT(onLineSymbolizerChanged()));

  // Point Symbolizer Widget
  m_pointWidget = new te::qt::widgets::PointSymbolizerProperty(m_ui->m_styleTabWidget);
  m_ui->m_styleTabWidget->addTab(m_pointWidget, tr("Points"));
  connect(m_pointWidget, SIGNAL(symbolizerChanged()), this, SLOT(onPointSymbolizerChanged()));

  connect(this, SIGNAL(symbolizerChanged(te::se::Symbolizer*)), m_styleController->getStyleExplorer(), SLOT(onSymbolizerChanged(te::se::Symbolizer*)));
  connect(this, SIGNAL(symbolizerChanged(te::se::Symbolizer*)), this, SLOT(onMapRefresh()));

  //preview
  m_symbolPreview = new te::qt::widgets::SymbolPreviewWidget(QSize(256, 256), m_ui->m_stylePreviewWidget);

  setTabStatus(false);
}

void ta::model::qt::CatalogManagerDialog::setTabStatus(bool status)
{
  m_ui->m_styleTabWidget->setTabEnabled(1, status); // Polygons
  m_ui->m_styleTabWidget->setTabEnabled(2, status); // Lines
  m_ui->m_styleTabWidget->setTabEnabled(3, status); // Points
}

bool ta::model::qt::CatalogManagerDialog::importLayer(te::map::AbstractLayerPtr layer)
{
  std::string dataSourceId = layer->getDataSourceId();
  te::da::DataSourceInfoPtr dsi = te::da::DataSourceInfoManager::getInstance().get(dataSourceId);

  if (dsi.get())
  {
    if (exchangeToDatabase(layer))
    {
      std::string layerName = validateLayerName(layer->getTitle());

      te::da::DataSourceInfoPtr dsiServer = te::da::DataSourceInfoManager::getInstance().get(m_activeServer->getId());

      std::string type = dsiServer->getType();
      std::string name = layerName;
      std::string uri = dsiServer->getConnInfoAsString();
      int permission = 1; //TODO: hard coded refator
      te::dt::TimeInstant* last_edition = new te::dt::TimeInstant(boost::posix_time::second_clock::local_time());
      std::string author = getActiveUserName();
      std::string source = "Created by layer import functionality.";
      std::string style = generateStyle(layer);
      bool locked = false;

      ta::model::Catalog* c = new ta::model::Catalog();
      c->setType(type);
      c->setName(name);
      c->setUri(uri);
      c->setPermission(permission);
      c->setLastEdition(last_edition);
      c->setSource(source);
      c->setAuthor(author);
      c->setStyle(style);
      c->setLocked(locked);

      return m_catalogFacade->save(c);
    }
  }

  return false;
}

bool ta::model::qt::CatalogManagerDialog::associateLayer(te::map::AbstractLayerPtr layer)
{
  std::string dataSourceId = layer->getDataSourceId();
  te::da::DataSourceInfoPtr dsi = te::da::DataSourceInfoManager::getInstance().get(dataSourceId);

  if (dsi.get())
  {
    std::string type = dsi->getType();

    if (type.compare("OGR") != 0)
    {
      std::string name = layer->getTitle();
      std::string uri = dsi->getConnInfoAsString();
      int permission = 1; //TODO: hard coded refator
      te::dt::TimeInstant* last_edition = new te::dt::TimeInstant(boost::posix_time::second_clock::local_time());
      std::string author = getActiveUserName();
      std::string source = "Created by layer association functionality.";
      std::string style = generateStyle(layer);
      bool locked = false;

      ta::model::Catalog* c = new ta::model::Catalog();
      c->setType(type);
      c->setName(name);
      c->setUri(uri);
      c->setPermission(permission);
      c->setLastEdition(last_edition);
      c->setSource(source);
      c->setAuthor(author);
      c->setStyle(style);
      c->setLocked(locked);

      return m_catalogFacade->save(c);
    }
    else
    {
      QMessageBox::warning(this, tr("TerraAmazon"), tr("It is not possible associate layers OGR type. Try import."));
    }
  }

  return false;
}

std::string ta::model::qt::CatalogManagerDialog::generateStyle(te::map::AbstractLayerPtr layer)
{
  std::string strStyle = "";

  std::auto_ptr<te::da::DataSetType> dsType = layer->getSchema();

  te::gm::GeometryProperty* gProp = te::da::GetFirstGeomProperty(dsType.get());

  if (gProp)
  {
    std::auto_ptr<te::se::Style> style(te::se::CreateFeatureTypeStyle(gProp->getGeometryType()));

    if (style.get())
    {
      strStyle = ta::common::Write(style.get());
    }
  }

  return strStyle;
}

bool ta::model::qt::CatalogManagerDialog::exchangeToDatabase(te::map::AbstractLayerPtr layer)
{
  te::da::DataSourcePtr targetDSPtr = m_activeServer;
  std::auto_ptr<te::da::DataSourceTransactor> transactor = targetDSPtr->getTransactor();

  std::string layerName = validateLayerName(layer->getTitle());

  try{
    std::vector<std::string> ds = transactor->getDataSetNames();
    for (std::size_t i = 0; i < ds.size(); ++i)
    {
      if (Trim(te::common::Convert2UCase(layerName)).compare(Trim(te::common::Convert2UCase(ds.at(i)))) == 0)
      {
        QMessageBox::warning(this, tr("TerraAmazon"), tr("This layer already exists on the current server. Try associate."));
        return false;
      }
    }
  }
  catch (const std::exception& /*e*/)
  {
    return false;
  }

  try
  {
    setCursor(Qt::WaitCursor);

    //create adapter
    std::auto_ptr<te::da::DataSetType> dsType = layer->getSchema();

    te::da::DataSetTypeConverter* converter = new te::da::DataSetTypeConverter(dsType.get(), targetDSPtr->getCapabilities(), targetDSPtr->getEncoding());

    converter->needConverter(dsType.get(), targetDSPtr->getCapabilities());

    te::da::DataSetType* dsTypeResult = converter->getResult();

    // Check dataset name
    if (!targetDSPtr->isDataSetNameValid(dsTypeResult->getName()))
    {
      int r = QMessageBox::question(this, tr("Exchanger"), tr("Layer name invalid for output datasource. Would you like to normalize the name?"), QMessageBox::Yes, QMessageBox::No);

      if (r == QMessageBox::Yes)
      {
        bool aux;
        std::string newName = te::common::ReplaceSpecialChars(dsTypeResult->getName(), aux);
        dsTypeResult->setName(newName);
      }
      else
      {
        throw te::common::Exception(tr("Layer name invalid for output datasource!").toStdString());
      }
    }

    // Check properties names
    std::vector<te::dt::Property* > props = dsTypeResult->getProperties();
    std::map<std::size_t, std::string> invalidNames;
    for (std::size_t i = 0; i < props.size(); ++i)
    {
      if (!targetDSPtr->isPropertyNameValid(props[i]->getName()))
      {
        invalidNames[i] = props[i]->getName();
      }
    }

    if (!invalidNames.empty())
    {
      int r = QMessageBox::question(this, tr("Exchanger"), tr("Some property name is invalid for output datasource. Would you like to normalize the name?"), QMessageBox::Yes, QMessageBox::No);

      if (r == QMessageBox::Yes)
      {
        std::map<std::size_t, std::string>::iterator it = invalidNames.begin();

        while (it != invalidNames.end())
        {
          bool aux;
          std::string newName = te::common::ReplaceSpecialChars(it->second, aux);

          props[it->first]->setName(newName);

          ++it;
        }
      }
      else
      {
        QString err(tr("Some property name is invalid for output datasource:\n\n"));

        std::map<std::size_t, std::string>::iterator it = invalidNames.begin();

        while (it != invalidNames.end())
        {
          err.append(" - ");
          err.append(it->second.c_str());

          ++it;
        }

        throw te::common::Exception(err.toStdString());
      }
    }

    //fix repeated names
    std::set<std::string> names;

    props = dsTypeResult->getProperties();

    for (std::size_t i = 0; i < props.size(); ++i)
    {
      //check if the property name its duplicated
      std::string propName = props[i]->getName();

      int count = 1;
      while (names.find(te::common::Convert2UCase(propName)) != names.end())
      {
        propName += "_";
        propName += te::common::Convert2String(count);
      }

      names.insert(te::common::Convert2UCase(propName));

      props[i]->setName(propName);
    }

    // set correct name with schema
    dsTypeResult->setName(layerName);

    if (dsType->getPrimaryKey())
    {
      te::da::PrimaryKey* pk = new te::da::PrimaryKey(dsTypeResult);

      std::string name = layerName + "_" + dsType->getPrimaryKey()->getName() + "_pk";

      boost::replace_all(name, ".", "_");

      pk->setName(name);

      std::vector<te::dt::Property*> props = dsType->getPrimaryKey()->getProperties();

      for (size_t t = 0; t < props.size(); ++t)
      {
        te::dt::Property* p = props[t]->clone();

        pk->add(p);
      }
    }

    //exchange
    std::map<std::string, std::string> nopt;
    std::auto_ptr<te::da::DataSet> dataset = layer->getData();

    transactor->begin();
    transactor->createDataSet(dsTypeResult, nopt);
    std::auto_ptr<te::da::DataSetAdapter> dsAdapter(te::da::CreateAdapter(dataset.get(), converter));
    if (dataset->moveBeforeFirst())
      transactor->add(dsTypeResult->getName(), dsAdapter.get(), targetDSPtr->getConnectionInfo());
    transactor->commit();

    setCursor(Qt::ArrowCursor);

    return true;
  }
  catch (const std::exception& e)
  {
    setCursor(Qt::ArrowCursor);
    transactor->rollBack();
    QString errMsg(tr("Error during import. The reported error is: %1"));
    errMsg = errMsg.arg(e.what());
    QMessageBox::information(this, tr("TerraAmazon"), errMsg);
  }

  return false;
}

std::string ta::model::qt::CatalogManagerDialog::validateLayerName(std::string title)
{
  std::size_t found = title.find("public.");
  if (found == std::string::npos)
  {
    return "public." + title;
  }
  return title;
}

std::vector<te::dt::Property*> ta::model::qt::CatalogManagerDialog::getAllowedAttributesFromLayer(ta::model::Catalog* catalog)
{
  std::vector<te::dt::Property*> result;

  std::auto_ptr<te::da::DataSetType> dsTypeOriginal = m_catalogFacade->getDataSetType(catalog);
  for (std::size_t t = 0; t < dsTypeOriginal->getProperties().size(); ++t)
  {
    int type = dsTypeOriginal->getProperties()[t]->getType();
    if (type == te::dt::INT16_TYPE || type == te::dt::UINT16_TYPE ||
      type == te::dt::INT32_TYPE || type == te::dt::UINT32_TYPE ||
      type == te::dt::INT64_TYPE || type == te::dt::UINT64_TYPE ||
      type == te::dt::STRING_TYPE || type == te::dt::STRING || type == te::dt::VAR_STRING ||
      type == te::dt::FIXED_STRING)
    {
      te::dt::Property* p = new te::dt::SimpleProperty(dsTypeOriginal->getProperties()[t]->getName(), dsTypeOriginal->getProperties()[t]->getType());
      result.push_back(p);
    }
  }
  return result;
}

std::string ta::model::qt::CatalogManagerDialog::getActiveUserName()
{
  if (m_activeServer)
  {
    return m_userName;
  }
  return "unknow user";
}
