/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of
    this application.
    */

// TerraAmazon
#include "../../../common/qt/CreateCellLayerUtils.h"
#include "../../../common/Utils.h"
#include "../../facade/UserFacade.h"
#include "../../facade/CatalogFacade.h"
#include "../../item/User.h"
#include "../../item/Catalog.h"
#include "../../Globals.h"
#include "ConceptualModelUtils.h"

// TerraLib
#include <terralib/common/Exception.h>
#include <terralib/common/PlatformUtils.h>
#include <terralib/common/Translator.h>
#include <terralib/cellspace/CellSpaceOperations.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>
#include <terralib/dataaccess/query/BinaryFunction.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/Insert.h>
#include <terralib/dataaccess/query/LiteralString.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/datatype/SimpleProperty.h>
#include <terralib/datatype/StringProperty.h>
#include <terralib/datatype/TimeInstant.h>
#include <terralib/geometry/Envelope.h>
#include <terralib/memory/DataSet.h>
#include <terralib/memory/DataSetItem.h>
#include <terralib/se/Utils.h>

// STL
#include <fstream>
#include <memory>

void ta::model::CreateConceptualModel(te::da::DataSourceInfoPtr targetDs, te::map::AbstractLayerPtr cellLayer, const double& res)
{
  if (!cellLayer)
    throw te::common::Exception(TE_TR("Selected Cell Layer invalid!"));

  validateDataForConceptualModel(targetDs, cellLayer->getSchema()->getName());

  CreateConceptualModel(targetDs, cellLayer->getSchema()->getName(), res);
}

void ta::model::CreateConceptualModel(te::da::DataSourceInfoPtr targetDs, const std::string& cellLayerName, const double& res)
{
  te::da::DataSourcePtr source = te::da::DataSourceManager::getInstance().get(targetDs->getId(), targetDs->getType(), targetDs->getConnInfo());
  source->open();

  ta::common::AdaptCellDataSet(source, cellLayerName);

  std::string file = ta::common::FindInTerraAmazonPath("share/sql/ScriptConceptualModel.sql");

  std::ifstream t(file);
  std::stringstream buffer;
  buffer << t.rdbuf();

  std::string strScript = buffer.str();

  std::unique_ptr<te::da::DataSourceTransactor> transactor = source->getTransactor();

  try
  {
    transactor->execute(strScript);

    ta::model::CatalogFacade cf(source);

    try
    {
      ta::model::Catalog* catalog = new ta::model::Catalog();
      ta::common::PrepareCatalogCellLayer(targetDs, cellLayerName, "Created with the Conceptual Model creation.", catalog);
      if (!catalog)
        throw;
      cf.saveAsCellLayer(catalog, res, true);
    }
    catch (const te::common::Exception& e)
    {
      throw e;
    }

    te::da::Field* field = new te::da::Field("*");
    te::da::Fields* fields = new te::da::Fields;
    fields->push_back(field);

    te::da::FromItem* fromItem = new te::da::DataSetName("pg_user");
    te::da::From* from = new te::da::From;
    from->push_back(fromItem);

    te::da::PropertyName* pName = new te::da::PropertyName("usename");

    te::da::Literal* literalName = new te::da::LiteralString(ta::model::Globals::sm_taUser);

    te::da::EqualTo* equalName = new te::da::EqualTo(pName, literalName);

    te::da::Where* where = new te::da::Where(equalName);

    te::da::Select* select = new te::da::Select(fields, from, where);

    std::unique_ptr<te::da::DataSet> res = source->query(select);

    if (res.get()->isEmpty())
    {
      std::string userSql = "CREATE USER " + ta::model::Globals::sm_taUser + " LOGIN ENCRYPTED PASSWORD '" + ta::model::Globals::sm_taPassword + "' SUPERUSER INHERIT CREATEDB CREATEROLE";

      source->execute(userSql);
    }

    ta::model::UserFacade uf(source);

    ta::model::User admin;
    admin.setName(ta::model::Globals::sm_taUser);
    admin.setPassword(sha256(ta::model::Globals::sm_taPassword));
    uf.save(&admin);
    uf.addAssociatedGroup(1, admin.getId());

    std::map<std::string, std::string> connInfo = source->getConnectionInfo();

    std::string dsUserName = connInfo["PG_USER"];
    std::string dsPwd = connInfo["PG_PASSWORD"];

    ta::model::User dbUser;
    dbUser.setName(dsUserName);
    dbUser.setPassword(sha256(dsPwd));
    uf.save(&dbUser);
    uf.addAssociatedGroup(1, dbUser.getId());

    transactor->commit();
  }
  catch (const te::common::Exception& e)
  {
    transactor->rollBack();
    throw e;
  }
  catch (const std::exception& e)
  {
    transactor->rollBack();
    throw e;
  }
  catch (...)
  {
    transactor->rollBack();
    throw;
  }
}

void ta::model::CreateConceptualModel(te::da::DataSourceInfoPtr targetDs, const std::string& newCellLayerName, const te::gm::Envelope& env, const double& res, const int& srid)
{
  validateDataForConceptualModel(targetDs, newCellLayerName);

  ta::common::CreateCellLayer(targetDs, newCellLayerName, env, res, srid);

  CreateConceptualModel(targetDs, newCellLayerName, res);
}

void ta::model::validateDataForConceptualModel(te::da::DataSourceInfoPtr targetDs, const std::string& /*newCellLayerName*/)
{
  te::da::DataSourcePtr source = te::da::DataSourceManager::getInstance().get(targetDs->getId(), targetDs->getType(), targetDs->getConnInfo());
  source->open();

  if (source->dataSetExists(Globals::sm_project))
  {
    throw te::common::Exception(TE_TR("Selected DataSource already has a conceptual model!"));
  }

  te::da::Field* field = new te::da::Field("*");
  te::da::Fields* fields = new te::da::Fields;
  fields->push_back(field);

  te::da::FromItem* fromItem = new te::da::DataSetName("pg_extension");
  te::da::From* from = new te::da::From;
  from->push_back(fromItem);

  te::da::PropertyName* pName = new te::da::PropertyName("extname");

  te::da::Literal* literalName = new te::da::LiteralString("postgis");

  te::da::EqualTo* equalName = new te::da::EqualTo(pName, literalName);

  te::da::Where* where = new te::da::Where(equalName);

  te::da::Select* select = new te::da::Select(fields, from, where);

  try
  {
    std::auto_ptr<te::da::DataSet> result = source->query(select);

    if (result->isEmpty())
    {
      source->execute("CREATE EXTENSION postgis");
    }
  }
  catch (const std::exception &e)
  {
    std::string err = e.what();
    err = TE_TR("Unable to create postgis extension: ") + err;
    throw te::common::Exception(err);
  }
}