/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of
    this application.
    */

// TerraAmazon
#include "ConceptualModelUtils.h"
#include "CreateConceptualModelDialog.h"
#include "ui_CreateConceptualModelDialogForm.h"

#include "../../../common/Utils.h"

// TerraLib
#include <terralib/common/Exception.h>
#include <terralib/common/Translator.h>
#include <terralib/qt/widgets/layer/utils/DataSet2Layer.h>
#include <terralib/qt/widgets/utils/ScopedCursor.h>

// Qt
#include <QMessageBox>
#include <QWidget>

// Boost
#include <boost/lexical_cast.hpp>

// STL
#include <fstream>
#include <memory>

ta::model::qt::CreateConceptualModelDialog::CreateConceptualModelDialog(QWidget* parent)
  : QDialog(parent),
    m_ui(new Ui::CreateConceptualModelDialogForm),
    m_cellLayerName("")
{
  // add controls
  m_ui->setupUi(this);

  std::string icon = ta::common::FindInTerraAmazonPath("share/icons/svg/conceptual-model-hint.svg");
  m_ui->m_imgLabel->setPixmap(QIcon(icon.c_str()).pixmap(48, 48));
  //m_ui->m_imgLabel->hide();

  m_widget = new ta::common::qt::CreateCellLayerWidget(m_ui->m_widget);

  QGridLayout* layout = new QGridLayout(m_ui->m_widget);
  layout->setContentsMargins(0, 0, 0, 0);
  layout->addWidget(m_widget);

  connect(m_ui->m_okPushButton, SIGNAL(clicked()), this, SLOT(onOkPushButtonClicked()));
}

ta::model::qt::CreateConceptualModelDialog::~CreateConceptualModelDialog()
{
}

void ta::model::qt::CreateConceptualModelDialog::onOkPushButtonClicked()
{
  te::qt::widgets::ScopedCursor c(Qt::WaitCursor);

  QString error;

  ta::common::qt::CreateCellLayerWidget* widget = getWidget();

  if (!widget->validateParameters(error))
  {
    QApplication::restoreOverrideCursor();
    QMessageBox::warning(this, tr("Create Conceptual Model"), error);
    return;
  }

  m_targetDs = widget->getCurrentDataSourceInfoPtr();

  try
  {
    m_cellLayerName = widget->getCurrentCellLayerName();
    te::gm::Envelope env = widget->getEnvelope();
    double res = widget->getCurrentResolution();
    int srid = widget->getSrid();
    ta::model::CreateConceptualModel(m_targetDs, m_cellLayerName, env, res, srid);

    QApplication::restoreOverrideCursor();
  }
  catch (const te::common::Exception& e)
  {
    QApplication::restoreOverrideCursor();
    QMessageBox::warning(this, tr("Create Conceptual Model"), e.what());
    return;
  }
  catch (const std::exception& e)
  {
    QApplication::restoreOverrideCursor();
    QMessageBox::warning(this, tr("Create Conceptual Model"), e.what());
    return;
  }
  catch (...)
  {
    QApplication::restoreOverrideCursor();
    QMessageBox::warning(this, tr("Create Conceptual Model"), tr("Error trying to create conceptual model!"));
    return;
  }

  accept();
}

ta::common::qt::CreateCellLayerWidget* ta::model::qt::CreateConceptualModelDialog::getWidget()
{
  return m_widget;
}

te::map::AbstractLayerPtr ta::model::qt::CreateConceptualModelDialog::getCellLayer()
{
  te::da::DataSourcePtr ds = te::da::DataSourceManager::getInstance().find(m_targetDs->getId());
  ds->open();

  if (!ds)
    throw te::common::Exception(TE_TR("Error while layer cration: DataSource invalid!"));

  std::auto_ptr<te::da::DataSetType> dst = ds->getDataSetType(m_cellLayerName);

  te::qt::widgets::DataSet2Layer converter(ds->getId());

  te::da::DataSetTypePtr dt(ds->getDataSetType(m_cellLayerName).release());

  te::map::AbstractLayerPtr layer = converter(dt);

  //layer->setSRID(m_bbSRID);

  return layer;
}
