/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/model/qt/conceptualModel/ProjectImportUtils.cpp

\brief
*/

// TerraAmazon
#include "ProjectImportUtils.h"
#include "../../item_mem/RuleRuleSetMem.h"
#include "../../facade/AoiLayerFacade.h"
#include "../../facade/ClassFacade.h"
#include "../../facade/CatalogFacade.h"
#include "../../facade/ProjectFacade.h"
#include "../../facade/RuleFacade.h"
#include "../../facade/RuleSetFacade.h"
#include "../../facade/RulesLayerColumnFacade.h"
#include "../../facade/RuleRuleSetFacade.h"

// TerraLib
#include <terralib/dataaccess/datasource/DataSourceTransactor.h>

ta::model::ProjectImportUtils::ProjectImportUtils()
{
}

ta::model::ProjectImportUtils::~ProjectImportUtils()
{
}

bool ta::model::ProjectImportUtils::importToDatabase(ta::model::ProjectMem* projectMem, std::string& errorMessage, std::string& warningMessage, te::da::DataSourcePtr& dataSource)
{
  std::unique_ptr<te::da::DataSourceTransactor> transactor = dataSource->getTransactor();

  //initialize transaction
  transactor->begin();

  //facades
  ta::model::AoiLayerFacade aoiLayerFacade(dataSource);
  ta::model::CatalogFacade catalogFacade(dataSource);
  ta::model::ClassFacade classFacade(dataSource);
  ta::model::ProjectFacade projectFacade(dataSource);
  ta::model::RuleFacade ruleFacade(dataSource);
  ta::model::RuleSetFacade ruleSetFacade(dataSource);
  ta::model::RuleRuleSetFacade ruleRuleSetFacade(dataSource);
  ta::model::RulesLayerColumnFacade rulesLayerColumnFacade(dataSource);

  if (projectFacade.existsByName(projectMem->getName()))
  {
    transactor->rollBack();
    errorMessage = "The project " + projectMem->getName() + " already exists.";
    return false;
  }
  else
  {
    if (!projectFacade.save(projectMem, transactor.get()))
    {
      transactor->rollBack();
      errorMessage = projectFacade.getErrorMessage();
      return false;
    }
  }

  //rulesets
  std::vector<ta::model::RuleSetMem*> rulesets = projectMem->getRuleSetMems();
  for (std::size_t i = 0; i < rulesets.size(); ++i)
  {
    //catalog
    ta::model::CatalogMem* catalog = rulesets.at(i)->getCatalogMem();
    if (!importAuxCatalog(catalog, transactor.get(), dataSource))
    {
      transactor->rollBack();
      errorMessage = "Inexistent or incompatible catalog: " + catalog->getName() + ".";
      return false;
    }

    //ruleset
    rulesets.at(i)->setCatalogId(catalog->getId());
    if (ruleSetFacade.existsByName(rulesets.at(i)->getName()))
    {
      transactor->rollBack();
      errorMessage = "The RuleSet " + rulesets.at(i)->getName() + " already exists.";
      return false;
    }
    else
    {
      if (!ruleSetFacade.save(rulesets.at(i), transactor.get()))
      {
        transactor->rollBack();
        errorMessage = ruleSetFacade.getErrorMessage();
        return false;
      }
    }

    //ruleset_project
    if (!projectFacade.addAssociateRuleSet(projectMem->getId(), rulesets.at(i)->getId(), transactor.get()))
    {
      transactor->rollBack();
      errorMessage = projectFacade.getErrorMessage();
      return false;
    }

    //rulestructs
    std::vector<ta::model::RuleStructMem*> rulestructs = rulesets.at(i)->getRuleStructMems();
    for (std::size_t j = 0; j < rulestructs.size(); ++j)
    {
      ta::model::RuleMem* rule = rulestructs.at(j)->getRuleMem();

      ta::model::CatalogMem* catalogRule = rule->getCatalogMem();
      if (!importAuxCatalog(catalogRule, transactor.get(), dataSource))
      {
        transactor->rollBack();
        errorMessage = "Inexistent or incompatible catalog: " + catalogRule->getName() + ".";
        return false;
      }

      //rule
      rule->setCatalogId(catalogRule->getId());
      if (ruleSetFacade.existsByName(rule->getName()))
      {
        transactor->rollBack();
        errorMessage = "The Rule " + rule->getName() + " already exists.";
        return false;
      }
      else
      {
        if (!ruleFacade.save(rule, transactor.get()))
        {
          transactor->rollBack();
          errorMessage = ruleFacade.getErrorMessage();
          return false;
        }
      }

      //rule_ruleset
      int priority = rulestructs.at(j)->getPriority();
      ta::model::RuleRuleSetMem* ruleRuleSetMem = new ta::model::RuleRuleSetMem();
      ruleRuleSetMem->setPriority(priority);
      ruleRuleSetMem->setRuleId(rule->getId());
      ruleRuleSetMem->setRuleSetId(rulesets.at(i)->getId());
      if (!ruleRuleSetFacade.save(ruleRuleSetMem, transactor.get()))
      {
        transactor->rollBack();
        errorMessage = ruleRuleSetFacade.getErrorMessage();
        return false;
      }

      //rules_layer_column
      std::vector<ta::model::RulesLayerColumnMem*> rulesLayerColumns = rulestructs.at(j)->getRulesLayerColumnMems();
      for (std::size_t k = 0; k < rulesLayerColumns.size(); ++k)
      {
        ta::model::RulesLayerColumnMem* rulesLayerColumn = rulesLayerColumns.at(k);
        rulesLayerColumn->setRuleId(rule->getId());
        rulesLayerColumn->setRulesetId(rulesets.at(i)->getId());

        bool existsColumnIn = false;
        bool existsColumnOut = false;

        int typeColumnIn = 0;
        int typeColumnOut = 0;

        std::auto_ptr<te::da::DataSetType> dsTypeIn = catalogFacade.getDataSetType(rule->getCatalogMem());
        for (std::size_t p = 0; p < dsTypeIn->getProperties().size(); p++)
        {
          std::string prop = dsTypeIn->getProperties()[p]->getName();
          if (rulesLayerColumn->getColumnIn() == prop){
            existsColumnIn = true;
            typeColumnIn = dsTypeIn->getProperties()[p]->getType();
            break;
          }
        }

        std::auto_ptr<te::da::DataSetType> dsTypeOut = catalogFacade.getDataSetType(rulesets.at(i)->getCatalogMem());
        for (std::size_t p = 0; p < dsTypeOut->getProperties().size(); p++)
        {
          std::string prop = dsTypeOut->getProperties()[p]->getName();
          if (rulesLayerColumn->getColumnOut() == prop){
            existsColumnOut = true;
            typeColumnOut = dsTypeOut->getProperties()[p]->getType();
            break;
          }
        }

        if (existsColumnIn && existsColumnOut && (typeColumnIn == typeColumnOut))
        {
          if (!rulesLayerColumnFacade.save(rulesLayerColumn, transactor.get()))
          {
            transactor->rollBack();
            errorMessage = ruleRuleSetFacade.getErrorMessage();
            return false;
          }
        }
        else
        {
          if (!existsColumnIn)
          {
            warningMessage += "Attribute not imported. The column " + rulesLayerColumn->getColumnIn() + " does not exists in " + rule->getCatalogMem()->getName() + ".\n";
          }
          if (!existsColumnOut)
          {
            warningMessage += "Attribute not imported. The column " + rulesLayerColumn->getColumnOut() + " does not exists in " + rulesets.at(i)->getCatalogMem()->getName() + ".\n";
          }
          if (existsColumnIn && existsColumnOut && (typeColumnIn != typeColumnOut))
          {
            warningMessage += "Attribute not imported. The column " + rulesLayerColumn->getColumnOut() + " has different type than the column " + rulesets.at(i)->getCatalogMem()->getName() + ".\n";
          }
        }
      }
    }

    //classes
    std::vector<ta::model::ClassMem*> classes = rulesets.at(i)->getClassMems();
    for (std::size_t m = 0; m < classes.size(); ++m)
    {
      //class
      if (classFacade.existsByName(classes.at(m)->getName(), transactor.get()))
      {
        int classId = classFacade.getByName(classes.at(m)->getName(), transactor.get());
        if (classId > 0)
        {
          classes.at(m)->setId(classId);
        }
      }

      if (classes.at(m)->getId() <= 0)
      {
        if (classFacade.existsByName(classes.at(m)->getName()))
        {
          int classId = classFacade.getByName(classes.at(m)->getName());
          if (classId > 0)
          {
            classes.at(m)->setId(classId);
          }
          else
          {
            transactor->rollBack();
            errorMessage = classFacade.getErrorMessage();
            return false;
          }
        }
        else
        {
          if (!classFacade.save(classes.at(m), transactor.get()))
          {
            transactor->rollBack();
            errorMessage = ruleFacade.getErrorMessage();
            return false;
          }
        }
      }

      //ruleset_class
      if (!ruleSetFacade.addRuleSetClass(rulesets.at(i)->getId(), classes.at(m)->getId(), transactor.get()))
      {
        transactor->rollBack();
        errorMessage = ruleSetFacade.getErrorMessage();
        return false;
      }
    }
  }

  transactor->commit();
  return true;
}

bool ta::model::ProjectImportUtils::importAuxCatalog(ta::model::CatalogMem* catalog, te::da::DataSourceTransactor* transactor, te::da::DataSourcePtr& dataSource)
{
  ta::model::CatalogFacade catalogFacade(dataSource);

  ta::model::Catalog* cat = catalogFacade.getByUriAndName(catalog->getUri(), catalog->getName());
  if (cat == 0)
  {
    return false;
  }
  catalog->setId(cat->getId());

  //rules_layer_params
  std::vector<ta::model::RulesLayerParamsMem*> rulesLayerParamsMems = catalog->getRulesLayerParamsMems();
  std::vector<std::string> classColumnsDB = catalogFacade.getClassColumns(catalog->getId());

  bool existClassColumns = true;
  for (std::size_t x = 0; x < rulesLayerParamsMems.size(); ++x)
  {
    bool existCC = false;
    ta::model::RulesLayerParamsMem* rulesLayerParamsMem = rulesLayerParamsMems.at(x);
    for (std::size_t y = 0; y < classColumnsDB.size(); ++y)
    {
      std::string classColumn = classColumnsDB.at(y);
      if (rulesLayerParamsMem->getClassColumn().compare(classColumn) == 0)
      {
        existCC = true;
        break;
      }
    }
    if (!existCC)
    {
      existClassColumns = false;
      break;
    }
  }

  if (!existClassColumns)
  {
    transactor->rollBack();
    //TODO: add error message
    return false;
  }

  //aoi_layers
  std::vector<ta::model::AoiLayerMem*> aoiLayerMems = catalog->getAoiLayerMems();
  std::vector<std::string> aoiIdColumns = catalogFacade.getAoiIdColumns(catalog->getId());

  bool existAoiLayers = true;
  for (std::size_t z = 0; z < aoiLayerMems.size(); ++z)
  {
    bool existAOI = false;
    for (std::size_t w = 0; w < aoiIdColumns.size(); ++w)
    {
      if (aoiLayerMems.at(z)->getAoiIdColumn().compare(aoiIdColumns.at(w)) == 0)
      {
        existAOI = true;
        break;
      }
    }
    if (!existAOI)
    {
      existAoiLayers = false;
      break;
    }
  }

  if (!existAoiLayers)
  {
    transactor->rollBack();
    //TODO: add error message
    return false;
  }

  return true;
}