/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/AOIControlProjectWizardPage.cpp

  \brief A Qt wizard page used to configure the project from AOI Control.
*/

// TerraLib
#include <terralib/dataaccess/datasource/DataSourceCapabilities.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/QueryCapabilities.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/datatype/Date.h>
#include <terralib/datatype/TimeInstant.h>
#include <terralib/qt/widgets/query/WhereClauseWidget.h>
#include <terralib/qt/widgets/utils/ScopedCursor.h>
#include "ui_AOIControlProjectWizardPageForm.h"
#include "AOIControlProjectWizardPage.h"

//TerraAmazon
#include "../../facade/ProjectFacade.h"
#include "../../facade/ProjectSceneFacade.h"
#include "../../facade/SceneFacade.h"
#include "../../item/Project.h"
#include "../../item/ProjectScene.h"
#include "../../item/Scene.h"
#include "model/Globals.h"

//Qt
#include <QCheckBox>
#include <QDialogButtonBox>
#include <QMessageBox>

ta::model::qt::AOIControlProjectWizardPage::AOIControlProjectWizardPage(QWidget* parent)
  : QWizardPage(parent),
  m_ui(new Ui::AOIControlProjectWizardPageForm)
{
  m_ui->setupUi(this);

  //configure page
  this->setTitle(tr("AOI Control Scenario"));
  this->setSubTitle(tr("Allows the user to associate a scenario with a project."));

  //mapper initializer
  m_lockMapper = new QSignalMapper(this);
  m_removeMapper = new QSignalMapper(this);

  //Scene table widget
  m_ui->m_scenarioTableWidget->setSelectionBehavior(QAbstractItemView::SelectRows);
  m_ui->m_scenarioTableWidget->setSelectionMode(QAbstractItemView::SingleSelection);

  //Scene & project table widget
  m_ui->m_scenarioProjectTableWidget->setSelectionBehavior(QAbstractItemView::SelectRows);
  m_ui->m_scenarioProjectTableWidget->setSelectionMode(QAbstractItemView::SingleSelection);

  // add button icons
  m_ui->m_scenarioFilterToolButton->setIcon(QIcon::fromTheme("view-filter"));
  m_ui->m_scenarioAddToolButton->setIcon(QIcon::fromTheme("list-add"));

  //Connect
  connect(m_lockMapper, SIGNAL(mapped(int)), this, SLOT(onLockSceneExecuted(int)));
  connect(m_ui->m_projectComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onProjectComboBoxIndexChanged(int)));
  connect(m_removeMapper, SIGNAL(mapped(int)), this, SLOT(onRemScenarioButtonClicked(int)));
  connect(m_ui->m_scenarioAddToolButton, SIGNAL(clicked()), this, SLOT(onAddScenarioButtonClicked()));
  connect(m_ui->m_scenarioFilterToolButton, SIGNAL(clicked()), this, SLOT(onSceneFilterButttonClicked()));

  m_ui->m_scenarioTableWidget->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
  m_ui->m_scenarioProjectTableWidget->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
}

ta::model::qt::AOIControlProjectWizardPage::~AOIControlProjectWizardPage()
{
}

bool ta::model::qt::AOIControlProjectWizardPage::isComplete() const
{
  return true;
}

void ta::model::qt::AOIControlProjectWizardPage::setServer(te::da::DataSourceInfoPtr server)
{
  if (server.get())
  {
    //get data source
    m_activeServer = te::da::DataSourceManager::getInstance().get(server->getId(), server->getType(), server->getConnInfo());

    if (!m_activeServer.get())
      return;

    m_activeServer->open();
    m_projectFacade.reset(new ta::model::ProjectFacade(m_activeServer));
    m_projSceneFacade.reset(new ta::model::ProjectSceneFacade(m_activeServer));
    m_sceneFacade.reset(new ta::model::SceneFacade(m_activeServer));
    loadData();
  }
}

void ta::model::qt::AOIControlProjectWizardPage::onProjectComboBoxIndexChanged(int /*index*/)
{
  te::qt::widgets::ScopedCursor cursor(Qt::WaitCursor);
  m_ui->m_scenarioProjectTableWidget->clearContents();
  m_ui->m_scenarioProjectTableWidget->setRowCount(0);

  std::string projectName = m_ui->m_projectComboBox->currentText().toStdString();
  int projId = m_ui->m_projectComboBox->currentData().toInt();
  std::vector<int> sceneIds = m_projSceneFacade->getScenes(projId);

  //Loading all the scenes associated with the selected project
  size_t scnNmbr = sceneIds.size();
  std::unique_ptr<ta::model::Scene> scene;

  //Loading all the projects scenes
  std::vector<ta::model::AbstractItem*> projScenes = m_projSceneFacade->get();
  std::vector<std::unique_ptr<ta::model::ProjectScene> > projScnPtrs;

  size_t projScnNmbr = projScenes.size();

  for (std::size_t t = 0; t < projScnNmbr; ++t)
    projScnPtrs.emplace_back(dynamic_cast<ta::model::ProjectScene*>(projScenes[t]));

  for (std::size_t t = 0; t < scnNmbr; ++t)
  {
    std::unique_ptr<ta::model::Scene> scene(dynamic_cast<ta::model::Scene*>(m_sceneFacade->get(sceneIds[t])));
    addScene(scene.get(), m_ui->m_scenarioProjectTableWidget);

    QCheckBox* lockCheck = dynamic_cast<QCheckBox*>(m_ui->m_scenarioProjectTableWidget->cellWidget((int)t, 1));
    for (size_t j = 0; j < projScnNmbr; ++j)
    {
      if (projScnPtrs[j]->getSceneId() == scene->getId())
        lockCheck->setChecked(projScnPtrs[j]->isLocked());
    }
  }
}

void ta::model::qt::AOIControlProjectWizardPage::onAddScenarioButtonClicked()
{
  int row = m_ui->m_scenarioTableWidget->currentRow();

  if (row == -1)
  {
    QMessageBox::warning(this, windowTitle(), tr("Select a scene to be associated with the project."));
    return;
  }

  int sceneId = m_ui->m_scenarioTableWidget->item(row, 0)->text().toInt();
  int projId = m_ui->m_projectComboBox->currentData().toInt();

  if (m_projSceneFacade->isAssociated(projId, sceneId))
  {
    QMessageBox::warning(this, windowTitle(), tr("This project is already associated with the selected scene."));
    return;
  }

  m_projectFacade->addAssocietedScene(projId, sceneId);
  addScene(dynamic_cast<ta::model::Scene*>(m_sceneFacade->get(sceneId)), m_ui->m_scenarioProjectTableWidget);
}

void ta::model::qt::AOIControlProjectWizardPage::onLockSceneExecuted(int row)
{
  int projId = m_ui->m_projectComboBox->currentData().toInt();
  int sceneId = m_ui->m_scenarioProjectTableWidget->item(row, 2)->text().toInt();
  QCheckBox* lockCheck = dynamic_cast<QCheckBox*>(m_ui->m_scenarioProjectTableWidget->cellWidget(row, 1));
  bool locked = lockCheck->isChecked();

  if(!m_projSceneFacade->isAssociated(projId, sceneId))
  {
    QMessageBox::warning(this, windowTitle(), tr("This project is not associated with the selected scene."));
    return;
  }

  if (!m_projSceneFacade->lock(projId, sceneId, locked))
  {
    QMessageBox::warning(this, windowTitle(), m_projSceneFacade->getErrorMessage().c_str());
  }
}

void ta::model::qt::AOIControlProjectWizardPage::onSceneFilterButttonClicked()
{
  std::string datasetName = ta::model::Globals::sm_scene;
  te::da::DataSetTypePtr dt(m_activeServer->getDataSetType(datasetName).release());
  std::unique_ptr<te::da::DataSet> dataset = filter(dt);
  if (dataset.get() && dataset->moveBeforeFirst())
  {
    te::qt::widgets::ScopedCursor cursor(Qt::WaitCursor);
    m_ui->m_scenarioTableWidget->clearContents();
    m_ui->m_scenarioTableWidget->setRowCount(0);
    while (dataset->moveNext())
    {
      int id = dataset->getInt32(ta::model::Scene::sm_id);
      ta::model::AbstractItem* item = m_sceneFacade->get(id);
      std::unique_ptr<ta::model::Scene> scene(dynamic_cast<ta::model::Scene*>(item));
      addScene(scene.get(), m_ui->m_scenarioTableWidget);
    }
  }
}

void ta::model::qt::AOIControlProjectWizardPage::onRemScenarioButtonClicked(int row)
{
  int projId = m_ui->m_projectComboBox->currentData().toInt();
  int sceneId = m_ui->m_scenarioProjectTableWidget->item(row, 2)->text().toInt();

  if(!m_projSceneFacade->isAssociated(projId, sceneId))
  {
    QMessageBox::warning(this, windowTitle(), tr("This project is not  associated with the selected scene."));
    return;
  }
  removed(row);
  m_projSceneFacade->remove(projId, sceneId);
  m_ui->m_scenarioProjectTableWidget->removeRow(row);
  m_ui->m_scenarioProjectTableWidget->resizeColumnsToContents();
}

void ta::model::qt::AOIControlProjectWizardPage::addScene(ta::model::Scene* scene, QTableWidget* sceneTable)
{
  int row = sceneTable->rowCount();
  sceneTable->insertRow(row);

  QTableWidgetItem* idItem = new QTableWidgetItem(QString::number(scene->getId()));
  idItem->setTextAlignment(Qt::AlignCenter);


  QTableWidgetItem* viewDateItem = new QTableWidgetItem(QString::fromStdString(scene->getViewDate()->toString()));
  viewDateItem->setTextAlignment(Qt::AlignCenter);


  QTableWidgetItem* inclusionDateItem = new QTableWidgetItem(QString::fromStdString(scene->getInclusion()->toString()));
  inclusionDateItem->setTextAlignment(Qt::AlignCenter);


  QTableWidgetItem* julDateItem = new QTableWidgetItem(QString::number(scene->getJulDay()));
  julDateItem->setTextAlignment(Qt::AlignCenter);


  QTableWidgetItem* aoiItem = new QTableWidgetItem(QString::fromStdString(scene->getAiObjectId()));
  aoiItem->setTextAlignment(Qt::AlignCenter);

  QTableWidgetItem* layerIdItem = new QTableWidgetItem(QString::number(scene->getAoiLayerId()));
  layerIdItem->setTextAlignment(Qt::AlignCenter);

  if (sceneTable->columnCount() > 6)
  {
    //action
    QWidget* wAction = new QWidget(sceneTable);

    //A ToolButton used to remove a scene from a project
    QToolButton* removeButton = new QToolButton(sceneTable);
    removeButton->setObjectName("REMOVE");
    removeButton->setAutoRaise(true);
    removeButton->setIcon(QIcon::fromTheme("list-remove"));

    QGridLayout* layout = new QGridLayout(wAction);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->addWidget(removeButton);

    //A checkbox used to know if the scene is looked for changes
    QCheckBox* lockedCheck = new QCheckBox(sceneTable);
    lockedCheck->setObjectName("LOCK");

    sceneTable->setCellWidget(row, 0, wAction);
    sceneTable->setCellWidget(row, 1, lockedCheck);
    sceneTable->setItem(row, 2, idItem);
    sceneTable->setItem(row, 3, viewDateItem);
    sceneTable->setItem(row, 4, inclusionDateItem);
    sceneTable->setItem(row, 5, julDateItem);
    sceneTable->setItem(row, 6, aoiItem);
    sceneTable->setItem(row, 7, layerIdItem);

    //Mapping the signals
    m_lockMapper->setMapping(lockedCheck, row);
    connect(lockedCheck, SIGNAL(stateChanged(int)), m_lockMapper, SLOT(map()));

    m_removeMapper->setMapping(removeButton, row);
    connect(removeButton, SIGNAL(pressed()), m_removeMapper, SLOT(map()));
  }
  else
  {
    sceneTable->setItem(row, 0, idItem);
    sceneTable->setItem(row, 1, viewDateItem);
    sceneTable->setItem(row, 2, inclusionDateItem);
    sceneTable->setItem(row, 3, julDateItem);
    sceneTable->setItem(row, 4, aoiItem);
    sceneTable->setItem(row, 5, layerIdItem);
  }

  sceneTable->resizeColumnToContents(0);
}

void ta::model::qt::AOIControlProjectWizardPage::loadData()
{
  if (!m_activeServer.get())
    return;

  //Loading all the scenes
  std::vector<ta::model::AbstractItem*> scenes = m_sceneFacade->get();
  std::vector<std::unique_ptr<ta::model::Scene> > scnPtrs;

  size_t scnNmbr = scenes.size();
  for (std::size_t t = 0; t < scnNmbr; ++t)
    scnPtrs.emplace_back(dynamic_cast<ta::model::Scene*>(scenes[t]));

  //Adding the scenes to the project, with existing associations if that is the case
  for (std::size_t i = 0; i < scnNmbr; ++i)
    addScene(scnPtrs[i].get(), m_ui->m_scenarioTableWidget);

  scnPtrs.clear();

  //Loading all the projects
  std::vector<ta::model::AbstractItem*> projects = m_projectFacade->get();
  std::vector<std::unique_ptr<ta::model::Project> > projPtrs;

  size_t projNmbr = projects.size();

  for (std::size_t t = 0; t < projNmbr; ++t)
    projPtrs.emplace_back(dynamic_cast<ta::model::Project*>(projects[t]));

  for (std::size_t i = 0; i < projNmbr; ++i)
  {
    m_ui->m_projectComboBox->addItem(QString::fromStdString(projPtrs[(int)i]->getName()));
    m_ui->m_projectComboBox->setItemData((int)i, QVariant::fromValue(projPtrs[(int)i]->getId()));
  }

  onProjectComboBoxIndexChanged(0);
  projPtrs.clear();
}

std::unique_ptr<te::da::DataSet> ta::model::qt::AOIControlProjectWizardPage::filter(te::da::DataSetTypePtr dataType)
{
  std::unique_ptr<QDialog> dialog(new QDialog(this));

  te::qt::widgets::WhereClauseWidget* whereWdg = new te::qt::widgets::WhereClauseWidget(dialog.get());
  whereWdg->setCriteriaTabEnabled(1, false);
  whereWdg->setDataSource(m_activeServer);

  //Fetching the attribute list
  size_t propNmb = dataType->size();
  std::vector<std::string> properties;

  for (size_t i = 0; i < propNmb; ++i)
  {
    std::string propName = dataType->getProperty(i)->getName();
    if (dataType->getProperty(i)->getType() != te::dt::GEOMETRY_TYPE && dataType->getProperty(i)->getType() != te::dt::DATETIME_TYPE)
    {
      properties.push_back(propName);
    }
  }

  whereWdg->setAttributeList(properties);

  //Getting the capabilities of the active server
  te::da::DataSourceCapabilities dsCap = m_activeServer->getCapabilities();
  te::da::QueryCapabilities queryCap = dsCap.getQueryCapabilities();

  std::vector<std::string> vecOperators;

  std::set<std::string>::iterator it;

  it = queryCap.getArithmeticOperators().begin();
  while (it != queryCap.getArithmeticOperators().end())
  {
    vecOperators.push_back(*it);
    ++it;
  }

  it = queryCap.getComparsionOperators().begin();
  while (it != queryCap.getComparsionOperators().end())
  {
    vecOperators.push_back(*it);
    ++it;
  }

  whereWdg->setOperatorsList(vecOperators);

  //adjusting the the fromItems
  std::vector<std::pair<std::string, std::string> > list;
  list.push_back(std::pair<std::string, std::string>(dataType->getName(), ""));

  whereWdg->setFromItems(list);

  // Adjusting...
  QGridLayout* layout = new QGridLayout(dialog.get());
  layout->addWidget(whereWdg);

  QDialogButtonBox* bbox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel, Qt::Horizontal, dialog.get());
  connect(bbox, SIGNAL(accepted()), dialog.get(), SLOT(accept()));
  connect(bbox, SIGNAL(rejected()), dialog.get(), SLOT(reject()));

  layout->addWidget(bbox);
  layout->setSizeConstraint(QLayout::SetFixedSize);

  std::unique_ptr<te::da::DataSet> dataset;

  int res = dialog->exec();

  if (res == QDialog::Accepted)
  {
    te::qt::widgets::ScopedCursor cursor(Qt::WaitCursor);
    te::da::Select* select = new te::da::Select();

    te::da::Fields* fields = new te::da::Fields;
    te::da::Field* f = new te::da::Field(new te::da::PropertyName("*"));
    fields->push_back(f);

    te::da::From* from = new te::da::From;
    te::da::FromItem* fi = new te::da::DataSetName(dataType->getName(), "");
    from->push_back(fi);

    te::da::Where* where = whereWdg->getWhere();

    select->setFields(fields);
    select->setFrom(from);
    select->setWhere(where);

    dataset.reset(m_activeServer->query(select).release());
  }
  return std::move(dataset);
}

void ta::model::qt::AOIControlProjectWizardPage::removed(int row)
{
  size_t rows = m_ui->m_scenarioProjectTableWidget->rowCount();

  for (size_t i = (row + 1); row <= rows; ++i)
  {
    QWidget* w = m_ui->m_scenarioProjectTableWidget->cellWidget((int)i, 6);

    if (!w)
      return;

    //Re-mapping the signals
    QToolButton* removeToolButton = w->findChild<QToolButton*>("REMOVE");
    m_removeMapper->setMapping(removeToolButton, ((int)i - 1));
  }
}
