/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/AOIControlScenarioWizardPage.cpp

  \brief A Qt wizard page used to configure the scenario from AOI Control.
*/

//Terralib
#include <terralib/dataaccess/dataset/DataSet.h>
#include <terralib/dataaccess/dataset/DataSetType.h>
#include <terralib/dataaccess/datasource/DataSourceCapabilities.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>
#include <terralib/dataaccess/datasource/DataSourceInfoManager.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/QueryCapabilities.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>
#include <terralib/dataaccess/utils/Utils.h>
#include <terralib/datatype/Date.h>
#include <terralib/datatype/TimeInstant.h>
#include <terralib/geometry/GeometryProperty.h>
#include <terralib/qt/widgets/query/WhereClauseWidget.h>
#include <terralib/qt/widgets/table/DataSetTableModel.h>
#include <terralib/qt/widgets/utils/ScopedCursor.h>
#include "ui_AOIControlScenarioWizardPageForm.h"
#include "AOIControlScenarioWizardPage.h"

//TerraAmazon
#include "../../facade/CatalogFacade.h"
#include "../../facade/AoiLayerFacade.h"
#include "../../facade/SceneFacade.h"
#include "../../item/AoiLayer.h"
#include "../../item/Catalog.h"
#include "../../item/Scene.h"
#include "model/Globals.h"

//QT
#include <QDialogButtonBox>
#include <QGridLayout>
#include <QMessageBox>

// Boost
#include <boost/date_time/gregorian/gregorian.hpp>

ta::model::qt::AOIControlScenarioWizardPage::AOIControlScenarioWizardPage(QWidget* parent)
  : QWizardPage(parent),
  m_ui(new Ui::AOIControlScenarioWizardPageForm)
{
  m_ui->setupUi(this);

  //configure page
  this->setTitle(tr("AOI Control Scenario"));
  this->setSubTitle(tr("Allows the user to associate a date to an area of interest."));

  //data table view
  m_tableModel = new te::qt::widgets::DataSetTableModel(m_ui->m_tableView);
  m_ui->m_tableView->setModel(m_tableModel);
  m_ui->m_tableView->setSelectionBehavior(QAbstractItemView::SelectRows);
  m_ui->m_tableView->setSelectionMode(QAbstractItemView::SingleSelection);

  //scene table view
  m_ui->m_sceneTableWidget->setSelectionMode(QAbstractItemView::NoSelection);

  //Current date
  m_ui->m_dateEdit->setDate(QDate::currentDate());

  // add button icons
  m_ui->m_aoiFilterToolButton->setIcon(QIcon::fromTheme("view-filter"));
  m_ui->m_aoiScenarioFilterToolButton->setIcon(QIcon::fromTheme("view-filter"));
  m_ui->m_aoiAddToolButton->setIcon(QIcon::fromTheme("list-add"));

  //connect
  connect(m_ui->m_aoiLayerComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onAOIComboBoxIndexChanged(int)));
  connect(m_ui->m_aoiAddToolButton, SIGNAL(clicked()), this, SLOT(onAddSceneButtonClicked()));
  connect(m_ui->m_aoiFilterToolButton, SIGNAL(clicked()), this, SLOT(onAOIFilterButttonClicked()));
  connect(m_ui->m_aoiScenarioFilterToolButton, SIGNAL(clicked()), this, SLOT(onSceneFilterButttonClicked()));

  m_ui->m_sceneTableWidget->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
}

ta::model::qt::AOIControlScenarioWizardPage::~AOIControlScenarioWizardPage()
{
}

bool ta::model::qt::AOIControlScenarioWizardPage::isComplete() const
{
  return true;
}

te::da::DataSourceInfoPtr ta::model::qt::AOIControlScenarioWizardPage::getServer()
{
  return te::da::DataSourceInfoManager::getInstance().get(m_activeServer->getId());
}

void ta::model::qt::AOIControlScenarioWizardPage::setServer(te::da::DataSourceInfoPtr server)
{
  if (server.get())
  {
    //get data source
    m_activeServer = te::da::DataSourceManager::getInstance().get(server->getId(), server->getType(), server->getConnInfo());

    if (!m_activeServer.get())
      return;

    m_activeServer->open();
    m_aoiLayerFacade.reset(new ta::model::AoiLayerFacade(m_activeServer));
    m_catalogFacade.reset(new ta::model::CatalogFacade(m_activeServer));
    m_sceneFacade.reset(new ta::model::SceneFacade(m_activeServer));
    loadData();
  }
}

void ta::model::qt::AOIControlScenarioWizardPage::onAOIComboBoxIndexChanged(int /*index*/)
{
  te::qt::widgets::ScopedCursor cursor(Qt::WaitCursor);
  m_ui->m_aoiIDComboBox->clear();

  m_ui->m_aoiIDComboBox->addItem(QString::fromStdString("Select..."));
  std::string datasetName = m_ui->m_aoiLayerComboBox->currentText().toStdString();
  te::da::DataSet* dataset(m_activeServer->getDataSet(datasetName).release());
  std::unique_ptr<te::da::DataSetType> dataType(m_activeServer->getDataSetType(datasetName).release());

  std::unique_ptr<ta::model::Catalog> catalog (dynamic_cast<ta::model::Catalog*>(m_catalogFacade->get(m_ui->m_aoiLayerComboBox->currentData().toInt())));

  std::vector<te::dt::Property*> props = dataType->getProperties();
  size_t numProps = props.size();

  for (size_t i = 0; i < numProps; ++i)
  {
    std::string propName = dataType->getProperty(i)->getName();
    if (dataType->getProperty(i)->getType() != te::dt::GEOMETRY_TYPE)
    {
      m_ui->m_aoiIDComboBox->addItem(QString::fromStdString(props[i]->getName()), QVariant((int)i));
    }
  }

  const std::string aoidColumn = m_aoiLayerFacade->getAoiIdColumn(catalog->getId());
  if (aoidColumn != "-")
  {
    m_ui->m_aoiIDComboBox->setCurrentText(QString::fromStdString(aoidColumn));
    m_ui->m_aoiIDComboBox->setEnabled(false);
  }
  else
    m_ui->m_aoiIDComboBox->setEnabled(true);

  if (dataset->moveBeforeFirst())
    m_tableModel->setDataSet(dataset, m_activeServer->getEncoding());
  else
    m_tableModel->setDataSet(0, m_activeServer->getEncoding());
}

void ta::model::qt::AOIControlScenarioWizardPage::onAddSceneButtonClicked()
{
  //check if exist items selected
  QItemSelectionModel* select = m_ui->m_tableView->selectionModel();

  if (!select->hasSelection())
  {
    QMessageBox::warning(this, windowTitle(), tr("Select a geometry as an area of interest."));
    return;
  }

  QModelIndexList indexList = m_ui->m_tableView->selectionModel()->selectedRows();
  QModelIndex rowIndex = indexList.at(0);
  QModelIndex index = m_tableModel->index(rowIndex.row(), m_ui->m_aoiIDComboBox->currentData().toInt(), QModelIndex());
  
  std::string aoiId = m_ui->m_tableView->model()->data(index).toString().toStdString();

  if (m_ui->m_aoiIDComboBox->isEnabled())
  {

    if (m_ui->m_aoiIDComboBox->currentText().toStdString() == "Select...")
    {
      QMessageBox::warning(this, windowTitle(), tr("Select thye column that will be used to identify an area of interest."));
      return;
    }

    //Creating the aoi loayer connection
    std::unique_ptr<ta::model::AoiLayer> aoiLayer(new ta::model::AoiLayer());
    aoiLayer->setCatalogId(m_ui->m_aoiLayerComboBox->currentData().toInt());
    aoiLayer->setAoiIdColumn(m_ui->m_aoiIDComboBox->currentText().toStdString());
    m_aoiLayerFacade->save(aoiLayer.get());
    m_ui->m_aoiIDComboBox->setEnabled(false);
  }

  //view date m_tableView
  int day, month, year;
  QDate viewQDate = m_ui->m_dateEdit->date();
  day = viewQDate.day();
  month = viewQDate.month();
  year = viewQDate.year();

  boost::gregorian::date boostViewDate(year, month, day);
  std::unique_ptr<te::dt::Date> viewDate(new te::dt::Date(boostViewDate));

  //Inclusion date
  boost::posix_time::ptime boostLocalTime = boost::posix_time::second_clock::local_time();
  std::unique_ptr<te::dt::TimeInstant> inclusionDate(new te::dt::TimeInstant(boostLocalTime));

  //julday
  QDate date = m_ui->m_dateEdit->date();
  int julday = date.dayOfYear();

  //Creating the new scene
  std::unique_ptr<ta::model::Scene> newScene(new ta::model::Scene());

  newScene->setInclusion(std::move(inclusionDate));
  newScene->setJulDay(julday);
  newScene->setViewDate(std::move(viewDate));
  newScene->setAoiLayerId(m_ui->m_aoiLayerComboBox->currentData().toInt());
  newScene->setAiObjectId(aoiId);

  if (!m_sceneFacade->save(newScene.get()))
    QMessageBox::warning(this, windowTitle(), m_sceneFacade->getErrorMessage().c_str());
  else
    addScene(newScene.get());
}

void ta::model::qt::AOIControlScenarioWizardPage::onAOIFilterButttonClicked()
{
  std::string datasetName = m_ui->m_aoiLayerComboBox->currentText().toStdString();
  te::da::DataSetTypePtr dt(m_activeServer->getDataSetType(datasetName).release());
  std::unique_ptr<te::da::DataSet> dataset = filter(dt);
  if (dataset.get() && dataset->moveBeforeFirst())
  {
    te::qt::widgets::ScopedCursor cursor(Qt::WaitCursor);
    m_tableModel->setDataSet(dataset.release(), m_activeServer->getEncoding());
  }
}

void ta::model::qt::AOIControlScenarioWizardPage::onSceneFilterButttonClicked()
{
  std::string datasetName = ta::model::Globals::sm_scene;
  te::da::DataSetTypePtr dt(m_activeServer->getDataSetType(datasetName).release());
  std::unique_ptr<te::da::DataSet> dataset = filter(dt);
  if (dataset.get() && dataset->moveBeforeFirst())
  {
    te::qt::widgets::ScopedCursor cursor(Qt::WaitCursor);
    m_ui->m_sceneTableWidget->clearContents();
    m_ui->m_sceneTableWidget->setRowCount(0);
    while (dataset->moveNext())
    {
      int id = dataset->getInt32(ta::model::Scene::sm_id);
      ta::model::AbstractItem* item = m_sceneFacade->get(id);
      std::unique_ptr<ta::model::Scene> scene(dynamic_cast<ta::model::Scene*>(item));
      addScene(scene.get());
    }
  }
}

void ta::model::qt::AOIControlScenarioWizardPage::addScene(ta::model::Scene* scene)
{
  int row = m_ui->m_sceneTableWidget->rowCount();
  m_ui->m_sceneTableWidget->insertRow(row);

  QTableWidgetItem* idItem = new QTableWidgetItem(QString::number(scene->getId()));
  idItem->setTextAlignment(Qt::AlignCenter);
  m_ui->m_sceneTableWidget->setItem(row, 0, idItem);

  QTableWidgetItem* viewDateItem = new QTableWidgetItem(QString::fromStdString(scene->getViewDate()->toString()));
  viewDateItem->setTextAlignment(Qt::AlignCenter);
  m_ui->m_sceneTableWidget->setItem(row, 1, viewDateItem);

  QTableWidgetItem* inclusionDateItem = new QTableWidgetItem(QString::fromStdString(scene->getInclusion()->toString()));
  inclusionDateItem->setTextAlignment(Qt::AlignCenter);
  m_ui->m_sceneTableWidget->setItem(row, 2, inclusionDateItem);

  QTableWidgetItem* julDateItem = new QTableWidgetItem(QString::number(scene->getJulDay()));
  julDateItem->setTextAlignment(Qt::AlignCenter);
  m_ui->m_sceneTableWidget->setItem(row, 3, julDateItem);

  QTableWidgetItem* aoiItem = new QTableWidgetItem(QString::fromStdString(scene->getAiObjectId()));
  aoiItem->setTextAlignment(Qt::AlignCenter);
  m_ui->m_sceneTableWidget->setItem(row, 4, aoiItem);

  QTableWidgetItem* layerIdItem = new QTableWidgetItem(QString::number(scene->getAoiLayerId()));
  layerIdItem->setTextAlignment(Qt::AlignCenter);
  m_ui->m_sceneTableWidget->setItem(row, 5, layerIdItem);
}

std::unique_ptr<te::da::DataSet> ta::model::qt::AOIControlScenarioWizardPage::filter(te::da::DataSetTypePtr dataType)
{
  std::unique_ptr<QDialog> dialog(new QDialog(this));

  te::qt::widgets::WhereClauseWidget* whereWdg = new te::qt::widgets::WhereClauseWidget(dialog.get());
  whereWdg->setCriteriaTabEnabled(1, false);
  whereWdg->setDataSource(m_activeServer);

  //Fetching the attribute list
  size_t propNmb = dataType->size();
  std::vector<std::string> properties;

  for (size_t i = 0; i < propNmb; ++i)
  {
    std::string propName = dataType->getProperty(i)->getName();
    if (dataType->getProperty(i)->getType() != te::dt::GEOMETRY_TYPE && dataType->getProperty(i)->getType() != te::dt::DATETIME_TYPE)
    {
      properties.push_back(propName);
    }
  }

  whereWdg->setAttributeList(properties);

  //Getting the capabilities of the active server
  te::da::DataSourceCapabilities dsCap = m_activeServer->getCapabilities();
  te::da::QueryCapabilities queryCap = dsCap.getQueryCapabilities();

  std::vector<std::string> vecOperators;

  std::set<std::string>::iterator it;

  it = queryCap.getArithmeticOperators().begin();
  while (it != queryCap.getArithmeticOperators().end())
  {
    vecOperators.push_back(*it);
    ++it;
  }

  it = queryCap.getComparsionOperators().begin();
  while (it != queryCap.getComparsionOperators().end())
  {
    vecOperators.push_back(*it);
    ++it;
  }

  whereWdg->setOperatorsList(vecOperators);

  //adjusting the the fromItems
  std::vector<std::pair<std::string, std::string> > list;
  list.push_back(std::pair<std::string, std::string>(dataType->getName(), ""));

  whereWdg->setFromItems(list);

  // Adjusting...
  QGridLayout* layout = new QGridLayout(dialog.get());
  layout->addWidget(whereWdg);

  QDialogButtonBox* bbox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel, Qt::Horizontal, dialog.get());
  connect(bbox, SIGNAL(accepted()), dialog.get(), SLOT(accept()));
  connect(bbox, SIGNAL(rejected()), dialog.get(), SLOT(reject()));

  layout->addWidget(bbox);
  layout->setSizeConstraint(QLayout::SetFixedSize);

  std::unique_ptr<te::da::DataSet> dataset;

  int res = dialog->exec();

  if (res == QDialog::Accepted)
  {
    te::qt::widgets::ScopedCursor cursor(Qt::WaitCursor);
    te::da::Select* select = new te::da::Select();

    te::da::Fields* fields = new te::da::Fields;
    te::da::Field* f = new te::da::Field(new te::da::PropertyName("*"));
    fields->push_back(f);

    te::da::From* from = new te::da::From;
    te::da::FromItem* fi = new te::da::DataSetName(dataType->getName(), "");
    from->push_back(fi);

    te::da::Where* where = whereWdg->getWhere();

    select->setFields(fields);
    select->setFrom(from);
    select->setWhere(where);

    dataset.reset(m_activeServer->query(select).release());
  }
  return std::move(dataset);
}

void ta::model::qt::AOIControlScenarioWizardPage::loadData()
{
  if (!m_activeServer.get())
    return;

  std::vector<ta::model::AbstractItem*> catalogs = m_catalogFacade->get();
  std::vector<std::unique_ptr<ta::model::Catalog> > catalogPtrs;

  size_t catalogNmbr = catalogs.size();
  for (std::size_t t = 0; t < catalogNmbr; ++t)
    catalogPtrs.emplace_back(dynamic_cast<ta::model::Catalog*>(catalogs[t]));

  for (std::size_t i = 0; i < catalogNmbr; ++i)
  {
    if (catalogPtrs[i]->isLocked())
      continue;

    te::da::DataSetTypePtr dt(m_activeServer->getDataSetType(catalogPtrs[i]->getName()).release());
    bool hasGeom = dt->hasGeom();

    if (hasGeom)
    {
      te::gm::GeometryProperty* gp = te::da::GetFirstGeomProperty(dt.get());
      if ((gp->getGeometryType() == te::gm::MultiPolygonType) || (gp->getGeometryType() == te::gm::PolygonType))
       m_ui->m_aoiLayerComboBox->addItem(QString::fromStdString(catalogPtrs[i]->getName()), QVariant(catalogPtrs[i]->getId()));
    }
  }

  std::vector<ta::model::AbstractItem*> scenes = m_sceneFacade->get();
  std::vector<std::unique_ptr<ta::model::Scene> > scnPtrs;

  size_t scnNmbr = scenes.size();
  for (std::size_t t = 0; t < scnNmbr; ++t)
    scnPtrs.emplace_back(dynamic_cast<ta::model::Scene*>(scenes[t]));

  for (std::size_t i = 0; i < scnNmbr; ++i)
    addScene(scnPtrs[i].get());

  catalogPtrs.clear();
  scnPtrs.clear();
}
