/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/AddNewTaskDialog.cpp

  \brief A Qt dialog used to configure...
*/

// TerraLib
#include <terralib/common/PlatformUtils.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>

// TerraAmazon
#include "../../Globals.h"
#include "../../facade/AoiLayerFacade.h"
#include "../../facade/CatalogFacade.h"
#include "../../facade/PhaseFacade.h"
#include "../../facade/SceneFacade.h"
#include "../../facade/TaskFacade.h"
#include "../../item/AoiLayer.h"
#include "../../item/Catalog.h"
#include "../../item/Phase.h"
#include "../../item/Scene.h"
#include "../../item/Task.h"
#include "../../../common/Utils.h"
#include "AddNewTaskDialog.h"
#include "ui_AddNewTaskDialogForm.h"

// Qt
#include <QGridLayout>
#include <QMessageBox>

ta::model::qt::AddNewTaskDialog::AddNewTaskDialog(QWidget* parent, Qt::WindowFlags f)
  : QDialog(parent, f),
  m_ui(new Ui::AddNewTaskDialogForm),
  m_hadAddedScene(false)
{
  m_ui->setupUi(this);
  
  std::string icon = ta::common::FindInTerraAmazonPath("share/icons/svg/terraAmazom-colorido-horizontal.svg");
  m_ui->m_imgLabel->setPixmap(QIcon(icon.c_str()).pixmap(112, 100));
  m_ui->m_imgLabel->hide();

  m_ui->m_sceneTable->setSortingEnabled(true);

  connect(m_ui->m_closePushButton, SIGNAL(clicked()), this, SLOT(onClosePushClicked()));
  connect(m_ui->m_addToolButton, SIGNAL(clicked()), this, SLOT(onAddToolPushClicked()));
  connect(m_ui->m_sceneTable, SIGNAL(cellPressed(int, int)), this, SLOT(onSceneTableClicked(int, int)));

}

ta::model::qt::AddNewTaskDialog::~AddNewTaskDialog()
{}

void ta::model::qt::AddNewTaskDialog::init()
{
  PhaseFacade phaseFacade(m_server);

  //get phases from selected data source (server)
  std::vector<ta::model::AbstractItem*> phases = phaseFacade.get();

  m_ui->m_phaseComboBox->clear();

  for (std::size_t t = 0; t < phases.size(); ++t)
  {
    ta::model::Phase* p = dynamic_cast<ta::model::Phase*>(phases[t]);

    m_ui->m_phaseComboBox->addItem(p->getDescription().c_str(), QVariant::fromValue(p->getId()));

  }

  listScenes();

}

void ta::model::qt::AddNewTaskDialog::listScenes()
{
  assert(m_server.get());

  while (m_ui->m_sceneTable->rowCount()> 0)
    m_ui->m_sceneTable->removeRow(0);

  TaskFacade tf(m_server);

  std::auto_ptr<te::da::DataSet> dataSet = tf.getTaskControl(m_projId);

  std::size_t row = 0;

  while (dataSet->moveNext())
  {
    m_ui->m_sceneTable->insertRow((int)row);

    QString sceneId = QString::number(dataSet->getInt32("ID"));
    QString date = dataSet->getDateTime("VIEW_DATE")->toString().c_str();
    QString aiObjectId = dataSet->getString("AI_OBJECT_ID").c_str();
    QString layer = dataSet->getString("NAME").c_str();

    m_ui->m_sceneTable->setItem((int)row, 0, new QTableWidgetItem(sceneId));
    m_ui->m_sceneTable->setItem((int)row, 1, new QTableWidgetItem(date));
    m_ui->m_sceneTable->setItem((int)row, 2, new QTableWidgetItem(aiObjectId));
    m_ui->m_sceneTable->setItem((int)row, 3, new QTableWidgetItem(layer));

    ++row;
  }

  m_ui->m_sceneTable->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
}

bool ta::model::qt::AddNewTaskDialog::hadAddedScene()
{
  return m_hadAddedScene;
}

void ta::model::qt::AddNewTaskDialog::setTask(te::da::DataSourcePtr ds, const int& projId, const int& userId)
{
  m_server = ds;
  m_projId = projId;
  m_userId = userId;

  init();
}

void ta::model::qt::AddNewTaskDialog::onClosePushClicked()
{
  close();
}

void ta::model::qt::AddNewTaskDialog::onAddToolPushClicked()
{
  Task task;
  TaskFacade tf(m_server);

  if (m_ui->m_sceneTable->rowCount() == 0 || m_ui->m_sceneTable->currentRow() < 0)
  {
    return;
  }
  
  if (m_ui->m_phaseComboBox->currentText() == "")
  {
    QMessageBox::warning(this, "TerraAmazon", tr("Phase not informed!"));
    return;
  }

  m_hadAddedScene = true;

  std::size_t projectId = m_projId;
  QVariant variant = m_ui->m_phaseComboBox->itemData(m_ui->m_phaseComboBox->currentIndex(), Qt::UserRole);
  std::size_t phaseId = variant.toString().toInt();
  std::size_t userId = m_userId;
  std::size_t sceneId = m_ui->m_sceneTable->item(m_ui->m_sceneTable->currentRow(), 0)->text().toInt();

  if (tf.loadTask((int)projectId, (int)phaseId, (int)userId, (int)sceneId))
  {
    QMessageBox::warning(this, "TerraAmazon", tr("This task already exists!"));
    return;
  }

  task.setProjectId((int)projectId);
  task.setSceneId((int)sceneId);
  task.setUserId((int)userId);
  task.setPhaseId((int)phaseId);

  if (!tf.save(&task))
  {
    QMessageBox::warning(this,"TerraAmazon", tr("Error adding task! Verify active project!"));
    return;
  }
  else
  {
    QMessageBox::warning(this,"TerraAmazon", tr("Task added with success."));
    return;
  }
}

void ta::model::qt::AddNewTaskDialog::onSceneTableClicked(int row, int /*col*/)
{
  m_ui->m_attributeTable->setRowCount(0);

  if (row == -1 || row >= m_ui->m_sceneTable->rowCount())
  {
    return;
  }

  std::size_t scnId = m_ui->m_sceneTable->item(row, 0)->text().toInt();

  SceneFacade sf(m_server);
  std::unique_ptr<ta::model::Scene> scn(dynamic_cast<ta::model::Scene*>(sf.get((int)scnId)));

  if (!scn.get())
  {
    QMessageBox::information(this, tr("TerraAmazon"), tr("Scene not found!"));
    return;
  }

  AoiLayerFacade aoif(m_server);
  std::unique_ptr<ta::model::AoiLayer> aoilayer(dynamic_cast<ta::model::AoiLayer*>(aoif.get(scn->getAoiLayerId())));

  if (!aoilayer.get())
  {
    QMessageBox::information(this, tr("TerraAmazon"), tr("AoiLayer not found!"));
    return;
  }

  CatalogFacade cf(m_server);
  std::auto_ptr<ta::model::Catalog> catalog(dynamic_cast<ta::model::Catalog*>(cf.get(aoilayer->getCatalogId())));

  if (!catalog.get())
  {
    QMessageBox::information(this, tr("TerraAmazon"), tr("Catalog not found!"));
    return;
  }

  std::unique_ptr<te::da::DataSetType> dsType = cf.getDataSetType(catalog.get());
  std::unique_ptr<te::da::DataSet> ds = cf.getDataSet(catalog.get(), scn->getAiObjectId(), aoilayer->getAoiIdColumn());

  if (!ds.get())
  {
    QMessageBox::information(this, tr("TerraAmazon"), tr("Data not found!"));
    return;
  }

  ds->moveBeforeFirst();

  assert(ds->getNumProperties() == dsType->getProperties().size());

  if (!ds->moveNext())
    return;

  for (std::size_t i = 0; i < dsType->getProperties().size(); ++i)
  {
    if (dsType->getProperties()[i]->getType() == te::dt::GEOMETRY_TYPE)
      continue;

    m_ui->m_attributeTable->insertRow((int)i);

    m_ui->m_attributeTable->setItem((int)i, 0, new QTableWidgetItem(dsType->getProperties()[i]->getName().c_str()));
    m_ui->m_attributeTable->setItem((int)i, 1, new QTableWidgetItem(ds->getValue(i)->toString().c_str()));
  }

  m_ui->m_attributeTable->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);

}