/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of
    this application.
    */

/*!
  \file terraamazon/AttributeControlWidget.cpp

  \brief A Qt widget used to configure the attribute control.
  */

// TerraAmazon
#include "AttributeControlWidget.h"
#include "../../facade/CatalogFacade.h"
#include "../../facade/ProjectFacade.h"
#include "../../facade/RuleFacade.h"
#include "../../facade/RuleSetFacade.h"
#include "../../facade/RuleRuleSetFacade.h"
#include "../../item/Catalog.h"
#include "../../item/Rule.h"
#include "../../item/RuleRuleSet.h"
#include "../../item/RuleSet.h"
#include "ui_AttributeControlWidgetForm.h"
#include "../../../common/Utils.h"

// TerraLib
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>
#include <terralib/datatype//SimpleProperty.h>
#include <terralib/dataaccess/utils/Utils.h>

// Qt
#include <QToolButton>
#include <QMessageBox>

ta::model::qt::AttributeControlWidget::AttributeControlWidget(QWidget* parent, Qt::WindowFlags f)
: QWidget(parent, f),
m_ui(new Ui::AttributeControlWidgetForm)
{
  m_ui->setupUi(this);

  m_propWidget = new te::qt::widgets::NewPropertyWidget(m_ui->m_widget);
  QGridLayout* layout = new QGridLayout(m_ui->m_widget);
  layout->addWidget(m_propWidget);
  layout->setContentsMargins(0, 0, 0, 0);

  m_ui->m_createAttributeToolButton->setIcon(QIcon::fromTheme("list-add"));

  //mapper initializer
  m_inputLayerComboBoxMapper = new QSignalMapper(this);
  m_removeButtonMapper = new QSignalMapper(this);
  m_addButtonMapper = new QSignalMapper(this);
  m_confirmButtonMapper = new QSignalMapper(this);
  m_cancelButtonMapper = new QSignalMapper(this);

  connect(m_ui->m_projectComboBox, SIGNAL(activated(int)), this, SLOT(onProjectComboBoxActivated(int)));
  connect(m_ui->m_outputLayerComboBox, SIGNAL(activated(int)), this, SLOT(triggerRefreshTableWidget(int)));
  connect(m_inputLayerComboBoxMapper, SIGNAL(mapped(int)), this, SLOT(onInputLayerComboBoxActivated(int)));
  connect(m_addButtonMapper, SIGNAL(mapped(int)), this, SLOT(onAddButtonClicked(int)));
  connect(m_removeButtonMapper, SIGNAL(mapped(int)), this, SLOT(onRemoveButtonClicked(int)));
  connect(m_confirmButtonMapper, SIGNAL(mapped(int)), this, SLOT(onConfirmButtonClicked(int)));
  connect(m_cancelButtonMapper, SIGNAL(mapped(int)), this, SLOT(onCancelButtonClicked(int)));
  connect(m_ui->m_createAttributeToolButton, SIGNAL(pressed()), this, SLOT(onCreateAttributeToolButtonPressed()));
}

ta::model::qt::AttributeControlWidget::~AttributeControlWidget()
{
}

void ta::model::qt::AttributeControlWidget::setServer(te::da::DataSourceInfoPtr server)
{
  if (server.get())
  {
    m_activeServer = te::da::DataSourceManager::getInstance().get(server->getId(), server->getType(), server->getConnInfo());

    if (!m_activeServer.get())
      return;

    m_activeServer->open();

    m_catalogFacade.reset(new ta::model::CatalogFacade(m_activeServer));
    m_projectFacade.reset(new ta::model::ProjectFacade(m_activeServer));
    m_ruleFacade.reset(new ta::model::RuleFacade(m_activeServer));
    m_ruleSetFacade.reset(new ta::model::RuleSetFacade(m_activeServer));
    m_rulesLayerColumnFacade.reset(new ta::model::RulesLayerColumnFacade(m_activeServer));

    listProjects();
  }
}

void ta::model::qt::AttributeControlWidget::onAddButtonClicked(int row)
{
  int rulesetId = m_ui->m_outputLayerComboBox->currentData().toInt();
  std::string columnOut = m_ui->m_tableWidget->item(row, 1)->text().toStdString();
  QComboBox* combRule = dynamic_cast<QComboBox*>(m_ui->m_tableWidget->cellWidget(row, 2));
  int ruleId = combRule->currentData().toInt();

  QWidget* wd = dynamic_cast<QWidget*>(m_ui->m_tableWidget->cellWidget(row, 0));
  std::string objectName = wd->objectName().toStdString();

  if (ruleId > 0)
  {
    ta::model::RulesLayerColumn* rlc = new RulesLayerColumn();
    validateInputLayerAttribute(rlc, row, columnOut, ruleId, rulesetId);

    if (!(rlc->getRuleId() > 0))
    {
      return;
    }

    if (objectName.compare("ADD-NEW") == 0)
    {
      if (m_currentPropertyNew != NULL)
      {
        ta::model::RuleSet* rs = dynamic_cast<ta::model::RuleSet*>(m_ruleSetFacade->get(rulesetId));
        ta::model::Catalog* c = dynamic_cast<ta::model::Catalog*>(m_catalogFacade->get(rs->getCatalogId()));

        std::string connInfo = c->getUri();
        te::da::DataSourceInfoPtr dsInfo = te::da::DataSourceInfoManager::getInstance().getByConnInfo(connInfo);

        if (dsInfo.get())
        {
          try
          {
            te::da::DataSourcePtr server = te::da::GetDataSource(dsInfo->getId());
            m_rulesLayerColumnFacade->save(rlc);
            server->addProperty(c->getName(), m_currentPropertyNew);
            m_currentPropertyNew = NULL;
            triggerRefreshTableWidget(0);
          }
          catch (const std::exception& e)
          {
            QMessageBox::warning(this, "TerraAmazon", tr(e.what()));
          }
          catch (...)
          {
            QMessageBox::warning(this, "TerraAmazon", tr("Unknown error while trying to add new attribute!"));
          }
        }
        else
        {
          QMessageBox::warning(this, "TerraAmazon", tr("Unknown error while trying to associate the attribute!"));
        }

        delete rs;
        delete c;
      }
    }
    else
    {
      try
      {
        m_rulesLayerColumnFacade->save(rlc);
        triggerRefreshTableWidget(0);
      }
      catch (const std::exception& e)
      {
        QMessageBox::warning(this, "TerraAmazon", tr(e.what()));
      }
      catch (...)
      {
        QMessageBox::warning(this, "TerraAmazon", tr("Unknown error while trying to associate the attribute!"));
      }
    }

    delete rlc;
  }
  else
  {
    QMessageBox::warning(this, "TerraAmazon", tr("Select a input layer!"));
  }
}

void ta::model::qt::AttributeControlWidget::validateInputLayerAttribute(ta::model::RulesLayerColumn* rlc, const int& row, const std::string& columnOut, const int& ruleId, const int& rulesetId)
{
  QComboBox* combColumnIn = dynamic_cast<QComboBox*>(m_ui->m_tableWidget->cellWidget(row, 3));
  std::string columnIn = combColumnIn->currentText().toStdString();
  if (combColumnIn->currentIndex() > 0)
  {
    rlc->setColumnIn(columnIn);
    rlc->setColumnOut(columnOut);
    rlc->setRuleId(ruleId);
    rlc->setRulesetId(rulesetId);
  }
  else
  {
    QMessageBox::warning(this, "TerraAmazon", tr("Select a input layer attribute!"));
  }
}

void ta::model::qt::AttributeControlWidget::onCancelButtonClicked(int row)
{
  m_ui->m_tableWidget->removeRow(row);
  triggerRefreshTableWidget(0);
}

void ta::model::qt::AttributeControlWidget::onConfirmButtonClicked(int row)
{
  onAddButtonClicked(row);
}

void ta::model::qt::AttributeControlWidget::onCreateAttributeToolButtonPressed()
{
  if (m_propWidget->buildProperty())
  {
    te::dt::Property* p = m_propWidget->getProperty();
    if (addNewAttributeToTableWidget(p))
    {
      m_ui->m_tabWidget->setCurrentIndex(0);
      m_ui->m_tabWidget->setTabEnabled(1, false);
    }
    else
    {
      QMessageBox::warning(this, "TerraAmazon", tr("This property name is not allowed!"));
    }
  }
  else
  {
    //QMessageBox::warning(this, "TerraAmazon", tr("Could not create the property!"));
  }
}

void ta::model::qt::AttributeControlWidget::onInputLayerComboBoxActivated(int row)
{
  int columnOutType = m_ui->m_tableWidget->item(row, 1)->data(Qt::UserRole).toInt();

  QComboBox* inputComb = dynamic_cast<QComboBox*>(m_ui->m_tableWidget->cellWidget(row, 2));

  std::string inputLayerName = inputComb->currentText().toStdString();
  int inputRuleId = inputComb->currentData().toInt();

  if (inputRuleId > 0)
  {
    ta::model::Rule* rule = dynamic_cast<ta::model::Rule*>(m_ruleFacade->get(inputRuleId));
    ta::model::Catalog* catalog = dynamic_cast<ta::model::Catalog*>(m_catalogFacade->get(rule->getCatalogId()));

    // get attribute names
    std::vector<te::dt::Property*> properties = getAllowedAttributesFromLayer(catalog);

    QComboBox* comb = new QComboBox(m_ui->m_tableWidget);
    comb->addItem("select...", QVariant::fromValue(-1));

    for (std::size_t i = 0; i < properties.size(); ++i)
    {
      if (properties.at(i)->getType() == columnOutType)
      {
        comb->addItem(properties.at(i)->getName().c_str(), QVariant(properties.at(i)->getName().c_str()));
      }
    }

    m_ui->m_tableWidget->setCellWidget(row, 3, comb);

    resizeTableWidget();

    delete rule;
    delete catalog;
    te::common::FreeContents(properties);
    properties.clear();
  }
}

void ta::model::qt::AttributeControlWidget::onProjectComboBoxActivated(int index)
{
  int projId = m_ui->m_projectComboBox->itemData(index).toInt();

  initComboBoxOutputLayers(projId);

  triggerRefreshTableWidget(0);
}

void ta::model::qt::AttributeControlWidget::triggerRefreshTableWidget(int /*idx*/)
{
  if (m_ui->m_outputLayerComboBox->currentData() > 0)
  {
    const int& rulesetId = m_ui->m_outputLayerComboBox->currentData().toInt();
    m_propWidget->setDataSourceId(m_activeServer->getId());
    m_ui->m_tabWidget->setTabEnabled(1, true);
    m_currentPropertyNew = NULL;
    updateAttributeTableWidget(rulesetId);
  }
  else if (m_ui->m_outputLayerComboBox->currentData() == -1)
  {
    m_ui->m_tableWidget->clearContents();
    m_ui->m_tableWidget->setRowCount(0);
    m_propWidget->setDataSourceId("");
    m_ui->m_tabWidget->setTabEnabled(1, false);
    m_currentPropertyNew = NULL;
    resizeTableWidget();
  }
}

void ta::model::qt::AttributeControlWidget::onRemoveButtonClicked(int row)
{
  std::string columnOut = m_ui->m_tableWidget->item(row, 1)->text().toStdString();
  const int& rulesetId = m_ui->m_outputLayerComboBox->currentData().toInt();

  ta::model::RulesLayerColumn* rlc = m_rulesLayerColumnFacade->getByRuleSetIdAndColumnOut(rulesetId, columnOut);
  m_rulesLayerColumnFacade->remove(rlc->getId());

  triggerRefreshTableWidget(0);

  delete rlc;
}

bool ta::model::qt::AttributeControlWidget::addNewAttributeToTableWidget(te::dt::Property* prop)
{
  const int& rulesetId = m_ui->m_outputLayerComboBox->currentData().toInt();
  ta::model::RuleSet* rs = dynamic_cast<ta::model::RuleSet*>(m_ruleSetFacade->get(rulesetId));
  ta::model::Catalog* c = dynamic_cast<ta::model::Catalog*>(m_catalogFacade->get(rs->getCatalogId()));
  std::string className;
  m_catalogFacade->getClassColumn(c->getId(), className);

  std::vector<std::string> vec;
  vec.push_back(className);

  std::string nameConvert = te::common::Convert2LCase(prop->getName());
  std::string propName = Trim(nameConvert);

  // verify property name
  if (!isValidNewPropertyName(propName, c, vec))
  {
    return false;
  }

  int newrow = m_ui->m_tableWidget->rowCount();

  // attribute column
  m_ui->m_tableWidget->insertRow(newrow);
  QTableWidgetItem* attributeCell = new QTableWidgetItem();
  attributeCell->setText(propName.c_str());
  attributeCell->setData(Qt::UserRole, QVariant::fromValue(prop->getType()));
  attributeCell->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
  m_ui->m_tableWidget->setItem(newrow, 1, attributeCell);

  m_currentPropertyNew = prop;

  // complete table widget
  fillTableWidgetOptions(newrow, rulesetId, true);

  resizeTableWidget();

  delete rs;
  delete c;

  return true;
}

void ta::model::qt::AttributeControlWidget::fillTableWidgetOptions(const int& row, const int &ruleSetId, bool newField)
{
  // verify
  QString text = m_ui->m_tableWidget->item(row, 1)->text();
  std::string columnOut = text.toStdString();

  ta::model::RulesLayerColumn* rlc = m_rulesLayerColumnFacade->getByRuleSetIdAndColumnOut(ruleSetId, columnOut);

  if (rlc->getRulesetId() > 0)
  {
    // not editable
    ta::model::Rule* rule = dynamic_cast<ta::model::Rule*>(m_ruleFacade->get(rlc->getRuleId()));

    QTableWidgetItem* inputLayerCell = new QTableWidgetItem();
    inputLayerCell->setText(dynamic_cast<ta::model::Catalog*>(m_catalogFacade->get(rule->getCatalogId()))->getName().c_str());
    inputLayerCell->setData(Qt::UserRole, QVariant::fromValue(rule->getId()));
    inputLayerCell->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
    m_ui->m_tableWidget->setItem(row, 2, inputLayerCell);

    QTableWidgetItem* inputAttributeCell = new QTableWidgetItem();
    inputAttributeCell->setText(rlc->getColumnIn().c_str());
    inputAttributeCell->setData(Qt::UserRole, QVariant(rlc->getColumnIn().c_str()));
    inputAttributeCell->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
    m_ui->m_tableWidget->setItem(row, 3, inputAttributeCell);

    QToolButton* removeButton = new QToolButton(m_ui->m_tableWidget);
    removeButton->setAutoRaise(true);
    removeButton->setCheckable(false);
    removeButton->setIcon(QIcon::fromTheme("list-remove"));
    m_ui->m_tableWidget->setCellWidget(row, 0, removeButton);

    m_removeButtonMapper->setMapping(removeButton, row);
    connect(removeButton, SIGNAL(pressed()), m_removeButtonMapper, SLOT(map()));

    delete rule;
  }
  else
  {
    // get rule to input layers
    std::vector<ta::model::Rule*> rules = m_ruleFacade->getRulesWithIntersectionByRuleSet(ruleSetId);

    // combobox editable input layer
    QComboBox* comb = new QComboBox(m_ui->m_tableWidget);
    comb->addItem("select...", QVariant::fromValue(-1));
    for (std::vector<int>::size_type i = 0; i != rules.size(); i++)
    {
      ta::model::Catalog* catalog = dynamic_cast<ta::model::Catalog*>(m_catalogFacade->get(rules[i]->getCatalogId()));
      comb->addItem(catalog->getName().c_str(), QVariant::fromValue(rules[i]->getId()));
    }
    m_ui->m_tableWidget->setCellWidget(row, 2, comb);
    m_inputLayerComboBoxMapper->setMapping(comb, row);
    connect(comb, SIGNAL(currentIndexChanged(int)), m_inputLayerComboBoxMapper, SLOT(map()));

    if (newField)
    {
      QWidget* wActions = new QWidget(m_ui->m_tableWidget);
      wActions->setObjectName("ADD-NEW");

      QToolButton* confirmButton = new QToolButton(m_ui->m_tableWidget);
      confirmButton->setAutoRaise(true);
      confirmButton->setCheckable(false);
      confirmButton->setIcon(QIcon::fromTheme("list-add"));

      QToolButton* cancelButton = new QToolButton(m_ui->m_tableWidget);
      cancelButton->setAutoRaise(true);
      cancelButton->setCheckable(false);
      cancelButton->setIcon(QIcon::fromTheme("list-remove"));

      QGridLayout* layout = new QGridLayout(wActions);
      layout->setContentsMargins(0, 0, 0, 0);

      layout->addWidget(confirmButton, 0, 0);
      layout->addWidget(cancelButton, 0, 1);

      m_ui->m_tableWidget->setCellWidget(row, 0, wActions);
      m_confirmButtonMapper->setMapping(confirmButton, row);
      m_cancelButtonMapper->setMapping(cancelButton, row);
      connect(confirmButton, SIGNAL(pressed()), m_confirmButtonMapper, SLOT(map()));
      connect(cancelButton, SIGNAL(pressed()), m_cancelButtonMapper, SLOT(map()));
    }
    else
    {
      QToolButton* addButton = new QToolButton(m_ui->m_tableWidget);
      addButton->setAutoRaise(true);
      addButton->setCheckable(false);
      addButton->setIcon(QIcon::fromTheme("list-add"));
      addButton->setObjectName("ADD");

      m_ui->m_tableWidget->setCellWidget(row, 0, addButton);
      m_addButtonMapper->setMapping(addButton, row);
      connect(addButton, SIGNAL(pressed()), m_addButtonMapper, SLOT(map()));
    }

    te::common::FreeContents(rules);
    rules.clear();
  }

  delete rlc;
}

std::vector<te::dt::Property*> ta::model::qt::AttributeControlWidget::getAllowedAttributesFromLayer(ta::model::Catalog* catalog)
{
  std::vector<te::dt::Property*> result;

  std::auto_ptr<te::da::DataSetType> dsTypeOriginal = m_catalogFacade->getDataSetType(catalog);
  //std::auto_ptr<te::da::DataSetType> dsTypeOriginal = m_activeServer->getDataSetType("public." + catalog->getName());
  for (std::size_t t = 0; t < dsTypeOriginal->getProperties().size(); ++t)
  {
    std::string prop = dsTypeOriginal->getProperties()[t]->getName();
    if (prop != "object_id" && prop != "scene_id"
      && prop != "task_id" && prop != "spatial_data")
    {
      te::dt::Property* p = new te::dt::SimpleProperty(dsTypeOriginal->getProperties()[t]->getName(), dsTypeOriginal->getProperties()[t]->getType());
      result.push_back(p);
    }
  }
  return result;
}

std::vector<te::dt::Property*> ta::model::qt::AttributeControlWidget::getAllowedAttributesFromOutputLayer(ta::model::Catalog* catalog, const std::vector<std::string>& class_columns)
{
  std::vector<te::dt::Property*> result;

  std::auto_ptr<te::da::DataSetType> dsTypeOriginal = m_catalogFacade->getDataSetType(catalog);

  for (std::size_t t = 0; t < dsTypeOriginal->getProperties().size(); t++)
  {
    std::string propName = dsTypeOriginal->getProperties()[t]->getName();
    int propType = dsTypeOriginal->getProperties()[t]->getType();
    if (propName != "object_id" && propName != "class_name"
      && propName != "scene_id" && propName != "task_id"
      && propName != "spatial_data"
      && propName != catalog->getCellColumn()
      && propType != te::dt::GEOMETRY_TYPE)
    {
      bool lockedField = false;
      for (std::size_t i = 0; i < class_columns.size(); i++)
      {
        if (propName == class_columns.at(i)){
          lockedField = true;
          break;
        }
      }
      if (!lockedField)
      {
        te::dt::Property* p = new te::dt::SimpleProperty(dsTypeOriginal->getProperties()[t]->getName(), dsTypeOriginal->getProperties()[t]->getType());
        result.push_back(p);
      }
    }
  }
  return result;
}

void ta::model::qt::AttributeControlWidget::resizeTableWidget()
{
  //m_ui->m_tableWidget->resizeColumnsToContents();
  m_ui->m_tableWidget->horizontalHeader()->setSectionResizeMode(1, QHeaderView::Stretch);
  m_ui->m_tableWidget->horizontalHeader()->setSectionResizeMode(2, QHeaderView::Stretch);
  m_ui->m_tableWidget->horizontalHeader()->setSectionResizeMode(3, QHeaderView::Stretch);
}

void ta::model::qt::AttributeControlWidget::updateAttributeTableWidget(const int &ruleSetId)
{
  ta::model::RuleSet* rs = dynamic_cast<ta::model::RuleSet*>(m_ruleSetFacade->get(ruleSetId));
  ta::model::Catalog* c = dynamic_cast<ta::model::Catalog*>(m_catalogFacade->get(rs->getCatalogId()));
  std::string className;
  m_catalogFacade->getClassColumn(c->getId(), className);

  std::vector<std::string> vec;
  vec.push_back(className);

  // get attribute names
  std::vector<te::dt::Property*> properties = getAllowedAttributesFromOutputLayer(c, vec);

  // reinitialize table widget
  m_ui->m_tableWidget->clearContents();
  m_ui->m_tableWidget->setRowCount(0);
  int newrow = m_ui->m_tableWidget->rowCount();

  for (std::size_t i = 0; i < properties.size(); i++)
  {
    newrow = m_ui->m_tableWidget->rowCount();

    // attribute column
    m_ui->m_tableWidget->insertRow(newrow);
    QTableWidgetItem* attributeCell = new QTableWidgetItem();
    attributeCell->setText(properties.at(i)->getName().c_str());
    attributeCell->setData(Qt::UserRole, QVariant::fromValue(properties.at(i)->getType()));
    attributeCell->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
    m_ui->m_tableWidget->setItem(newrow, 1, attributeCell);

    // complete table widget
    fillTableWidgetOptions(newrow, ruleSetId);
  }

  delete rs;
  delete c;
  te::common::FreeContents(properties);
  properties.clear();

  resizeTableWidget();
}

void ta::model::qt::AttributeControlWidget::initComboBoxOutputLayers(const int &projectId)
{
  std::vector<int> rulesets = m_projectFacade->getRuleSets(projectId);

  // fill output layer combobox
  QComboBox *comb = dynamic_cast<QComboBox*>(m_ui->m_outputLayerComboBox);
  comb->clear();
  m_ui->m_attributeControlGroupBox->setEnabled(true);

  for (std::vector<int>::size_type i = 0; i != rulesets.size(); i++)
  {
    ta::model::RuleSet* rs = dynamic_cast<ta::model::RuleSet*>(m_ruleSetFacade->get(rulesets[i]));

    ta::model::RuleSet* output;

    if (m_ruleSetFacade->isOutputWithIntersection(rs->getId(), output))
    {
      ta::model::Catalog* catalog = dynamic_cast<ta::model::Catalog*>(m_catalogFacade->get(output->getCatalogId()));
      comb->addItem(catalog->getName().c_str(), QVariant::fromValue(output->getId()));
    }

    delete rs;
  }

  if (comb->count() == 0)
  {
    m_ui->m_attributeControlGroupBox->setEnabled(false);
  }
}

bool ta::model::qt::AttributeControlWidget::isValidNewPropertyName(std::string propName, ta::model::Catalog* catalog, const std::vector<std::string>& class_columns)
{
  std::auto_ptr<te::da::DataSetType> dsType = m_catalogFacade->getDataSetType(catalog);
  //std::auto_ptr<te::da::DataSetType> dsType = m_activeServer->getDataSetType("public." + catalog->getName());

  if (!propName.empty() && propName != "linkcolumn" && propName != "scene_id"
    && propName != "task_id" && propName != "class_name"
    && propName != "geom_id" && propName != "object_id"
    && propName != "spatial_data" && propName != catalog->getCellColumn())
  {

    for (std::size_t i = 0; i < class_columns.size(); i++)
    {
      if (propName == class_columns.at(i)){
        return false;
      }
    }

    for (std::size_t t = 0; t < dsType->getProperties().size(); t++)
    {
      if (propName == dsType->getProperties()[t]->getName()){
        return false;
      }
    }

    return true;
  }

  return false;
}

void ta::model::qt::AttributeControlWidget::listProjects()
{
  std::vector<ta::model::AbstractItem*> projs = m_projectFacade->get();

  m_ui->m_projectComboBox->clear();

  for (std::size_t t = 0; t < projs.size(); ++t)
  {
    ta::model::Project* p = dynamic_cast<ta::model::Project*>(projs[t]);

    m_ui->m_projectComboBox->addItem(p->getName().c_str(), QVariant(p->getId()));
  }

  if (!projs.empty())
    onProjectComboBoxActivated(0);

  te::common::FreeContents(projs);
  projs.clear();
}