/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of
    this application.
    */

/*!
  \file terraamazon/ClassControlWidget.cpp

  \brief A Qt widget used to configure the class control.
  */

// TerraAmazon
#include "../../facade/CatalogFacade.h"
#include "../../facade/ClassFacade.h"
#include "../../facade/ProjectFacade.h"
#include "../../facade/RuleSetFacade.h"
#include "../../item/Catalog.h"
#include "../../item/Class.h"
#include "../../item/Project.h"
#include "ClassControlWidget.h"
#include "ui_ClassControlWidgetForm.h"

#include "../../../common/Utils.h"

// TerraLib
#include <terralib/common/STLUtils.h>
#include <terralib/datatype/StringProperty.h>
#include <terralib/dataaccess/utils/Utils.h>

// Qt
#include <QMessageBox>

ta::model::qt::ClassControlWidget::ClassControlWidget(QWidget* parent, Qt::WindowFlags f)
: QWidget(parent, f),
m_ui(new Ui::ClassControlWidgetForm)
{
  m_ui->setupUi(this);

  // Color Picker
  m_colorPicker = new te::qt::widgets::ColorPickerToolButton(this);
  m_colorPicker->setFixedSize(70, 24);
  m_colorPicker->setColor(QColor(255, 255, 0, 128));

  QGridLayout* colorPickerLayout = new QGridLayout(m_ui->m_colorWidget);
  colorPickerLayout->setContentsMargins(0, 0, 0, 0);
  colorPickerLayout->addWidget(m_colorPicker);

  //mapper initializer
  m_removeMapper = new QSignalMapper(this);

  // add button icons
  m_ui->m_classAddToolButton->setIcon(QIcon::fromTheme("list-add"));
  m_ui->m_addClassColumnToolButton->setIcon(QIcon::fromTheme("check"));
  m_ui->m_addToolButton->setIcon(QIcon::fromTheme("go-next"));
  m_ui->m_addAllToolButton->setIcon(QIcon::fromTheme("go-last"));
  m_ui->m_removeToolButton->setIcon(QIcon::fromTheme("go-previous"));
  m_ui->m_removeAllToolButton->setIcon(QIcon::fromTheme("go-first"));

  //connects
  connect(m_ui->m_projectComboBox, SIGNAL(activated(int)), this, SLOT(onProjectComboBoxActivated(int)));
  connect(m_ui->m_layersComboBox, SIGNAL(activated(int)), this, SLOT(onLayersComboBoxActivated(int)));
  connect(m_ui->m_classAddToolButton, SIGNAL(pressed()), this, SLOT(onClassAddToolButtonPressed()));
  connect(m_ui->m_addClassColumnToolButton, SIGNAL(pressed()), this, SLOT(onClassColumnAddToolButtonPressed()));
  connect(m_ui->m_classNameComboBoxEdit, SIGNAL(activated(int)), this, SLOT(onClassNameComboBoxEditActivated(int)));

  connect(m_ui->m_addToolButton, SIGNAL(pressed()), this, SLOT(onAddClassToolButtonPressed()));
  connect(m_ui->m_removeToolButton, SIGNAL(pressed()), this, SLOT(onRemoveClassToolButtonPressed()));
  connect(m_ui->m_addAllToolButton, SIGNAL(pressed()), this, SLOT(onAddAllClassToolButtonPressed()));
  connect(m_ui->m_removeAllToolButton, SIGNAL(pressed()), this, SLOT(onRemoveAllClassToolButtonPressed()));

  connect(m_ui->m_classTableWidget, SIGNAL(itemPressed(QTableWidgetItem*)), this, SLOT(setActionsState()));
  connect(m_ui->m_classInTableWidget, SIGNAL(itemPressed(QTableWidgetItem*)), this, SLOT(setActionsState()));

  connect(m_removeMapper, SIGNAL(mapped(int)), this, SLOT(onRemoveClassExecuted(int)));
}

ta::model::qt::ClassControlWidget::~ClassControlWidget()
{
}

void ta::model::qt::ClassControlWidget::setActiveServer(te::da::DataSourceInfoPtr dsInfo)
{
  m_server = te::da::GetDataSource(dsInfo->getId());

  listProjects();

  fillNames();
}

void ta::model::qt::ClassControlWidget::onProjectComboBoxActivated(int index)
{
  int projId = m_ui->m_projectComboBox->itemData(index).toInt();

  //get project output layers
  ta::model::ProjectFacade pf(m_server);
  ta::model::CatalogFacade cf(m_server);

  std::vector<int> catalogIds = pf.getOutputCatalogs(projId);

  m_ui->m_layersComboBox->clear();

  for (std::size_t t = 0; t < catalogIds.size(); ++t)
  {
    ta::model::Catalog* catalog = dynamic_cast<ta::model::Catalog*>(cf.get(catalogIds[t]));

    m_ui->m_layersComboBox->addItem(catalog->getName().c_str(), QVariant(catalog->getId()));

    delete catalog;
  }

  if (catalogIds.empty())
  {
    m_ui->m_classColumnComboBox->clear();
    m_ui->m_classColumnComboBox->setEnabled(false);
    m_ui->m_addClassColumnToolButton->setEnabled(false);

    m_ui->m_classInTableWidget->setRowCount(0);

    setActionsState();

    listNotAssociatedClasses(0);
  }

  else
  {
    onLayersComboBoxActivated(0);
  }
}

void ta::model::qt::ClassControlWidget::onLayersComboBoxActivated(int /*index*/)
{
  int catalogId = m_ui->m_layersComboBox->currentData().toInt();

  ta::model::CatalogFacade cf(m_server);

  ta::model::Catalog* catalog = dynamic_cast<ta::model::Catalog*>(cf.get(catalogId));

  //get atrributes
  m_ui->m_classColumnComboBox->clear();

  std::auto_ptr<te::da::DataSetType> dsType = cf.getDataSetType(catalog);

  delete catalog;

  if (!dsType.get())
    return;

  //get properties names
  QStringList propNamesList;
  for (std::size_t t = 0; t < dsType->getProperties().size(); ++t)
    propNamesList.append(dsType->getProperties()[t]->getName().c_str());

  m_ui->m_classColumnComboBox->addItems(propNamesList);

  //check if catalog (CatalogFacade) alredy has a class column
  std::string classColumn = "";
  if (cf.getClassColumn(catalogId, classColumn))
  {
    m_ui->m_classColumnComboBox->setCurrentText(classColumn.c_str());
    m_ui->m_classColumnComboBox->setEnabled(false);
    m_ui->m_addClassColumnToolButton->setEnabled(false);
  }
  else
  {
    m_ui->m_classColumnComboBox->setEnabled(true);
    m_ui->m_addClassColumnToolButton->setEnabled(true);
  }

  //list classes
  listAssociatedClasses(catalogId);
}

void ta::model::qt::ClassControlWidget::onClassAddToolButtonPressed()
{
  QString text = m_ui->m_classNameComboBoxEdit->currentText();
  std::string textStd = text.toStdString();

  if (text.isEmpty())
  {
    QMessageBox::warning(this, windowTitle(), tr("Class name not defined."));
    return;
  }

  ta::model::ClassFacade cf(m_server);

  ta::model::Class* c = new ta::model::Class();
  c->setName(textStd);
  c->setStyle(m_colorPicker->getColor().name().toStdString());

  if (!cf.existsByName(textStd))
  {
    if (!cf.save(c))
    {
      delete c;
      QMessageBox::warning(this, windowTitle(), cf.getErrorMessage().c_str());
      return;
    }
  }
  else
  {
    QMessageBox::warning(this, "TerraAmazon", tr("Class already exists."));
  }

  delete c;

  listNotAssociatedClasses(0);

  m_ui->m_classNameComboBoxEdit->setCurrentIndex(0);
}

void ta::model::qt::ClassControlWidget::onClassColumnAddToolButtonPressed()
{
  if (m_ui->m_classColumnComboBox->currentText().isEmpty())
  {
    QMessageBox::warning(this, windowTitle(), tr("Column name not defined."));
    return;
  }

  std::string classColumnName = m_ui->m_classColumnComboBox->currentText().toStdString();

  //get catalog
  int catalogId = m_ui->m_layersComboBox->currentData().toInt();

  ta::model::CatalogFacade cf(m_server);

  ta::model::Catalog* catalog = dynamic_cast<ta::model::Catalog*>(cf.get(catalogId));

  std::auto_ptr<te::da::DataSetType> dsType = cf.getDataSetType(catalog);

  delete catalog;

  //check if using a exit property o create a new column.
  bool newProp = true;

  for (std::size_t t = 0; t < dsType->getProperties().size(); ++t)
  {
    if (dsType->getProperties()[t]->getName() == classColumnName)
    {
      newProp = false;
      break;
    }
  }


  if (newProp)
  {
    //create column
    te::dt::Property* p = new te::dt::StringProperty(classColumnName, te::dt::VAR_STRING, 255);

    try
    {
      m_server->addProperty(dsType->getName(), p);
    }
    catch (const std::exception& e)
    {
      QMessageBox::warning(this, windowTitle(), e.what());
      return;
    }
    catch (...)
    {
      QMessageBox::warning(this, windowTitle(), tr("Error creating new property."));
      return;
    }
  }


  //associate column
  if (!cf.addClassColumn(catalogId, classColumnName))
  {
    QMessageBox::warning(this, windowTitle(), cf.getErrorMessage().c_str());
    return;
  }

  m_ui->m_classColumnComboBox->setEnabled(false);
  m_ui->m_addClassColumnToolButton->setEnabled(false);

  QMessageBox::information(this, windowTitle(), tr("Class Column Associated."));
}

void ta::model::qt::ClassControlWidget::onClassNameComboBoxEditActivated(int /*index*/)
{
  QComboBox* comb = m_ui->m_classNameComboBoxEdit;
  QVariant var = comb->currentData(Qt::UserRole);
  if (var.isValid()){
    std::string color = var.toString().toStdString();
    m_colorPicker->setColor(QColor(color.c_str()));
  }
}

void ta::model::qt::ClassControlWidget::onRemoveClassExecuted(int row)
{
  QVariant var = m_ui->m_classTableWidget->item(row, 2)->data(Qt::UserRole);

  if (!var.isValid())
    return;

  int classId = var.toInt();

  //remove class
  ta::model::ClassFacade cf(m_server);

  if (!cf.remove(classId))
  {
    QMessageBox::warning(this, windowTitle(), cf.getErrorMessage().c_str());

    return;
  }

  listNotAssociatedClasses(0);
}

void ta::model::qt::ClassControlWidget::onAddClassToolButtonPressed()
{
  //check if exist items selected
  QItemSelectionModel* select = m_ui->m_classTableWidget->selectionModel();

  if (!select->hasSelection())
  {
    QMessageBox::warning(this, windowTitle(), tr("Select at least one class to associate."));
    return;
  }

  QVariant var = m_ui->m_layersComboBox->currentData();

  if (!var.isValid())
  {
    QMessageBox::warning(this, windowTitle(), tr("Output layer not defined."));
    return;
  }

  int catalogId = var.toInt();

  ta::model::RuleSetFacade rsf(m_server);

  QModelIndexList indexList = m_ui->m_classTableWidget->selectionModel()->selectedRows(2);

  for (int i = 0; i < indexList.count(); ++i)
  {
    QModelIndex index = indexList.at(i);

    int row = index.row();

    QVariant var = m_ui->m_classTableWidget->item(row, 2)->data(Qt::UserRole);

    if (var.isValid())
    {
      int classId = var.toInt();

      if (!rsf.addRuleSetClass(catalogId, classId))
      {
        QMessageBox::warning(this, windowTitle(), rsf.getErrorMessage().c_str());
        return;
      }
    }
  }

  listAssociatedClasses(catalogId);
}

void ta::model::qt::ClassControlWidget::onRemoveClassToolButtonPressed()
{
  //check if exist items selected
  QItemSelectionModel* select = m_ui->m_classInTableWidget->selectionModel();

  if (!select->hasSelection())
  {
    QMessageBox::warning(this, windowTitle(), tr("Select at least on class to remove."));
    return;
  }

  int catalogId = m_ui->m_layersComboBox->currentData().toInt();

  ta::model::RuleSetFacade rsf(m_server);

  QModelIndexList indexList = m_ui->m_classInTableWidget->selectionModel()->selectedRows(1);

  for (int i = 0; i < indexList.count(); ++i)
  {
    QModelIndex index = indexList.at(i);

    int row = index.row();

    QVariant var = m_ui->m_classInTableWidget->item(row, 1)->data(Qt::UserRole);

    if (var.isValid())
    {
      int classId = var.toInt();

      if (!rsf.removeRuleSetClass(catalogId, classId))
      {
        QMessageBox::warning(this, windowTitle(), rsf.getErrorMessage().c_str());
        return;
      }
    }
  }

  listAssociatedClasses(catalogId);
}

void ta::model::qt::ClassControlWidget::onAddAllClassToolButtonPressed()
{
  //check if exist items selected
  if (m_ui->m_classTableWidget->rowCount() <= 0)
  {
    QMessageBox::warning(this, windowTitle(), tr("No class to associate."));
    return;
  }

  int catalogId = m_ui->m_layersComboBox->currentData().toInt();

  ta::model::RuleSetFacade rsf(m_server);

  for (int i = 0; i < m_ui->m_classTableWidget->rowCount(); ++i)
  {
    QVariant var = m_ui->m_classTableWidget->item(i, 2)->data(Qt::UserRole);

    if (var.isValid())
    {
      int classId = var.toInt();

      if (!rsf.addRuleSetClass(catalogId, classId))
      {
        QMessageBox::warning(this, windowTitle(), rsf.getErrorMessage().c_str());
        return;
      }
    }
  }

  listAssociatedClasses(catalogId);
}

void ta::model::qt::ClassControlWidget::onRemoveAllClassToolButtonPressed()
{
  //check if exist items selected
  if (m_ui->m_classInTableWidget->rowCount() <= 0)
  {
    QMessageBox::warning(this, windowTitle(), tr("No class to remove."));
    return;
  }

  QVariant var = m_ui->m_layersComboBox->currentData();

  if (!var.isValid())
  {
    QMessageBox::warning(this, windowTitle(), tr("Output layer not defined."));
    return;
  }

  int catalogId = var.toInt();

  ta::model::RuleSetFacade rsf(m_server);

  for (int i = 0; i < m_ui->m_classInTableWidget->rowCount(); ++i)
  {
    QVariant var = m_ui->m_classInTableWidget->item(i, 1)->data(Qt::UserRole);

    if (var.isValid())
    {
      int classId = var.toInt();

      if (!rsf.removeRuleSetClass(catalogId, classId))
      {
        QMessageBox::warning(this, windowTitle(), rsf.getErrorMessage().c_str());
        return;
      }
    }
  }

  listAssociatedClasses(catalogId);
}

void ta::model::qt::ClassControlWidget::setActionsState()
{
  m_ui->m_addToolButton->setEnabled(false);
  m_ui->m_removeToolButton->setEnabled(false);
  m_ui->m_addAllToolButton->setEnabled(false);
  m_ui->m_removeAllToolButton->setEnabled(false);

  QVariant var = m_ui->m_layersComboBox->currentData();

  bool isValid = var.isValid();

  if (m_ui->m_classTableWidget->rowCount() > 0 && isValid)
  {
    m_ui->m_addAllToolButton->setEnabled(true);

    QModelIndexList indexList = m_ui->m_classTableWidget->selectionModel()->selectedRows(2);

    if (indexList.count() > 0)
    {
      m_ui->m_addToolButton->setEnabled(true);
    }
  }

  if (m_ui->m_classInTableWidget->rowCount() > 0)
  {
    m_ui->m_removeAllToolButton->setEnabled(true);

    QModelIndexList indexList = m_ui->m_classInTableWidget->selectionModel()->selectedRows(2);

    if (indexList.count() > 0)
    {
      m_ui->m_removeToolButton->setEnabled(true);
    }
  }
}

void ta::model::qt::ClassControlWidget::listProjects()
{
  ta::model::ProjectFacade pf(m_server);

  std::vector<ta::model::AbstractItem*> projs = pf.get();

  m_ui->m_projectComboBox->clear();

  for (std::size_t t = 0; t < projs.size(); ++t)
  {
    ta::model::Project* p = dynamic_cast<ta::model::Project*>(projs[t]);

    m_ui->m_projectComboBox->addItem(p->getName().c_str(), QVariant(p->getId()));
  }

  if (!projs.empty())
    onProjectComboBoxActivated(0);
  else
    listNotAssociatedClasses(0);

  te::common::FreeContents(projs);
  projs.clear();

  setActionsState();
}

void ta::model::qt::ClassControlWidget::listNotAssociatedClasses(const int& catologId)
{
  ta::model::ClassFacade classF(m_server);

  //get list of all not associated classes
  std::vector<ta::model::Class*> items = classF.getNotAssociatedClasses(catologId);

  m_ui->m_classTableWidget->setRowCount(0);
  for (std::size_t t = 0; t < items.size(); ++t)
  {
    ta::model::Class* c = items[t];

    std::string className = c->getName();
    std::string strColorHex = c->getStyle();

    //fill table
    int row = m_ui->m_classTableWidget->rowCount();
    m_ui->m_classTableWidget->insertRow(row);

    //button remove
    {
      QToolButton* removeToolButton = new QToolButton(m_ui->m_classTableWidget);
      removeToolButton->setIcon(QIcon::fromTheme("list-remove"));
      bool hasAssociation = classF.hasAssociation(items[t]->getId());
      removeToolButton->setEnabled(!hasAssociation);
      m_ui->m_classTableWidget->setCellWidget(row, 0, removeToolButton);

      m_removeMapper->setMapping(removeToolButton, row);
      connect(removeToolButton, SIGNAL(pressed()), m_removeMapper, SLOT(map()));
    }

    //style
    {
      QPixmap pix = QPixmap(QSize(24, 24));
      pix.fill(QColor(strColorHex.c_str()));

      QIcon icon(pix);

      QTableWidgetItem* item = new QTableWidgetItem(icon, "");
      item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
      m_ui->m_classTableWidget->setItem(row, 1, item);
    }

    //class name
    {
      QTableWidgetItem* item = new QTableWidgetItem();
      item->setText(className.c_str());
      item->setData(Qt::UserRole, QVariant(c->getId()));
      item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
      m_ui->m_classTableWidget->setItem(row, 2, item);
    }
  }

  m_ui->m_classTableWidget->resizeColumnsToContents();
  m_ui->m_classTableWidget->horizontalHeader()->setSectionResizeMode(2, QHeaderView::Stretch);

  te::common::FreeContents(items);
  items.clear();

  setActionsState();
}

void ta::model::qt::ClassControlWidget::listAssociatedClasses(const int& catalogId)
{
  ta::model::ClassFacade classF(m_server);
  ta::model::RuleSetFacade rsf(m_server);

  //get all classes associated with this catalog
  std::vector<int> catalogClasses = rsf.getRuleSetClasses(catalogId);

  m_ui->m_classInTableWidget->setRowCount(0);
  for (std::size_t t = 0; t < catalogClasses.size(); ++t)
  {
    ta::model::Class* c = dynamic_cast<ta::model::Class*>(classF.get(catalogClasses[t]));

    std::string className = c->getName();
    std::string strColorHex = c->getStyle();
    int classId = c->getId();

    delete c;

    //fill table
    int row = m_ui->m_classInTableWidget->rowCount();
    m_ui->m_classInTableWidget->insertRow(row);

    //style
    {
      QPixmap pix = QPixmap(QSize(24, 24));
      pix.fill(QColor(strColorHex.c_str()));

      QIcon icon(pix);

      QTableWidgetItem* item = new QTableWidgetItem(icon, "");
      item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
      m_ui->m_classInTableWidget->setItem(row, 0, item);
    }

    //class name
    {
      QTableWidgetItem* item = new QTableWidgetItem();
      item->setText(className.c_str());
      item->setData(Qt::UserRole, QVariant(classId));
      item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
      m_ui->m_classInTableWidget->setItem(row, 1, item);
    }
  }

  m_ui->m_classInTableWidget->resizeColumnsToContents();
  m_ui->m_classInTableWidget->horizontalHeader()->setSectionResizeMode(1, QHeaderView::Stretch);

  listNotAssociatedClasses(catalogId);
}

void ta::model::qt::ClassControlWidget::fillNames()
{
  std::vector<std::string> keys;
  keys.push_back("name");
  keys.push_back("color");
  std::string error = "";
  std::vector<std::vector<std::string>> hints = ta::common::loadHintConfig("classes", keys, error);

  if (error.empty()){
    QComboBox* combNames = m_ui->m_classNameComboBoxEdit;
    combNames->clear();
    combNames->addItem("");
    for (std::vector<int>::size_type i = 0; i < hints.size(); ++i)
    {
      std::string name = hints.at(i).at(0);
      std::string color = hints.at(i).at(1);
      combNames->addItem(name.c_str(), QVariant(color.c_str()));
    }
  }
  else{
    //TODO: launch error
  }
}

