/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/EndProcessDialog.cpp

  \brief A Qt dialog used to configure the end process.
*/

// TerraLib
#include <terralib/common/PlatformUtils.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>
#include <terralib/datatype/Date.h>
#include <terralib/datatype/TimeInstant.h>

// TerraAmazon
#include "EndProcessDialog.h"
#include "ui_EndProcessDialogForm.h"

#include "../../../common/Utils.h"

// Boost
#include <boost/date_time/gregorian/gregorian.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>

// QT
#include <QMessageBox>

ta::model::qt::EndProcessDialog::EndProcessDialog(QWidget* parent, Qt::WindowFlags f)
  : QDialog(parent, f),
  m_ui(new Ui::EndProcessDialogForm)
{
  m_ui->setupUi(this);

  std::string icon = ta::common::FindInTerraAmazonPath("share/icons/svg/terraAmazom-colorido-horizontal.svg");
  m_ui->m_imgLabel->setPixmap(QIcon(icon.c_str()).pixmap(112, 100));
  m_ui->m_imgLabel->hide();

  connect(m_ui->m_closeButton, SIGNAL(clicked()), this, SLOT(onCloseButtonClicked()));

  connect(m_ui->m_openRadioButton, SIGNAL(clicked()), this, SLOT(onOpenRadioButtonClicked()));
  connect(m_ui->m_closeRadioButton, SIGNAL(clicked()), this, SLOT(onCloseRadioButtonClicked()));
  connect(m_ui->m_reviewRadioButton, SIGNAL(clicked()), this, SLOT(onReviewRadioButtonClicked()));
  connect(m_ui->m_alluserscheckBox, SIGNAL(clicked()), this, SLOT(onAllUsersCheckBoxClicked()));
}

ta::model::qt::EndProcessDialog::~EndProcessDialog()
{}

void ta::model::qt::EndProcessDialog::onCloseButtonClicked()
{
  std::string currentStatus = "";

  if (m_ui->m_closeRadioButton->isChecked() == true)
  {
    currentStatus = "CLOSED";
  }
  else if (m_ui->m_openRadioButton->isChecked() == true)
  {
    currentStatus = "OPENED";
  }
  else if (m_ui->m_reviewRadioButton->isChecked() == true)
  {
    currentStatus = "IN REVIEW";
  }

  if (currentStatus.empty())
  {
    close();
    return;
  }

  std::string obs = "";

  if (m_ui->obsTextEdit->toPlainText().length() > 0)
  {
    obs = m_ui->obsTextEdit->toPlainText().toStdString();
  }

  if (m_ui->m_closeRadioButton->isChecked() == true)
  {
    //TODO: REVISION....
    //insert into ta_revision
  }
  else if (m_ui->m_reviewRadioButton->isChecked() == true)
  {
    if (m_ui->m_userComboBox->currentText() == "")
    {
      return;
    }

    //TODO: NOTIFICATION
  }

  //UPDATE TASKLOG
  CellFacade cf(m_server);
  ProcessFacade pf(m_server);
  TaskLogFacade tkl(m_server);
  TaskLog* tasklog;

  try
  {
    tasklog = tkl.getLastTaskLog(m_process->getTaskId());

    boost::posix_time::ptime boostLocalTime = boost::posix_time::second_clock::local_time();
    te::dt::TimeInstant* finalTime = new te::dt::TimeInstant(boostLocalTime);

    tasklog->setStatus(currentStatus);
    tasklog->setFinalTime(finalTime);
    tasklog->setObs(obs);
    tasklog->setTimer((5 * 60) + tasklog->getTimer()); //TODO: INCREASE IN SECONDS 5*60 ?!?

    tkl.update(tasklog);

    if (!cf.unlockCells(m_process->getId()))
      return;

    pf.remove(m_process.get()->getId());

    accept();

  }
  catch (const std::exception& e)
  {
    QMessageBox::warning(this, "TerraAmazon", tr("Error: ") + e.what());
  }

}

void ta::model::qt::EndProcessDialog::setEndProcess(te::da::DataSourceInfoPtr dsInfo, int processId)
{
  m_server = te::da::DataSourceManager::getInstance().get(dsInfo->getId(), dsInfo->getType(), dsInfo->getConnInfo());
  
  ta::model::ProcessFacade pf(m_server);
  m_process.reset(dynamic_cast<ta::model::Process*>(pf.get(processId)));
}

void ta::model::qt::EndProcessDialog::onAllUsersCheckBoxClicked()
{
  m_ui->m_userComboBox->clear();

  std::vector<User*> vecUsers;
  ProcessFacade pf(m_server);
  UserFacade uf(m_server);

  User* currentUser = dynamic_cast<User*>(uf.get(dynamic_cast<Task*>(pf.getTask(m_process.get()))->getUserId()));

  if (m_ui->m_alluserscheckBox->isChecked())
  {
    for (std::size_t i = 0; i < uf.get().size(); i++)
    {
      User* user = dynamic_cast<User*>(uf.get()[i]);

      if (user->getName() != "admin" && user->getId() != currentUser->getId())
        vecUsers.push_back(user);
    }
  }
  else
  {
    vecUsers = pf.getAssociatedUsersByScene(dynamic_cast<Task*>(pf.getTask(m_process.get()))->getSceneId());
  }

  for (std::size_t i = 0; i < vecUsers.size(); i++)
  {
    if (dynamic_cast<User*>(vecUsers.at(i))->getId() != currentUser->getId())
    {
      m_ui->m_userComboBox->insertItem(dynamic_cast<User*>(vecUsers.at(i))->getId(), dynamic_cast<User*>(vecUsers.at(i))->getName().c_str());
    }
  }

}

void ta::model::qt::EndProcessDialog::onOpenRadioButtonClicked()
{
  m_ui->m_alluserscheckBox->setEnabled(false);
  m_ui->m_userComboBox->setEnabled(false);
}

void ta::model::qt::EndProcessDialog::onCloseRadioButtonClicked()
{
  m_ui->m_alluserscheckBox->setEnabled(false);
  m_ui->m_userComboBox->setEnabled(false);
}

void ta::model::qt::EndProcessDialog::onReviewRadioButtonClicked()
{
  m_ui->m_alluserscheckBox->setEnabled(true);
  m_ui->m_userComboBox->setEnabled(true);
}