/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

// TerraAmazon
#include "LockedCellsWidget.h"
#include "ui_LockedCellsWidgetForm.h"
#include "../../item/Catalog.h"
#include "../../item/User.h"
#include "../../item/CellLock.h"
#include "../../../common/Utils.h"

// TerraLib
#include <terralib/common/PlatformUtils.h>
#include <terralib/common/StringUtils.h>
#include <terralib/common/STLUtils.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>

// Qt
#include <QComboBox>
#include <QCheckBox>
#include <QLineEdit>
#include <QMessageBox>
#include <QWidget>

// Boost
#include <boost/lexical_cast.hpp>

// STL
#include <fstream>
#include <memory>

ta::model::qt::LockedCellsWidget::LockedCellsWidget(QWidget* parent, Qt::WindowFlags f)
: QWidget(parent, f),
m_ui(new Ui::LockedCellsWidgetForm)
{
  m_ui->setupUi(this);

  m_ui->m_clearToolButton->setIcon(QIcon::fromTheme("edit-clear"));
  m_ui->m_applyToolButton->setIcon(QIcon::fromTheme("check"));

  connect(m_ui->m_applyToolButton, SIGNAL(clicked()), this, SLOT(onApplyToolButtonClicked()));
  connect(m_ui->m_clearToolButton, SIGNAL(clicked()), this, SLOT(onClearToolButtonClicked()));
  connect(m_ui->m_unlockCellsPushButton, SIGNAL(clicked()), this, SLOT(onUnlockCellsPushButtonClicked()));
}

ta::model::qt::LockedCellsWidget::~LockedCellsWidget()
{
}

void ta::model::qt::LockedCellsWidget::setServer(te::da::DataSourceInfoPtr server)
{
  if (server.get())
  {
    m_activeServer = te::da::DataSourceManager::getInstance().get(server->getId(), server->getType(), server->getConnInfo());

    if (!m_activeServer.get())
      return;

    m_activeServer->open();

    m_catalogFacade.reset(new ta::model::CatalogFacade(m_activeServer));
    m_userFacade.reset(new ta::model::UserFacade(m_activeServer));
    m_processFacade.reset(new ta::model::ProcessFacade(m_activeServer));

    listUsers();
    listLockedCells();
  }
}

void ta::model::qt::LockedCellsWidget::listUsers()
{
  std::vector<AbstractItem*> users = m_userFacade->get();

  QComboBox* comb = m_ui->m_userComboBox;
  comb->addItem("select...", QVariant::fromValue(-1));

  for (std::size_t i = 0; i < users.size(); ++i)
  {
    std::auto_ptr<User> user(dynamic_cast<ta::model::User*>(users.at(i)));
    comb->addItem(user->getName().c_str(), QVariant(user->getId()));
  }
}

void ta::model::qt::LockedCellsWidget::listLockedCells()
{
  int catalogId = m_catalogFacade->getActiveCellLayer();
  m_activeCellLayer = catalogId;

  ta::model::Catalog* catalog = dynamic_cast<ta::model::Catalog*>(m_catalogFacade->get(catalogId));

  if (!catalog)
    return;

  std::string cellOid = m_ui->m_cellOidLineEdit->text().toStdString();
  int userId = m_ui->m_userComboBox->currentData(Qt::UserRole).toInt();

  std::vector<ta::model::CellLock*> vec = m_processFacade->getLockedCells(catalog->getName(), cellOid, userId);

  QTableWidget* tw = m_ui->m_lockedCellsTableWidget;

  // reinitialize table widget
  tw->clearContents();
  tw->setRowCount(0);
  int newrow = tw->rowCount();

  for (std::size_t i = 0; i < vec.size(); i++)
  {

    tw->insertRow(newrow);

    QTableWidgetItem* cellOidItem = new QTableWidgetItem();
    cellOidItem->setText(vec.at(i)->getCellOid().c_str());
    cellOidItem->setData(Qt::UserRole, QVariant(vec.at(i)->getCellOid().c_str()));
    cellOidItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
    tw->setItem(newrow, 0, cellOidItem);

    QTableWidgetItem* userNameItem = new QTableWidgetItem();
    userNameItem->setText(vec.at(i)->getUserName().c_str());
    userNameItem->setData(Qt::UserRole, QVariant(vec.at(i)->getUserName().c_str()));
    userNameItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
    tw->setItem(newrow, 1, userNameItem);

    QTableWidgetItem* dateTimeItem = new QTableWidgetItem();
    dateTimeItem->setText(vec.at(i)->getDateTime()->toString().c_str());
    dateTimeItem->setData(Qt::UserRole, QVariant(vec.at(i)->getDateTime()->toString().c_str()));
    dateTimeItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
    tw->setItem(newrow, 2, dateTimeItem);

    QTableWidgetItem* projectNameItem = new QTableWidgetItem();
    projectNameItem->setText(vec.at(i)->getProjetName().c_str());
    projectNameItem->setData(Qt::UserRole, QVariant(vec.at(i)->getProjetName().c_str()));
    projectNameItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
    tw->setItem(newrow, 3, projectNameItem);

    newrow = tw->rowCount();
  }

  tw->resizeColumnsToContents();
  tw->horizontalHeader()->setStretchLastSection(true);

  delete catalog;
  te::common::FreeContents(vec);
  vec.clear();
}

void ta::model::qt::LockedCellsWidget::onApplyToolButtonClicked()
{
  listLockedCells();
}

void ta::model::qt::LockedCellsWidget::onClearToolButtonClicked()
{
  m_ui->m_cellOidLineEdit->clear();
  m_ui->m_userComboBox->setCurrentIndex(0);
}

void ta::model::qt::LockedCellsWidget::onUnlockCellsPushButtonClicked()
{
  QTableWidget* tw = m_ui->m_lockedCellsTableWidget;

  QList<QTableWidgetItem*> list = tw->selectedItems();

  std::size_t i = 0;
  std::vector<std::string> cellOids;
  
  if (!list.empty())
  {
    while (i < list.size())
    {
      QTableWidgetItem* item = list.at((int)i);
      std::string cellOid = item->text().toStdString();
      cellOids.push_back(cellOid);
      i += tw->columnCount();
    }
    m_catalogFacade->unlockCellLayerItems(m_activeCellLayer, cellOids);
    listLockedCells();
  }

  cellOids.clear();
}
