/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of
    this application.
    */

/*!
  \file terraamazon/PhaseControlDialog.cpp

  \brief A Qt dialog used to configure the phase control.
  */

// Qt
#include <QMessageBox>
#include <QComboBox>
#include <QListWidgetItem>
#include <QToolButton>

// TerraAmazon
#include "PhaseControlDialog.h"
#include "ui_PhaseControlDialogForm.h"
#include "../../item/Group.h"
#include "../../item/Phase.h"
#include "../../facade/GroupFacade.h"
#include "../../facade/PhaseFacade.h"

#include "../../../common/Utils.h"

// TerraLib
#include <terralib/common/PlatformUtils.h>
#include <terralib/common/STLUtils.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>

ta::model::qt::PhaseControlDialog::PhaseControlDialog(QWidget* parent, Qt::WindowFlags f)
: QDialog(parent, f),
m_ui(new Ui::PhaseControlDialogForm)
{
  m_ui->setupUi(this);

  std::string icon = ta::common::FindInTerraAmazonPath("share/icons/svg/terraAmazom-colorido-horizontal.svg");
  m_ui->m_imgLabel->setPixmap(QIcon(icon.c_str()).pixmap(112, 100));
  m_ui->m_imgLabel->hide();

  m_ui->m_addPhaseToolButton->setIcon(QIcon::fromTheme("list-add"));

  connect(m_ui->m_addPhaseToolButton, SIGNAL(clicked()), this, SLOT(onAddPhaseButtonClicked()));
  connect(m_ui->m_finishPushButton, SIGNAL(clicked()), this, SLOT(onFinishPushButtonClicked()));
}

ta::model::qt::PhaseControlDialog::~PhaseControlDialog()
{
}

void ta::model::qt::PhaseControlDialog::setServer(te::da::DataSourceInfoPtr server)
{
  if (server.get())
  {
    m_activeServer = te::da::DataSourceManager::getInstance().get(server->getId(), server->getType(), server->getConnInfo());

    if (!m_activeServer.get())
      return;

    m_activeServer->open();

    updateTWGroupPhase("");

    fillNames();
  }
}

void ta::model::qt::PhaseControlDialog::onAddPhaseButtonClicked()
{
  if (m_activeServer.get())
  {
    ta::model::PhaseFacade phaseFacade(m_activeServer);

    QString text = m_ui->m_phaseComboBoxEdit->currentText();
    std::string textStd = text.toStdString();

    if (!text.isEmpty())
    {
      Phase* phase = new Phase();
      phase->setDescription(textStd);

      if (!phaseFacade.existsByDescription(textStd))
      {
        phaseFacade.save(phase);
        
        m_ui->m_phaseComboBoxEdit->setCurrentIndex(0);

        updateTWGroupPhase(textStd);
      }
      else
      {
        QMessageBox::warning(this, "TerraAmazon", tr("Phase already exists."));
      }
    }
    else
    {
      QMessageBox::warning(this, "TerraAmazon", tr("Phase name not defined."));
    }
  }
}

void ta::model::qt::PhaseControlDialog::onAddAssociationToolButtonClicked()
{
  if (m_activeServer.get())
  {

    int row = getRowByWidgetSearch();

    if (row >= 0)
    {
      QComboBox* phaseBox = dynamic_cast<QComboBox*>(m_ui->tableWidget->cellWidget(row, 1));
      int phaseId = phaseBox->currentData().toInt();

      QComboBox* groupBox = dynamic_cast<QComboBox*>(m_ui->tableWidget->cellWidget(row, 2));
      int groupId = groupBox->currentData().toInt();

      ta::model::GroupFacade groupFacade(m_activeServer);

      if (groupId != -1 && phaseId != -1)
      {
        if (groupFacade.addAssociatePhase(groupId, phaseId))
        {
          m_ui->tableWidget->removeRow(row);
        }
        else
        {
          QMessageBox::critical(this, "TerraAmazon", tr("Error adding associate phase!"));
        }
        updateTWGroupPhase("");
      }
      else
      {
        QMessageBox::warning(this, "TerraAmazon", tr("Select the phase and group!"));
      }

    }
  }
}

void ta::model::qt::PhaseControlDialog::onRemoveAssociationToolButtonClicked()
{
  if (m_activeServer.get())
  {

    int row = getRowByWidgetSearch();

    if (row >= 0)
    {
      int phaseId = m_ui->tableWidget->item(row, 1)->data(Qt::UserRole).toInt();
      int groupId = m_ui->tableWidget->item(row, 2)->data(Qt::UserRole).toInt();

      ta::model::GroupFacade groupFacade(m_activeServer);
      if (groupFacade.removeAssociatePhase(groupId, phaseId))
      {
        m_ui->tableWidget->removeRow(row);
      }
      else
      {
        QMessageBox::critical(this, "TerraAmazon", tr("Error remove associate phase!"));
      }
    }

    updateTWGroupPhase("");
  }
}

void ta::model::qt::PhaseControlDialog::onFinishPushButtonClicked()
{
  close();
}

void ta::model::qt::PhaseControlDialog::updateTWGroupPhase(const std::string& lastAddedPhase)
{
  if (m_activeServer.get())
  {
    // reinitialize table widget
    m_ui->tableWidget->clearContents();
    m_ui->tableWidget->setRowCount(0);
    int newrow = m_ui->tableWidget->rowCount();

    // get facades
    ta::model::GroupFacade groupFacade(m_activeServer);
    ta::model::PhaseFacade phaseFacade(m_activeServer);

    // get allowed groups from database
    std::vector<ta::model::Group*> groups = groupFacade.getAllowedGroups();

    // create table items ---

    for (std::vector<int>::size_type i = 0; i != groups.size(); i++)
    {
      std::vector<ta::model::Phase*> associatedPhases = groupFacade.getAssociatedPhases(m_activeServer, groups[i]->getId());

      for (std::vector<int>::size_type j = 0; j != associatedPhases.size(); j++)
      {
        newrow = m_ui->tableWidget->rowCount();
        m_ui->tableWidget->insertRow(newrow);

        // button column
        QToolButton* removeAssociation = new QToolButton(m_ui->tableWidget);
        removeAssociation->setAutoRaise(true);
        removeAssociation->setIcon(QIcon::fromTheme("list-remove"));

        // phase column
        QTableWidgetItem* phaseCell = new QTableWidgetItem();
        phaseCell->setText(associatedPhases[j]->getDescription().c_str());
        phaseCell->setData(Qt::UserRole, QVariant::fromValue(associatedPhases[j]->getId()));
        phaseCell->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);

        // group column
        QTableWidgetItem* groupCell = new QTableWidgetItem();
        groupCell->setText(groups[i]->getName().c_str());
        groupCell->setData(Qt::UserRole, QVariant::fromValue(groups[i]->getId()));
        groupCell->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);

        // set widgets ui and connects
        m_ui->tableWidget->setCellWidget(newrow, 0, removeAssociation);
        m_ui->tableWidget->setItem(newrow, 1, phaseCell);
        m_ui->tableWidget->setItem(newrow, 2, groupCell);
        connect(removeAssociation, SIGNAL(clicked()), this, SLOT(onRemoveAssociationToolButtonClicked()));
      }

      // free memory
      te::common::FreeContents(associatedPhases);
      associatedPhases.clear();
    }

    // create dynamic combobox ---

    newrow = m_ui->tableWidget->rowCount();
    m_ui->tableWidget->insertRow(newrow);

    // button column
    QToolButton* addAssociation = new QToolButton(m_ui->tableWidget);
    addAssociation->setAutoRaise(true);
    addAssociation->setIcon(QIcon::fromTheme("list-add"));
    QComboBox* combGroup = new QComboBox(m_ui->tableWidget);
    QComboBox* combPhase = new QComboBox(m_ui->tableWidget);

    // group combobox column
    combGroup->addItem("select...", QVariant::fromValue(-1));
    for (std::vector<int>::size_type i = 0; i != groups.size(); i++)
    {
      combGroup->addItem(groups[i]->getName().c_str(), QVariant::fromValue(groups[i]->getId()));
    }

    // get not associated phases from database
    std::vector<ta::model::Phase*> phases = phaseFacade.getNotAssociatedPhases();

    // phase combobox column
    combPhase->addItem("select...", QVariant::fromValue(-1));

    for (std::vector<int>::size_type i = 0; i != phases.size(); i++)
    {
      std::string description = phases[i]->getDescription();
      int id = phases[i]->getId();

      combPhase->addItem(description.c_str(), QVariant::fromValue(id));

      if (lastAddedPhase.compare(description) == 0)
      {
        combPhase->setCurrentIndex(i + 1);
      }
    }

    // set widgets ui and connects
    m_ui->tableWidget->setCellWidget(newrow, 0, addAssociation);
    m_ui->tableWidget->setCellWidget(newrow, 1, combPhase);
    m_ui->tableWidget->setCellWidget(newrow, 2, combGroup);
    connect(addAssociation, SIGNAL(clicked()), this, SLOT(onAddAssociationToolButtonClicked()));

    m_ui->tableWidget->scrollToBottom();

    // free memory
    te::common::FreeContents(groups);
    te::common::FreeContents(phases);
    groups.clear();
    phases.clear();

    m_ui->tableWidget->resizeColumnsToContents();
    m_ui->tableWidget->horizontalHeader()->setSectionResizeMode(1, QHeaderView::Stretch);
    m_ui->tableWidget->horizontalHeader()->setSectionResizeMode(2, QHeaderView::Stretch);
  }
}

int ta::model::qt::PhaseControlDialog::getRowByWidgetSearch()
{
  int row = -1;
  QToolButton* button = dynamic_cast<QToolButton*>(sender());
  if (button)
  {
    for (int i = 0; i < m_ui->tableWidget->rowCount(); ++i)
    {
      QWidget* w = m_ui->tableWidget->cellWidget(i, 0);
      if (button == w)
      {
        row = i;
        break;
      }
    }
  }
  return row;
}

void ta::model::qt::PhaseControlDialog::fillNames()
{
  std::vector<std::string> keys;
  keys.push_back("name");
  std::string error = "";
  std::vector<std::vector<std::string>> hints = ta::common::loadHintConfig("phases", keys, error);

  if (error.empty()){
    QComboBox* combNames = m_ui->m_phaseComboBoxEdit;
    combNames->clear();
    combNames->addItem("");
    for (std::vector<int>::size_type i = 0; i < hints.size(); ++i)
    {
      std::string name = hints.at(i).at(0);
      combNames->addItem(name.c_str());
    }
  }
  else{
    //TODO: launch error
  }
}