/*  This file is part of the TerraAmazon application.

See the Copyright file for details about the license of
this application.
*/

/*!
\file terraamazon/ProcessCellManagerDialog.cpp

\brief A Qt dialog used to configure the process cell manager.
*/

// TerraLib
#include <terralib/cellspace/CellSpaceOperations.h>

// TerraAmazon
#include "ProcessCellManagerDialog.h"
#include "ui_ProcessCellManagerDialogForm.h"
#include "../../../common/Utils.h"
#include "../../../common/qt/CreateCellLayerUtils.h"
#include "../../item/CellLayer.h"

// QT
#include <QMessageBox>
#include <QRadioButton>

// Boost
#include <boost/lexical_cast.hpp>

ta::model::qt::ProcessCellManagerDialog::ProcessCellManagerDialog(QWidget* parent, Qt::WindowFlags f)
: QDialog(parent, f),
m_ui(new Ui::ProcessCellManagerDialogForm),
m_rowActiveCellLayer(-1)
{
  m_ui->setupUi(this);

  std::string icon = ta::common::FindInTerraAmazonPath("share/icons/svg/terraAmazom-colorido-horizontal.svg");
  m_ui->m_imgLabel->setPixmap(QIcon(icon.c_str()).pixmap(112, 100));
  m_ui->m_imgLabel->hide();

  m_createCellLayerWidget = new ta::common::qt::CreateCellLayerWidget(m_ui->m_createCellLayerWidget);
  QGridLayout* layout1 = new QGridLayout(m_ui->m_createCellLayerWidget);
  layout1->setContentsMargins(0, 0, 0, 0);
  layout1->addWidget(m_createCellLayerWidget);

  m_lockedCellsWidget = new ta::model::qt::LockedCellsWidget(m_ui->m_lockedCellsWidget);
  QGridLayout* layout2 = new QGridLayout(m_ui->m_lockedCellsWidget);
  layout2->setContentsMargins(0, 0, 0, 0);
  layout2->addWidget(m_lockedCellsWidget);

  m_radioButtonMapper = new QSignalMapper(this);

  connect(m_ui->m_okCreateCellLayerPushButton, SIGNAL(clicked()), this, SLOT(onOkCreateCellLayerPushButtonClicked()));
  connect(m_ui->m_updatePushButton, SIGNAL(clicked()), this, SLOT(onUpdatePushButtonClicked()));
  connect(m_radioButtonMapper, SIGNAL(mapped(int)), this, SLOT(onRadioButtonChecked(int)));
}

ta::model::qt::ProcessCellManagerDialog::~ProcessCellManagerDialog()
{
}

void ta::model::qt::ProcessCellManagerDialog::setServer(te::da::DataSourceInfoPtr server)
{
  if (server.get())
  {
    m_activeServer = te::da::DataSourceManager::getInstance().get(server->getId(), server->getType(), server->getConnInfo());

    if (!m_activeServer.get())
      return;

    m_activeServer->open();

    m_catalogFacade.reset(new ta::model::CatalogFacade(m_activeServer));

    getLockedCellsWidget()->setServer(server);

    listCellLayers();
  }
}

void ta::model::qt::ProcessCellManagerDialog::listCellLayers()
{
  std::vector<ta::model::CellLayer*> layers = m_catalogFacade->getCellLayers();

  if (!layers.empty())
  {
    QTableWidget* tw = m_ui->m_cellLayersTableWidget;

    // reinitialize table widget
    tw->clearContents();
    tw->setRowCount(0);
    int newrow = tw->rowCount();

    for (std::size_t i = 0; i < layers.size(); i++)
    {
      newrow = tw->rowCount();
      tw->insertRow(newrow);

      // radio button
      QRadioButton* radioItem = new QRadioButton(tw);
      bool active = layers.at(i)->getActive();
      if (active){
        radioItem->setChecked(true);
        onRadioButtonChecked(newrow);
      }
      m_radioButtonMapper->setMapping(radioItem, newrow);
      connect(radioItem, SIGNAL(clicked()), m_radioButtonMapper, SLOT(map()));
      tw->setCellWidget(newrow, 0, radioItem);

      // name attribute
      QTableWidgetItem* nameItem = new QTableWidgetItem();
      nameItem->setText(layers.at(i)->getName().c_str());
      nameItem->setData(Qt::UserRole, QVariant::fromValue(layers.at(i)->getCatalogId()));
      nameItem->setFlags(Qt::ItemIsEnabled);
      tw->setItem(newrow, 1, nameItem);

      // resolution attribute
      QTableWidgetItem* resItem = new QTableWidgetItem();
      resItem->setText(QString::number(layers.at(i)->getResolution(), 'f', 2));
      resItem->setData(Qt::UserRole, QVariant::fromValue(layers.at(i)->getResolution()));
      resItem->setFlags(Qt::ItemIsEnabled);
      tw->setItem(newrow, 2, resItem);
    }
  }

  m_ui->m_cellLayersTableWidget->resizeColumnsToContents();
  m_ui->m_cellLayersTableWidget->horizontalHeader()->setSectionResizeMode(1, QHeaderView::Stretch);
  m_ui->m_cellLayersTableWidget->horizontalHeader()->setSectionResizeMode(2, QHeaderView::Stretch);
}

ta::common::qt::CreateCellLayerWidget* ta::model::qt::ProcessCellManagerDialog::getCreateCellLayerWidget()
{
  return m_createCellLayerWidget;
}

ta::model::qt::LockedCellsWidget* ta::model::qt::ProcessCellManagerDialog::getLockedCellsWidget()
{
  return m_lockedCellsWidget;
}

void ta::model::qt::ProcessCellManagerDialog::onOkCreateCellLayerPushButtonClicked()
{
  QString error;

  ta::common::qt::CreateCellLayerWidget* widget = getCreateCellLayerWidget();

  if (!widget->validateParameters(error))
  {
    QMessageBox::warning(this, tr("Process Cell Manager"), error);
    return;
  }

  try
  {
    te::da::DataSourceInfoPtr targetDs = widget->getCurrentDataSourceInfoPtr();
    std::string newCellLayerName = widget->getCurrentCellLayerName();
    te::gm::Envelope env = widget->getEnvelope();
    double res = widget->getCurrentResolution();
    int srid = widget->getSrid();

    te::da::DataSourcePtr source = te::da::DataSourceManager::getInstance().get(targetDs->getId(), targetDs->getType(), targetDs->getConnInfo());
    source->open();

    ta::common::CreateCellLayer(targetDs, newCellLayerName, env, res, srid);
    ta::common::AdaptCellDataSet(source, newCellLayerName);

    ta::model::Catalog* catalog = new ta::model::Catalog();
    ta::common::PrepareCatalogCellLayer(targetDs, newCellLayerName, "Created with the Cell Layer Manager.", catalog);
    if (!catalog)
      throw;
    m_catalogFacade->saveAsCellLayer(catalog, res, false);

    m_ui->m_tabWidget->setCurrentIndex(0);
    listCellLayers();
  }
  catch (const te::common::Exception& e)
  {
    QMessageBox::warning(this, tr("Process Cell Manager"), e.what());
    return;
  }
  catch (const std::exception& e)
  {
    QMessageBox::warning(this, tr("Process Cell Manager"), e.what());
    return;
  }
  catch (...)
  {
    QMessageBox::warning(this, tr("Process Cell Manager"), tr("Error trying to create cell layer!"));
    return;
  }
}

void ta::model::qt::ProcessCellManagerDialog::onRadioButtonChecked(const int& row)
{
  m_rowActiveCellLayer = row;
}

void ta::model::qt::ProcessCellManagerDialog::onUpdatePushButtonClicked()
{
  QTableWidget* tw = m_ui->m_cellLayersTableWidget;

  if (m_rowActiveCellLayer < 0){
    QMessageBox::warning(this, windowTitle(), tr("Invalid parameter. Select a cell layer."));
    return;
  }

  QTableWidgetItem* item = tw->item(m_rowActiveCellLayer, 1);
  QVariant var = item->data(Qt::UserRole);

  if (!var.isValid())
  {
    QMessageBox::warning(this, windowTitle(), tr("Catalog id is invalid."));
    return;
  }

  if (m_catalogFacade->setActiveCellLayer(var.toInt()))
  {
    QMessageBox::information(this, windowTitle(), tr("The selected cell layer has been successfully activated."));
  }
  else
  {
    QMessageBox::warning(this, windowTitle(), tr("The selected cell layer has not activated."));
  }
}