/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/ProcessControlDialog.cpp

  \brief A Qt dialog used to configure the process control.
*/

// TerraLib
#include <terralib/common/PlatformUtils.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/datatype/Date.h>
#include <terralib/datatype/TimeInstant.h>

// TerraAmazon
#include "AddNewTaskDialog.h"
#include "EndProcessDialog.h"
#include "ProcessControlDialog.h"
#include "ui_ProcessControlDialogForm.h"

#include "../../../common/Utils.h"

// QT
#include <QMessageBox>

ta::model::qt::ProcessControlDialog::ProcessControlDialog(QWidget* parent, Qt::WindowFlags f)
  : QDialog(parent, f),
  m_ui(new Ui::ProcessControlDialogForm),
  m_selectedRow(-1),
  m_manualSelect(false)
{
  m_ui->setupUi(this);

  std::string icon = ta::common::FindInTerraAmazonPath("share/icons/svg/terraAmazom-colorido-horizontal.svg");
  m_ui->m_imgLabel->setPixmap(QIcon(icon.c_str()).pixmap(112, 100));
  m_ui->m_imgLabel->hide();

  m_ui->m_attributeTable->setEditTriggers(QAbstractItemView::NoEditTriggers);
  m_ui->m_sceneTable->setEditTriggers(QAbstractItemView::NoEditTriggers);
  m_ui->m_sceneTable->resizeColumnsToContents();

  connect(m_ui->m_addnewtask, SIGNAL(clicked()), this, SLOT(onAddNewTaskButtonClicked()));
  connect(m_ui->m_startprocessButton, SIGNAL(clicked()), this, SLOT(onStartProcessButtonClicked()));
  connect(m_ui->m_endcurrentprocessButton, SIGNAL(clicked()), this, SLOT(onEndProcessButtonClicked()));
  connect(m_ui->m_closeButton, SIGNAL(clicked()), this, SLOT(onCloseButtonClicked()));
  connect(m_ui->m_sceneTable, SIGNAL(cellPressed(int, int)), this, SLOT(onSceneTableClicked(int, int)));
  connect(m_ui->m_sceneTable->selectionModel(), SIGNAL(selectionChanged(const QItemSelection &, const QItemSelection &)), this, SLOT(onSelectionChanged(const QItemSelection &, const QItemSelection &)));

}

void ta::model::qt::ProcessControlDialog::redrawSelection_slot()
{
  redrawSelection();
}

ta::model::qt::ProcessControlDialog::~ProcessControlDialog()
{}

void ta::model::qt::ProcessControlDialog::setProcess(te::da::DataSourceInfoPtr dsInfo, int userId, int processId)
{
  m_server = te::da::DataSourceManager::getInstance().get(dsInfo->getId(), dsInfo->getType(), dsInfo->getConnInfo());

  if (!m_server.get())
    return;

  if (m_server->isOpened() == false)
    m_server->open();

  m_dsInfo = dsInfo;

  ta::model::UserFacade uf(m_server);
  m_user.reset(dynamic_cast<ta::model::User*>(uf.get(userId)));

  ta::model::ProcessFacade pf(m_server);
  m_process.reset(dynamic_cast<ta::model::Process*>(pf.get(processId)));

  fillSceneTable();
  updateSelectedRow();
}

void ta::model::qt::ProcessControlDialog::onAddNewTaskButtonClicked()
{
  ta::model::qt::AddNewTaskDialog dlg(this);

  dlg.setTask(m_server, m_user->getProjectId(), m_user->getId());

  dlg.exec();

  if (dlg.hadAddedScene())
  {
    fillSceneTable();
    updateSelectedRow();
  }
}

void ta::model::qt::ProcessControlDialog::onStartProcessButtonClicked()
{
  if (m_ui->m_sceneTable->rowCount() == 0)
  {
    QMessageBox::information(this, "TerraAmazon", tr("There is no Process to be started!"));
    return;
  }

  if (m_ui->m_sceneTable->currentRow() == -1)
    return;

  ProcessFacade pf(m_server);
  SceneFacade sf(m_server);
  TaskLogFacade tlk(m_server);

  std::size_t candidateRow = m_ui->m_sceneTable->currentRow();
  int sceneId = m_ui->m_sceneTable->item((int)candidateRow, 1)->text().toInt();
  int taskId = m_ui->m_sceneTable->item((int)candidateRow, 6)->text().toInt();

  // CHECKS IF THE SCENE IS LOCKED
  bool lockState = false;
  if (!sf.isSceneLocked(sceneId, m_user->getProjectId(), lockState))
  {
    QMessageBox::warning(this, "TerraAmazon", tr("Error checking if cell is locked"));
    return;
  }
  if (lockState == true)
  {
    QMessageBox::information(this, "TerraAmazon", tr("This Scene was locked by the administrator and cannot be started!"));
    return;
  }

  // CHECKS IF THE PROCESS ALREADY EXISTS
  std::vector<Process*> vecProcess;
  if (pf.loadProcess(m_user->getProjectId(), m_user->getId(), vecProcess))
  {
    if (taskId == vecProcess[0]->getTaskId())
    {
      QMessageBox::warning(this, "TerraAmazon", tr("You have already started this process!"));
      return;
    }

    return;
  }

  vecProcess.clear();

  ta::model::Process process;
  ta::model::TaskLog* tasklog = new TaskLog();

  // CREATE TASKLOG AND SAVE
  boost::posix_time::ptime boostLocalTime = boost::posix_time::second_clock::local_time();
  te::dt::TimeInstant* initialTime = new te::dt::TimeInstant(boostLocalTime);

  tasklog->setTaskId(taskId);
  tasklog->setInitialTime(initialTime);
  tasklog->setStatus("STARTED");

  tlk.save(tasklog);

  std::vector<ta::model::AbstractItem*> tasklogs = tlk.get();

  // CREATE PROCESS AND SAVE
  tasklog = tlk.getLastTaskLog(taskId);

  process.setTaskId(tasklog->getTaskId());
  process.setTaskLogId(tasklog->getId());

  pf.save(&process);

  m_process.reset(dynamic_cast<ta::model::Process*>(pf.get(dynamic_cast<ta::model::AbstractItem*>(&process)->getId())));

  m_manualSelect = false;
  m_selectedRow = -1;
  m_ui->m_sceneTable->clearSelection();
  m_manualSelect = true;

  m_selectedRow = m_ui->m_sceneTable->currentRow();

  fillSceneTable();

  redrawSelection();
}

void ta::model::qt::ProcessControlDialog::onEndProcessButtonClicked()
{
  if (m_process.get() == 0)
  {
    QMessageBox::warning(this, "TerraAmazon", tr("There is not an active proccess. Start one first!"));
    return;
  }

  ta::model::qt::EndProcessDialog dlg(this);
  dlg.setEndProcess(m_dsInfo, m_process->getId());

  int resultExec = dlg.exec();

  if (resultExec == QDialog::Accepted)
  {
    m_manualSelect = false;
    m_selectedRow = -1;
    m_ui->m_sceneTable->clearSelection();
    m_manualSelect = true;

    //delete m_process;
    m_process = 0;

    fillSceneTable();

    return;
  }

  dlg.close();
}

void ta::model::qt::ProcessControlDialog::onCloseButtonClicked()
{
  close();
}

void ta::model::qt::ProcessControlDialog::fillSceneTable()
{
  assert(m_server.get());

  while (m_ui->m_sceneTable->rowCount() > 0)
  {
    m_ui->m_sceneTable->removeRow(0);
  }

  ProcessFacade pf(m_server);

  std::auto_ptr<te::da::DataSet> ds = pf.getProcessControl(m_user->getProjectId(), m_user->getId());

  std::size_t row = 0;

  while (ds->moveNext())
  {
    m_ui->m_sceneTable->insertRow((int)row);

    QString sceneId = QString::number(ds->getInt32("SCENE_ID"));
    QString date = ds->getDateTime("VIEW_DATE")->toString().c_str();
    QString aiObjectId = ds->getString("AI_OBJECT_ID").c_str();
    QString phase = ds->getString("DESCRIPTION").c_str();
    QString status = ds->getString("STATUS").c_str();
    QString taskId = QString::number(ds->getInt32("ID"));

    QTableWidgetItem *icon_item = new QTableWidgetItem;
    icon_item->setIcon(getStatusIconPixmap(status.toStdString()));
    m_ui->m_sceneTable->setItem((int)row, 0, icon_item);

    m_ui->m_sceneTable->setItem((int)row, 1, new QTableWidgetItem(sceneId));
    m_ui->m_sceneTable->setItem((int)row, 2, new QTableWidgetItem(date));
    m_ui->m_sceneTable->setItem((int)row, 3, new QTableWidgetItem(aiObjectId));
    m_ui->m_sceneTable->setItem((int)row, 4, new QTableWidgetItem(phase));
    m_ui->m_sceneTable->setItem((int)row, 5, new QTableWidgetItem(status));
    m_ui->m_sceneTable->setItem((int)row, 6, new QTableWidgetItem(taskId));
    
    ++row;
  }

  m_ui->m_sceneTable->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);

  updateSelectedRow();

}

void ta::model::qt::ProcessControlDialog::onSceneTableClicked(int row, int /*col*/)
{
  m_ui->m_attributeTable->setRowCount(0);

  if (m_ui->m_sceneTable->rowCount() == 0)
  {
    return;
  }
  if (row < 0)
  {
    return;
  }

  std::size_t scnId = m_ui->m_sceneTable->item(row, 1)->text().toInt();

  SceneFacade sf(m_server);
  std::unique_ptr<ta::model::Scene> scn(dynamic_cast<ta::model::Scene*>(sf.get((int)scnId)));

  if (!scn.get())
  {
    QMessageBox::information(this, tr("TerraAmazon"), tr("Scene not found!"));
    return;
  }

  AoiLayerFacade aoif(m_server);
  std::unique_ptr<ta::model::AoiLayer> aoilayer(dynamic_cast<ta::model::AoiLayer*>(aoif.get(scn->getAoiLayerId())));

  if (!aoilayer.get())
  {
    QMessageBox::information(this, tr("TerraAmazon"), tr("AoiLayer not found!"));
    return;
  }

  CatalogFacade cf(m_server);
  std::auto_ptr<ta::model::Catalog> catalog(dynamic_cast<ta::model::Catalog*>(cf.get(aoilayer->getCatalogId())));

  if (!catalog.get())
  {
    QMessageBox::information(this, tr("TerraAmazon"), tr("Catalog not found!"));
    return;
  }

  std::unique_ptr<te::da::DataSetType> dsType = cf.getDataSetType(catalog.get());
  std::unique_ptr<te::da::DataSet> ds = cf.getDataSet(catalog.get(), scn->getAiObjectId(), aoilayer->getAoiIdColumn());

  if (!ds.get())
  {
    QMessageBox::information(this, tr("TerraAmazon"), tr("Data not found!"));
    return;
  }

  ds->moveBeforeFirst();

  assert(ds->getNumProperties() == dsType->getProperties().size());

  if (!ds->moveNext())
    return;

  for (std::size_t i = 0; i < dsType->getProperties().size(); ++i)
  {
    if (dsType->getProperties()[i]->getType() == te::dt::GEOMETRY_TYPE)
      continue;

    m_ui->m_attributeTable->insertRow((int)i);

    m_ui->m_attributeTable->setItem((int)i, 0, new QTableWidgetItem(dsType->getProperties()[i]->getName().c_str()));
    m_ui->m_attributeTable->setItem((int)i, 1, new QTableWidgetItem(ds->getValue(i)->toString().c_str()));

  }

  m_ui->m_attributeTable->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);

}

void ta::model::qt::ProcessControlDialog::onSelectionChanged(const QItemSelection & /*newSelection*/, const QItemSelection & /*oldSelection*/)
{
  redrawSelection();
}

void ta::model::qt::ProcessControlDialog::redrawSelection()
{
  if (!m_manualSelect)
  {
    m_manualSelect = true;
    return;
  }

  if (m_selectedRow != -1)
  {
    m_manualSelect = false;
    
    QTableWidgetItem *icon_item = new QTableWidgetItem;
    icon_item->setIcon(getStatusIconPixmap("STARTED"));
    m_ui->m_sceneTable->setItem(m_selectedRow, 0, icon_item);
    m_ui->m_sceneTable->selectRow(m_selectedRow);

    QModelIndex topLeft = m_ui->m_sceneTable->model()->index(m_selectedRow, 0, QModelIndex());
    QModelIndex bottomRight = m_ui->m_sceneTable->model()->index(m_selectedRow, 6, QModelIndex());

    QItemSelection selection(topLeft, bottomRight);
    m_ui->m_sceneTable->selectionModel()->select(selection, QItemSelectionModel::Select);

  }
}

void ta::model::qt::ProcessControlDialog::updateSelectedRow()
{
  if (m_process)
  {
    int taskId = m_process->getTaskId();

    for (int i = 0; i < m_ui->m_sceneTable->rowCount(); i++)
    {
      if (m_ui->m_sceneTable->item(i, 6)->text().toInt() == taskId)
      {
        m_selectedRow = i;
        redrawSelection();
        break;
      }
    }
  }
}

QIcon ta::model::qt::ProcessControlDialog::getStatusIconPixmap(std::string status)
{

  if (strcmp(status.c_str(), "CLOSED") == 0)
  {
    return QIcon(ta::common::FindInTerraAmazonPath("share/icons/processIcon-closed.bmp").c_str());
  }
  else if (strcmp(status.c_str(), "OPENED") == 0)
  {
    return QIcon(ta::common::FindInTerraAmazonPath("share/icons/processIcon-opened.bmp").c_str());
  }
  else if (strcmp(status.c_str(), "IN REVIEW") == 0)
  {
    return QIcon(ta::common::FindInTerraAmazonPath("share/icons/processIcon-revision.bmp").c_str());
  }
  else if (strcmp(status.c_str(), "STARTED") == 0)
  {
    return QIcon(ta::common::FindInTerraAmazonPath("share/icons/processIcon-started.bmp").c_str());
  }
  else
    return QIcon(ta::common::FindInTerraAmazonPath("share/icons/processIcon-new.bmp").c_str());

}

int ta::model::qt::ProcessControlDialog::getProcess()
{
  if (m_process)
  {
    return m_process.get()->getId();
  }

  return -1;
}