/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/ProjectActivationWidget.cpp

  \brief A Qt widget used to configure the project settings.
*/

// TerraAmazon
#include "../../../common/Utils.h"
#include "../../facade/ProjectFacade.h"
#include "../../item/Project.h"
#include "ProjectActivationWidget.h"
#include "ui_ProjectActivationWidgetForm.h"

// TerraLib
#include <terralib/common/STLUtils.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>
#include <terralib/datatype/DateTimeInstant.h>


// STL
#include <string>

// Qt
#include <QListWidgetItem>
#include <QMessageBox>
#include <QValidator>

ta::model::qt::ProjectActivationWidget::ProjectActivationWidget(QWidget* parent, Qt::WindowFlags f)
  : QWidget(parent, f),
  m_ui(new Ui::ProjectActivationWidgetForm)
{
  m_ui->setupUi(this);

  m_ui->m_projectComboBox->addItem("");

  std::string projActivated = ta::common::FindInTerraAmazonPath("share/icons/svg/project-activated.svg");
  m_ui->m_activeToolButton->setIcon(QIcon(projActivated.c_str()));

  std::string projDeactivated = ta::common::FindInTerraAmazonPath("share/icons/svg/project-deactivated.svg");
  m_ui->m_deactiveToolButton->setIcon(QIcon(projDeactivated.c_str()));

  connect(m_ui->m_activeToolButton, SIGNAL(pressed()), this, SLOT(onActivateTriggered()));
  connect(m_ui->m_deactiveToolButton, SIGNAL(pressed()), this, SLOT(onDeactivateTriggered()));
}

ta::model::qt::ProjectActivationWidget::~ProjectActivationWidget()
{
}

void ta::model::qt::ProjectActivationWidget::setServer(te::da::DataSourceInfoPtr dsInfo)
{
  //get data source
  m_server = te::da::DataSourceManager::getInstance().get(dsInfo->getId(), dsInfo->getType(), dsInfo->getConnInfo());

  if (!m_server.get())
    return;

  if (m_server->isOpened() == false)
    m_server->open();

  listProjects();
}

int ta::model::qt::ProjectActivationWidget::getActiveProjectId()
{
  if (m_activeProject)
    return m_activeProject->getId();

  return -1;
}

void ta::model::qt::ProjectActivationWidget::setActiveProjectId(int projId)
{
  ta::model::ProjectFacade pf(m_server);
  m_activeProject.reset(dynamic_cast<ta::model::Project*>(pf.get(projId)));
  m_ui->m_projectLineEdit->setText(m_activeProject->getName().c_str());
}

void ta::model::qt::ProjectActivationWidget::onActivateTriggered()
{
  if (m_ui->m_projectComboBox->count() < 1)
  {
    QMessageBox::warning(this, windowTitle(), tr("There is no available project."));
    return;
  }

  QVariant var = m_ui->m_projectComboBox->itemData(m_ui->m_projectComboBox->currentIndex());

  if (!var.isValid())
  {
    QMessageBox::warning(this, windowTitle(), tr("Invalid project selected."));
    return;
  }

  int id = var.toInt();

  ta::model::ProjectFacade pf(m_server);

  ta::model::Project* p = dynamic_cast<ta::model::Project*>(pf.get(id));

  //check if current project can be deactivated

  //active new project
  m_activeProject.reset(p);

  m_ui->m_projectLineEdit->setText(m_activeProject->getName().c_str());
}

void ta::model::qt::ProjectActivationWidget::onDeactivateTriggered()
{
  m_activeProject.reset(0);

  m_ui->m_projectLineEdit->clear();
}

void ta::model::qt::ProjectActivationWidget::listProjects()
{
  if (!m_server.get())
    return;

  ta::model::ProjectFacade pf(m_server);

  //get projects from selected data source (server)
  std::vector<ta::model::AbstractItem*> projects = pf.get();

  m_ui->m_projectComboBox->clear();

  for (std::size_t t = 0; t < projects.size(); ++t)
  {
    ta::model::Project* p = dynamic_cast<ta::model::Project*>(projects[t]);

    m_ui->m_projectComboBox->addItem(p->getName().c_str(), QVariant::fromValue(p->getId()));
  }

  te::common::FreeContents(projects);
  projects.clear();
}
