/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of
    this application.
    */

/*!
  \file terraamazon/ProjectControlWidget.cpp

  \brief A Qt widget used to configure the project settings.
  */

// TerraAmazon
#include "../../../common/Utils.h"
#include "../../item/Class.h"
#include "../../item/Project.h"
#include "../../item/Rule.h"
#include "../../item/RulesLayerColumn.h"
#include "../../item/RuleSet.h"
#include "../../item_mem/ProjectMem.h"
#include "ProjectControlWidget.h"
#include "ui_ProjectControlWidgetForm.h"

// TerraLib
#include <terralib/common/STLUtils.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>
#include <terralib/dataaccess/utils/Utils.h>
#include <terralib/datatype/Date.h>
#include <terralib/datatype/TimeInstant.h>

// STL
#include <string>

// Boost
#include <boost/date_time/gregorian/gregorian.hpp>
#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/property_tree/xml_parser.hpp>

// Qt
#include <QComboBox>
#include <QDateTime>
#include <QFileDialog>
#include <QListWidgetItem>
#include <QMessageBox>
#include <QValidator>

Q_DECLARE_METATYPE(te::da::DataSourceInfoPtr);

ta::model::qt::ProjectControlWidget::ProjectControlWidget(QWidget* parent, Qt::WindowFlags f)
: QWidget(parent, f),
m_ui(new Ui::ProjectControlWidgetForm)
{
  m_ui->setupUi(this);

  // add button icons
  m_ui->m_addToolButton->setIcon(QIcon::fromTheme("list-add"));
  m_ui->m_importToolButton->setIcon(QIcon::fromTheme("document-open"));

  //mappers
  m_removeButtonMapper = new QSignalMapper(this);
  m_updateButtonMapper = new QSignalMapper(this);
  m_exportButtonMapper = new QSignalMapper(this);

  // validators
  m_ui->m_minScaleLineEdit->setValidator(new QDoubleValidator(this));
  m_ui->m_maxScaleLineEdit->setValidator(new QDoubleValidator(this));
  m_ui->m_areaMinLineEdit->setValidator(new QDoubleValidator(this));

  //set dates
  m_ui->m_createdDateEdit->setDate(QDate::currentDate());
  m_ui->m_updatedDateEdit->setDate(QDate::currentDate());

  //connects
  connect(m_ui->m_addToolButton, SIGNAL(pressed()), this, SLOT(onAddToolButtonPressed()));
  connect(m_ui->m_importToolButton, SIGNAL(pressed()), this, SLOT(onImportToolButtonPressed()));

  connect(m_removeButtonMapper, SIGNAL(mapped(int)), this, SLOT(onRemoveToolButtonPressed(int)));
  connect(m_updateButtonMapper, SIGNAL(mapped(int)), this, SLOT(onUpdateToolButtonPressed(int)));
  connect(m_exportButtonMapper, SIGNAL(mapped(int)), this, SLOT(onExportToolButtonPressed(int)));

  connect(m_ui->m_tableWidget, SIGNAL(cellClicked(int, int)), this, SLOT(onTableCellClicked(int, int)));
}

ta::model::qt::ProjectControlWidget::~ProjectControlWidget()
{
}

void ta::model::qt::ProjectControlWidget::onAddToolButtonPressed()
{
  //check project
  if (m_ui->m_nameLineEdit->text().isEmpty())
  {
    QMessageBox::warning(this, "TerraAmazon", tr("Project name not defined."));
    return;
  }

  ta::model::ProjectFacade pf(m_server);

  //check if already exists a project with this name
  std::string projName = m_ui->m_nameLineEdit->text().toStdString();

  if (pf.existsByName(projName))
  {
    QMessageBox::warning(this, tr("Warning"), tr("Project already exists."));
    return;
  }

  //check scale
  if (m_ui->m_scaleGroupBox->isChecked())
  {
    if (m_ui->m_minScaleLineEdit->text().isEmpty() || m_ui->m_maxScaleLineEdit->text().isEmpty())
    {
      QMessageBox::warning(this, "TerraAmazon", tr("Scale values not defined."));
      return;
    }
  }

  //create project and save
  ta::model::Project p;

  p.setName(projName);
  p.setMaxCells((int)m_ui->m_numCellsSpinBox->value());
  p.setActive(m_ui->m_availableUsersCheckBox->isChecked());

  if (m_ui->m_scaleGroupBox->isChecked())
  {
    p.setMinScale(m_ui->m_minScaleLineEdit->text().toDouble());
    p.setMaxScale(m_ui->m_maxScaleLineEdit->text().toDouble());
  }

  boost::posix_time::ptime boostLocalTime = boost::posix_time::second_clock::local_time();
  boost::gregorian::date d = boostLocalTime.date();
  te::dt::Date* createdTime = new te::dt::Date(d);
  te::dt::Date* updatedTime = new te::dt::Date(d);

  p.setCreated(createdTime);
  p.setUpdated(updatedTime);

  if (!pf.save(&p))
  {
    QMessageBox::warning(this, "TerraAmazon", tr("Project not created."));

    return;
  }

  QMessageBox::information(this, "TerraAmazon", tr("Project Created."));

  listProjects();
}

void ta::model::qt::ProjectControlWidget::onUpdateToolButtonPressed(int row)
{
  //check project
  QVariant var = m_ui->m_tableWidget->item(row, 1)->data(Qt::UserRole);

  if (!var.isValid())
  {
    QMessageBox::warning(this, "TerraAmazon", tr("Error getting project information."));
    return;
  }

  int id = var.toInt();

  //get name
  if (m_ui->m_tableWidget->item(row, 1)->text().isEmpty())
  {
    QMessageBox::warning(this, "TerraAmazon", tr("Project name not defined."));
    return;
  }
  std::string name = m_ui->m_tableWidget->item(row, 1)->text().toStdString();

  //get cells
  if (m_ui->m_tableWidget->item(row, 2)->text().isEmpty())
  {
    QMessageBox::warning(this, "TerraAmazon", tr("Number of cells not defined."));
    return;
  }

  bool ok = true;
  int nCells = m_ui->m_tableWidget->item(row, 2)->text().toInt(&ok);

  if (!ok)
  {
    QMessageBox::warning(this, "TerraAmazon", tr("Invalid value for cells."));
    return;
  }

  //get min area
  if (m_ui->m_tableWidget->item(row, 3)->text().isEmpty())
  {
    QMessageBox::warning(this, "TerraAmazon", tr("Minimum area not defined."));
    return;
  }

  if (!ok)
  {
    QMessageBox::warning(this, "TerraAmazon", tr("Invalid value for minimum area."));
    return;
  }

  // scale
  bool useScale = true;
  if (m_ui->m_tableWidget->item(row, 4)->text().isEmpty() && m_ui->m_tableWidget->item(row, 5)->text().isEmpty())
  {
    useScale = false;
  }
  else
  {
    if (m_ui->m_tableWidget->item(row, 4)->text().isEmpty() || m_ui->m_tableWidget->item(row, 5)->text().isEmpty())
    {
      QMessageBox::warning(this, "TerraAmazon", tr("Values not defined for scale."));
      return;
    }
  }

  double scaleMin = 0.;
  double scaleMax = 0.;
  if (useScale)
  {
    scaleMin = m_ui->m_tableWidget->item(row, 4)->text().toDouble(&ok);

    if (!ok)
    {
      QMessageBox::warning(this, "TerraAmazon", tr("Invalid value for minimum scale."));
      return;
    }

    scaleMax = m_ui->m_tableWidget->item(row, 5)->text().toDouble(&ok);

    if (!ok)
    {
      QMessageBox::warning(this, "TerraAmazon", tr("Invalid value for maximum scale."));
      return;
    }
  }

  // is available
  QComboBox* cmbBox = dynamic_cast<QComboBox*>(m_ui->m_tableWidget->cellWidget(row, 6));

  if (!cmbBox)
  {
    QMessageBox::warning(this, "TerraAmazon", tr("Error getting available information."));
    return;
  }

  bool isAvailable = true;

  int availableValue = cmbBox->currentData().toInt();

  if (availableValue == 0)
    isAvailable = false;


  //load project and save
  ta::model::ProjectFacade pf(m_server);

  ta::model::Project* p = dynamic_cast<ta::model::Project*>(pf.get(id));

  p->setName(name);
  p->setMaxCells(nCells);
  p->setActive(isAvailable);

  if (useScale)
  {
    p->setMinScale(scaleMin);
    p->setMaxScale(scaleMax);
  }
  else
  {
    pf.setNoScale(p);
  }

  //update date
  boost::posix_time::ptime boostLocalTime = boost::posix_time::second_clock::local_time();
  boost::gregorian::date d = boostLocalTime.date();

  te::dt::Date* updatedTime = new te::dt::Date(d);

  p->setUpdated(updatedTime);

  if (!pf.update(p))
  {
    delete p;

    QMessageBox::warning(this, "TerraAmazon", tr("Project not updated."));

    return;
  }

  delete p;

  QMessageBox::information(this, "TerraAmazon", tr("Project Updated."));

  listProjects();
}

void ta::model::qt::ProjectControlWidget::onRemoveToolButtonPressed(int row)
{
  //check project
  QVariant var = m_ui->m_tableWidget->item(row, 1)->data(Qt::UserRole);

  if (!var.isValid())
  {
    QMessageBox::warning(this, "TerraAmazon", tr("Error getting project information."));
    return;
  }

  int id = var.toInt();

  ta::model::ProjectFacade pf(m_server);

  //remove project
  if (!pf.remove(id))
  {
    QMessageBox::information(this, "TerraAmazon", tr("Project not Removed."));

    return;
  }

  QMessageBox::information(this, "TerraAmazon", tr("Project Removed."));

  listProjects();
}

void ta::model::qt::ProjectControlWidget::onExportToolButtonPressed(int row)
{
  try
  {
    // get project
    QTableWidgetItem* item = m_ui->m_tableWidget->item(row, 1);
    int projId = item->data(Qt::UserRole).toInt();
    ta::model::Project* proj = dynamic_cast<ta::model::Project*>(m_projectFacade->get(projId));

    //TODO: get recent path

    std::string projName = "project_" + proj->getName() + ".xml";
    QString fileName = QFileDialog::getSaveFileName(this, tr("Save as..."), projName.c_str(), tr("XML file (*.xml *.XML);;"), 0);

    if (fileName.isEmpty())
      return;

    // create xml file from project
    boost::property_tree::ptree mainFile;

    // project
    boost::property_tree::ptree project;
    project.put(ta::model::Project::sm_name, proj->getName());
    project.put(ta::model::Project::sm_active, proj->isActive());
    project.put(ta::model::Project::sm_minScale, proj->getMinScale());
    project.put(ta::model::Project::sm_maxScale, proj->getMaxScale());
    project.put(ta::model::Project::sm_created, proj->getCreated()->getDate());
    project.put(ta::model::Project::sm_updated, proj->getUpdated()->getDate());
    project.put(ta::model::Project::sm_maxCells, proj->getMaxCells());

    // rulesets
    std::vector<int> ruleSetIds = m_projectFacade->getRuleSets(projId);

    boost::property_tree::ptree rulesets;

    for (int i = 0; i < ruleSetIds.size(); ++i)
    {
      ta::model::RuleSet* rs = dynamic_cast<ta::model::RuleSet*>(m_rulesetFacade->get(ruleSetIds.at(i)));
      boost::property_tree::ptree ruleset;
      ruleset.put(ta::model::RuleSet::sm_name, rs->getName());

      //catalog
      boost::property_tree::ptree catalog = exportCatalogAux(rs->getCatalogId());

      //class
      std::vector<int> classIds = m_rulesetFacade->getRuleSetClasses(rs->getCatalogId());
      boost::property_tree::ptree classes;
      if (!classIds.empty()){
        for (int i = 0; i < classIds.size(); ++i)
        {
          ta::model::Class* c = dynamic_cast<ta::model::Class*>(m_classFacade->get(classIds.at(i)));
          boost::property_tree::ptree clazz;
          clazz.put(ta::model::Class::sm_name, c->getName());
          clazz.put(ta::model::Class::sm_description, c->getDescription());
          clazz.put(ta::model::Class::sm_style, c->getStyle());
          classes.push_back(std::make_pair("class", clazz));
        }
      }

      //rule structures
      std::vector<ta::model::RuleRuleSet*> rrs = m_ruleRuleSetFacade->getByRuleSetId(rs->getId());
      boost::property_tree::ptree ruleStructs;
      if (!rrs.empty())
      {
        for (int i = 0; i < rrs.size(); ++i)
        {
          boost::property_tree::ptree ruleStruct;

          //rule
          ta::model::Rule* r = dynamic_cast<ta::model::Rule*>(m_ruleFacade->get(rrs.at(i)->getRuleId()));
          boost::property_tree::ptree rule;
          rule.put(ta::model::Rule::sm_name, r->getName());
          rule.put(ta::model::Rule::sm_operation_id, r->getOperationId());

          //catalog rule
          boost::property_tree::ptree catalogRule = exportCatalogAux(r->getCatalogId());
          rule.push_back(std::make_pair("catalog", catalogRule));

          //rule ruleset
          int priority = rrs.at(i)->getPriority();

          ruleStruct.push_back(std::make_pair("rule", rule));
          ruleStruct.put(ta::model::RuleRuleSet::sm_priority, priority);

          //rules layer column
          std::vector<ta::model::RulesLayerColumn*> rlc = m_rulesLayerColumnFacade->getByRuleSetIdAndRuleId(rs->getId(), r->getId());
          boost::property_tree::ptree rulesLayerColumns;
          if (!rlc.empty())
          {
            for (int i = 0; i < rlc.size(); ++i)
            {
              boost::property_tree::ptree rulesLayerColumn;
              rulesLayerColumn.put(ta::model::RulesLayerColumn::sm_column_in, rlc.at(i)->getColumnIn());
              rulesLayerColumn.put(ta::model::RulesLayerColumn::sm_column_out, rlc.at(i)->getColumnOut());
              rulesLayerColumns.push_back(std::make_pair("ruleslayercolumn", rulesLayerColumn));
            }
            ruleStruct.add_child("ruleslayercolumns", rulesLayerColumns);
          }
          ruleStructs.push_back(std::make_pair("rulestruct", ruleStruct));
        }
      }

      ruleset.push_back(std::make_pair("catalog", catalog));
      ruleset.add_child("classes", classes);
      ruleset.add_child("rulestructs", ruleStructs);

      rulesets.push_back(std::make_pair("ruleset", ruleset));
    }

    project.add_child("rulesets", rulesets);

    mainFile.add_child("project", project);

    boost::property_tree::xml_parser::write_xml(fileName.toStdString(), mainFile);
    //boost::property_tree::xml_parser::write_xml(fileName.toStdString(), mainFile, std::locale(), boost::property_tree::xml_writer_make_settings<std::string>('\t', 1));
  }
  catch (boost::property_tree::xml_parser::xml_parser_error &je)
  {
    std::string filename = je.filename().c_str();
    std::string errmsg = "Error parsing " + je.filename() + " - line " + boost::lexical_cast<std::string>(je.line()) + ": " + je.message().c_str();
    QMessageBox::warning(this, "TerraAmazon", tr(errmsg.c_str()));
  }
  catch (std::exception const& e)
  {
    std::string errmsg = e.what();
    QMessageBox::warning(this, "TerraAmazon", tr(errmsg.c_str()));
  }

  QMessageBox::warning(this, "TerraAmazon", tr("Project Exported."));

  return;
}

void ta::model::qt::ProjectControlWidget::onImportToolButtonPressed()
{
  QString fileName = QFileDialog::getOpenFileName(this, tr("Import project..."), QString(), tr("XML file (*.xml *.XML);;"), 0);

  if (fileName.isEmpty())
    return;

  try
  {
    boost::property_tree::ptree input;
    boost::property_tree::xml_parser::read_xml(fileName.toStdString(), input);

    boost::property_tree::ptree project;
    ta::model::ProjectMem* projectMem = new ta::model::ProjectMem();

    project = input.get_child("project");

    std::string name = project.get<std::string>(ta::model::Project::sm_name);
    bool active = project.get<bool>(ta::model::Project::sm_active);
    double minScale = project.get<double>(ta::model::Project::sm_minScale);
    double maxScale = project.get<double>(ta::model::Project::sm_maxScale);
    std::string sCreatedOn = project.get<std::string>(ta::model::Project::sm_created);
    boost::gregorian::date dCreatedOn(boost::gregorian::from_simple_string(sCreatedOn));
    te::dt::Date* dtCreatedOn = new te::dt::Date(dCreatedOn);
    std::string sUpdateOn = project.get<std::string>(ta::model::Project::sm_updated);
    boost::gregorian::date dUpdatedOn(boost::gregorian::from_simple_string(sUpdateOn));
    te::dt::Date* dtUpdatedOn = new te::dt::Date(dUpdatedOn);
    int maxCells = project.get<int>(ta::model::Project::sm_maxCells);

    projectMem->setName(name);
    projectMem->setActive(active);
    projectMem->setMinScale(minScale);
    projectMem->setMaxScale(maxScale);
    projectMem->setCreated(dtCreatedOn);
    projectMem->setUpdated(dtUpdatedOn);
    projectMem->setMaxCells(maxCells);

    //rulesets
    boost::property_tree::ptree rulesets;
    try
    {
      rulesets = project.get_child("rulesets");
    }
    catch (std::exception const& /*e*/)
    {
      //...
    }
    if (!rulesets.empty())
    {
      for (boost::property_tree::ptree::const_iterator itrs = rulesets.begin(); itrs != rulesets.end(); ++itrs)
      {
        ta::model::RuleSetMem* rulesetMem = new ta::model::RuleSetMem();

        //name
        std::string name = itrs->second.get<std::string>(ta::model::RuleSet::sm_name);
        rulesetMem->setName(name);

        //catalog
        boost::property_tree::ptree catalog = itrs->second.get_child("catalog");
        ta::model::CatalogMem* catalogMem = importCatalogAux(catalog);
        if (catalogMem != 0)
        {
          rulesetMem->setCatalogMem(catalogMem);
        }

        //classes
        boost::property_tree::ptree classes;
        try
        {
          classes = itrs->second.get_child("classes");
        }
        catch (std::exception const& /*e*/)
        {
          //...
        }
        if (!classes.empty())
        {
          for (boost::property_tree::ptree::const_iterator itc = classes.begin(); itc != classes.end(); ++itc)
          {
            std::string name = itc->second.get<std::string>(ta::model::Class::sm_name);
            std::string description = itc->second.get<std::string>(ta::model::Class::sm_description);
            std::string style = itc->second.get<std::string>(ta::model::Class::sm_style);

            ta::model::ClassMem* classMem = new ta::model::ClassMem();
            classMem->setName(name);
            classMem->setDescription(description);
            classMem->setStyle(style);

            rulesetMem->getClassMems().push_back(classMem);
          }
        }

        //rulestructs
        boost::property_tree::ptree rulestructs;
        try
        {
          rulestructs = itrs->second.get_child("rulestructs");
        }
        catch (std::exception const& /*e*/)
        {
          //...
        }
        if (!rulestructs.empty())
        {
          for (boost::property_tree::ptree::const_iterator itrst = rulestructs.begin(); itrst != rulestructs.end(); ++itrst)
          {
            ta::model::RuleStructMem* ruleStructMen = new ta::model::RuleStructMem();

            int priority = itrst->second.get<int>(ta::model::RuleStructMem::sm_priority);
            ruleStructMen->setPriority(priority);

            boost::property_tree::ptree rule = itrst->second.get_child("rule");
            std::string name = rule.get<std::string>(ta::model::Rule::sm_name);
            int operationId = rule.get<int>(ta::model::Rule::sm_operation_id);
            boost::property_tree::ptree ruleCatalog = rule.get_child("catalog");
            ta::model::CatalogMem* ruleCatalogMem = importCatalogAux(ruleCatalog);

            ta::model::RuleMem* ruleMem = new ta::model::RuleMem();
            ruleMem->setName(name);
            ruleMem->setOperationId(operationId);
            ruleMem->setCatalogMem(ruleCatalogMem);
            ruleStructMen->setRuleMem(ruleMem);

            boost::property_tree::ptree ruleslayercolumns;
            try
            {
              ruleslayercolumns = itrst->second.get_child("ruleslayercolumns");
            }
            catch (std::exception const& /*e*/)
            {
              //...
            }
            if (!ruleslayercolumns.empty())
            {
              for (boost::property_tree::ptree::const_iterator itrlc = ruleslayercolumns.begin(); itrlc != ruleslayercolumns.end(); ++itrlc)
              {
                std::string column_in = itrlc->second.get<std::string>(ta::model::RulesLayerColumnMem::sm_column_in);
                std::string column_out = itrlc->second.get<std::string>(ta::model::RulesLayerColumnMem::sm_column_out);
                ta::model::RulesLayerColumnMem* rulesLayerColumnMem = new ta::model::RulesLayerColumnMem();
                rulesLayerColumnMem->setColumnIn(column_in);
                rulesLayerColumnMem->setColumnOut(column_out);
                ruleStructMen->getRulesLayerColumnMems().push_back(rulesLayerColumnMem);
              }
            }

            rulesetMem->getRuleStructMems().push_back(ruleStructMen);
          }
        }

        projectMem->getRuleSetMems().push_back(rulesetMem);
      }
    }
    std::string errorMessage = "";
    std::string warningMessage = "";
    if (m_projectFacade->importToDatabase(projectMem, errorMessage, warningMessage))
    {
      if (!warningMessage.empty())
      {
        QMessageBox::warning(this, "TerraAmazon", tr(warningMessage.c_str()));
      }
      listProjects();
    }
    else
    {
      QMessageBox::warning(this, "TerraAmazon", tr(errorMessage.c_str()));
      return;
    }
  }
  catch (boost::property_tree::xml_parser::xml_parser_error &je)
  {
    std::string filename = je.filename().c_str();
    std::string errmsg = "Error parsing " + je.filename() + " - line " + boost::lexical_cast<std::string>(je.line()) + ": " + je.message().c_str();
    QMessageBox::warning(this, "TerraAmazon", tr(errmsg.c_str()));
    return;
  }
  catch (std::exception const& e)
  {
    std::string errmsg = e.what();
    QMessageBox::warning(this, "TerraAmazon", tr(errmsg.c_str()));
    return;
  }

  QMessageBox::warning(this, "TerraAmazon", tr("Project Imported."));
}

ta::model::CatalogMem* ta::model::qt::ProjectControlWidget::importCatalogAux(const boost::property_tree::ptree& catalog)
{
  ta::model::CatalogMem* result = new ta::model::CatalogMem();

  std::string name = catalog.get<std::string>(ta::model::Catalog::sm_name);
  std::string uri = catalog.get<std::string>(ta::model::Catalog::sm_uri);
  std::string source_type = catalog.get<std::string>(ta::model::Catalog::sm_type);
  int permission = catalog.get<int>(ta::model::Catalog::sm_permission);
  std::string slastEdition = catalog.get<std::string>(ta::model::Catalog::sm_lastEdition);
  boost::posix_time::ptime bLastEdition(boost::posix_time::time_from_string(slastEdition));
  te::dt::TimeInstant* last_editon = new te::dt::TimeInstant(bLastEdition);
  std::string author = catalog.get<std::string>(ta::model::Catalog::sm_author);
  std::string source = catalog.get<std::string>(ta::model::Catalog::sm_source);
  std::string description = catalog.get<std::string>(ta::model::Catalog::sm_description);
  double scale_min = catalog.get<double>(ta::model::Catalog::sm_scaleMin);
  double scale_max = catalog.get<double>(ta::model::Catalog::sm_scaleMax);
  std::string style = catalog.get<std::string>(ta::model::Catalog::sm_style);
  bool locked = catalog.get<bool>(ta::model::Catalog::sm_locked);
  std::string cell_column = catalog.get<std::string>(ta::model::Catalog::sm_cellColumn);

  result->setName(name);
  result->setUri(uri);
  result->setType(source_type);
  result->setPermission(permission);
  result->setLastEdition(last_editon);
  result->setAuthor(author);
  result->setSource(source);
  result->setDescription(description);
  result->setScaleMin(scale_min);
  result->setScaleMax(scale_max);
  result->setStyle(style);
  result->setLocked(locked);
  result->setCellColumn(cell_column);

  //rules layer params
  boost::property_tree::ptree rulelayerparamss;
  try
  {
    rulelayerparamss = catalog.get_child("rulelayerparamss");
  }
  catch (std::exception const& /*e*/)
  {
    //...
  }

  if (!rulelayerparamss.empty())
  {
    for (boost::property_tree::ptree::const_iterator itrlp = rulelayerparamss.begin(); itrlp != rulelayerparamss.end(); ++itrlp)
    {
      std::string class_column = itrlp->second.get<std::string>(ta::model::RulesLayerParamsMem::sm_classColumn);
      ta::model::RulesLayerParamsMem* rulesLayerParamsMem = new ta::model::RulesLayerParamsMem();
      rulesLayerParamsMem->setClassColumn(class_column);
      result->getRulesLayerParamsMems().push_back(rulesLayerParamsMem);
    }
  }

  //cell layer
  boost::property_tree::ptree cellLayer;
  try
  {
    cellLayer = catalog.get_child("cellLayer");
  }
  catch (std::exception const& /*e*/)
  {
    //...
  }

  if (!cellLayer.empty())
  {
    bool active = cellLayer.get<bool>(ta::model::CellLayerMem::sm_active);
    double resolution = cellLayer.get<double>(ta::model::CellLayerMem::sm_resolution);
    ta::model::CellLayerMem* cellLayerMem = new ta::model::CellLayerMem();
    cellLayerMem->setActive(active);
    cellLayerMem->setResolution(resolution);
    result->setCellLayerMem(cellLayerMem);
  }

  //aoi layer
  boost::property_tree::ptree aoilayers;
  try
  {
    aoilayers = catalog.get_child("aoilayers");
  }
  catch (std::exception const& /*e*/)
  {
    //...
  }

  if (!aoilayers.empty())
  {
    for (boost::property_tree::ptree::const_iterator itaoi = aoilayers.begin(); itaoi != aoilayers.end(); ++itaoi)
    {
      std::string aoiIdColumn = itaoi->second.get<std::string>(ta::model::AoiLayerMem::sm_aoiIdColumn);
      ta::model::AoiLayerMem* aoiLayerMem = new ta::model::AoiLayerMem();
      aoiLayerMem->setAoiIdColumn(aoiIdColumn);
      result->getAoiLayerMems().push_back(aoiLayerMem);
    }
  }

  return result;
}

boost::property_tree::ptree ta::model::qt::ProjectControlWidget::exportCatalogAux(const int& catalogId)
{
  ta::model::Catalog* ct = dynamic_cast<ta::model::Catalog*>(m_catalogFacade->get(catalogId));
  boost::property_tree::ptree catalog;
  catalog.put(ta::model::Catalog::sm_name, ct->getName());
  catalog.put(ta::model::Catalog::sm_uri, ct->getUri());
  catalog.put(ta::model::Catalog::sm_type, ct->getType());
  catalog.put(ta::model::Catalog::sm_permission, ct->getPermission());
  catalog.put(ta::model::Catalog::sm_lastEdition, ct->getLastEdition()->getTimeInstant());
  catalog.put(ta::model::Catalog::sm_author, ct->getAuthor());
  catalog.put(ta::model::Catalog::sm_source, ct->getSource());
  catalog.put(ta::model::Catalog::sm_description, ct->getDescription());
  catalog.put(ta::model::Catalog::sm_scaleMin, ct->getScaleMin());
  catalog.put(ta::model::Catalog::sm_scaleMax, ct->getScaleMax());
  catalog.put(ta::model::Catalog::sm_style, ct->getStyle());
  catalog.put(ta::model::Catalog::sm_locked, ct->isLocked());
  catalog.put(ta::model::Catalog::sm_cellColumn, ct->getCellColumn());

  // rules_layer_params
  std::vector<std::string> classColumns = m_catalogFacade->getClassColumns(ct->getId());
  boost::property_tree::ptree ruleLayerParamsList;
  if (!classColumns.empty()){
    for (int i = 0; i < classColumns.size(); ++i)
    {
      boost::property_tree::ptree ruleLayerParamsItem;
      ruleLayerParamsItem.put(ta::model::CatalogFacade::sm_rulesLayerParams_classColumn, classColumns.at(i));
      ruleLayerParamsList.push_back(std::make_pair("rulelayerparams", ruleLayerParamsItem));
    }
    catalog.add_child("rulelayerparamss", ruleLayerParamsList);
  }

  // cell_layer
  ta::model::CellLayer* cl = m_catalogFacade->getCellLayerData(ct->getId());
  if (cl != NULL)
  {
    boost::property_tree::ptree cellLayer;
    cellLayer.put(ta::model::CellLayer::sm_active, cl->getActive());
    cellLayer.put(ta::model::CellLayer::sm_resolution, cl->getResolution());
    catalog.push_back(std::make_pair("celllayer", cellLayer));
  }

  //aoi_layer
  std::vector<std::string> aoiIdColumns = m_catalogFacade->getAoiIdColumns(ct->getId());
  boost::property_tree::ptree aoiLayerList;
  if (!aoiIdColumns.empty()){
    for (int i = 0; i < aoiIdColumns.size(); ++i)
    {
      boost::property_tree::ptree aoiLayerItem;
      aoiLayerItem.put(ta::model::CatalogFacade::sm_aoiLayer_aoiIdColumn, aoiIdColumns.at(i));
      aoiLayerList.push_back(std::make_pair("aoilayer", aoiLayerItem));
    }
    catalog.add_child("aoilayers", aoiLayerList);
  }

  return catalog;
}

void  ta::model::qt::ProjectControlWidget::onTableCellClicked(int row, int /*column*/)
{
  //check project
  QVariant var = m_ui->m_tableWidget->item(row, 1)->data(Qt::UserRole);

  if (!var.isValid())
  {
    QMessageBox::warning(this, "TerraAmazon", tr("Error getting project information."));
    return;
  }

  int id = var.toInt();

  ta::model::ProjectFacade pf(m_server);

  ta::model::Project* p = dynamic_cast<ta::model::Project*>(pf.get(id));

  QDate createdDate(p->getCreated()->getYear(),
    p->getCreated()->getMonth(),
    p->getCreated()->getDay());

  QDate updatedDate(p->getUpdated()->getYear(),
    p->getUpdated()->getMonth(),
    p->getUpdated()->getDay());

  m_ui->m_createdDateEdit->setDate(createdDate);
  m_ui->m_updatedDateEdit->setDate(updatedDate);

  delete p;
}

void ta::model::qt::ProjectControlWidget::setActiveServer(te::da::DataSourceInfoPtr dsInfo)
{
  //get data source
  m_server = te::da::GetDataSource(dsInfo->getId());

  if (!m_server.get())
    return;

  m_catalogFacade.reset(new ta::model::CatalogFacade(m_server));
  m_ruleFacade.reset(new ta::model::RuleFacade(m_server));
  m_ruleRuleSetFacade.reset(new ta::model::RuleRuleSetFacade(m_server));
  m_rulesLayerColumnFacade.reset(new ta::model::RulesLayerColumnFacade(m_server));
  m_rulesetFacade.reset(new ta::model::RuleSetFacade(m_server));
  m_projectFacade.reset(new ta::model::ProjectFacade(m_server));
  m_classFacade.reset(new ta::model::ClassFacade(m_server));

  listProjects();
}

te::da::DataSourcePtr ta::model::qt::ProjectControlWidget::getServer()
{
  return m_server;
}

void ta::model::qt::ProjectControlWidget::listProjects()
{
  if (!m_server.get())
    return;

  ta::model::ProjectFacade pf(m_server);

  //get projects from selected data source (server)
  std::vector<ta::model::AbstractItem*> projects = pf.get();

  m_ui->m_tableWidget->setRowCount(0);

  for (std::size_t t = 0; t < projects.size(); ++t)
  {
    ta::model::Project* p = dynamic_cast<ta::model::Project*>(projects[t]);

    int id = p->getId();
    std::string name = p->getName();
    int nCells = (int)p->getMaxCells();
    double minArea = 0.;
    bool hasScale = pf.hasScaleDefined(p);
    double scaleMin = p->getMinScale();
    double scaleMax = p->getMaxScale();
    bool isAvailable = p->isActive();

    //fill table
    int row = m_ui->m_tableWidget->rowCount();
    m_ui->m_tableWidget->insertRow(row);

    {
      QWidget* wActions = new QWidget(m_ui->m_tableWidget);
      wActions->setObjectName("PROJ_CONTROLS");

      QToolButton* updateButton = new QToolButton(m_ui->m_tableWidget);
      updateButton->setCheckable(false);
      updateButton->setIcon(QIcon::fromTheme("view-refresh"));

      QToolButton* removeButton = new QToolButton(m_ui->m_tableWidget);
      removeButton->setCheckable(false);
      removeButton->setIcon(QIcon::fromTheme("list-remove"));

      QToolButton* exportButton = new QToolButton(m_ui->m_tableWidget);
      exportButton->setCheckable(false);
      exportButton->setIcon(QIcon::fromTheme("document-export"));

      QGridLayout* layout = new QGridLayout(wActions);
      layout->setContentsMargins(0, 0, 0, 0);

      layout->addWidget(updateButton, 0, 0);
      layout->addWidget(removeButton, 0, 1);
      layout->addWidget(exportButton, 0, 2);

      m_ui->m_tableWidget->setCellWidget(row, 0, wActions);

      m_updateButtonMapper->setMapping(updateButton, row);
      connect(updateButton, SIGNAL(pressed()), m_updateButtonMapper, SLOT(map()));

      m_removeButtonMapper->setMapping(removeButton, row);
      connect(removeButton, SIGNAL(pressed()), m_removeButtonMapper, SLOT(map()));

      m_exportButtonMapper->setMapping(exportButton, row);
      connect(exportButton, SIGNAL(pressed()), m_exportButtonMapper, SLOT(map()));
    }

    //proj name
    {
      QTableWidgetItem* item = new QTableWidgetItem();
      item->setText(name.c_str());
      item->setData(Qt::UserRole, QVariant(id));
      item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsEditable);
      m_ui->m_tableWidget->setItem(row, 1, item);
    }

    //proj cells
    {
      QTableWidgetItem* item = new QTableWidgetItem();
      item->setText(QString::number(nCells));
      item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsEditable);
      m_ui->m_tableWidget->setItem(row, 2, item);
    }

    //proj area min
    {
      QTableWidgetItem* item = new QTableWidgetItem();
      item->setText(QString::number(minArea));
      item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsEditable);
      m_ui->m_tableWidget->setItem(row, 3, item);
    }

    //proj scale min
    {
      QTableWidgetItem* item = new QTableWidgetItem();
      if (hasScale)
        item->setText(QString::number(scaleMin));
      else
        item->setText("");
      item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsEditable);
      m_ui->m_tableWidget->setItem(row, 4, item);
    }

    //proj scale max
    {
      QTableWidgetItem* item = new QTableWidgetItem();
      if (hasScale)
        item->setText(QString::number(scaleMax));
      else
        item->setText("");
      item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsEditable);
      m_ui->m_tableWidget->setItem(row, 5, item);
    }

    //proj available
    {
      QComboBox* cmbBox = new QComboBox(m_ui->m_tableWidget);
      cmbBox->addItem(tr("TRUE"), QVariant(1));
      cmbBox->addItem(tr("FALSE"), QVariant(0));

      if (isAvailable)
        cmbBox->setCurrentIndex(0);
      else
        cmbBox->setCurrentIndex(1);

      m_ui->m_tableWidget->setCellWidget(row, 6, cmbBox);
    }
  }

  m_ui->m_tableWidget->resizeColumnsToContents();
  m_ui->m_tableWidget->horizontalHeader()->setSectionResizeMode(1, QHeaderView::Stretch);
  m_ui->m_tableWidget->horizontalHeader()->setSectionResizeMode(2, QHeaderView::Stretch);
  m_ui->m_tableWidget->horizontalHeader()->setSectionResizeMode(3, QHeaderView::Stretch);
  m_ui->m_tableWidget->horizontalHeader()->setSectionResizeMode(4, QHeaderView::Stretch);
  m_ui->m_tableWidget->horizontalHeader()->setSectionResizeMode(5, QHeaderView::Stretch);
  m_ui->m_tableWidget->horizontalHeader()->setSectionResizeMode(6, QHeaderView::Stretch);

  te::common::FreeContents(projects);
}
