/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
\file terraamazon/ProjectInformationDialog.cpp

\brief A Qt dialog used for project description.
*/

// TerraLib
#include <terralib/common/STLUtils.h>
#include <terralib/common/StringUtils.h>
#include <terralib/dataaccess/utils/Utils.h>
#include <terralib/geometry/GeometryProperty.h>
#include <terralib/srs/SpatialReferenceSystemManager.h>

// TerraAmazon
#include "../../../common/Utils.h"
#include "../../facade/CatalogFacade.h"
#include "../../facade/ClassFacade.h"
#include "../../facade/ProjectFacade.h"
#include "../../facade/RuleFacade.h"
#include "../../facade/RuleSetFacade.h"
#include "../../item/Catalog.h"
#include "../../item/Class.h"
#include "../../item/Project.h"
#include "../../item/Rule.h"
#include "../../item/RuleSet.h"
#include "../../Utils.h"
#include "ProjectInformationDialog.h"
#include "ui_ProjectInformationDialogForm.h"

// Qt
#include <QFont>

ta::model::qt::ProjectInformationDialog::ProjectInformationDialog(QWidget* parent, Qt::WindowFlags f)
  : QDialog(parent, f),
  m_ui(new Ui::ProjectInfoDialogForm)
{
  m_ui->setupUi(this);

  std::string icon = ta::common::FindInTerraAmazonPath("share/icons/svg/terraAmazom-colorido-horizontal.svg");
  m_ui->m_imgLabel->setPixmap(QIcon(icon.c_str()).pixmap(112, 100));
  m_ui->m_imgLabel->hide();

  //connects
  connect(m_ui->m_projectComboBox, SIGNAL(activated(int)), this, SLOT(onProjectComboBoxActivated(int)));

  //setup form
  m_ui->m_projectTreeWidget->header()->setSectionResizeMode(QHeaderView::Stretch);
}

ta::model::qt::ProjectInformationDialog::~ProjectInformationDialog()
{
}

void ta::model::qt::ProjectInformationDialog::setActiveServer(te::da::DataSourceInfoPtr server)
{
  if (server.get())
  {
    m_activeServer = te::da::GetDataSource(server->getId());
  }

  listProjects();
}

void ta::model::qt::ProjectInformationDialog::listProjects()
{
  if (!m_activeServer.get())
    return;

  m_ui->m_projectComboBox->clear();

  ta::model::ProjectFacade pf(m_activeServer);

  //get projects from selected data source (server)
  std::vector<ta::model::AbstractItem*> projects = pf.get();

  for (std::size_t t = 0; t < projects.size(); ++t)
  {
    ta::model::Project* p = dynamic_cast<ta::model::Project*>(projects[t]);

    m_ui->m_projectComboBox->addItem(p->getName().c_str(), QVariant(p->getId()));
  }

  if (!projects.empty())
    onProjectComboBoxActivated(0);

  te::common::FreeContents(projects);
}

std::string ta::model::qt::ProjectInformationDialog::getCatalogProjection(ta::model::Catalog* catalog)
{
  assert(catalog);

  std::string projStr = "";

  ta::model::CatalogFacade cf(m_activeServer);

  std::auto_ptr<te::da::DataSetType> dsType = cf.getDataSetType(catalog);

  te::gm::GeometryProperty* geomProp = te::da::GetFirstGeomProperty(dsType.get());

  if (geomProp)
  {
    int srid = geomProp->getSRID();

    projStr += te::common::Convert2String(srid);
    projStr += " - ";
    projStr += te::srs::SpatialReferenceSystemManager::getInstance().getName(srid);
  }

  return projStr;
}

void ta::model::qt::ProjectInformationDialog::onProjectComboBoxActivated(int index)
{
  m_ui->m_projectTreeWidget->clear();

  int projId = m_ui->m_projectComboBox->itemData(index).toInt();

  ta::model::ProjectFacade pf(m_activeServer);

  std::auto_ptr<ta::model::Project> p(dynamic_cast<ta::model::Project*>(pf.get(projId)));

  //get project informations
  QTreeWidgetItem* projItem = new QTreeWidgetItem(m_ui->m_projectTreeWidget);
  projItem->setText(0, tr("Project"));
  QFont font = projItem->font(0);
  font.setBold(true);
  projItem->setFont(0, font);

  QTreeWidgetItem* projItemName = new QTreeWidgetItem(projItem);
  projItemName->setText(0, tr("Name:"));
  projItemName->setText(1, p->getName().c_str());

  QTreeWidgetItem* projItemCells = new QTreeWidgetItem(projItem);
  projItemCells->setText(0, tr("Cells:"));
  projItemCells->setText(1, QString::number(p->getMaxCells()));

  QTreeWidgetItem* projItemMinScale = new QTreeWidgetItem(projItem);
  projItemMinScale->setText(0, tr("Min Scale:"));
  projItemMinScale->setText(1, QString::number(p->getMinScale()));

  QTreeWidgetItem* projItemMaxScale = new QTreeWidgetItem(projItem);
  projItemMaxScale->setText(0, tr("Max Scale:"));
  projItemMaxScale->setText(1, QString::number(p->getMaxScale()));

  //get rulesets
  QTreeWidgetItem* rsItem = new QTreeWidgetItem(m_ui->m_projectTreeWidget);
  rsItem->setText(0, tr("Output Layers"));
  font = rsItem->font(0);
  font.setBold(true);
  rsItem->setFont(0, font);

  std::vector<int> ruleSets = pf.getRuleSets(p->getId());

  for (std::size_t t = 0; t < ruleSets.size(); ++t)
  {
    ta::model::RuleSetFacade rsf(m_activeServer);
    ta::model::CatalogFacade cf(m_activeServer);

    std::auto_ptr<ta::model::RuleSet> rs(dynamic_cast<ta::model::RuleSet*>(rsf.get(ruleSets[t])));

    QTreeWidgetItem* rItem = new QTreeWidgetItem(rsItem);
    rItem->setText(0, tr("Layer ") + QString::number(t));

    std::auto_ptr<ta::model::Catalog> c(dynamic_cast<ta::model::Catalog*>(cf.get(rs->getCatalogId())));

    QTreeWidgetItem* cName = new QTreeWidgetItem(rItem);
    cName->setText(0, tr("Name:"));
    cName->setText(1, c->getName().c_str());

    QTreeWidgetItem* cCellName = new QTreeWidgetItem(rItem);
    cCellName->setText(0, tr("Cell Column:"));
    cCellName->setText(1, c->getCellColumn().c_str());
    
    //get classes
    QTreeWidgetItem* classesItem = new QTreeWidgetItem(rItem);
    classesItem->setText(0, tr("Classes"));

    std::vector<int> classes = rsf.getRuleSetClasses(c->getId());
    ta::model::ClassFacade classf(m_activeServer);
    for (std::size_t p = 0; p < classes.size(); ++p)
    {
      std::auto_ptr<ta::model::Class> classObj(dynamic_cast<ta::model::Class*>(classf.get(classes[p])));
      
      QPixmap pix(16, 16);
      pix.fill(QColor(classObj->getStyle().c_str()));

      QTreeWidgetItem* className = new QTreeWidgetItem(classesItem);
      className->setText(0, classObj->getName().c_str());
      className->setIcon(1, QIcon(pix));
    }

    //get rules
    QTreeWidgetItem* rulesItem = new QTreeWidgetItem(rItem);
    rulesItem->setText(0, tr("Rules"));

    std::vector<int> rules = rsf.getRulesRuleSet(rs->getId());
    ta::model::RuleFacade rf(m_activeServer);
    for (std::size_t q = 0; q < rules.size(); ++q)
    {
      QTreeWidgetItem* ruleItem = new QTreeWidgetItem(rulesItem);
      ruleItem->setText(0, tr("Rule ") + QString::number(q));

      std::auto_ptr<ta::model::Rule> rule(dynamic_cast<ta::model::Rule*>(rf.get(rules[q])));

      QTreeWidgetItem* ruleName = new QTreeWidgetItem(ruleItem);
      ruleName->setText(0, tr("Name:"));
      ruleName->setText(1, rule->getName().c_str());

      QTreeWidgetItem* ruleOperation = new QTreeWidgetItem(ruleItem);
      std::string ruleType = ta::model::GetSpatialOperationString(rule->getOperationId());
      ruleOperation->setText(0, tr("Operation:"));
      ruleOperation->setText(1, ruleType.c_str());

      QTreeWidgetItem* ruleLayer = new QTreeWidgetItem(ruleItem);
      std::auto_ptr<ta::model::Catalog> c(dynamic_cast<ta::model::Catalog*>(cf.get(rule->getCatalogId())));
      ruleLayer->setText(0, tr("Layer:"));
      ruleLayer->setText(1, c->getName().c_str());

      QTreeWidgetItem* ruleLayerProj = new QTreeWidgetItem(ruleItem);
      
      ruleLayerProj->setText(0, tr("Projection:"));
      ruleLayerProj->setText(1, getCatalogProjection(c.get()).c_str());
    }
  }

  m_ui->m_projectTreeWidget->expandAll();
}
