/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of
    this application.
    */

/*!
  \file terraamazon/RulesControlWidget.cpp

  \brief A Qt widget used to configure the rules control.
  */

// TerraAmazon
#include "../../../common/Utils.h"
#include "../../Enums.h"
#include "../../Globals.h"
#include "../../Utils.h"
#include "../../facade/CatalogFacade.h"
#include "../../facade/ProjectFacade.h"
#include "../../facade/RuleFacade.h"
#include "../../facade/RuleRuleSetFacade.h"
#include "../../facade/RuleSetFacade.h"
#include "../../item/Catalog.h"
#include "../../item/Project.h"
#include "../../item/Rule.h"
#include "../../item/RuleRuleSet.h"
#include "../../item/RuleSet.h"
#include "RulesControlWidget.h"
#include "ui_RulesControlWidgetForm.h"

// TerraLib
#include <terralib/common/STLUtils.h>
#include <terralib/dataaccess/utils/Utils.h>

// Qt
#include <QCheckBox>
#include <QMessageBox>

ta::model::qt::RulesControlWidget::RulesControlWidget(QWidget* parent, Qt::WindowFlags f)
: QWidget(parent, f),
m_ui(new Ui::RulesControlWidgetForm),
m_user("")
{
  m_ui->setupUi(this);

  // add button icons
  m_ui->m_addRuleToolButton->setIcon(QIcon::fromTheme("list-add"));
  m_ui->m_associateLayersToolButton->setIcon(QIcon::fromTheme("check"));
  m_ui->m_associateRulesToolButton->setIcon(QIcon::fromTheme("check"));
  m_ui->m_createOutputLayerToolButton->setIcon(QIcon::fromTheme("list-add"));

  std::string iconDsAmazon = ta::common::FindInTerraAmazonPath("share/icons/svg/terraAmazom-icon.svg");

  m_ui->m_pixLabel->setPixmap(QIcon(iconDsAmazon.c_str()).pixmap(16, 16));



  //create layers double list widget
  m_layersDoubleListWidget = new te::qt::widgets::DoubleListWidget(m_ui->m_layersAssociationWidget);

  m_layersDoubleListWidget->setLeftLabel("Catalog Layers");
  m_layersDoubleListWidget->setRightLabel("Output Layers");
  m_layersDoubleListWidget->setVisibledOrderButtons(false);

  QGridLayout* layerDisplayLayout = new QGridLayout(m_ui->m_layersAssociationWidget);
  layerDisplayLayout->setContentsMargins(0, 0, 0, 0);
  layerDisplayLayout->addWidget(m_layersDoubleListWidget);

  //create rules double list widget
  m_rulesDoubleListWidget = new te::qt::widgets::DoubleListWidget(m_ui->m_rulesAssociationWidget);

  m_rulesDoubleListWidget->setLeftLabel("Available Rules");
  m_rulesDoubleListWidget->setRightLabel("Associated Rules");

  QGridLayout* rulesDisplayLayout = new QGridLayout(m_ui->m_rulesAssociationWidget);
  rulesDisplayLayout->setContentsMargins(0, 0, 0, 0);
  rulesDisplayLayout->addWidget(m_rulesDoubleListWidget);

  //mapper initializer
  m_removeMapper = new QSignalMapper(this);

  //connects
  connect(m_ui->m_addRuleToolButton, SIGNAL(pressed()), this, SLOT(onAddRuleToolButtonPressed()));
  connect(m_ui->m_associateLayersToolButton, SIGNAL(pressed()), this, SLOT(onAssociateLayersToolButtonPressed()));
  connect(m_ui->m_associateRulesToolButton, SIGNAL(pressed()), this, SLOT(onAssociateRulesToolButtonPressed()));

  connect(m_ui->m_catalogLayerComboBox, SIGNAL(activated(int)), this, SLOT(onCatalogLayerComboBoxActivated(int)));
  connect(m_ui->m_projectsComboBox, SIGNAL(activated(int)), this, SLOT(onProjectsComboBoxActivated(int)));
  connect(m_ui->m_projectRulesComboBox, SIGNAL(activated(int)), this, SLOT(onProjectRulesComboBoxActivated(int)));
  connect(m_ui->m_projectLayersComboBox, SIGNAL(activated(int)), this, SLOT(onProjectLayersComboBoxActivated(int)));

  connect(m_ui->m_catalogLayerComboBox, SIGNAL(activated(int)), this, SLOT(setRuleName()));
  connect(m_ui->m_operationComboBox, SIGNAL(activated(int)), this, SLOT(setRuleName()));

  connect(m_removeMapper, SIGNAL(mapped(int)), this, SLOT(onRemoveRuleExecuted(int)));

  connect(m_ui->m_createOutputLayerToolButton, SIGNAL(pressed()), this, SLOT(onCreateLayerToolButtonPressed()));


  //set operations
  setOperations();
}

ta::model::qt::RulesControlWidget::~RulesControlWidget()
{
}

void ta::model::qt::RulesControlWidget::setActiveServer(te::da::DataSourceInfoPtr dsInfo)
{
  m_server = te::da::GetDataSource(dsInfo->getId());

  //list catalog layers
  listCatalogs();

  setRuleName();

  //list rules
  listRules();

  //list projects
  listProjects();
}

void ta::model::qt::RulesControlWidget::setUser(const std::string& user)
{
  m_user = user;
}

void ta::model::qt::RulesControlWidget::onAddRuleToolButtonPressed()
{
  if (m_ui->m_ruleNameLineEdit->text().isEmpty())
  {
    QMessageBox::warning(this, this->windowTitle(), tr("Rule name not defined."));
    return;
  }

  int catalogId = m_ui->m_catalogLayerComboBox->currentData().toInt();
  int soId = m_ui->m_operationComboBox->currentData().toInt();
  std::string ruleName = m_ui->m_ruleNameLineEdit->text().toStdString();

  ta::model::RuleFacade rf(m_server);

  ta::model::Rule r;
  r.setName(ruleName);
  r.setCatalogId(catalogId);
  r.setOperationId(soId);

  if (!rf.save(&r))
  {
    QMessageBox::warning(this, windowTitle(), rf.getErrorMessage().c_str());
    return;
  }

  //associate cell column
  ta::model::CatalogFacade cf(m_server);
  ta::model::Catalog* c = dynamic_cast<ta::model::Catalog*>(cf.get(catalogId));

  if (c && c->getCellColumn().empty())
  {
    c->setCellColumn(m_ui->m_cellColumnComboBox->currentText().toStdString());

    if (!cf.update(c))
    {
      QMessageBox::warning(this, this->windowTitle(), cf.getErrorMessage().c_str());
      return;
    }

    m_ui->m_cellColumnComboBox->setEnabled(false);
  }

  listRules();

  setRuleName();

  onProjectLayersComboBoxActivated(m_ui->m_projectLayersComboBox->currentIndex());

  QMessageBox::information(this, windowTitle(), tr("Rule added."));
}

void ta::model::qt::RulesControlWidget::onAssociateLayersToolButtonPressed()
{
  std::vector<int> layers = m_layersDoubleListWidget->getOutputDataValues();

  //check if layer was removed
  int projId = m_ui->m_projectsComboBox->currentData().toInt();

  ta::model::ProjectFacade pf(m_server);
  std::vector<int> catalogIds = pf.getOutputCatalogs(projId);

  for (std::size_t t = 0; t < catalogIds.size(); ++t)
  {
    int curCatalogId = catalogIds[t];

    bool found = false;
    //check if still in layers list
    for (std::size_t p = 0; p < layers.size(); ++p)
    {
      if (curCatalogId == layers[p])
      {
        found = true;
        break;
      }
    }

    if (!found)
    {
      int ruleSetId;

      ta::model::RuleSetFacade rsf(m_server);

      if (rsf.getRuleSetId(curCatalogId, ruleSetId))
        pf.removeAssociateRuleSet(projId, ruleSetId);
    }
  }

  //check if layer was added
  catalogIds = pf.getOutputCatalogs(projId);

  for (std::size_t p = 0; p < layers.size(); ++p)
  {
    int curCatalogId = layers[p];

    bool found = false;
    //check if already associated with project
    for (std::size_t t = 0; t < catalogIds.size(); ++t)
    {
      if (curCatalogId == catalogIds[t])
      {
        found = true;
        break;
      }
    }

    if (!found)
    {
      //create ruleSet
      ta::model::RuleSetFacade rsf(m_server);

      if (!rsf.hasCatalog(curCatalogId))
      {
        ta::model::RuleSet rs;
        rs.setCatalogId(curCatalogId);
        rs.setName(rsf.createRuleSetName(curCatalogId));

        if (!rsf.save(&rs))
        {
          QMessageBox::warning(this, windowTitle(), rsf.getErrorMessage().c_str());
          return;
        }

        //create clean rule
        if (!rsf.createOutputRule(rs.getId(), curCatalogId, ta::model::SO_CLEAN))
        {
          QMessageBox::warning(this, windowTitle(), rsf.getErrorMessage().c_str());
          return;
        }
      }

      int ruleSetId;
      if (rsf.getRuleSetId(curCatalogId, ruleSetId))
      {
        //create project ruleset association
        if (!pf.addAssociateRuleSet(projId, ruleSetId))
        {
          QMessageBox::warning(this, windowTitle(), pf.getErrorMessage().c_str());
          return;
        }
      }
    }
  }

  onProjectsComboBoxActivated(m_ui->m_projectsComboBox->currentIndex());
  onProjectRulesComboBoxActivated(m_ui->m_projectRulesComboBox->currentIndex());


  QMessageBox::information(this, windowTitle(), tr("Layers Associated."));
}

void ta::model::qt::RulesControlWidget::onAssociateRulesToolButtonPressed()
{
  int catalogId = m_ui->m_projectLayersComboBox->currentData().toInt();

  //get ruleset
  ta::model::RuleSetFacade rsf(m_server);
  int ruleSetId;

  if (!rsf.getRuleSetId(catalogId, ruleSetId))
  {
    QMessageBox::warning(this, windowTitle(), tr("Rule Set not found."));
    return;
  }

  //get rules ids associated
  std::vector<int> outputRules = m_rulesDoubleListWidget->getOutputDataValues();

  //clear associated rules
  ta::model::RuleRuleSetFacade rrsf(m_server);

  std::vector<int> rules = rrsf.getRules(ruleSetId);

  for (std::size_t t = 0; t < rules.size(); ++t)
  {
    int curRule = rules[t];

    if (!rsf.removeAssociation(curRule, ruleSetId))
    {
      QMessageBox::warning(this, windowTitle(), rsf.getErrorMessage().c_str());
      return;
    }
  }

  //associated rules
  for (std::size_t p = 0; p < outputRules.size(); ++p)
  {
    int curRule = outputRules[p];

    if (!rsf.createAssociation(curRule, ruleSetId))
    {
      QMessageBox::warning(this, windowTitle(), rsf.getErrorMessage().c_str());
      return;
    }
  }

  onProjectLayersComboBoxActivated(m_ui->m_projectLayersComboBox->currentIndex());

  listRules();

  QMessageBox::information(this, windowTitle(), tr("Rules Associated."));
}

void ta::model::qt::RulesControlWidget::onCatalogLayerComboBoxActivated(int /*index*/)
{
  int catalogId = m_ui->m_catalogLayerComboBox->currentData().toInt();

  ta::model::CatalogFacade cf(m_server);

  ta::model::Catalog* catalog = dynamic_cast<ta::model::Catalog*>(cf.get(catalogId));

  std::auto_ptr<te::da::DataSetType> dsType = cf.getDataSetType(catalog);

  if (!dsType.get())
    return;

  //get properties names
  QStringList propNamesList;
  for (std::size_t t = 0; t < dsType->getProperties().size(); ++t)
    propNamesList.append(dsType->getProperties()[t]->getName().c_str());

  m_ui->m_cellColumnComboBox->clear();
  m_ui->m_cellColumnComboBox->addItems(propNamesList);

  //disable cell column combo box
  if (!catalog->getCellColumn().empty())
  {
    m_ui->m_cellColumnComboBox->setCurrentText(catalog->getCellColumn().c_str());
    m_ui->m_cellColumnComboBox->setEnabled(false);
  }
  else
  {
    m_ui->m_cellColumnComboBox->setEnabled(true);
  }

  delete catalog;
}

void ta::model::qt::RulesControlWidget::onProjectsComboBoxActivated(int index)
{
  int projId = m_ui->m_projectsComboBox->itemData(index).toInt();

  ta::model::CatalogFacade cf(m_server);

  //get project output layers
  ta::model::ProjectFacade pf(m_server);

  std::vector<int> catalogIds = pf.getOutputCatalogs(projId);

  std::vector<std::string> outputLayers;
  std::vector<int> outputLayersIds;

  for (std::size_t t = 0; t < catalogIds.size(); ++t)
  {
    ta::model::Catalog* catalog = dynamic_cast<ta::model::Catalog*>(cf.get(catalogIds[t]));

    outputLayers.push_back(catalog->getName());
    outputLayersIds.push_back(catalog->getId());

    delete catalog;
  }

  //get catalog layers
  std::vector<ta::model::AbstractItem*> items = cf.get();

  std::vector<std::string> inputLayers;
  std::vector<int> inputLayersIds;

  for (std::size_t t = 0; t < items.size(); ++t)
  {
    ta::model::Catalog* catalog = dynamic_cast<ta::model::Catalog*>(items[t]);

    std::string catalogName = catalog->getName();

    if (!pf.hasCatalog(projId, catalog->getId()))
    {
      inputLayers.push_back(catalog->getName());
      inputLayersIds.push_back(catalog->getId());
    }
  }

  m_layersDoubleListWidget->setInputDataValues(inputLayers, inputLayersIds);
  m_layersDoubleListWidget->setOutputDataValues(outputLayers, outputLayersIds);

  te::common::FreeContents(items);
  items.clear();
}

void ta::model::qt::RulesControlWidget::onProjectRulesComboBoxActivated(int index)
{
  int projId = m_ui->m_projectRulesComboBox->itemData(index).toInt();

  //get project output layers
  ta::model::ProjectFacade pf(m_server);
  ta::model::CatalogFacade cf(m_server);

  std::vector<int> catalogIds = pf.getOutputCatalogs(projId);

  m_ui->m_projectLayersComboBox->clear();

  for (std::size_t t = 0; t < catalogIds.size(); ++t)
  {
    ta::model::Catalog* catalog = dynamic_cast<ta::model::Catalog*>(cf.get(catalogIds[t]));

    m_ui->m_projectLayersComboBox->addItem(catalog->getName().c_str(), QVariant(catalog->getId()));

    delete catalog;
  }

  if (catalogIds.empty())
  {
    m_rulesDoubleListWidget->clearInputValues();
    m_rulesDoubleListWidget->clearOutputValues();
  }
  else
  {
    onProjectLayersComboBoxActivated(0);
  }
}

void ta::model::qt::RulesControlWidget::onProjectLayersComboBoxActivated(int index)
{
  m_rulesDoubleListWidget->clearInputValues();
  m_rulesDoubleListWidget->clearOutputValues();

  int layerId = m_ui->m_projectLayersComboBox->itemData(index).toInt();

  ta::model::RuleRuleSetFacade rrsf(m_server);

  //get rules associate with this project
  ta::model::RuleSetFacade rsf(m_server);
  int ruleSetId;

  if (!rsf.getRuleSetId(layerId, ruleSetId))
  {
    return;
  }

  ta::model::RuleFacade rf(m_server);

  std::vector<int> layerRules = rrsf.getRules(ruleSetId);
  std::vector<std::string> layerRulesNames;
  std::vector<int> layerRulesIds;

  for (std::size_t t = 0; t < layerRules.size(); ++t)
  {
    ta::model::Rule* rule = dynamic_cast<ta::model::Rule*>(rf.get(layerRules[t]));

    layerRulesNames.push_back(rule->getName());
    layerRulesIds.push_back(rule->getId());

    delete rule;
  }

  //get all rules not associated with this project
  std::vector<ta::model::AbstractItem*> rulesItems = rf.get();

  std::vector<std::string> rulesNames;
  std::vector<int> rulesIds;

  for (std::size_t t = 0; t < rulesItems.size(); ++t)
  {
    ta::model::Rule* rule = dynamic_cast<ta::model::Rule*>(rulesItems[t]);

    bool hasRule = false;

    for (std::size_t p = 0; p < layerRules.size(); ++p)
    {
      if (layerRules[p] == rule->getId())
      {
        hasRule = true;
        break;
      }
    }

    if (!hasRule)
    {
      rulesNames.push_back(rule->getName());
      rulesIds.push_back(rule->getId());
    }
  }

  //fill double list widget
  m_rulesDoubleListWidget->setInputDataValues(rulesNames, rulesIds);
  m_rulesDoubleListWidget->setOutputDataValues(layerRulesNames, layerRulesIds);

  te::common::FreeContents(rulesItems);
  rulesItems.clear();
}

void ta::model::qt::RulesControlWidget::onRemoveRuleExecuted(int row)
{
  QVariant var = m_ui->m_ruleTableWidget->item(row, 4)->data(Qt::UserRole);

  if (var.isValid())
  {
    int ruleId = var.toInt();

    ta::model::RuleFacade rf(m_server);

    if (!rf.remove(ruleId))
    {
      QMessageBox::warning(this, windowTitle(), rf.getErrorMessage().c_str());
    }

    QMessageBox::information(this, windowTitle(), tr("Rule removed."));
  }

  listRules();

  onProjectLayersComboBoxActivated(m_ui->m_projectLayersComboBox->currentIndex());
}

void ta::model::qt::RulesControlWidget::setRuleName()
{
  int catalogId = m_ui->m_catalogLayerComboBox->currentData().toInt();
  int soId = m_ui->m_operationComboBox->currentData().toInt();

  ta::model::RuleFacade rf(m_server);

  std::string ruleName = rf.createRuleName(catalogId, soId);

  m_ui->m_ruleNameLineEdit->setText(ruleName.c_str());
}

void ta::model::qt::RulesControlWidget::onCreateLayerToolButtonPressed()
{
  if (m_ui->m_createOutputLayerLineEdit->text().isEmpty())
  {
    QMessageBox::warning(this, windowTitle(), tr("Output layer name not defined."));
    return;
  }

  //check name if is valid
  ta::model::CatalogFacade cf(m_server);

  //create new layer
  int newLayerId = 0;
  if (!cf.createAmazonLayer(m_ui->m_createOutputLayerLineEdit->text().toStdString(), m_user, newLayerId))
  {
    QMessageBox::warning(this, windowTitle(), tr("Error creating Amazon Layer."));
    return;
  }

  m_ui->m_createOutputLayerLineEdit->clear();

  //reload list os catalog layers
  listCatalogs();

  //list projects
  listProjects();

  if (newLayerId > 0)
  {
    QComboBox* comb = m_ui->m_catalogLayerComboBox;
    int numLayers = comb->count();
    for (int t = 0; t < numLayers; ++t)
    {
      if (comb->itemData(t, Qt::UserRole).toInt() == newLayerId){
        comb->setCurrentIndex(t);
        break;
      }
    }
  }
}

void ta::model::qt::RulesControlWidget::setOperations()
{
  m_ui->m_operationComboBox->clear();

  m_ui->m_operationComboBox->addItem((ta::model::Globals::sm_soDifference).c_str(), QVariant(ta::model::SO_DIFFERENCE));
  m_ui->m_operationComboBox->addItem((ta::model::Globals::sm_soIntersection).c_str(), QVariant(ta::model::SO_INTERSECTION));
  m_ui->m_operationComboBox->addItem((ta::model::Globals::sm_soUnion).c_str(), QVariant(ta::model::SO_UNION));
}

void ta::model::qt::RulesControlWidget::listCatalogs()
{
  ta::model::CatalogFacade cf(m_server);

  std::vector<ta::model::AbstractItem*> items = cf.get();

  //fill combo with catalog items
  m_ui->m_catalogLayerComboBox->clear();

  for (std::size_t t = 0; t < items.size(); ++t)
  {
    ta::model::Catalog* catalog = dynamic_cast<ta::model::Catalog*>(items[t]);

    if (catalog)
    {
      m_ui->m_catalogLayerComboBox->addItem(catalog->getName().c_str(), QVariant(catalog->getId()));
    }
  }

  if (!items.empty())
    onCatalogLayerComboBoxActivated(0);

  te::common::FreeContents(items);
  items.clear();
}

void ta::model::qt::RulesControlWidget::listProjects()
{
  ta::model::ProjectFacade pf(m_server);

  std::vector<ta::model::AbstractItem*> projs = pf.get();

  m_ui->m_projectsComboBox->clear();
  m_ui->m_projectRulesComboBox->clear();

  for (std::size_t t = 0; t < projs.size(); ++t)
  {
    ta::model::Project* p = dynamic_cast<ta::model::Project*>(projs[t]);

    m_ui->m_projectsComboBox->addItem(p->getName().c_str(), QVariant(p->getId()));
    m_ui->m_projectRulesComboBox->addItem(p->getName().c_str(), QVariant(p->getId()));
  }

  if (!projs.empty())
  {
    onProjectsComboBoxActivated(0);
    onProjectRulesComboBoxActivated(0);
  }

  te::common::FreeContents(projs);
  projs.clear();
}

void ta::model::qt::RulesControlWidget::listRules()
{
  m_ui->m_ruleTableWidget->setRowCount(0);

  ta::model::RuleFacade rf(m_server);
  std::vector<ta::model::AbstractItem*> items = rf.get();

  ta::model::CatalogFacade cf(m_server);
  ta::model::RuleRuleSetFacade rrsf(m_server);

  for (std::size_t t = 0; t < items.size(); ++t)
  {
    ta::model::Rule* r = dynamic_cast<ta::model::Rule*>(items[t]);

    ta::model::Catalog* catalog = dynamic_cast<ta::model::Catalog*>(cf.get(r->getCatalogId()));

    std::string ruleName = r->getName();
    std::string catalogName = catalog->getName();
    std::string ruleType = ta::model::GetSpatialOperationString(r->getOperationId());
    std::string cellColumn = catalog->getCellColumn();

    bool hasRuleDependecy = rrsf.hasRuleDependecy(r->getId());

    //fill table
    int row = m_ui->m_ruleTableWidget->rowCount();
    m_ui->m_ruleTableWidget->insertRow(row);

    //button remove
    {
      QToolButton* removeToolButton = new QToolButton(m_ui->m_ruleTableWidget);
      removeToolButton->setEnabled(!hasRuleDependecy);
      removeToolButton->setIcon(QIcon::fromTheme("list-remove"));
      m_ui->m_ruleTableWidget->setCellWidget(row, 0, removeToolButton);

      m_removeMapper->setMapping(removeToolButton, row);
      connect(removeToolButton, SIGNAL(pressed()), m_removeMapper, SLOT(map()));
    }

    //layer name
    {
      QTableWidgetItem* item = new QTableWidgetItem();
      item->setText(catalogName.c_str());
      item->setFlags(Qt::ItemIsEnabled);
      m_ui->m_ruleTableWidget->setItem(row, 1, item);
    }

    //rule type
    {
      QTableWidgetItem* item = new QTableWidgetItem();
      item->setText(ruleType.c_str());
      item->setFlags(Qt::ItemIsEnabled);
      m_ui->m_ruleTableWidget->setItem(row, 2, item);
    }

    //cell column
    {
      QTableWidgetItem* item = new QTableWidgetItem();
      item->setText(cellColumn.c_str());
      item->setFlags(Qt::ItemIsEnabled);
      m_ui->m_ruleTableWidget->setItem(row, 3, item);
    }

    //rule name
    {
      QTableWidgetItem* item = new QTableWidgetItem();
      item->setText(ruleName.c_str());
      item->setFlags(Qt::ItemIsEnabled);
      item->setData(Qt::UserRole, QVariant(r->getId()));
      m_ui->m_ruleTableWidget->setItem(row, 4, item);
    }

    delete catalog;
  }

  m_ui->m_ruleTableWidget->resizeColumnsToContents();
  m_ui->m_ruleTableWidget->horizontalHeader()->setSectionResizeMode(4, QHeaderView::Stretch);

  te::common::FreeContents(items);
  items.clear();
}
