/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
 * \file terralib/qt/af/XMLFormatter.h
 *
 * \brief A class for xml serialization formatting strings.
 */

#ifndef __TERRAAMAZON_WORKSPACE_XMLFORMATTER_H
#define __TERRAAMAZON_WORKSPACE_XMLFORMATTER_H

#include <terralib/BuildConfig.h>

#include <terralib/maptools/AbstractLayer.h>

//STL
#include <list>
#include <string>

namespace te {

  //Forward declarations
  namespace da
  {
    class DataSourceInfo;
  }
}

namespace ta
{
  namespace workspace
  {
    struct WorkspaceMetadata;

    /*!
    \class XMLFormatter

    \brief A class that formats strings.

    Use this class for changing strings, contained in some TerraLib objects, to XML percentage encode format and/or to human readable format. Use XML formatting before serializing the object to a XML file, for example, and human readable format after read the object from a XML.

    \ingroup af
    */
    class XMLFormatter
    {
    public:

      /*!
      \brief Formats the project informations.

      \param p The project.

      \param encode \a Pass true to change for XML percentage format and \a false for human readable format.
      */
      static void format(WorkspaceMetadata* p, const std::list<te::map::AbstractLayerPtr>& layers, const bool& encode);

      /*!
      \brief Formats the layer informations.

      \param l The layer.

      \param encode \a Pass true to change for XML percentage format and \a false for human readable format.

      \note The sublayers will be formatted too, using the \a encode passed.
      */
      static void format(te::map::AbstractLayer* l, const bool& encode);

      /*!
      \brief Formats the data source informations.

      \param d The data source info.

      \param encode \a Pass true to change for XML percentage format and \a false for human readable format.
      */
      static void format(te::da::DataSourceInfo* d, const bool& encode);

      /*!
      \brief Formats all data source informations registered in the \a te::da::DataSourceInfoManager object.

      \param encode \a Pass true to change for XML percentage format and \a false for human readable format.

      \sa te::da::DataSourceInfoManager
      */
      static void formatDataSourceInfos(const bool& encode);

      /*!
      \brief Formats the string.

      \param s The string.

      \param encode \a Pass true to change for XML percentage format and \a false for human readable format.
      */
      static std::string format(const std::string& s, const bool& encode);
    };
  } // end namespace workspace
}   // end namespace ta

#endif // __TERRAAMAZON_WORKSPACE_XMLFORMATTER_H
