/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */
 
/*!
\file terraamazon/ServerLoginDialog.cpp

\brief A Qt dialog used to configure which servers the user is connected.
*/

// TerraLib
#include <terralib/common/PlatformUtils.h>
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>
#include <terralib/dataaccess/datasource/DataSourceFactory.h>

// TerraAmazon
#include "../../common/Utils.h"
#include "../../model/facade/UserFacade.h"
#include "../../model/Globals.h"
#include "../../model/item/User.h"
#include "../../model/Utils.h"
#include "ServerLoginDialog.h"
#include "ServerRegisterDialog.h"
#include "ui_ServerLoginDialogForm.h"

//Qt
#include <QMessageBox>

Q_DECLARE_METATYPE(te::da::DataSourceInfoPtr);

ta::workspace::qt::ServerLoginDialog::ServerLoginDialog(QWidget* parent, Qt::WindowFlags f)
  : QDialog(parent, f),
  m_ui(new Ui::ServerLoginDialogForm)
{
  m_ui->setupUi(this);
  
  m_userType = ta::common::TA_GIS_USER;
  std::string icon = ta::common::FindInTerraAmazonPath("share/icons/svg/terraAmazom-colorido-horizontal.svg");  
  m_ui->m_imgLabel->setPixmap(QIcon(icon.c_str()).pixmap(112, 100));
  m_ui->m_imgLabel->hide();

  m_ui->m_passLineEdit->setEchoMode(QLineEdit::Password);
  m_ui->m_userLineEdit->setFocus();
  m_ui->m_newPassLineEdit->setEchoMode(QLineEdit::Password);
  m_ui->m_confirmPassLineEdit->setEchoMode(QLineEdit::Password);
  m_ui->m_passGroupBox->hide();

  // add button icons
  m_ui->m_addServerToolButton->setIcon(QIcon::fromTheme("datasource"));

  connect(m_ui->m_okPushButton, SIGNAL(clicked()), this, SLOT(onOkPushButtonClicked()));
  connect(m_ui->m_addServerToolButton, SIGNAL(clicked(bool)), this, SLOT(onAddToolButtonClicked(bool)));
}

ta::workspace::qt::ServerLoginDialog::~ServerLoginDialog()
{
}

te::da::DataSourceInfoPtr ta::workspace::qt::ServerLoginDialog::getActiveServer()
{
  return m_activeServer;
}

int ta::workspace::qt::ServerLoginDialog::getActiveUserId()
{
  if (m_activeServer)
  {
    if (m_activeUser)
      return m_activeUser->getId();
  }
  return -1;
}

void ta::workspace::qt::ServerLoginDialog::setActiveServer(te::da::DataSourceInfoPtr server)
{
  if (server.get())
  {
    m_activeServer = server;

    te::da::DataSourcePtr ds =  te::da::DataSourceManager::getInstance().get(server->getId(), server->getType(), server->getConnInfo());

    if(!ds->isOpened())
      ds->open();

    m_facade.reset(new ta::model::UserFacade(ds));
  }
}

bool ta::workspace::qt::ServerLoginDialog::login(std::unique_ptr<ta::model::User> user)
{
  bool tst1, tst2, tst3, tst4;
  tst1 = m_ui->m_passLineEdit->text().isEmpty();
  tst2 = m_ui->m_passLineEdit->text().toStdString() == ta::model::Globals::sm_taDefaultPassword;
  tst3 = user->getPassword() == sha256(ta::model::Globals::sm_taDefaultPassword);
  tst4 = user->getPassword() == sha256(m_ui->m_passLineEdit->text().toStdString());

  if (tst1)
  {
    QMessageBox::warning(this, windowTitle(), tr("A password must be provided."));
    return false;
  }
  else if (tst2 && tst3)
  {
    QMessageBox::warning(this, windowTitle(), tr("The password must be changed, it can not match the default password."));
    m_ui->m_passGroupBox->show();
    return false;
  }
  else if (!m_ui->m_passGroupBox->isVisible() && !tst4)
  {
    QMessageBox::warning(this, windowTitle(), tr("The password is incorrect."));
    return false;
  }

  if (m_ui->m_passGroupBox->isVisible())
  {
    if (!m_ui->m_newPassLineEdit->text().isEmpty() && (m_ui->m_newPassLineEdit->text() == m_ui->m_confirmPassLineEdit->text()))
      user->setPassword(sha256(m_ui->m_newPassLineEdit->text().toStdString()));
    else
    {
      QMessageBox::warning(this, windowTitle(), tr("The password and it's confirmation provided do not match."));
      return false;
    }

    if (m_ui->m_emaiLlineEdit->text().isEmpty())
    {
      QMessageBox::warning(this, windowTitle(), tr("An e-mail must be provided."));
      return false;
    }
    else
      user->setEMail(m_ui->m_emaiLlineEdit->text().toStdString());

    if (!m_facade->update(user.get()))
    {
      QMessageBox::warning(this, windowTitle(), m_facade->getErrorMessage().c_str());
      return false;
    }
  }

  m_ui->m_passGroupBox->hide();
  m_activeUser = std::move(user);
  return true;
}

std::vector<te::da::DataSourceInfoPtr> ta::workspace::qt::ServerLoginDialog::getServers()
{
  std::vector<te::da::DataSourceInfoPtr> infos;

  for (int i = 0; i < m_ui->m_serverComboBox->count(); ++i)
  {
    QVariant v = m_ui->m_serverComboBox->itemData(i, Qt::UserRole);
    infos.push_back(v.value<te::da::DataSourceInfoPtr>());
  }

  return infos;
}

void ta::workspace::qt::ServerLoginDialog::setServers(std::vector<te::da::DataSourceInfoPtr> servers)
{
  m_ui->m_serverComboBox->clear();

  for (std::size_t i = 0; i < servers.size(); ++i)
  {
    te::da::DataSourceInfoPtr ds = servers[i];
    m_ui->m_serverComboBox->addItem(QString::fromStdString(ds->getTitle()), QVariant::fromValue(ds));
  }
}

void ta::workspace::qt::ServerLoginDialog::setUserType(ta::common::TaUserType userType)
{
  m_userType = userType;
}

void ta::workspace::qt::ServerLoginDialog::onAddToolButtonClicked(bool)
{
  ta::workspace::qt::ServerRegisterDialog dlg;

  if (dlg.exec() == QDialog::Accepted)
  {
    te::da::DataSourceInfoPtr server = dlg.getServer();

    if (server.get())
    {
      bool newServer = te::da::DataSourceInfoManager::getInstance().add(server);
      if (newServer)
      {
        te::da::DataSourcePtr ds = te::da::DataSourceFactory::make(server->getType());
        ds->setId(server->getId());
        ds->setConnectionInfo(server->getConnInfo());
        te::da::DataSourceManager::getInstance().insert(ds);

        m_ui->m_serverComboBox->addItem(QString::fromStdString(server->getTitle()), QVariant::fromValue(server));
        m_ui->m_serverComboBox->setCurrentIndex(m_ui->m_serverComboBox->count() -1);

        m_activeUser = std::move(dlg.getActiveUser());
        if (m_activeUser.get())
        {
          m_ui->m_userLineEdit->setText(QString::fromStdString(m_activeUser->getName()));
          m_ui->m_passLineEdit->setText(QString::fromStdString(m_activeUser->getPassword()));
        }
      }
      else
        QMessageBox::information(this, windowTitle(), tr("The selected server already exists."));
    }
  }
}

void ta::workspace::qt::ServerLoginDialog::onOkPushButtonClicked()
{
  te::da::DataSourceInfoPtr info;
  QVariant v = m_ui->m_serverComboBox->currentData();
  info = v.value<te::da::DataSourceInfoPtr>();
  if (info.get())
  {
    std::map<std::string, std::string> connInfo = info->getConnInfo();
    std::string username = m_ui->m_userLineEdit->text().toStdString();
    std::string pass = sha256(m_ui->m_passLineEdit->text().toStdString());

    //Using the provided userName and password on the connection info
    info->setConnInfo(connInfo);
    setActiveServer(info);

    std::unique_ptr<ta::model::User> taUser = m_facade->getByName(username);

    if (taUser.get())
    {
      if (login(std::move(taUser)))
      {
        this->accept();
      }
    }
    else
      QMessageBox::information(this, windowTitle(), tr("Server activation failed, username is incorrect."));
  }
  else
    QMessageBox::information(this, windowTitle(), tr("Server activation failed, no valid server selected."));
}


