/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */
 
 /*!
\file terraamazon/ServerManagerrDialog.cpp

\brief A Qt dialog used to configure which servers the user is connected to and which is the active server.
*/

// TerraLib
#include <terralib/common/PlatformUtils.h>
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceManager.h>
#include <terralib/dataaccess/datasource/DataSourceFactory.h>

// TerraAmazon
#include "../../common/Utils.h"
#include "../../model/facade/UserFacade.h"
#include "../../model/Globals.h"
#include "../../model/item/User.h"
#include "ServerManagerDialog.h"
#include "ServerRegisterDialog.h"
#include "ui_ServerManagerDialogForm.h"

//Qt
#include <QMessageBox>

Q_DECLARE_METATYPE(te::da::DataSourceInfoPtr);

ta::workspace::qt::ServerManagerDialog::ServerManagerDialog(QWidget* parent, Qt::WindowFlags f)
  : QDialog(parent, f),
  m_ui(new Ui::ServerManagerDialogForm),
  m_hasChanges(false)
{
  m_ui->setupUi(this);

  std::string icon = ta::common::FindInTerraAmazonPath("share/icons/svg/terraAmazom-colorido-horizontal.svg");  
  m_ui->m_imgLabel->setPixmap(QIcon(icon.c_str()).pixmap(112, 100));
  m_ui->m_imgLabel->hide();

  m_ui->m_newPassLineEdit->setEchoMode(QLineEdit::Password);
  m_ui->m_confirmPassLineEdit->setEchoMode(QLineEdit::Password);

  //temporally disabling an unused widgegt
  m_ui->tabWidget->setTabEnabled(1, false);

  // add button icons
  m_ui->m_addToolButton->setIcon(QIcon::fromTheme("list-add"));
  m_ui->m_removeToolButton->setIcon(QIcon::fromTheme("list-remove"));
  m_ui->m_updateToolButton->setIcon(QIcon::fromTheme("preferences-system"));
  m_ui->m_removeToolButton->setIcon(QIcon::fromTheme("list-remove"));

  connect(m_ui->m_okPushButton, SIGNAL(clicked()), this, SLOT(onOkPushButtonClicked()));
  connect(m_ui->m_addToolButton, SIGNAL(clicked(bool)), this, SLOT(onAddToolButtonClicked(bool)));
  connect(m_ui->m_updateToolButton, SIGNAL(clicked(bool)), this, SLOT(onUpdateToolButtonClicked(bool)));
  connect(m_ui->m_removeToolButton, SIGNAL(clicked(bool)), this, SLOT(onRemoveToolButtonClicked(bool)));
}

ta::workspace::qt::ServerManagerDialog::~ServerManagerDialog()
{
}

te::da::DataSourceInfoPtr ta::workspace::qt::ServerManagerDialog::getActiveServer()
{
  return m_activeServer;
}

void ta::workspace::qt::ServerManagerDialog::setActiveServer(te::da::DataSourceInfoPtr server)
{
  if (server.get())
  {
    m_activeServer = server;

    te::da::DataSourcePtr ds =  te::da::DataSourceManager::getInstance().get(server->getId(), server->getType(), server->getConnInfo());

    if(!ds->isOpened())
      ds->open();

    m_ui->m_serverLineEdit->setText(QString::fromStdString(m_activeServer->getTitle()));
    m_facade.reset(new ta::model::UserFacade(ds));
  }
}

std::vector<te::da::DataSourceInfoPtr> ta::workspace::qt::ServerManagerDialog::getServers()
{
  std::vector<te::da::DataSourceInfoPtr> infos;

  for (int i = 0; i < m_ui->m_serversListWidget->count(); ++i)
  {
    QListWidgetItem* item = m_ui->m_serversListWidget->item(i);

    QVariant v = item->data(Qt::UserRole);
    infos.push_back(v.value<te::da::DataSourceInfoPtr>());
  }

  return infos;
}

void ta::workspace::qt::ServerManagerDialog::setServers(std::vector<te::da::DataSourceInfoPtr> servers)
{
  m_ui->m_serversListWidget->clear();

  for (std::size_t i = 0; i < servers.size(); ++i)
  {
    te::da::DataSourceInfoPtr ds = servers[i];

    QListWidgetItem* item = new QListWidgetItem(ds->getTitle().c_str());
    item->setData(Qt::UserRole, QVariant::fromValue(ds));

    m_ui->m_serversListWidget->addItem(item);
    //m_ui->m_serverComboBox->addItem(QString::fromStdString(ds->getTitle()), QVariant::fromValue(ds));
  }
}

int ta::workspace::qt::ServerManagerDialog::getActiveUserId()
{
  if (m_activeServer)
  {
    if (m_activeUser)
      return m_activeUser->getId();
  }
  return -1;
}

void ta::workspace::qt::ServerManagerDialog::setActiveUser(std::unique_ptr<ta::model::User> user)
{
  m_ui->m_userLineEdit->setText(QString::fromStdString(user->getName()));
  m_ui->m_emaiLlineEdit->setText(QString::fromStdString(user->getEMail()));
  m_activeUser = std::move(user);
}

bool ta::workspace::qt::ServerManagerDialog::hasChanges()
{
  return m_hasChanges;
}

void ta::workspace::qt::ServerManagerDialog::onAddToolButtonClicked(bool)
{
  ta::workspace::qt::ServerRegisterDialog dlg;

  if (dlg.exec() == QDialog::Accepted)
  {
    te::da::DataSourceInfoPtr server = dlg.getServer();

    if (server.get())
    {
      bool newServer = te::da::DataSourceInfoManager::getInstance().add(server);
      if (newServer)
      {
        te::da::DataSourcePtr ds = te::da::DataSourceFactory::make(server->getType());
        ds->setId(server->getId());
        ds->setConnectionInfo(server->getConnInfo());
        te::da::DataSourceManager::getInstance().insert(ds);

        QListWidgetItem* item = new QListWidgetItem(server->getTitle().c_str());
        item->setData(Qt::UserRole, QVariant::fromValue(server));

        m_ui->m_serversListWidget->addItem(item);
        //m_ui->m_serverComboBox->addItem(QString::fromStdString(server->getTitle()), QVariant::fromValue(server));
        m_hasChanges = true;
        setActiveUser(dlg.getActiveUser());
      }
      else
        QMessageBox::information(this, windowTitle(), tr("The selected server already exists."));
    }
  }
}

void ta::workspace::qt::ServerManagerDialog::onOkPushButtonClicked()
{
  //Acquiring the name and the eMail of the cconfigured user
  std::string name, eMail, password;
  name = m_ui->m_userLineEdit->text().toStdString();
  eMail = m_ui->m_emaiLlineEdit->text().toStdString();
  password = m_ui->m_newPassLineEdit->text().toStdString();

  if (!name.empty() && (name != m_activeUser->getName()))
    m_activeUser->setName(name);
  else
    m_ui->m_userLineEdit->setText(QString::fromStdString(m_activeUser->getName()));

  if (!eMail.empty() && (eMail != m_activeUser->getEMail()))
    m_activeUser->setEMail(eMail);
  else
    m_ui->m_userLineEdit->setText(QString::fromStdString(m_activeUser->getEMail()));

  bool tst1, tst2, tst3, tst4;
  tst1 = !password.empty(); // Has the user provided a new password?
  tst2 = password != ta::model::Globals::sm_taDefaultPassword; // Is the password different from the default password?
  tst3 = password == m_ui->m_confirmPassLineEdit->text().toStdString(); // Does the new password matches the confirmation?
  tst4 = m_activeUser->getPassword() != sha256(password); // Is the password different from the user's current password?

  if (tst1)
  {
    if (tst2)
    {
      if (tst3)
      {
        if (tst4)
          m_activeUser->setPassword(sha256(password));
        else
        {
          QMessageBox::warning(this, windowTitle(), tr("The password coulod not be updated, it matches the current password."));
          return;
        }
      }
      else
      {
        QMessageBox::warning(this, windowTitle(), tr("The password coulod not be updated, the password and the confirmation do not match."));
        return;
      }
    }
    else
    {
      QMessageBox::warning(this, windowTitle(), tr("The password coulod not be updated, it can not match the default password."));
      return;
    }
  }

  m_facade->update(m_activeUser.get());
  this->accept();
}

void ta::workspace::qt::ServerManagerDialog::onUpdateToolButtonClicked(bool)
{
  ta::workspace::qt::ServerRegisterDialog dlg;

  QListWidgetItem* oldItem = m_ui->m_serversListWidget->selectedItems()[0];
  QVariant v = oldItem->data(Qt::UserRole);
  te::da::DataSourceInfoPtr oldServer = v.value<te::da::DataSourceInfoPtr>();

  dlg.setServer(oldServer);

  dlg.exec();

  te::da::DataSourceInfoPtr server = dlg.getServer();

  //TODO: Verify the need to relace this function, or remove it entirely
  //if (server.get())
  //{
  //  int index = m_ui->m_serverComboBox->findData(oldItem->data(Qt::UserRole));
  //  m_ui->m_serverComboBox->removeItem(index);
  //  delete oldItem;
  //  QListWidgetItem* item = new QListWidgetItem(server->getTitle().c_str());
  //  item->setData(Qt::UserRole, QVariant::fromValue(server));

  //  m_ui->m_serversListWidget->addItem(item);
  //  m_ui->m_serverComboBox->addItem(QString::fromStdString(server->getTitle()), QVariant::fromValue(server));
  //}

  delete oldItem;
  m_hasChanges = true;
}

void ta::workspace::qt::ServerManagerDialog::onRemoveToolButtonClicked(bool)
{
  QList<QListWidgetItem*> items = m_ui->m_serversListWidget->selectedItems();

  for (int i = 0; i < items.size(); ++i)
  {
    //TODO: Verify the need to relace this function, or remove it entirely
    //int index = m_ui->m_serverComboBox->findData(items.at(i)->data(Qt::UserRole));
    //delete items.at(i);
    //m_ui->m_serverComboBox->removeItem(index);
  }

  m_hasChanges = true;
}

