/*  This file is part of the TerraAmazon application.

    See the Copyright file for details about the license of 
    this application.
 */

/*!
  \file terraamazon/ServerRegisterDialog.cpp

  \brief A Qt dialog used to configure the Server settings.
*/

// TerraAmazon
#include "../../common/Utils.h"
#include "../../model/facade/UserFacade.h"
#include "../../model/item/User.h"
#include "../../model/Globals.h"
#include "../../model/Utils.h"
#include "ServerRegisterWidget.h"
#include "ServerRegisterDialog.h"
#include "ui_ServerRegisterDialogForm.h"

// TerraLib
#include <terralib/common/Exception.h>
#include <terralib/common/PlatformUtils.h>
#include <terralib/common/Translator.h>
#include <terralib/dataaccess/dataset/DataSet.h>
#include <terralib/dataaccess/datasource/DataSource.h>
#include <terralib/dataaccess/datasource/DataSourceFactory.h>
#include <terralib/dataaccess/query/And.h>
#include <terralib/dataaccess/query/BinaryFunction.h>
#include <terralib/dataaccess/query/DataSetName.h>
#include <terralib/dataaccess/query/EqualTo.h>
#include <terralib/dataaccess/query/Field.h>
#include <terralib/dataaccess/query/Fields.h>
#include <terralib/dataaccess/query/From.h>
#include <terralib/dataaccess/query/FromItem.h>
#include <terralib/dataaccess/query/LiteralString.h>
#include <terralib/dataaccess/query/PropertyName.h>
#include <terralib/dataaccess/query/Select.h>
#include <terralib/dataaccess/query/Where.h>

// Boost
#include <boost/uuid/random_generator.hpp>
#include <boost/uuid/uuid_io.hpp>

// Qt
#include <QMessageBox>
#include <QGridLayout>

ta::workspace::qt::ServerRegisterDialog::ServerRegisterDialog(QWidget* parent, Qt::WindowFlags f)
  : QDialog(parent, f),
  m_ui(new Ui::ServerRegisterDialogForm)
{
  m_ui->setupUi(this);

  QGridLayout* layout = new QGridLayout(m_ui->m_widget);
  layout->setContentsMargins(0, 0, 0, 0);

  m_widget = new ServerRegisterWidget(m_ui->m_widget);

  layout->addWidget(m_widget);

  std::string icon = ta::common::FindInTerraAmazonPath("share/icons/svg/terraAmazom-colorido-horizontal.svg");
  m_ui->m_imgLabel->setPixmap(QIcon(icon.c_str()).pixmap(112, 100));
  m_ui->m_imgLabel->hide();

  connect(m_ui->m_okPushButton, SIGNAL(clicked()), this, SLOT(onOkPushButtonClicked()));
}

ta::workspace::qt::ServerRegisterDialog::~ServerRegisterDialog()
{
}

ta::workspace::qt::ServerRegisterWidget* ta::workspace::qt::ServerRegisterDialog::getWidget()
{
  return m_widget;
}

std::string ta::workspace::qt::ServerRegisterDialog::getHost()
{
  return m_widget->getHost();
}

std::string ta::workspace::qt::ServerRegisterDialog::getPort()
{
  return m_widget->getPort();
}

std::string ta::workspace::qt::ServerRegisterDialog::getUserName()
{
  return m_widget->getUser();
}

std::string ta::workspace::qt::ServerRegisterDialog::getPassword()
{
  return m_widget->getPassword();
}

std::string ta::workspace::qt::ServerRegisterDialog::getServerName()
{
  return m_widget->getServerName();
}

void ta::workspace::qt::ServerRegisterDialog::setServer(te::da::DataSourceInfoPtr server)
{
  std::map<std::string, std::string> connInfo = server->getConnInfo();
  m_widget->setHost(connInfo["PG_HOST"]);
  m_widget->setPort(connInfo["PG_PORT"]);
  m_widget->setServerName(connInfo["PG_DB_NAME"]);
}

std::map<std::string, std::string> ta::workspace::qt::ServerRegisterDialog::makeConnInfo(const std::string& host,
                                                                                         const std::string& port,
                                                                                         const std::string& user,
                                                                                         const std::string& password,
                                                                                         const std::string& dsName)
{
  std::map<std::string, std::string> connInfo;

  connInfo["PG_CONNECT_TIMEOUT"] = "5";
  connInfo["PG_HIDE_RASTER_TABLES"] = "FALSE";
  connInfo["PG_HIDE_SPATIAL_METADATA_TABLES"] = "FALSE";
  connInfo["PG_MAX_POOL_SIZE"] = "4";
  connInfo["PG_MIN_POOL_SIZE"] = "1";

  if (!host.empty())
    connInfo["PG_HOST"] = host;
  else
    connInfo["PG_HOST"] = "localhost";

  if (!port.empty())
    connInfo["PG_PORT"] = port;
  else
    connInfo["PG_PORT"] = "5432";

  if (!user.empty())
    connInfo["PG_USER"] = user;

  if (!password.empty())
    connInfo["PG_PASSWORD"] = password;

  if (!dsName.empty())
    connInfo["PG_DB_NAME"] = dsName;
  else
    throw te::common::Exception(TE_TR("Select a server!"));

  return connInfo;
}


te::da::DataSourceInfoPtr ta::workspace::qt::ServerRegisterDialog::getServer()
{
  return m_server;
}

std::unique_ptr<ta::model::User> ta::workspace::qt::ServerRegisterDialog::getActiveUser()
{
  return std::move(m_user);
}

void ta::workspace::qt::ServerRegisterDialog::onOkPushButtonClicked()
{
  std::string host = m_widget->getHost();
  std::string port = m_widget->getPort();
  std::string user = m_widget->getUser();
  std::string password = m_widget->getPassword();
  std::string dsName = m_widget->getServerName();
  std::string alias = m_widget->getServerAlias();
  std::map<std::string, std::string> connInfo;

  try
  {
    connInfo = makeConnInfo(host, port, ta::model::Globals::sm_taUser, ta::model::Globals::sm_taPassword, dsName);
  }
  catch (te::common::Exception e)
  {
    QMessageBox::warning(this, windowTitle(), e.what());
    return;
  }

  if (alias.empty())
  {
    alias = host + "@" + dsName;
  }

  te::da::DataSourcePtr ds(te::da::DataSourceFactory::make("POSTGIS"));

  try
  {
    ds->setConnectionInfo(connInfo);
    ds->open();
  }
  catch (...)
  {
    QMessageBox::warning(this, windowTitle(), tr("This is not a TerraAmazon Server."));
    return;
  }
  
  bool isTaUser =  ta::model::HasTerraAmazonUser(ds);

  if (!isTaUser)
  {
    QMessageBox::warning(this, windowTitle(), tr("This is not a TerraAmazon user"));
    return;
  }

  std::unique_ptr<ta::model::UserFacade> facade(new ta::model::UserFacade(ds));
  m_user = std::move(facade->getByName(user));

  if (m_user.get())
    m_user->setPassword(password);

  if (m_server.get() == 0)
  {
    m_server.reset(new te::da::DataSourceInfo);

    m_server->setConnInfo(connInfo);

    boost::uuids::basic_random_generator<boost::mt19937> gen;
    boost::uuids::uuid u = gen();
    std::string dsId = boost::uuids::to_string(u);

    m_server->setId(dsId);
    m_server->setTitle(alias);
    m_server->setAccessDriver("POSTGIS");
    m_server->setType("POSTGIS");
  }
  else
  {
    m_server->setConnInfo(connInfo);
    m_server->setTitle(alias);
  }

  accept();
}